/*

This is a simple SPyRO XMLRPC Client,
It uses the same uri to make http requests

Eric Sadit Tellez Avila donsadit@gmail.com or sadit@lsc.fie.umich.mx
3kong Corporation
This file is released in terms of LGPL licence

*/

var __SPyRO_URL = null;
var __SPyRO_ASYNC = true;

/* 
function SPyRO_init(optionlist)
Initializes the SPyRO module, the optionlist is a list of options name1=value1,name2=value2,...
The valid options are:
|url| is the location of the remote SPyRO server, it must be in the same domain of
the current location

*/
function SPyRO_init(optionlist)
{
    var url = null;
    eval(optionlist);
    __SPyRO_URL = url;
}

/*
function SPyRO_fatalError(TheError)
Is called to inform fatal errors

function SPyRO_errorHandler(errmsg, handler, args, request, response)
Global error handler
*/

function SPyRO_errorHandler(errmsg, handler, args, request, response)
{
    DOM3K_showError(errmsg);
}

function SPyRO_fatalError(TheError)
{
    throw TheError;
}

/*
function SPyRO_info(infodata)
This function is called to show info |info|
*/
function SPyRO_info(infodata)
{
    /*infodata;*/
}

function SPyRO_infodone(infodata)
{
    /**/
}

/*
function SPyRO_warning(TheWarning)
This function is called to show a warning |TheWarning|
*/
function SPyRO_warning(TheWarning)
{
    throw TheWarning;
}

/*
function SPyRO_getElementArray(elem)
Returns nodes Element of elem as an Array (preserve order)
*/
function SPyRO_getElementArray(elem)
{
    var a = new Array();
    var nodes = elem.childNodes;
    /*alert("nodes:" + nodes + ",x:" + elem.tagName+", type:"+elem.nodeType);*/
    for (var j = 0; j < nodes.length; j++) {
	var y = nodes.item(j);
	if (y.nodeType == elem.nodeType) {
	    a[a.length] = y;
	}
    }
    return a;
}

/*
function SPyRO_getTextData(elem)
Returns the text in all text nodes in |elem|
*/
function SPyRO_getTextData(elem)
{
    var sbuff = "";
    var nodes = elem.childNodes;
    for (var j = 0; j < nodes.length; j++) {
	var y = nodes.item(j);
	if (y.nodeType != elem.nodeType) {
	    sbuff += y.data;
	}
    }
    return sbuff;
}

/*
function SPyRO_decodeArray(arrayelem)

Change xmlrpc array to a javascript array
*/
function SPyRO_decodeArray(arrayelem)
{
    var res = new Array();
    var nodes = SPyRO_getElementArray(SPyRO_getElementArray(arrayelem)[0]);
    for (i in nodes) {
	res[res.length] = SPyRO_decodeObject(SPyRO_getElementArray(nodes[i])[0]);
    }
    return res;
}

/*
function SPyRO_decodeDict(structelem)

Change xmlrpc struct to a javascript associative array
*/

function SPyRO_decodeDict(structelem)
{
    var resxx = {};
    var nodes = SPyRO_getElementArray(structelem);
    var name;
    var value;
    for (m in nodes) {
	var members = SPyRO_getElementArray(nodes[m]);
	name = members[0];
	value = members[1];
	if (value.tagName == 'name') {
	    name = members[1];
	    value = members[0];
	}
	name = SPyRO_getTextData(name);
	value = SPyRO_decodeObject(SPyRO_getElementArray(value)[0]);
	/*axlert("name:" + name);
	axlert("value:" + value);*/
	resxx[name] = value;
    }
    return resxx;
}

/*
function SPyRO_decodeObject(elem)

Change xmlrpc value to a javascript value (guess type)
*/

function SPyRO_decodeObject(elem)
{
    var tagName = elem.tagName;

    if (tagName == 'struct') {
	return SPyRO_decodeDict(elem);
    } else if (tagName == 'array') {
	return SPyRO_decodeArray(elem);
    } else if (tagName == 'int') {
	return parseInt(SPyRO_getTextData(elem));
    } else if (tagName == 'float' || tagName == 'double') {
	return parseFloat(SPyRO_getTextData(elem));
    } else if (tagName == 'string') {
	return SPyRO_getTextData(elem);
    } else if (tagName == 'boolean') {
	x = SPyRO_getTextData(elem);
	return x == '1';
    } else if (tagName == 'nil') {
	return null;
    }
    SPyRO_fatalError('SPyRO_decodeObject: Unknown type ' + tagName);
    throw "Unknown type " + tagName;
}

/*

function SPyRO_decodeMsg(handler, errorhandler, args, request, response)

Change xmlrpc messages to the javascript representation
|handler| The handler to be called to manage the result of the response
|errorhandler| The errorhandler is called when an error occurres in the call.
         The prototype:
	     errorhandler(error_explanation, handler, args, request, response)
	 Where error_exception is the explanation of the error, and
	 handler, args, request and response are part of the spyro's request message
|args| Arguments to be passed to the handler => handler(dataofresponse,args)
|request| The original request
|httpreq| The XMLHttpRequest object performing the connection to the SPyRO server

*/

function SPyRO_decodeMsg(handler, errorhandler, args, request, httpreq)
{
    /*root.normalize();*/
    var root = httpreq.responseXML.documentElement;
    var res;
    res = SPyRO_getElementArray(SPyRO_getElementArray(root)[0])[0];
    res = SPyRO_getElementArray(SPyRO_getElementArray(res)[0])[0];
    res = SPyRO_decodeObject(res);
    return res;
}

/*
function SPyRO_encodeArray(arrayobj)

encode a javascript array to a xmlrpc array
*/
function SPyRO_encodeArray(arrayobj)
{
    var params = "";
    for (x in arrayobj) {
        var y = SPyRO_encodeObject(arrayobj[x]);
        /*alert(y);*/
        params+= "<value>" + y + "</value>\n";
    }
    return "<array><data>" + params + "</data></array>\n"; 
}

/*
function SPyRO_encodeField(argname, argvalue)

encode a javascript key-value pair to a xmlrpc field's struct
*/

function SPyRO_encodeField(argname, argvalue)
{
    // if argvalue type != string get type
    var arg = "<member>\n";
    arg+= "  <name>" + argname + "</name>\n";
    arg+= "  <value>" + SPyRO_encodeObject(argvalue) + "</value>\n";
    arg+= "</member>\n";
    return arg;
}

/*
function SPyRO_encodeDict(dictobj)
encode a javascript associative array to a xmlrpc struct
*/

function SPyRO_encodeDict(dictobj)
{
    var params = "";
    for (x in dictobj) {
	params += SPyRO_encodeField(x, dictobj[x]);
    }
    return "<struct>\n" + params + "</struct>\n";
}

/*
function SPyRO_encodeObject(obj)
encode a javascript value to its representation in xmlrpc
*/

function SPyRO_encodeObject(obj)
{
    var enc = null;
    var arreglo = 0;
    var typeofobj = typeof(obj);
    /*alert(typeofobj + obj)*/
    if (obj == null) {
	enc = "<nil/>";
    } else if (typeofobj == 'boolean') {
	if (obj) {
	    obj = 1;
	} else {
	    obj = 0;
	}
	enc = "<boolean>" + obj + "</boolean>"
    } else if (typeofobj == 'number') {
	if (obj % 1.0 == 0) {
	    enc = "<int>" + obj + "</int>";
	} else {
	    enc = "<double>" + obj + "</double>";
	}
    } else if (typeofobj == 'string') {
	enc = obj.replace(/&/g,"&amp;").replace(/</g,"&lt;").replace(/>/g,"&gt;");
    } else if (typeofobj == 'object') {
	arreglo = 1;
	for (var x in obj) {
	   if (x == '__fake__') {
	      arreglo = 0;
	      obj = new Array();
	      break;
	   }
           if ((parseInt(x) % 1) != 0) {
              arreglo = 0;
              break;
           }
        }

        if (arreglo) {
            enc = SPyRO_encodeArray(obj);
        } else {
            enc = SPyRO_encodeDict(obj);
        }
    }
    return enc;
}

/* 
function SPyRO_encodeMsg(msg, methodname)
Encodes the final message */
function SPyRO_encodeMsg(msg,methodname)
{
  var xmsg = "<methodCall><methodName>" + methodname + "</methodName>\n<params><param><value>\n" + SPyRO_encodeObject(msg) + "\n</value></param></params></methodCall>\n";
  return xmsg;
}

/*
function SPyRO_createXMLHttpRequest()
Creates a new XMLHttpRequest
modified from:
http://ajaxpatterns.org/XMLHttpRequest_Call#XML_Responses

*/
function SPyRO_createXMLHttpRequest()
{
   try {
       return new XMLHttpRequest();
   } catch(e) {}
   try {
       return new ActiveXObject("Msxml2.XMLHTTP");
   } catch (e) {}
   try {
       return new ActiveXObject("Microsoft.XMLHTTP");
   } catch (e) {}
   SPyRO_warning("XMLHttpRequest not supported");
   throw 'XMLHttpRequest not supported';
   return null;
}

/* function SPyRO_nextCallSync()
The next call to SPyRO will be in SYNC mode

*/
function SPyRO_nextCallSync()
{
    __SPyRO_ASYNC = false;
}

/*
function SPyRO_sendRequest(handler, errorhandler, args, spyrorequest, spyrooptions)
Send a request to the SPyRO server
|handler| Function to handle the result
|errorhandler| Function to handle errors
|args| Arguments to the handler
|spyrorequest| The request (in javascript format)
|spyrooptions| Options to modify the behaviour of the request. The options are explained in the documentation
               of the callMethod function
*/


function SPyRO_sendRequest(handler, errorhandler, args, requestargs, spyrooptions)
{
    var httpreq = SPyRO_createXMLHttpRequest();
    if (spyrooptions == null) spyrooptions = new Array();
    if (spyrooptions["url"] == null) spyrooptions["url"] = __SPyRO_URL;
    if (spyrooptions["async"] == null) spyrooptions["async"] = true;
    if (spyrooptions["encoder"] == null) spyrooptions["encoder"] = SPyRO_encodeMsg;
    if (spyrooptions["decoder"] == null) spyrooptions["decoder"] = SPyRO_decodeMsg;
    if (spyrooptions["querystring"] == null) spyrooptions["querystring"] = "?XMLRPC";
    if (spyrooptions["httpmethod"] == null) spyrooptions["httpmethod"] = "POST";
    // var msg = SPyRO_encodeMsg(params,"SPyRO_execute");
    var spyrorequest = spyrooptions["encoder"](requestargs[0], requestargs[1], spyrooptions);
    SPyRO_info("Sending request");
    httpreq.open(spyrooptions["httpmethod"], spyrooptions["url"] + spyrooptions["querystring"], spyrooptions["async"]);
    httpreq.send(spyrorequest);
    httpreq.onreadystatechange = function () {
        if (httpreq.readyState == 4) {
            SPyRO_infodone("Done");
		try {
			if (httpreq.status != 200) {
			    errorhandler("HTTP status " + httpreq.status + ": " + httpreq.statusText , handler, args, spyrorequest, httpreq);
			    return;
			}
		} catch (e) {
		    try {
			errorhandler("Error in the connection: " + httpreq.status + ", " + httpreq.statusText, handler, args, spyrorequest, httpreq);
			return;
		    } catch(ee) {
			errorhandler("Not connected: " + ee, handler, args, spyrorequest, httpreq);
		    }
		}
            var res = spyrooptions["decoder"](handler, errorhandler, args, spyrorequest, httpreq);
            var errname;
            var errcode;
            if (res['errcode'] != 0) { /* only for debug */
                errcode = res['errcode'];
                if (errcode == 0) {
                    errname =  'Successful';
                } else if (errcode == 1) {
                    errname = 'ErrorUnknownObject';
                } else if (errcode == 2) {
                    errname = 'ErrorUnknownAttribute';
                } else if (errcode == 3) {
                    errname = 'ErrorExceptionThrowed';
                } else if (errcode == 4) {
                    errname = 'ErrorUnknownRequestType';
                } else if (errcode == 5) {
                    errname = 'ObjectIsCallable';
                } else if (errcode == 6) {
                    errname = 'ErrorInTheAuthentication';
                } else {
                    errname = 'ErrorUnknownErrorCode(' + errcode + ')';
                }
                var errname = "SPyRO returns " + errname + "("+errcode+"):" + res['retvalue'];
                errorhandler(errname, handler, args, spyrorequest, httpreq, errcode, res['retvalue']);
                return;
            }
            if (handler) handler(res['retvalue'], args);
        }
    }
}

/* Methods of SPyRO

All SPyRO remote calls (callMethod, setAttr, getAttr) receives as first two arguments:
|handler| is the Handler to manage the response of SPyRO
|handlerargs| is an argument to be passed as second argument
of the  handler (the fist argument is the response of the call).

*/


/*
function callMethod(handler, handlerargs, objname, attrname, args, kwargs, auth, errorhandler)
Call a remote method. Arguments are:

handler. It's the handler to manage the response of the method call. It must receive 2 arguments (dataFromResponse, handlerargs)
handlerargs. It's the second argument to be passed to the handler, it can be any object.
objname. The name of the remote object. 
attrname. The name of the method to be called.
args. It's an array of arguments. It can be null
kwargs. It's a dictionary of arguments (keyword arguments). It can be null.
auth. It's a dictionary with arguments to perform authentication to requests. It can be null.
errorhandler. It's a function that handles errors in the call. If it's null a generic global handler (SPyRO_errorhandler) will be used.
spyrooptions. A set of options that modifies the behaviour of spyro remote calls. It's a hash set {name : value}
    url:
          It's the remote handler. Under tipical access privileges to the browser,
          only url under the same host are allowed. But you can still getting access
	  to different locations of web services.
    async:
          If true the request it's asynchronous, if it's false it will be synchronous.
*/
function callMethod(handler, handlerargs, objname, attrname, args, kwargs, auth, errorhandler, spyrooptions)
{
    if (args == null) {
    	args = new Array();
    }
    if (kwargs == null) {
    	kwargs = {'__fake__' : '__fake__'};
    }
    var params = new Array();
    params = {
        'objname' : objname,
        'attrname' : attrname,
	'args' : args,
	'kwargs' : kwargs,
	'auth' : auth,
	'local_server' : null,
	'reqtype' : 'CALL',
	'rettype' : 'VAL'
    };
    /*alert(params);*/
    if (errorhandler == null) errorhandler = SPyRO_errorHandler;
    SPyRO_sendRequest(handler, errorhandler, handlerargs, [params, "SPyRO_execute"], spyrooptions);
}

/*
function getAttr(handler, handlerargs, objname, attrname, auth, errorhandler, spyrooptions)
Get an attribute.

handler. Is the handler to manage the response of the method call. It must receive 2 arguments (dataFromResponse, handlerargs)
handlerargs. It's the second argument to be passed to the handler, it can be any object.
objname. The name of the remote object. 
attrname. The name of the attribute to be requested.
auth. It's a dictionary with arguments to perform authentication to requests. It can be null.
errorhandler. It's a function that handles errors in the call. If it's null a generic global handler (SPyRO_errorhandler) will be used.
spyrooptions. The same that callMethod
*/
function getAttr(handler, handlerargs, objname, attrname, auth, errorhandler, spyrooptions)
{
    var params = {
        'objname' : objname,
        'attrname': attrname,
	'auth' : auth,
	'local_server' : null,
	'reqtype' : 'GET',
	'rettype' : 'VAL'
    };
	/*    var msg = SPyRO_encodeMsg(params,"SPyRO_execute");*/
    if (errorhandler == null) errorhandler = SPyRO_errorHandler;
    SPyRO_sendRequest(handler, errorhandler, handlerargs, [params, "SPyRO_execute"], spyrooptions);
}

/* 
function setAttr(handler, handlerargs, objname, attrname, value, auth, errorhandler, spyrooptions)

Set an attribute

handler. Is the handler to manage the response of the method call. It must receive 2 arguments (dataFromResponse, handlerargs)
handlerargs. It's the second argument to be passed to the handler, it can be any object.
objname. The name of the remote object. 
attrname. The name of the attribute to set.
value. The value to set.
auth. It's a dictionary with arguments to perform authentication to requests. It can be null.
errorhandler. It's a function that handles errors in the call. If it's null a generic global handler (SPyRO_errorhandler) will be used.
spyrooptions. The same that callMethod

*/
function setAttr(handler, handlerargs, objname, attrname, setvalue, auth, errorhandler, spyrooptions)
{
    var params = {
        'objname' : objname,
        'attrname': attrname,
        'value': setvalue,
	'auth' : auth,
	'local_server' : null,
	'reqtype' : 'SET',
	'rettype' : 'VAL'
    };
    /*var msg = SPyRO_encodeMsg(params,"SPyRO_execute");*/
    if (errorhandler == null) errorhandler = SPyRO_errorHandler;
    SPyRO_sendRequest(handler, errorhandler, handlerargs, [params, "SPyRO_execute"], spyrooptions);
}

