import template3k
import warn 
import db3k
import time
from shttp import http
import SPyRO as spyro
import auth3k
import auth3k.authweb as authweb
import appconfig
_spyro_authentication = authweb.Auth()
# To set permisions to the global module or object, uncomment the following
# [ [allowed users list], [ allowed groups list], [ denied user list] ]
_spyro_authentication.perms['catalogos.cortes'] = [[], ['caja_admin'], []]

import catalogos
catalogos.index()
db = db3k.Connection(name="db/catalogos", database="db/catalogos", manager="sqlite3")
date_fmt = "%Y-%m-%d"

def _make_table(fields, results, outmap=None):
    res = ["""<table style="width: 100%"><tbody><tr>"""]
    for h in fields:
        res.append("<th>%s</th>"%h)
    res.append("</tr>")
    for row in results:
        res.append("<tr>")
        for idx, col in enumerate(row):
            if outmap:
                col = outmap(fields[idx], col)
            res.append("<td>%s</td>"%col)
        res.append("</tr>")
    return "".join(res)

def _make_html(**kwargs):
    temp = template3k.get_template(appconfig.template_admin)
    kwargs['CommentsCell'] = ''
    return temp.merge(kwargs)

def index(fechaini=None, fechafin=None, **kwargs):
    res = [
        """<a href="javascript: DOM3K_printById('Content');">Imprimir datos</a>"""
        """<h1 class="thirdtitle">Corte de Caja</h1>"""
        """<form action="javascript: void(0);" """
        """ onsubmit="callto({'handler': handle_commands}, 'catalogos.cortes', 'corte_rango', null, Easyform_formToDict(this));">"""
        """<a href="javascript: callto({'handler': handle_commands}, 'catalogos.cortes', 'corte_diario'); void(0);">Corte del d&iacute;a</a>. &nbsp; """
        """Rango fechas: """
        """Inicial: <input name="fechaini:date:mandatory" value="" type="text" onclick="Easyform_frameReadDate(DOM3K_readInput(this), DOM3K_updateInput, this, event.clientY, event.clientX);" />, """
        """Final: <input name="fechafin:date:mandatory" value="" type="text" onclick="Easyform_frameReadDate(DOM3K_readInput(this), DOM3K_updateInput, this, event.clientY, event.clientX);" />"""
        """<input name=":string" value="Corte" type="submit" />"""
        """</form>"""
        ]
    return _make_html(Title="Cortes de Caja",
                      Content="\n".join(res))

def _lista_venta_rango(fechaini, fechafin, **kwargs):
    fields = "xid, vendedorid, startdate, estado_venta, estado_fiscal, datos_factura"
    res = db.query("""SELECT %s FROM venta WHERE startdate >= ? AND startdate < ? ORDER BY xid DESC"""%fields,
                   (fechaini,fechafin))
    w = db.query("""SELECT vendedorid, nombre, email, SUM(cantidad*costo_final) as monto, COUNT(*) as numero FROM venta """
                 """LEFT JOIN venta_productos ON (venta.xid = idventa) """
                 """LEFT JOIN persona ON (vendedorid = persona.xid) """
                 """WHERE startdate >= ? AND startdate < ? GROUP BY vendedorid""",
                 (fechaini,fechafin))
    x = db.query("""SELECT estado_venta, SUM(cantidad*costo_final) as monto, COUNT(*) as numero FROM venta """
                 """LEFT JOIN venta_productos ON (xid = idventa) """
                 """WHERE startdate >= ? AND startdate < ? GROUP BY estado_venta""",
                 (fechaini,fechafin))
    y = db.query("""SELECT estado_fiscal, SUM(cantidad*costo_final) as monto, COUNT(*) as numero FROM venta """
                 """INNER JOIN venta_productos ON (xid = idventa) """
                 """WHERE startdate >= ? AND startdate < ? AND estado_venta = 'PAGADA' """
                 """GROUP BY estado_fiscal""",
                 (fechaini,fechafin))
    z = db.query("""SELECT nombre, SUM(cantidad*costo_final) as monto, cantidad, costo_final FROM venta """
                 """INNER JOIN venta_productos ON (venta.xid = idventa) """
                 """INNER JOIN catalogo_producto ON (venta_productos.idproducto = catalogo_producto.idproducto) """
                 """INNER JOIN producto ON (catalogo_producto.idproducto = producto.xid) """
                 """WHERE startdate >= ? AND startdate < ? AND estado_venta = 'PAGADA' GROUP BY producto.xid""",
                 (fechaini,fechafin))
    zz = db.query("""SELECT nombre, SUM(cantidad*costo_final) as monto, cantidad, costo_final FROM venta """
                  """INNER JOIN venta_productos ON (venta.xid = idventa) """
                  """INNER JOIN catalogo_producto ON (venta_productos.idproducto = catalogo_producto.idproducto) """
                  """INNER JOIN producto ON (catalogo_producto.idproducto = producto.xid) """
                  """WHERE startdate >= ? AND startdate < ? AND estado_venta <> 'PAGADA' GROUP BY producto.xid""",
                  (fechaini,fechafin))
    def gencontrol(loc):
        xid = loc['q'][0]
        return ["""<a href="javascript: callto({'handler': handle_commands}, 'catalogos.cortes', 'ver_pagos', ['%s']); void(0);">Ver&nbsp;Pagos</a>"""%xid,
                """<a href="javascript: callto({'handler': handle_commands}, 'catalogos.cortes', 'ver_productos', ['%s']); void(0);">"""
                """Ver&nbsp;Productos</a>"""%xid ]
    config={'control_generator': gencontrol}
    rowclasses = (('cell1', 'cell2'), 'header')
    return ( ('insert-list', 'Fechas de Corte',  "fecha inicial <= fecha ventas < fecha final" + 
              _make_table(['Estado', 'Fecha'],
                          [["Fecha Inicial", fechaini],
                           ["Fecha Final", fechafin]]), 'fechas', rowclasses),
             ('insert-list', 'Estad&iacute;sticas Estados Ventas',
              _make_table(['Estado Venta', 'Monto', 'Numero Ventas'], x), 'estadisticas-ventas', rowclasses),
             ('insert-list', 'Estad&iacute;sticas Estados Fiscales (Solo Pagadas)',
              _make_table(['Estado Fiscal', 'Monto', 'Numero Ventas'], y), 'estadisticas-fiscales', rowclasses),
             ('insert-list', 'Estad&iacute;sticas Vendedores (Solo Pagadas)',
              _make_table(['Id', 'Nombre', 'Email', 'Monto', 'Numero Ventas'], w), 'estadisticas-vendedores', rowclasses),
             ('insert-list', 'Productos Pagados',
              _make_table(['Nombre', 'Monto', 'Cantidad', 'Costo x Unidad'], z), 'estadisticas-productos-pagados', rowclasses),
             ('insert-list', 'Productos NO Pagados',
              _make_table(['Nombre', 'Monto', 'Cantidad', 'Costo x Unidad'], zz), 'estadisticas-productos-no-pagados', rowclasses),
             ('insert-list', 'Lista de Ventas', catalogos._out_list('venta', fields.split(", "), res, config=config), 'venta', rowclasses),
            )

def ver_productos(idventa):
    # config = {'indexable': False, 'editable': False, 'insertable': False, 'structured_search': False}
    res = catalogos.list_venta_productos({'idventa': idventa})
    return (('frame-list', 'Productos venta %s'%idventa, res, '', rowclasses),
            )

def ver_pagos(idventa):
    fields = "idventa, fecha_pago, costo, txid, type"
    res = db.query("SELECT %s type FROM pagos WHERE idventa = ?"%fields, (idventa,))
    res = [ [ str(x) for x in row ] for row in res ]
    config = {'indexable': lambda: False, 'editable': lambda: False, 'insertable': lambda: False, 'structured_search': lambda: False}
    res = catalogos._out_list('pagos', fields.split(", "), res, config=config)
    return (('frame-list', 'Pagos a venta %s'%idventa, res, '', rowclasses),
            )

import datetime
def corte_rango(fechaini, fechafin, **kwargs):
    fechafin = datetime.datetime(*time.strptime(fechafin, date_fmt)[:3]) + datetime.timedelta(1)
    fechafin = str(fechafin)
    return _lista_venta_rango(fechaini, fechafin)

def corte_diario():
    fechaini = time.strftime(date_fmt, time.localtime(time.time()))
    # (tm_year,tm_mon,tm_day,tm_hour,tm_min,tm_sec,tm_wday,tm_yday,tm_isdst)
    return corte_rango(fechaini, fechaini)
