#coding: utf8;

__doc__ = ''' Easyform server to lib3k.lib3k '''

import auth3k
import auth3k.authweb as authweb
_spyro_authentication = authweb.Auth()
# To set permisions to the global module or object, uncomment the following
# [ [allowed users list], [ allowed groups list], [ denied user list] ]
_spyro_authentication.permsobject['lib3k.lib3k'] = [[],['admin'],[]]
# Note: In module objects the object name must be the name of the object. If an authobject is
# manages many objects, there may exists many object names

# For method level management
# _spyro_authentication.perms['methodname'] = [[], ['admin'], []]

# Gets automatically the connection because is already charged
import re
import warn
import db3k
import time
import locale3k
import template3k
import SPyRO as spyro
import appconfig
import ft3k
import types
# import marshal

dbmanager='sqlite3'
if dbmanager.startswith('sqlite'): # param style qmark
    # sqlite can't use indexes with like operator,  we use a work around (see _like_cond function for details)
    likechar = None 
    _kwargs = {'timeout': 60 * 60}
else:
    likechar = '%%'
    _kwargs = {}
# elif param style = qmark: likechar = '%s'

# db = db3k.Connection(name='db/lib3k',
#                      database='db/lib3k',
#                      user='',
#                      password='',
#                      host=None,
#                      manager=dbmanager,
#                      kwargs=_kwargs)

ftname="ft%s"%('db/lib3k')
def ftpool_create(name):
    conn = db3k.create2(name=name)
    spyro.register_exit_handler(ftpool_exit_handler)
    return conn

ftpool=db3k.ThreadPool(ftname, dbmanager, ftpool_create)
dbftindex = db3k.Connection(
                     name=ftname,
                     database='db/lib3k',
                     user='',
                     password='',
                     host=None,
                     manager=dbmanager,
                     pool=ftpool)
db = dbftindex # transactional database connection

def ftpool_exit_handler():
    try:
        ftpool.delconn(1)
    except KeyError, e:
        warn.debug("ftpool_exit_handler ", 'lib3k.lib3k', ": ", e)

#######################################
#######################################
#############
############  INDEX
#############
#######################################
#######################################
def index(**kwargs):
    objname = 'lib3k.lib3k'
    title = 'Lib3K.Lib3K'
    temp = template3k.get_template(appconfig.template_admin)
    return temp.merge(PageTitle=objname.title(),
                      Menu='''<td><a href="index">%s</a></td>'''%objname.title(),
                      JavascriptInit='''callto({'handler': handle_commands},'lib3k.lib3k','showpage')''',
                      DevelQuote='',
                      Title='Module ' + title,
                      Content=''
                      )

def showpage(**kwargs):
    objname = 'lib3k.lib3k'
    itime = time.time()
    warn.debug("""If you want to index for a first time an existing database, """
               """you must enable the following piece of code (%s.showpage)"""%objname)
    if False:
        for x in ['lib3k_user', 'lib3k_resource', 'lib3k_annotation']:
            # print "DDDDD manage ft: ", x
            eval("manage_fulltext_%s"%x)(0, 1)
    print "***********************************> index time: ", time.time() - itime
    
    res = []
    res.append(modconfig['lib3k_user']['output_showpage']('Lib3K User',list_lib3k_user(),'lib3k_user'))
    res.append(modconfig['lib3k_resource']['output_showpage']('Lib3K Resource',list_lib3k_resource(),'lib3k_resource'))
    res.append(modconfig['lib3k_annotation']['output_showpage']('Lib3K Annotation',list_lib3k_annotation(),'lib3k_annotation'))
    return res


#######################################
#####
####   COMMON CUSTOMIZABLE FUNCTIONS
#####
#######################################
def _sql_quote(f):
    return _strip_field(f).replace("'","''")

def _like_cond(f, a):
    if likechar is None:
        b = a[:-1] + chr(ord(a[-1]) + 1)
        return "(%s >= '%s' AND %s < '%s')"%(f, a, f, b)
    else:
        return "%s LIKE '%s%s'"%(f, a, likechar)

def _join_cond(d,joinmethod=None):
    try:
        items = d.items()
    except AttributeError:
        return ' 1=1 '
    if len(items) == 0:
        return ' 1=1 '
    else:
        if joinmethod == 'LIKE':
            return ' AND '.join([ _like_cond(_sql_quote(k), _sql_quote(v)) for k, v in d.items() ])
        else:
            return ' AND '.join([ "%s = '%s'"%(_sql_quote(k), _sql_quote(v)) for k, v in d.items() ])

def _quote_html(f):
    return f.replace("<","&lt;").replace(">","&gt;") #.replace("&","&amp;")

def _strip_field(value):
    if value is None:
        return ''
    elif type(value) in types.StringTypes:
        for enc in ("utf8","latin1","latin2","latin3","utf16"):
            try:
                return value.encode(enc).decode(enc)
                # return value.encode(enc)
            except (UnicodeDecodeError,UnicodeEncodeError), e:
                warn.debug("Ignoring and Retrying on UnicodeError (enc, value, error): ", repr((enc, value, e)))
        enc = "utf8"
        return value.decode(enc,"replace")
        # return value.encode(enc,"replace")
    else:
        return str(value)

def _strip_dict(d):
    for k,v in d.items():
        try:
            d[k] = _strip_field(d[k])
        except AttributeError:
            pass
    return d

def _pager(begin, count, total, template, num=15):
    xres = [ "Viewing %s elements (starting at %sth) from %s total entries "%(min(count,total-begin), begin, total) ]
    start = max(begin - num * count, 0)
    for x in range(num*2):
        page_number = start / count + 1
        if start == begin:
            xres.append("%s&nbsp;"%page_number)
        else:
            xres.append( template % locals() )
        start += count
        if start > total:
            if page_number == 1: return []
            break
    return xres

def _get_config(config, tabname):
    if config is None:
        return modconfig[tabname]
    try:
        config['__sentinel__']
    except KeyError:
        _config = dict(modconfig[tabname])
        _config.update(config)
    else:
        _config = config
    return _config
        
def _control_generator(xlocals):
    res = [
        (r"""<a href="javascript: showformmodify('%(objname)s', '%(suffix)s', """
        r"""['update', %(rowid)s]);void(0);">%(_moditem)s</a>"""%xlocals),
        (r"""<a href="javascript: calldelete('%(objname)s', '%(suffix)s',  """
        r"""[%(rowid)s]);void(0);">%(_delitem)s</a>"""%xlocals)
        ]
    if xlocals['config']['indexable']():
        res.append("""<a href="javascript: void(0)" """
                   """onclick="guigo_related(this, '%(objname)s', '%(tabname)s');">%(_related)s</a>"""%xlocals)
    return res

######
##### Global data
######

objname = 'lib3k.lib3k'

#######################################################################################
#######################################################################################
######
######   FORMS
######
#######################################################################################
#######################################################################################
modconfig = {}

def _map_result(tabname, fieldname, value, outmap=None, config=None):
    objname = 'lib3k.lib3k'
    config = _get_config(config, tabname)
    foreigns = config['foreign_mapper']
    try:
        # This is the default behaviour, you must create caches of this values in order to speed up
        # this requests
        foreigntable, foreignfield = foreigns[fieldname]
        res = _preview(foreigntable, foreignfield, {foreignfield:value})
        display = _strip_field(res[0][1])
        if outmap is None:
            return display
        else:
            return outmap(locals())            
    except (KeyError, IndexError), e:
        return _strip_field(value)

def _preview(tabname, id, condition_dict=None, display=None, config=None, preview_maxsize=None, **kwargs):
    # tabname = <-python:keyword id="QuotedTabname" />
    config = _get_config(config, tabname)
    if condition_dict is None: condition_dict = {}
    # auth = spyro.getrequest()['auth']
    # assert _spyro_authentication._rawauth(auth,  objname, preview_<-python:keyword id="Tabname" />)
    # cond = ' 1=1 '
    condition = _join_cond(condition_dict)
    if display is None:
        fields = config['preview_fields']
        display = ",".join(fields)
    else:
        fields = None
    res = []
    internal = config['internal_fields']
    if not preview_maxsize:
        preview_maxsize = config['preview_maxsize']
    for qres in db.hquery('SELECT %s,%s FROM %s WHERE %s'%(_sql_quote(id), _sql_quote(display), tabname, condition)):
        row = []
        if fields is None:
            fields = qres.keys()
            fields.sort()
        for qk in fields:
            qv = qres[qk]
            if qk in internal:
                continue
            if qv:
                row.append(_map_result(tabname, qk, qv)) # posible infinite loops due to foreign resolution
        res.append([qres[id], ", ".join(row)[:preview_maxsize]])
    return res

def _get_rowid(tabname, cond, config=None):
    """ Returns the rowid identifing any row in the table, it's calculated
    using the condition or the insertion data. This method must be replaced
    using knowledge about the table. You must override this function with
    the correct behaviour if the default is not what you need.
    """
    config = _get_config(config, tabname)
    idname = config['list_fields'][0]
    if isinstance(cond, types.DictType):
        try:
            data = cond[idname]
        except KeyError:
            return repr(_strip_dict(cond)).replace("u'","'")
    else:
        data = cond[0]
    x = repr({str(idname): _strip_field(data)})
    x = x.replace("u'","'")
    return x

def _out_list(tabname, fields, query, dopager=None, config=None):
    objname = 'lib3k.lib3k'
    suffix = tabname
    config = _get_config(config, tabname)
    locale = locale3k.getweblocale()
    _newitem = locale3k.getname("actions", "new", locale)
    _delitem = locale3k.getname("actions", "delete", locale)
    _moditem = locale3k.getname("actions", "modify", locale)
    _related = locale3k.getname("actions","related",locale)
    editable = config['editable']()
    internal = config['internal_fields']
    try:
        hgen = config['header_generator']
        # it must has the same prototype than locale3k.getname and can be None, when it's None
        # the generation of headers will be skipped
    except KeyError:
        hgen = locale3k.getname
    out = []
    l = []
    viewfields = []
    for i, f in enumerate(fields):
        if f in internal:
            continue
        viewfields.append((i, f))
        if hgen:
            out.append(hgen(tabname, f, locale))
    l.append( config['output_table_header'](out) )
    try:
        outmap = config['outmap']
    except KeyError:
        outmap = lambda x: """<a href="javascript: callto({'showresults': DOM3K_createFrame}, '%(objname)s', 'list_%(foreigntable)s', [{'%(foreignfield)s':  '%(value)s'}]); void(0);">%(display)s</a>"""%x
    for q in query:
        if len(q) > 1:
            index = 1
        else:
            index = 0
        name_name = fields[index]
        data_name = q[index]
        r = []
        try:
            dgen = config['map_result']
        except KeyError:
            dgen = _map_result
        for i, f in viewfields:
            r.append(dgen(tabname, f, q[i], outmap, config))
        l.append(config['output_table_row'](r, q, fields))
        if editable:
            rowid = _get_rowid(tabname, q, config)
            controls = config['control_generator'](locals())
            l.append(config['output_table_control_row'](r, controls, q, fields))
            # l.append('<tr><td style="text-align: right;" colspan="%s">%s</td></tr>'%(len(r), " &nbsp; ".join(controls)))
    # cursor.close()
    if editable:
        newitemlink = r"""<a href="javascript: showformmodify('%(objname)s', '%(suffix)s'); ">%(_newitem)s</a>"""%locals()
    else:
        newitemlink = ""
    res = []
    searchname = locale3k.getname("actions","search",locale)
    insertable=config['insertable']()
    if insertable:
        res.append(newitemlink)
    if config['structured_search']():
        searchlink = r""" <a href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\'Search %(tabname)s\'')}}, '%(objname)s', 'form_search_%(tabname)s'); void(0);">%(searchname)s</a> &nbsp; """%locals()
    else:
        searchlink = ''
    if config['indexable']():
        searchname_fulltext = locale3k.getname("actions","search_fulltext",locale)
        searchlink += r""" <a href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\'Search Fulltext on %(tabname)s\'')}}, '%(objname)s', 'form_search_fulltext_%(tabname)s'); void(0);">%(searchname_fulltext)s</a> &nbsp;"""%locals()
    if dopager:
        searchlink += dopager() 
        res.append(searchlink)
    res.append(config['output_table'](l, query, fields))
    if insertable:
        res.append(newitemlink)
    res.append(" &nbsp; ")
    res.append(searchlink)
    return "".join(res)

def _list_select(db, tabname, config, condition, orderby, begin, count):
    limits = 'LIMIT %d OFFSET %d'%(count,begin)
    if orderby is None:
        orderby = ''
    else:
        orderby = 'ORDER BY ' + (','.join(map(_sql_quote, orderby)))
    rowcount = db.query('SELECT COUNT(*) FROM %s WHERE %s %s'%(tabname, condition, orderby))[0][0]
    query = db.query('SELECT %s FROM %s WHERE %s %s %s'%(",".join(config['list_fields']), tabname, condition, orderby, limits))
    return query, rowcount

def _list(tabname, condition_dict=None, orderby=None, usecondmethod=None, list_condition_dict=None, begin=None, count=None, config=None, **kwargs):
    objname = 'lib3k.lib3k'
    suffix = tabname
    config = _get_config(config, tabname)
    # auth = spyro.getrequest()['auth']
    emptycond = False
    if condition_dict is None:
        condition_dict = {}
        emptycond = True
    if (list_condition_dict is None) or (len(list_condition_dict) == 0):
        list_condition_dict = [condition_dict]
    else:
        emptycond = False
    _orderby = orderby
    fields = config['list_fields']
    if count is None:
        count = config['count']
    if begin is None:
        begin = 0
    condition = " OR ".join( [ "(%s)"%_join_cond(cd, usecondmethod) for cd in list_condition_dict ] )
    # Using cursors yields to a very poor performance (maybe we use it in a wrong way).
    # Using count and limit to enhance performance (2 queries :( )
    # cursor = db.conn.cursor()
    # cursor.execute('SELECT %s FROM %s WHERE %s %s'%(",".join(fields),tabname,condition,orderby))
    # This was tested in postgresql
    if config['list_with_empty_condition']() or not emptycond:
        query, rowcount = config['select'](db, tabname, config, condition, orderby, begin, count)
    else:
        rowcount = 0
        query = []
    if usecondmethod is None:
        strcondmethod = 'null'
    else:
        strcondmethod = repr(usecondmethod)
    if len(list_condition_dict) == 0 or len(list_condition_dict[0]) == 0:
        str_list_condition_dict = 'null'
    else:
        str_list_condition_dict = repr(list_condition_dict)
    if _orderby is None:
        strorderby = 'null'
    else:
        strorderby = repr(_orderby)
    locs = locals()
    def dopager():
        res = []
        if len(list_condition_dict) > 0:
            qx = []
            for qc in list_condition_dict:
                try:
                    items = qc.items()
                except AttributeError:
                    items = ()
                qx.append(", ".join([ "%s: %s"%(k,v) for k,v in items ]))
            if len(qx) > 0:
                res.append("<div>%s</div>"%". ".join(qx))
        tem = r"""<a href="javascript: calllist('%(objname)s', '%(suffix)s', [null,%(strorderby)s,%(strcondmethod)s,%(str_list_condition_dict)s], { 'begin' : %%(start)s, 'count' : %%(count)s } ) ; void(0);" >%%(page_number)s</a> """%locs
        res.extend(_pager(begin, count, rowcount, tem))
        return "".join(res)
    return _out_list(tabname, fields, query, dopager, config)

def _out_choose_search(tabname, fields, query, idtag, previewsize, config=None):
    config = _get_config(config, tabname)
    # auth = spyro.getrequest()['auth']
    # assert _spyro_authentication._rawauth(auth,  objname, list_<-python:keyword id="Tabname" />)
    locale = locale3k.getweblocale()
    l = [ '<table><tbody><tr>' ]
    internal = config['internal_fields']    
    for f in fields:
        if f in internal:
            continue
        l.append("<th>%s</th>"%locale3k.getname(tabname, f, locale))
    l.append('</tr>')
    # if begin > 0: cursor.scroll(begin)
    # for q in cursor.fetchmany(count):
    outmap = lambda x: """<a href="javascript: callto({'showresults': DOM3K_createFrame}, '%(objname)s', 'list_%(foreigntable)s', [{'%(foreignfield)s':  '%(value)s'}]); void(0);">%(display)s</a>"""%x
    for q in query:
        name_id = fields[0]
        data_id = str(q[0])
        r = []
        for i,f in enumerate(fields):
            if f in internal:
                continue
            r.append(_map_result(tabname, f, q[i])) # outmap
        name = ". ".join(r)
        r.append("""<a href="javascript: void(0);" onclick="DOM3K_replaceListOptions([[%s,'%s']], %s); DOM3K_deleteFrame(this); void(0);">%s</a>"""%(repr(data_id),_strip_field(name),repr(idtag),locale3k.getname("general","choose_this",locale)))
        l.append('<tr>' + " ".join(map(lambda s: "<td>%s</td>"%s, r)) + '</tr>')
    if len(query) >= previewsize:
        l.append("""<tr><td>...</td><td class="thirdtitle" colspan="%s">%s</td></tr>"""%(len(fields),locale3k.getname("general","too_many_results", locale)))
    l.append("</tbody></table>")
    return "".join(l)

def _form(tabname, mode, condition_dict, config=None, **kwargs):
    config = _get_config(config, tabname)
    if mode is None:
        mode = 'insert'
    dict_default = dict(config['dict_default'])
    if condition_dict is None:
        cond = '{}'
    else:
        cond = condition_dict
    if config['editable']():
        dict_default['__onsubmit__'] = r"DOM3K_commitform(this, '%s','%s',['%s',Easyform_formToDict(this),%s]);"%(objname,tabname,mode,cond)
    else:
        dict_default['__onsubmit__'] = r"DOM3K_deleteFrame(this);"
        dict_default['__editmode__'] = ""
    if mode == 'update':
        qupdate = 'SELECT * FROM %s WHERE %s'%(tabname,_join_cond(condition_dict))
        dict_default.update(db.hquery(qupdate)[0])
        _strip_dict(dict_default)
    elif mode != 'insert':
        raise AttributeError, "Attribute mode is set to an unknown value (insert|update) '%s'"%mode
    dict_default['__id__'] = str(time.time())
    return config['form_element'].merge(dict_default)

####
##
## Particular methods
##
###################################################################################################
###############################################################################################
###
### Table lib3k_user UI
###

### Configuring lib3k_user

# TODO: Change all values of editable, insertable, indexable, etc to predicates
# TODO: in order to be able to give a behaviour dependable of the user, or auth

modconfig['lib3k_user'] = {
    'internal_fields' : [u'xid'],
    'dict_default': { '' : '<python:locale id="general.submit" />','city' : '','xid' : '','email' : '','country' : '',
                     '__objname__': 'lib3k.lib3k',
                     '__tabname__': 'lib3k_user',
                     #'__fulltextsearch__': '', # disables the fulltext search in the modify frame
                    },
    'editable': lambda: True,
    'insertable': lambda: True,
    'indexable': lambda: True,
    'structured_search': lambda: True,
    'ftindex': ft3k.sqlindex(name='lib3k_user', conn=dbftindex),
    'ftsearch_size': 10,
    'count': 30,
    'list_fields': ['xid', 'email', 'city', 'country'],
    'preview_fields': ['xid', 'email', 'city', 'country'],
    'list_with_empty_condition': lambda: True,
    'control_generator': _control_generator,
    'preview_maxsize': 64,
    'output_table_header': lambda r: "<tr>%s</tr>"%("<th>%s</th>"%("</th><th>".join(r))),
    'output_table_row': lambda r, q, fields: "<tr>%s</tr>"%("<td>%s</td>"%("</td><td>".join(r))),
    'output_table_control_row': lambda r, controls, q, fields: '<tr><td style="text-align: right;" colspan="%s">%s</td></tr>'%(len(r), " &nbsp; ".join(controls)),
    'output_table': lambda r, query, fields: """<table style="width: 100%%"><tbody>%s</tbody></table>"""%("".join(r)),
    'output_showpage': lambda title, data, id: ('insert',
                                                       {'title': title,
                                                        'data': data,
                                                        'id': id,
                                                        'rowclasses': (['cell1', 'cell1', 'cell2', 'cell2'], 'header')
                                                       }),
    'select': _list_select,
    'foreign_mapper': {},
    'form_element': template3k.get_template("""lib3k_user:form_element""",
                                            """
            <strong>Lib3K User</strong>
            <form method="POST" enctype="multipart/form-data" id="form-lib3k_user<python:keyword id='__id__' />" action="javascript: void(0);" onsubmit="<python:keyword id="__onsubmit__" />" name="lib3k_user" >
                <table><tbody>
            
<tr><td>
<python:locale id="lib3k_user.email">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Email'} ]
</python:locale>:</td><td><input name="email:string:" type="text" value="<python:keyword id="email" />" size="40" /></td></tr>


<tr><td>
<python:locale id="lib3k_user.city">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'City'} ]
</python:locale>:</td><td><input name="city:string:" type="text" value="<python:keyword id="city" />" size="40" /></td></tr>


<tr><td>
<python:locale id="lib3k_user.country">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Country'} ]
</python:locale>:</td><td><input name="country:string:" type="text" value="<python:keyword id="country" />" size="40" /></td></tr>

<tr><td><input type="submit" name=":submit" value="<python:locale id="general.submit" />" /></td></tr>
                </tbody></table>
            </form>
            <python:keyword id="__bottom__"></python:keyword>
            """),
    'form_element_search': template3k.get_template("""lib3k_user:form_element_search""",
                                                   """
            <strong>Lib3K User</strong>
            <form method="POST" enctype="multipart/form-data" id="form-lib3k_user<python:keyword id='__id__' />" action="javascript: void(0);" onsubmit="<python:keyword id="__onsubmit__" />" name="lib3k_user" >
                <table><tbody>
            
<tr><td>
<python:locale id="lib3k_user.email">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Email'} ]
</python:locale>:</td><td><input name="email:string:" type="text" value="<python:keyword id="email" />" size="40" /></td></tr>


<tr><td>
<python:locale id="lib3k_user.city">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'City'} ]
</python:locale>:</td><td><input name="city:string:" type="text" value="<python:keyword id="city" />" size="40" /></td></tr>


<tr><td>
<python:locale id="lib3k_user.country">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Country'} ]
</python:locale>:</td><td><input name="country:string:" type="text" value="<python:keyword id="country" />" size="40" /></td></tr>

<tr><td><input type="submit" name=":submit" value="<python:locale id="general.submit" />" /></td></tr>
                </tbody></table>
            </form>
            <python:keyword id="__bottom__"></python:keyword>
            """),
    '__sentinel__': True
    }

### End configuration

###
### Beginning methods/functions

#_spyro_authentication.perms['form_lib3k_user'] = False
#_spyro_authentication.perms['list_lib3k_user'] = False
#_spyro_authentication.perms['modify_lib3k_user'] = False
#_spyro_authentication.perms['delete_lib3k_user'] = False
 

def _list_method_lib3k_user(mode, condition_dict, config=None):
    tabname = 'lib3k_user'
    config = _get_config(config, tabname)
    return (config['output_showpage'](tabname, _list(tabname, condition_dict, config=config), tabname), )

def form_lib3k_user(mode = None, condition_dict = None, **kwargs):
    tabname = 'lib3k_user'
    kwargs['config'] = _get_config(None, tabname)
    return _form(tabname, mode, condition_dict, **kwargs)

def modify_lib3k_user(mode, data, cond, dbconn=None, doreturn=1, **kwargs):
    tabname = 'lib3k_user'
    config = _get_config(None, tabname)
    assert config['editable'](), "View only entity, can't %s"%mode
    tabname = 'lib3k_user'
    return _modify_lib3k_user(mode, data, cond, dbconn, doreturn, config=config, **kwargs)

def _modify_lib3k_user(mode, data, cond, dbconn=None, doreturn=1, config=None, **kwargs):
    tabname = 'lib3k_user'
    config = _get_config(config, tabname)
    if dbconn is None:
        dbconn = db
    # auth = spyro.getrequest()['auth']
    # assert _spyro_authentication._rawauth(auth,  objname, modify_lib3k_user)
    if mode == 'update':
        if None in (data,cond): raise Exception("Unable to update table %s with empty data"%tabname)
        if len(cond) == 0: raise Exception("Unable to update %s without condition"%tabname)
        # It's a good idea overwrite user id fields with information of the authentications
        # For example:
        #cond['userid'] = spyro.getrequest()['auth'][0]
        x = dbconn.update(tabname, _strip_dict(data), cond)
        if config['indexable']():
            rowid = _get_rowid(tabname, cond, config)
            ftindex = config['ftindex']
            ftindex.handle_update(_preview(tabname, '1', data, config=config, preview_maxsize=512)[0][1], rowid)
    elif mode == 'insert':
        if data is None: raise Exception("Unable to insert into table %s with empty data"%tabname)
        #cond['userid'] = spyro.getrequest()['auth'][0]
        x = dbconn.insert(tabname, _strip_dict(data))
        if config['indexable']():
            xdata = _preview(tabname, config['list_fields'][0], data, config=config, preview_maxsize=512)[0]
            rowid = _get_rowid(tabname, xdata, config)
            ftindex = config['ftindex']
            ftindex.handle_insert(xdata[1], rowid)
    else:
        raise Exception("Unknown action to modify %s"%tabname)
    if doreturn:
        return _list_method_lib3k_user(mode, cond)
    else:
        return None

def delete_lib3k_user(cond, dbconn = None, doreturn = 1, **kwargs):
    tabname = 'lib3k_user'
    config = _get_config(None, tabname)
    assert config['editable'](), "View only entity, can't delete"
    return _delete_lib3k_user(cond, dbconn, doreturn, config=config, **kwargs)

def _delete_lib3k_user(cond, dbconn = None, doreturn = 1, config = None, **kwargs):
    tabname = 'lib3k_user'
    config = _get_config(config, tabname)
    if None is cond: raise Exception("Unable to delete table with empty condition")
    if len(cond) == 0: raise Exception("Unable to delete without condition")
    # auth = spyro.getrequest()['auth']
    # assert _spyro_authentication._rawauth(auth,  objname, delete_lib3k_user)
    if dbconn is None:
        dbconn = db
    if config['indexable']():
        rowid = _get_rowid(tabname, cond, config)
        try:
            xdata = _preview(tabname, '1', cond, config=config, preview_maxsize=512)[0][1]
        except IndexError:
            pass
        else:
            ftindex = config['ftindex']
            ftindex.handle_delete(xdata, rowid)
    x = dbconn.execute('DELETE FROM %s WHERE %s'%(tabname, _join_cond(cond)))
    if doreturn:
        return _list_method_lib3k_user("delete", cond)
    else:
        return None

def list_lib3k_user(*args, **kwargs):
    tabname = 'lib3k_user'
    kwargs['config'] = _get_config(None, tabname)
    # assert _spyro_authentication._rawauth(auth,  objname, list_lib3k_user)
    return _list(tabname, *args, **kwargs)


def form_search_lib3k_user(method=None, *args, **kwargs):
    tabname = 'lib3k_user'
    config = _get_config(None, tabname)
    dict_default = dict(config['dict_default']) # dictionary
    dict_default['__editmode__'] = ""
    if method == "choose_search":
        idtag = args[0]
        residtag = 'choose-search-%s'%idtag
        dict_default['__onsubmit__'] = (r"callto({'showresults': function(d){ DOM3K_updateElement(d,'%s'); """
                                        r"""DOM3K_BackgroundRowColors('%s',['rgb(200,200,230);','inherit']);}}, """
                                        r"""'%s','choose_search_%s',null,{'condition_dict': Easyform_formToDict(this), 'idtag': %s });"""
                                        r""" """%(residtag,residtag,objname,tabname,repr(idtag)))
        dict_default['__bottom__'] = """<div id="%s"></div>"""%residtag
    else:
        dict_default['__onsubmit__'] = r"callto({'handler': handle_commands}, '%s','search_%s',[Easyform_formToDict(this), null, 'LIKE']);"%(objname,tabname)
    dict_default['__id__'] = str(time.time())
    x = config['form_element_search'].merge(dict_default)
    return x

def manage_fulltext_lib3k_user(drop_index, reindex):
    tabname = 'lib3k_user'
    config = _get_config(None, tabname)
    ftindex = config['ftindex']
    if drop_index:
        ftindex.drop_index()
    if reindex:
        ftindex.create_index()
        for x in _preview(tabname, config['list_fields'][0], config=config, preview_maxsize=512):
            _rowid = _get_rowid(tabname, x)
            ftindex.handle_insert(x[1], _rowid)

def form_search_fulltext_lib3k_user(output='list', idtag='', query='',**kwargs):
    tabname = 'lib3k_user'
    dict_default = {'query': query, 'output': output, 'idtag': idtag}
    if output == "choose_search":
        residtag = 'choose-search-%s'%idtag
        dict_default['__onsubmit__'] = (r"""callto({'handler': handle_commands_closure('command=\'frame\'')}, '%s', 'search_fulltext_%s', """
                                        r"""null, Easyform_formToDict(this));"""%(objname, tabname))
        dict_default['__bottom__'] = """<div id="%s"></div>"""%residtag
        dict_default['__idtag__'] = """<input name="idtag:string:mandatory" value="%s" type="hidden" />"""%idtag
        dict_default['residtag'] = residtag
    else:
        dict_default['residtag'] = tabname
        dict_default['__onsubmit__'] = r"callto({'handler': handle_commands}, '%s', 'search_fulltext_%s', null, Easyform_formToDict(this));"%(objname, tabname)
        dict_default['__idtag__'] = ""
        dict_default['__bottom__'] = ""
    dict_default['__id__'] = str(time.time())
    return """<form action="javascript: void(0);" onsubmit="%(__onsubmit__)s">
    <input name="output:string:mandatory" value="%(output)s" type="hidden" />
    <input name="query:string:mandatory" value="%(query)s" size="40" type="text" />
    <input name="residtag:string" value="residtag" type="hidden" />
    %(__idtag__)s
    <input name=":submit" value="Search" type="submit" />
    </form>
    %(__bottom__)s
    """%dict_default
    
def search_fulltext_lib3k_user(query, output="", idtag="", residtag="", **kwargs):
    tabname = 'lib3k_user'
    config = _get_config(None, tabname)
    ftindex = config['ftindex']
    size = config['ftsearch_size']
    searchres = ftindex.search(query, size)
    # if len(searchres) == 0:
    #     return ""
    res = []
    fields = config['list_fields']
    for rank, cond in searchres:
        cond = eval(cond)
        res.extend( db.query("""SELECT %s FROM %s WHERE %s LIMIT 1"""%(", ".join(fields),tabname,_join_cond(cond,"="))) )
    # we loose the ranking, but we standardize the output
    if output == 'raw':
        return res
    if output == 'choose_search':
        return _out_choose_search(tabname, fields, res, idtag, size)
    else:
        x = _out_list(tabname, fields, res, idtag)
        if len(res) >= size:
            x+="<br/>" + locale3k.getname("general","too_many_results")
        return (config['output_showpage'](tabname, x, tabname), )

def search_lib3k_user(cond,orderby=None,usecondmethod=None,**kwargs):
    tabname = 'lib3k_user'
    config = _get_config(None, tabname)
    x = list_lib3k_user(cond, orderby, usecondmethod=usecondmethod)
    return (config['output_showpage']('Search results', x, tabname), )

def choose_search_lib3k_user(idtag, condition_dict = None, method="LIKE", outmethod="simple",
                                                  previewsize=10, list_condition_dict = None, **kwargs):
    tabname = 'lib3k_user'
    config = _get_config(None, tabname)
    # auth = spyro.getrequest()['auth']
    # assert _spyro_authentication._rawauth(auth,  objname, list_lib3k_user)
    emptycond = False
    if condition_dict is None:
        condition_dict = {}
        emptycond = True
    if (list_condition_dict is None) or (len(list_condition_dict) == 0):
        list_condition_dict = [condition_dict]
    else:
        emptycond = False
    fields = ['xid', 'email', 'city', 'country']
    count = previewsize
    begin = 0
    condition = " OR ".join( [ "(%s)"%_join_cond(cd, method) for cd in list_condition_dict ] )
    limits = 'LIMIT %d OFFSET %d'%(count,begin)
    if config['list_with_empty_condition']() or not emptycond:
        query = db.query('SELECT %s FROM %s WHERE %s %s'%(",".join(fields),tabname,condition,limits))
    else:
        query = []
    return _out_choose_search(tabname, fields, query, idtag, previewsize)

def preview_lib3k_user(*args, **kwargs):
    tabname = 'lib3k_user'
    kwargs['config'] = _get_config(None, tabname)
    # kwargs['display'] = 'email'
    return _preview(tabname, *args, **kwargs)

###
### end of lib3k_user UI
###
###############################################################################################
###############################################################################################
###############################################################################################
###############################################################################################
###
### Table lib3k_resource UI
###

### Configuring lib3k_resource

# TODO: Change all values of editable, insertable, indexable, etc to predicates
# TODO: in order to be able to give a behaviour dependable of the user, or auth

modconfig['lib3k_resource'] = {
    'internal_fields' : [u'xid', u'sha1', u'contenttype'],
    'dict_default': { '' : '<python:locale id="general.submit" />','sha1' : '','contenttype' : '','description' : '','url' : '','keywords' : '','owner' : '','xid' : '','access_policies' : '','foreignid' : '',
                     '__objname__': 'lib3k.lib3k',
                     '__tabname__': 'lib3k_resource',
                     #'__fulltextsearch__': '', # disables the fulltext search in the modify frame
                    },
    'editable': lambda: True,
    'insertable': lambda: True,
    'indexable': lambda: True,
    'structured_search': lambda: True,
    'ftindex': ft3k.sqlindex(name='lib3k_resource', conn=dbftindex),
    'ftsearch_size': 10,
    'count': 30,
    'list_fields': ['xid', 'foreignid', 'sha1', 'url', 'contenttype', 'owner', 'description', 'keywords', 'access_policies'],
    'preview_fields': ['xid', 'foreignid', 'sha1', 'url', 'contenttype', 'owner', 'description', 'keywords', 'access_policies'],
    'list_with_empty_condition': lambda: True,
    'control_generator': _control_generator,
    'preview_maxsize': 64,
    'output_table_header': lambda r: "<tr>%s</tr>"%("<th>%s</th>"%("</th><th>".join(r))),
    'output_table_row': lambda r, q, fields: "<tr>%s</tr>"%("<td>%s</td>"%("</td><td>".join(r))),
    'output_table_control_row': lambda r, controls, q, fields: '<tr><td style="text-align: right;" colspan="%s">%s</td></tr>'%(len(r), " &nbsp; ".join(controls)),
    'output_table': lambda r, query, fields: """<table style="width: 100%%"><tbody>%s</tbody></table>"""%("".join(r)),
    'output_showpage': lambda title, data, id: ('insert',
                                                       {'title': title,
                                                        'data': data,
                                                        'id': id,
                                                        'rowclasses': (['cell1', 'cell1', 'cell2', 'cell2'], 'header')
                                                       }),
    'select': _list_select,
    'foreign_mapper': {u'owner': (u'lib3k_user', u'xid')},
    'form_element': template3k.get_template("""lib3k_resource:form_element""",
                                            """
            <strong>Lib3K Resource</strong>
            <form method="POST" enctype="multipart/form-data" id="form-lib3k_resource<python:keyword id='__id__' />" action="javascript: void(0);" onsubmit="<python:keyword id="__onsubmit__" />" name="lib3k_resource" >
                <table><tbody>
            
<tr><td>
<python:locale id="lib3k_resource.foreignid">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Foreignid'} ]
</python:locale>:</td><td><input name="foreignid:int:" type="text" value="<python:keyword id="foreignid" />" size="40" /></td></tr>


<tr><td>
<python:locale id="lib3k_resource.url">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Url'} ]
</python:locale>:</td><td><input name="url:string:" type="text" value="<python:keyword id="url" />" size="40" /></td></tr>


<tr><td>
<python:locale id="lib3k_resource.owner">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Owner'} ]
</python:locale>:</td><td><select onfocus="callto({'rawhandler': createhandlerselect('<python:keyword id="owner" />'), 'handlerargs': this},'<python:keyword id="__objname__" />','preview_lib3k_user',['xid'])" name="owner:int"  id="select-lib3k_user_xid_<python:keyword id='__id__' />" ></select>
<a style="text-decoration: none;" href="javascript: DOM3K_replaceListOptions([['','']],'select-lib3k_user_xid_<python:keyword id="__id__" />'); void(0);"><python:locale id="actions.clear" /></a><python:keyword id="__editmode__">

/
<a style="text-decoration: none;" href="javascript: showformmodify('<python:keyword id="__objname__" />', 'lib3k_user'); void(0);"><python:locale id="actions.new" /></a></python:keyword>

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_lib3k_user', ['choose_search','select-lib3k_user_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search" /></a><python:keyword id="__fulltextsearch__">

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_fulltext_lib3k_user', ['choose_search','select-lib3k_user_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search_fulltext" /></a></python:keyword>

</td></tr>


<tr><td>
<python:locale id="lib3k_resource.description">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Description'} ]
</python:locale>:</td><td><textarea cols="50" rows="12" name="description:text:" ><python:keyword id="description" /></textarea></td></tr>


<tr><td>
<python:locale id="lib3k_resource.keywords">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Keywords'} ]
</python:locale>:</td><td><textarea cols="50" rows="12" name="keywords:text:" ><python:keyword id="keywords" /></textarea></td></tr>


<tr><td>
<python:locale id="lib3k_resource.access_policies">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Access Policies'} ]
</python:locale>:</td><td><input name="access_policies:string:" type="text" value="<python:keyword id="access_policies" />" size="40" /></td></tr>

<tr><td><input type="submit" name=":submit" value="<python:locale id="general.submit" />" /></td></tr>
                </tbody></table>
            </form>
            <python:keyword id="__bottom__"></python:keyword>
            """),
    'form_element_search': template3k.get_template("""lib3k_resource:form_element_search""",
                                                   """
            <strong>Lib3K Resource</strong>
            <form method="POST" enctype="multipart/form-data" id="form-lib3k_resource<python:keyword id='__id__' />" action="javascript: void(0);" onsubmit="<python:keyword id="__onsubmit__" />" name="lib3k_resource" >
                <table><tbody>
            
<tr><td>
<python:locale id="lib3k_resource.foreignid">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Foreignid'} ]
</python:locale>:</td><td><input name="foreignid:int:" type="text" value="<python:keyword id="foreignid" />" size="40" /></td></tr>


<tr><td>
<python:locale id="lib3k_resource.url">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Url'} ]
</python:locale>:</td><td><input name="url:string:" type="text" value="<python:keyword id="url" />" size="40" /></td></tr>


<tr><td>
<python:locale id="lib3k_resource.owner">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Owner'} ]
</python:locale>:</td><td><select onfocus="callto({'rawhandler': createhandlerselect('<python:keyword id="owner" />'), 'handlerargs': this},'<python:keyword id="__objname__" />','preview_lib3k_user',['xid'])" name="owner:int"  id="select-lib3k_user_xid_<python:keyword id='__id__' />" ></select>
<a style="text-decoration: none;" href="javascript: DOM3K_replaceListOptions([['','']],'select-lib3k_user_xid_<python:keyword id="__id__" />'); void(0);"><python:locale id="actions.clear" /></a><python:keyword id="__editmode__">

/
<a style="text-decoration: none;" href="javascript: showformmodify('<python:keyword id="__objname__" />', 'lib3k_user'); void(0);"><python:locale id="actions.new" /></a></python:keyword>

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_lib3k_user', ['choose_search','select-lib3k_user_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search" /></a><python:keyword id="__fulltextsearch__">

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_fulltext_lib3k_user', ['choose_search','select-lib3k_user_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search_fulltext" /></a></python:keyword>

</td></tr>


<tr><td>
<python:locale id="lib3k_resource.description">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Description'} ]
</python:locale>:</td><td><textarea cols="50" rows="12" name="description:text:" ><python:keyword id="description" /></textarea></td></tr>


<tr><td>
<python:locale id="lib3k_resource.keywords">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Keywords'} ]
</python:locale>:</td><td><textarea cols="50" rows="12" name="keywords:text:" ><python:keyword id="keywords" /></textarea></td></tr>


<tr><td>
<python:locale id="lib3k_resource.access_policies">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Access Policies'} ]
</python:locale>:</td><td><input name="access_policies:string:" type="text" value="<python:keyword id="access_policies" />" size="40" /></td></tr>

<tr><td><input type="submit" name=":submit" value="<python:locale id="general.submit" />" /></td></tr>
                </tbody></table>
            </form>
            <python:keyword id="__bottom__"></python:keyword>
            """),
    '__sentinel__': True
    }

### End configuration

###
### Beginning methods/functions

#_spyro_authentication.perms['form_lib3k_resource'] = False
#_spyro_authentication.perms['list_lib3k_resource'] = False
#_spyro_authentication.perms['modify_lib3k_resource'] = False
#_spyro_authentication.perms['delete_lib3k_resource'] = False
 

def _list_method_lib3k_resource(mode, condition_dict, config=None):
    tabname = 'lib3k_resource'
    config = _get_config(config, tabname)
    return (config['output_showpage'](tabname, _list(tabname, condition_dict, config=config), tabname), )

def form_lib3k_resource(mode = None, condition_dict = None, **kwargs):
    tabname = 'lib3k_resource'
    kwargs['config'] = _get_config(None, tabname)
    return _form(tabname, mode, condition_dict, **kwargs)

def modify_lib3k_resource(mode, data, cond, dbconn=None, doreturn=1, **kwargs):
    tabname = 'lib3k_resource'
    config = _get_config(None, tabname)
    assert config['editable'](), "View only entity, can't %s"%mode
    tabname = 'lib3k_resource'
    return _modify_lib3k_resource(mode, data, cond, dbconn, doreturn, config=config, **kwargs)

def _modify_lib3k_resource(mode, data, cond, dbconn=None, doreturn=1, config=None, **kwargs):
    tabname = 'lib3k_resource'
    config = _get_config(config, tabname)
    if dbconn is None:
        dbconn = db
    # auth = spyro.getrequest()['auth']
    # assert _spyro_authentication._rawauth(auth,  objname, modify_lib3k_resource)
    if mode == 'update':
        if None in (data,cond): raise Exception("Unable to update table %s with empty data"%tabname)
        if len(cond) == 0: raise Exception("Unable to update %s without condition"%tabname)
        # It's a good idea overwrite user id fields with information of the authentications
        # For example:
        #cond['userid'] = spyro.getrequest()['auth'][0]
        x = dbconn.update(tabname, _strip_dict(data), cond)
        if config['indexable']():
            rowid = _get_rowid(tabname, cond, config)
            ftindex = config['ftindex']
            ftindex.handle_update(_preview(tabname, '1', data, config=config, preview_maxsize=512)[0][1], rowid)
    elif mode == 'insert':
        if data is None: raise Exception("Unable to insert into table %s with empty data"%tabname)
        #cond['userid'] = spyro.getrequest()['auth'][0]
        x = dbconn.insert(tabname, _strip_dict(data))
        if config['indexable']():
            xdata = _preview(tabname, config['list_fields'][0], data, config=config, preview_maxsize=512)[0]
            rowid = _get_rowid(tabname, xdata, config)
            ftindex = config['ftindex']
            ftindex.handle_insert(xdata[1], rowid)
    else:
        raise Exception("Unknown action to modify %s"%tabname)
    if doreturn:
        return _list_method_lib3k_resource(mode, cond)
    else:
        return None

def delete_lib3k_resource(cond, dbconn = None, doreturn = 1, **kwargs):
    tabname = 'lib3k_resource'
    config = _get_config(None, tabname)
    assert config['editable'](), "View only entity, can't delete"
    return _delete_lib3k_resource(cond, dbconn, doreturn, config=config, **kwargs)

def _delete_lib3k_resource(cond, dbconn = None, doreturn = 1, config = None, **kwargs):
    tabname = 'lib3k_resource'
    config = _get_config(config, tabname)
    if None is cond: raise Exception("Unable to delete table with empty condition")
    if len(cond) == 0: raise Exception("Unable to delete without condition")
    # auth = spyro.getrequest()['auth']
    # assert _spyro_authentication._rawauth(auth,  objname, delete_lib3k_resource)
    if dbconn is None:
        dbconn = db
    if config['indexable']():
        rowid = _get_rowid(tabname, cond, config)
        try:
            xdata = _preview(tabname, '1', cond, config=config, preview_maxsize=512)[0][1]
        except IndexError:
            pass
        else:
            ftindex = config['ftindex']
            ftindex.handle_delete(xdata, rowid)
    x = dbconn.execute('DELETE FROM %s WHERE %s'%(tabname, _join_cond(cond)))
    if doreturn:
        return _list_method_lib3k_resource("delete", cond)
    else:
        return None

def list_lib3k_resource(*args, **kwargs):
    tabname = 'lib3k_resource'
    kwargs['config'] = _get_config(None, tabname)
    # assert _spyro_authentication._rawauth(auth,  objname, list_lib3k_resource)
    return _list(tabname, *args, **kwargs)


def form_search_lib3k_resource(method=None, *args, **kwargs):
    tabname = 'lib3k_resource'
    config = _get_config(None, tabname)
    dict_default = dict(config['dict_default']) # dictionary
    dict_default['__editmode__'] = ""
    if method == "choose_search":
        idtag = args[0]
        residtag = 'choose-search-%s'%idtag
        dict_default['__onsubmit__'] = (r"callto({'showresults': function(d){ DOM3K_updateElement(d,'%s'); """
                                        r"""DOM3K_BackgroundRowColors('%s',['rgb(200,200,230);','inherit']);}}, """
                                        r"""'%s','choose_search_%s',null,{'condition_dict': Easyform_formToDict(this), 'idtag': %s });"""
                                        r""" """%(residtag,residtag,objname,tabname,repr(idtag)))
        dict_default['__bottom__'] = """<div id="%s"></div>"""%residtag
    else:
        dict_default['__onsubmit__'] = r"callto({'handler': handle_commands}, '%s','search_%s',[Easyform_formToDict(this), null, 'LIKE']);"%(objname,tabname)
    dict_default['__id__'] = str(time.time())
    x = config['form_element_search'].merge(dict_default)
    return x

def manage_fulltext_lib3k_resource(drop_index, reindex):
    tabname = 'lib3k_resource'
    config = _get_config(None, tabname)
    ftindex = config['ftindex']
    if drop_index:
        ftindex.drop_index()
    if reindex:
        ftindex.create_index()
        for x in _preview(tabname, config['list_fields'][0], config=config, preview_maxsize=512):
            _rowid = _get_rowid(tabname, x)
            ftindex.handle_insert(x[1], _rowid)

def form_search_fulltext_lib3k_resource(output='list', idtag='', query='',**kwargs):
    tabname = 'lib3k_resource'
    dict_default = {'query': query, 'output': output, 'idtag': idtag}
    if output == "choose_search":
        residtag = 'choose-search-%s'%idtag
        dict_default['__onsubmit__'] = (r"""callto({'handler': handle_commands_closure('command=\'frame\'')}, '%s', 'search_fulltext_%s', """
                                        r"""null, Easyform_formToDict(this));"""%(objname, tabname))
        dict_default['__bottom__'] = """<div id="%s"></div>"""%residtag
        dict_default['__idtag__'] = """<input name="idtag:string:mandatory" value="%s" type="hidden" />"""%idtag
        dict_default['residtag'] = residtag
    else:
        dict_default['residtag'] = tabname
        dict_default['__onsubmit__'] = r"callto({'handler': handle_commands}, '%s', 'search_fulltext_%s', null, Easyform_formToDict(this));"%(objname, tabname)
        dict_default['__idtag__'] = ""
        dict_default['__bottom__'] = ""
    dict_default['__id__'] = str(time.time())
    return """<form action="javascript: void(0);" onsubmit="%(__onsubmit__)s">
    <input name="output:string:mandatory" value="%(output)s" type="hidden" />
    <input name="query:string:mandatory" value="%(query)s" size="40" type="text" />
    <input name="residtag:string" value="residtag" type="hidden" />
    %(__idtag__)s
    <input name=":submit" value="Search" type="submit" />
    </form>
    %(__bottom__)s
    """%dict_default
    
def search_fulltext_lib3k_resource(query, output="", idtag="", residtag="", **kwargs):
    tabname = 'lib3k_resource'
    config = _get_config(None, tabname)
    ftindex = config['ftindex']
    size = config['ftsearch_size']
    searchres = ftindex.search(query, size)
    # if len(searchres) == 0:
    #     return ""
    res = []
    fields = config['list_fields']
    for rank, cond in searchres:
        cond = eval(cond)
        res.extend( db.query("""SELECT %s FROM %s WHERE %s LIMIT 1"""%(", ".join(fields),tabname,_join_cond(cond,"="))) )
    # we loose the ranking, but we standardize the output
    if output == 'raw':
        return res
    if output == 'choose_search':
        return _out_choose_search(tabname, fields, res, idtag, size)
    else:
        x = _out_list(tabname, fields, res, idtag)
        if len(res) >= size:
            x+="<br/>" + locale3k.getname("general","too_many_results")
        return (config['output_showpage'](tabname, x, tabname), )

def search_lib3k_resource(cond,orderby=None,usecondmethod=None,**kwargs):
    tabname = 'lib3k_resource'
    config = _get_config(None, tabname)
    x = list_lib3k_resource(cond, orderby, usecondmethod=usecondmethod)
    return (config['output_showpage']('Search results', x, tabname), )

def choose_search_lib3k_resource(idtag, condition_dict = None, method="LIKE", outmethod="simple",
                                                  previewsize=10, list_condition_dict = None, **kwargs):
    tabname = 'lib3k_resource'
    config = _get_config(None, tabname)
    # auth = spyro.getrequest()['auth']
    # assert _spyro_authentication._rawauth(auth,  objname, list_lib3k_resource)
    emptycond = False
    if condition_dict is None:
        condition_dict = {}
        emptycond = True
    if (list_condition_dict is None) or (len(list_condition_dict) == 0):
        list_condition_dict = [condition_dict]
    else:
        emptycond = False
    fields = ['xid', 'foreignid', 'sha1', 'url', 'contenttype', 'owner', 'description', 'keywords', 'access_policies']
    count = previewsize
    begin = 0
    condition = " OR ".join( [ "(%s)"%_join_cond(cd, method) for cd in list_condition_dict ] )
    limits = 'LIMIT %d OFFSET %d'%(count,begin)
    if config['list_with_empty_condition']() or not emptycond:
        query = db.query('SELECT %s FROM %s WHERE %s %s'%(",".join(fields),tabname,condition,limits))
    else:
        query = []
    return _out_choose_search(tabname, fields, query, idtag, previewsize)

def preview_lib3k_resource(*args, **kwargs):
    tabname = 'lib3k_resource'
    kwargs['config'] = _get_config(None, tabname)
    # kwargs['display'] = 'foreignid'
    return _preview(tabname, *args, **kwargs)

###
### end of lib3k_resource UI
###
###############################################################################################
###############################################################################################
###############################################################################################
###############################################################################################
###
### Table lib3k_annotation UI
###

### Configuring lib3k_annotation

# TODO: Change all values of editable, insertable, indexable, etc to predicates
# TODO: in order to be able to give a behaviour dependable of the user, or auth

modconfig['lib3k_annotation'] = {
    'internal_fields' : [],
    'dict_default': { '' : '<python:locale id="general.submit" />','resourceid' : '','annotation_owner' : '','annotation' : '',
                     '__objname__': 'lib3k.lib3k',
                     '__tabname__': 'lib3k_annotation',
                     #'__fulltextsearch__': '', # disables the fulltext search in the modify frame
                    },
    'editable': lambda: True,
    'insertable': lambda: True,
    'indexable': lambda: True,
    'structured_search': lambda: True,
    'ftindex': ft3k.sqlindex(name='lib3k_annotation', conn=dbftindex),
    'ftsearch_size': 10,
    'count': 30,
    'list_fields': ['resourceid', 'annotation_owner', 'annotation'],
    'preview_fields': ['resourceid', 'annotation_owner', 'annotation'],
    'list_with_empty_condition': lambda: True,
    'control_generator': _control_generator,
    'preview_maxsize': 64,
    'output_table_header': lambda r: "<tr>%s</tr>"%("<th>%s</th>"%("</th><th>".join(r))),
    'output_table_row': lambda r, q, fields: "<tr>%s</tr>"%("<td>%s</td>"%("</td><td>".join(r))),
    'output_table_control_row': lambda r, controls, q, fields: '<tr><td style="text-align: right;" colspan="%s">%s</td></tr>'%(len(r), " &nbsp; ".join(controls)),
    'output_table': lambda r, query, fields: """<table style="width: 100%%"><tbody>%s</tbody></table>"""%("".join(r)),
    'output_showpage': lambda title, data, id: ('insert',
                                                       {'title': title,
                                                        'data': data,
                                                        'id': id,
                                                        'rowclasses': (['cell1', 'cell1', 'cell2', 'cell2'], 'header')
                                                       }),
    'select': _list_select,
    'foreign_mapper': {u'resourceid': (u'lib3k_resource', u'xid'), u'annotation_owner': (u'lib3k_user', u'xid')},
    'form_element': template3k.get_template("""lib3k_annotation:form_element""",
                                            """
            <strong>Lib3K Annotation</strong>
            <form method="POST" enctype="multipart/form-data" id="form-lib3k_annotation<python:keyword id='__id__' />" action="javascript: void(0);" onsubmit="<python:keyword id="__onsubmit__" />" name="lib3k_annotation" >
                <table><tbody>
            
<tr><td>
<python:locale id="lib3k_annotation.resourceid">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Resourceid'} ]
</python:locale>:</td><td><select onfocus="callto({'rawhandler': createhandlerselect('<python:keyword id="resourceid" />'), 'handlerargs': this},'<python:keyword id="__objname__" />','preview_lib3k_resource',['xid'])" name="resourceid:int"  id="select-lib3k_resource_xid_<python:keyword id='__id__' />" ></select>
<a style="text-decoration: none;" href="javascript: DOM3K_replaceListOptions([['','']],'select-lib3k_resource_xid_<python:keyword id="__id__" />'); void(0);"><python:locale id="actions.clear" /></a><python:keyword id="__editmode__">

/
<a style="text-decoration: none;" href="javascript: showformmodify('<python:keyword id="__objname__" />', 'lib3k_resource'); void(0);"><python:locale id="actions.new" /></a></python:keyword>

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_lib3k_resource', ['choose_search','select-lib3k_resource_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search" /></a><python:keyword id="__fulltextsearch__">

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_fulltext_lib3k_resource', ['choose_search','select-lib3k_resource_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search_fulltext" /></a></python:keyword>

</td></tr>


<tr><td>
<python:locale id="lib3k_annotation.annotation_owner">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Annotation Owner'} ]
</python:locale>:</td><td><select onfocus="callto({'rawhandler': createhandlerselect('<python:keyword id="annotation_owner" />'), 'handlerargs': this},'<python:keyword id="__objname__" />','preview_lib3k_user',['xid'])" name="annotation_owner:int"  id="select-lib3k_user_xid_<python:keyword id='__id__' />" ></select>
<a style="text-decoration: none;" href="javascript: DOM3K_replaceListOptions([['','']],'select-lib3k_user_xid_<python:keyword id="__id__" />'); void(0);"><python:locale id="actions.clear" /></a><python:keyword id="__editmode__">

/
<a style="text-decoration: none;" href="javascript: showformmodify('<python:keyword id="__objname__" />', 'lib3k_user'); void(0);"><python:locale id="actions.new" /></a></python:keyword>

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_lib3k_user', ['choose_search','select-lib3k_user_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search" /></a><python:keyword id="__fulltextsearch__">

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_fulltext_lib3k_user', ['choose_search','select-lib3k_user_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search_fulltext" /></a></python:keyword>

</td></tr>


<tr><td>
<python:locale id="lib3k_annotation.annotation">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Annotation'} ]
</python:locale>:</td><td><textarea cols="50" rows="12" name="annotation:text:" ><python:keyword id="annotation" /></textarea></td></tr>

<tr><td><input type="submit" name=":submit" value="<python:locale id="general.submit" />" /></td></tr>
                </tbody></table>
            </form>
            <python:keyword id="__bottom__"></python:keyword>
            """),
    'form_element_search': template3k.get_template("""lib3k_annotation:form_element_search""",
                                                   """
            <strong>Lib3K Annotation</strong>
            <form method="POST" enctype="multipart/form-data" id="form-lib3k_annotation<python:keyword id='__id__' />" action="javascript: void(0);" onsubmit="<python:keyword id="__onsubmit__" />" name="lib3k_annotation" >
                <table><tbody>
            
<tr><td>
<python:locale id="lib3k_annotation.resourceid">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Resourceid'} ]
</python:locale>:</td><td><select onfocus="callto({'rawhandler': createhandlerselect('<python:keyword id="resourceid" />'), 'handlerargs': this},'<python:keyword id="__objname__" />','preview_lib3k_resource',['xid'])" name="resourceid:int"  id="select-lib3k_resource_xid_<python:keyword id='__id__' />" ></select>
<a style="text-decoration: none;" href="javascript: DOM3K_replaceListOptions([['','']],'select-lib3k_resource_xid_<python:keyword id="__id__" />'); void(0);"><python:locale id="actions.clear" /></a><python:keyword id="__editmode__">

/
<a style="text-decoration: none;" href="javascript: showformmodify('<python:keyword id="__objname__" />', 'lib3k_resource'); void(0);"><python:locale id="actions.new" /></a></python:keyword>

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_lib3k_resource', ['choose_search','select-lib3k_resource_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search" /></a><python:keyword id="__fulltextsearch__">

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_fulltext_lib3k_resource', ['choose_search','select-lib3k_resource_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search_fulltext" /></a></python:keyword>

</td></tr>


<tr><td>
<python:locale id="lib3k_annotation.annotation_owner">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Annotation Owner'} ]
</python:locale>:</td><td><select onfocus="callto({'rawhandler': createhandlerselect('<python:keyword id="annotation_owner" />'), 'handlerargs': this},'<python:keyword id="__objname__" />','preview_lib3k_user',['xid'])" name="annotation_owner:int"  id="select-lib3k_user_xid_<python:keyword id='__id__' />" ></select>
<a style="text-decoration: none;" href="javascript: DOM3K_replaceListOptions([['','']],'select-lib3k_user_xid_<python:keyword id="__id__" />'); void(0);"><python:locale id="actions.clear" /></a><python:keyword id="__editmode__">

/
<a style="text-decoration: none;" href="javascript: showformmodify('<python:keyword id="__objname__" />', 'lib3k_user'); void(0);"><python:locale id="actions.new" /></a></python:keyword>

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_lib3k_user', ['choose_search','select-lib3k_user_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search" /></a><python:keyword id="__fulltextsearch__">

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_fulltext_lib3k_user', ['choose_search','select-lib3k_user_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search_fulltext" /></a></python:keyword>

</td></tr>


<tr><td>
<python:locale id="lib3k_annotation.annotation">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Annotation'} ]
</python:locale>:</td><td><textarea cols="50" rows="12" name="annotation:text:" ><python:keyword id="annotation" /></textarea></td></tr>

<tr><td><input type="submit" name=":submit" value="<python:locale id="general.submit" />" /></td></tr>
                </tbody></table>
            </form>
            <python:keyword id="__bottom__"></python:keyword>
            """),
    '__sentinel__': True
    }

### End configuration

###
### Beginning methods/functions

#_spyro_authentication.perms['form_lib3k_annotation'] = False
#_spyro_authentication.perms['list_lib3k_annotation'] = False
#_spyro_authentication.perms['modify_lib3k_annotation'] = False
#_spyro_authentication.perms['delete_lib3k_annotation'] = False
 

def _list_method_lib3k_annotation(mode, condition_dict, config=None):
    tabname = 'lib3k_annotation'
    config = _get_config(config, tabname)
    return (config['output_showpage'](tabname, _list(tabname, condition_dict, config=config), tabname), )

def form_lib3k_annotation(mode = None, condition_dict = None, **kwargs):
    tabname = 'lib3k_annotation'
    kwargs['config'] = _get_config(None, tabname)
    return _form(tabname, mode, condition_dict, **kwargs)

def modify_lib3k_annotation(mode, data, cond, dbconn=None, doreturn=1, **kwargs):
    tabname = 'lib3k_annotation'
    config = _get_config(None, tabname)
    assert config['editable'](), "View only entity, can't %s"%mode
    tabname = 'lib3k_annotation'
    return _modify_lib3k_annotation(mode, data, cond, dbconn, doreturn, config=config, **kwargs)

def _modify_lib3k_annotation(mode, data, cond, dbconn=None, doreturn=1, config=None, **kwargs):
    tabname = 'lib3k_annotation'
    config = _get_config(config, tabname)
    if dbconn is None:
        dbconn = db
    # auth = spyro.getrequest()['auth']
    # assert _spyro_authentication._rawauth(auth,  objname, modify_lib3k_annotation)
    if mode == 'update':
        if None in (data,cond): raise Exception("Unable to update table %s with empty data"%tabname)
        if len(cond) == 0: raise Exception("Unable to update %s without condition"%tabname)
        # It's a good idea overwrite user id fields with information of the authentications
        # For example:
        #cond['userid'] = spyro.getrequest()['auth'][0]
        x = dbconn.update(tabname, _strip_dict(data), cond)
        if config['indexable']():
            rowid = _get_rowid(tabname, cond, config)
            ftindex = config['ftindex']
            ftindex.handle_update(_preview(tabname, '1', data, config=config, preview_maxsize=512)[0][1], rowid)
    elif mode == 'insert':
        if data is None: raise Exception("Unable to insert into table %s with empty data"%tabname)
        #cond['userid'] = spyro.getrequest()['auth'][0]
        x = dbconn.insert(tabname, _strip_dict(data))
        if config['indexable']():
            xdata = _preview(tabname, config['list_fields'][0], data, config=config, preview_maxsize=512)[0]
            rowid = _get_rowid(tabname, xdata, config)
            ftindex = config['ftindex']
            ftindex.handle_insert(xdata[1], rowid)
    else:
        raise Exception("Unknown action to modify %s"%tabname)
    if doreturn:
        return _list_method_lib3k_annotation(mode, cond)
    else:
        return None

def delete_lib3k_annotation(cond, dbconn = None, doreturn = 1, **kwargs):
    tabname = 'lib3k_annotation'
    config = _get_config(None, tabname)
    assert config['editable'](), "View only entity, can't delete"
    return _delete_lib3k_annotation(cond, dbconn, doreturn, config=config, **kwargs)

def _delete_lib3k_annotation(cond, dbconn = None, doreturn = 1, config = None, **kwargs):
    tabname = 'lib3k_annotation'
    config = _get_config(config, tabname)
    if None is cond: raise Exception("Unable to delete table with empty condition")
    if len(cond) == 0: raise Exception("Unable to delete without condition")
    # auth = spyro.getrequest()['auth']
    # assert _spyro_authentication._rawauth(auth,  objname, delete_lib3k_annotation)
    if dbconn is None:
        dbconn = db
    if config['indexable']():
        rowid = _get_rowid(tabname, cond, config)
        try:
            xdata = _preview(tabname, '1', cond, config=config, preview_maxsize=512)[0][1]
        except IndexError:
            pass
        else:
            ftindex = config['ftindex']
            ftindex.handle_delete(xdata, rowid)
    x = dbconn.execute('DELETE FROM %s WHERE %s'%(tabname, _join_cond(cond)))
    if doreturn:
        return _list_method_lib3k_annotation("delete", cond)
    else:
        return None

def list_lib3k_annotation(*args, **kwargs):
    tabname = 'lib3k_annotation'
    kwargs['config'] = _get_config(None, tabname)
    # assert _spyro_authentication._rawauth(auth,  objname, list_lib3k_annotation)
    return _list(tabname, *args, **kwargs)


def form_search_lib3k_annotation(method=None, *args, **kwargs):
    tabname = 'lib3k_annotation'
    config = _get_config(None, tabname)
    dict_default = dict(config['dict_default']) # dictionary
    dict_default['__editmode__'] = ""
    if method == "choose_search":
        idtag = args[0]
        residtag = 'choose-search-%s'%idtag
        dict_default['__onsubmit__'] = (r"callto({'showresults': function(d){ DOM3K_updateElement(d,'%s'); """
                                        r"""DOM3K_BackgroundRowColors('%s',['rgb(200,200,230);','inherit']);}}, """
                                        r"""'%s','choose_search_%s',null,{'condition_dict': Easyform_formToDict(this), 'idtag': %s });"""
                                        r""" """%(residtag,residtag,objname,tabname,repr(idtag)))
        dict_default['__bottom__'] = """<div id="%s"></div>"""%residtag
    else:
        dict_default['__onsubmit__'] = r"callto({'handler': handle_commands}, '%s','search_%s',[Easyform_formToDict(this), null, 'LIKE']);"%(objname,tabname)
    dict_default['__id__'] = str(time.time())
    x = config['form_element_search'].merge(dict_default)
    return x

def manage_fulltext_lib3k_annotation(drop_index, reindex):
    tabname = 'lib3k_annotation'
    config = _get_config(None, tabname)
    ftindex = config['ftindex']
    if drop_index:
        ftindex.drop_index()
    if reindex:
        ftindex.create_index()
        for x in _preview(tabname, config['list_fields'][0], config=config, preview_maxsize=512):
            _rowid = _get_rowid(tabname, x)
            ftindex.handle_insert(x[1], _rowid)

def form_search_fulltext_lib3k_annotation(output='list', idtag='', query='',**kwargs):
    tabname = 'lib3k_annotation'
    dict_default = {'query': query, 'output': output, 'idtag': idtag}
    if output == "choose_search":
        residtag = 'choose-search-%s'%idtag
        dict_default['__onsubmit__'] = (r"""callto({'handler': handle_commands_closure('command=\'frame\'')}, '%s', 'search_fulltext_%s', """
                                        r"""null, Easyform_formToDict(this));"""%(objname, tabname))
        dict_default['__bottom__'] = """<div id="%s"></div>"""%residtag
        dict_default['__idtag__'] = """<input name="idtag:string:mandatory" value="%s" type="hidden" />"""%idtag
        dict_default['residtag'] = residtag
    else:
        dict_default['residtag'] = tabname
        dict_default['__onsubmit__'] = r"callto({'handler': handle_commands}, '%s', 'search_fulltext_%s', null, Easyform_formToDict(this));"%(objname, tabname)
        dict_default['__idtag__'] = ""
        dict_default['__bottom__'] = ""
    dict_default['__id__'] = str(time.time())
    return """<form action="javascript: void(0);" onsubmit="%(__onsubmit__)s">
    <input name="output:string:mandatory" value="%(output)s" type="hidden" />
    <input name="query:string:mandatory" value="%(query)s" size="40" type="text" />
    <input name="residtag:string" value="residtag" type="hidden" />
    %(__idtag__)s
    <input name=":submit" value="Search" type="submit" />
    </form>
    %(__bottom__)s
    """%dict_default
    
def search_fulltext_lib3k_annotation(query, output="", idtag="", residtag="", **kwargs):
    tabname = 'lib3k_annotation'
    config = _get_config(None, tabname)
    ftindex = config['ftindex']
    size = config['ftsearch_size']
    searchres = ftindex.search(query, size)
    # if len(searchres) == 0:
    #     return ""
    res = []
    fields = config['list_fields']
    for rank, cond in searchres:
        cond = eval(cond)
        res.extend( db.query("""SELECT %s FROM %s WHERE %s LIMIT 1"""%(", ".join(fields),tabname,_join_cond(cond,"="))) )
    # we loose the ranking, but we standardize the output
    if output == 'raw':
        return res
    if output == 'choose_search':
        return _out_choose_search(tabname, fields, res, idtag, size)
    else:
        x = _out_list(tabname, fields, res, idtag)
        if len(res) >= size:
            x+="<br/>" + locale3k.getname("general","too_many_results")
        return (config['output_showpage'](tabname, x, tabname), )

def search_lib3k_annotation(cond,orderby=None,usecondmethod=None,**kwargs):
    tabname = 'lib3k_annotation'
    config = _get_config(None, tabname)
    x = list_lib3k_annotation(cond, orderby, usecondmethod=usecondmethod)
    return (config['output_showpage']('Search results', x, tabname), )

def choose_search_lib3k_annotation(idtag, condition_dict = None, method="LIKE", outmethod="simple",
                                                  previewsize=10, list_condition_dict = None, **kwargs):
    tabname = 'lib3k_annotation'
    config = _get_config(None, tabname)
    # auth = spyro.getrequest()['auth']
    # assert _spyro_authentication._rawauth(auth,  objname, list_lib3k_annotation)
    emptycond = False
    if condition_dict is None:
        condition_dict = {}
        emptycond = True
    if (list_condition_dict is None) or (len(list_condition_dict) == 0):
        list_condition_dict = [condition_dict]
    else:
        emptycond = False
    fields = ['resourceid', 'annotation_owner', 'annotation']
    count = previewsize
    begin = 0
    condition = " OR ".join( [ "(%s)"%_join_cond(cd, method) for cd in list_condition_dict ] )
    limits = 'LIMIT %d OFFSET %d'%(count,begin)
    if config['list_with_empty_condition']() or not emptycond:
        query = db.query('SELECT %s FROM %s WHERE %s %s'%(",".join(fields),tabname,condition,limits))
    else:
        query = []
    return _out_choose_search(tabname, fields, query, idtag, previewsize)

def preview_lib3k_annotation(*args, **kwargs):
    tabname = 'lib3k_annotation'
    kwargs['config'] = _get_config(None, tabname)
    # kwargs['display'] = 'annotation_owner'
    return _preview(tabname, *args, **kwargs)

###
### end of lib3k_annotation UI
###
###############################################################################################
###############################################################################################
