############################################################################
#                                                                          #
#    This file 'auth3k/__init__.py'                                        #
#    is part of 'SPyRO: Simple Python Distributed Indexing'                #
#    Copyright (C) 2004-2005 by Eric Sadit Tellez Avila                    #
#    Copyright (C) 2006      by Eric Sadit Tellez Avila                    #
#    Copyright (C) 2007      by Eric Sadit Tellez Avila                    #
#    Copyright (C) 2008      by Eric Sadit Tellez Avila                    #
#    sadit@lsc.fie.umich.mx or donsadit@gmail.com                          #
#                                                                          #
#    This program is free software; you can redistribute it and#or modify  #
#    it under the terms of the GNU General Public License as published by  #
#    the Free Software Foundation; either version 2 of the License, or     #
#    (at your option) any later version.                                   #
#                                                                          #
#    This program is distributed in the hope that it will be useful,       #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of        #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
#    GNU General Public License for more details.                          #
#                                                                          #
#    You should have received a copy of the GNU General Public License     #
#    along with this program; if not, write to the                         #
#    Free Software Foundation, Inc.,                                       #
#    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             #
############################################################################
"""
This module support authentication for unix's PAM system. It works just for
login and password.


"""
#coding: utf8;

import PAM
import warn

def authunix(login, password):
    """  Alias to validate. This alias exists due to historical reasons.
    The parameters and functionality are the same that for the L{validate} function.
    """
    return validate(login, password)

def validate(login, password):
    """ Authentication based on Unix login, the process must have root
    access. You must have this in mind. You can change the uid and gid
    to give root to this function or put it in an SPyRO server.
    
    @param login: User name to authenticate
    @type login: string
    
    @param password: The password to use in the authentication
    @type password: string
    
    @return: Just the validation result (is validated = True or not validated = False)
    @rtype: boolean
    
    """
    def getpassword(auth, query_list, args):
        resp = []
        for query, t in query_list:
            if t in (PAM.PAM_PROMPT_ECHO_ON, PAM.PAM_PROMPT_ECHO_OFF):
                resp.append((password, 0))
            elif t in (PAM.PAM_PROMPT_ERROR_MSG, PAM.PAM_PROMPT_TEXT_INFO):
                warn.log("AUTH:",query)
                resp.append(('', 0));
            else:
                return None
        return resp    
    pam = PAM.pam()
    pam.start('ftp')
    pam.set_item(PAM.PAM_USER, login)
    pam.set_item(PAM.PAM_CONV, getpassword)
    isauth = False
    try:
        pam.authenticate()
        isauth = True
    except PAM.error, err:
        warn.log(login, " ", err)
    return isauth
    
if __name__ == '__main__':
    print authunix('xxx','xxx')
