#coding: utf8;
############################################################################
#                                                                          #
#    This file 'auth3k/authpop.py'                                          #
#    is part of 'SPyRO: Simple Python Distributed Indexing'                #
#    Copyright (C) 2004-2005 by Eric Sadit Tellez Avila                    #
#    Copyright (C) 2006      by Eric Sadit Tellez Avila                    #
#    Copyright (C) 2007      by Eric Sadit Tellez Avila                    #
#    Copyright (C) 2008      by Eric Sadit Tellez Avila                    #
#    sadit@lsc.fie.umich.mx or donsadit@gmail.com                          #
#                                                                          #
#    This program is free software; you can redistribute it and#or modify  #
#    it under the terms of the GNU General Public License as published by  #
#    the Free Software Foundation; either version 2 of the License, or     #
#    (at your option) any later version.                                   #
#                                                                          #
#    This program is distributed in the hope that it will be useful,       #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of        #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
#    GNU General Public License for more details.                          #
#                                                                          #
#    You should have received a copy of the GNU General Public License     #
#    along with this program; if not, write to the                         #
#    Free Software Foundation, Inc.,                                       #
#    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             #
############################################################################
"""
Validation via POP3. It can user SSL and TLS (via pyOpenSSL).

The validation is performed against a POP3 server. This module uses blocking
sockets (it uses the poplib package)
"""
import poplib
import warn

class POP3Auth:
    def __init__(self, remotehost, port = None, usingssl=True):
        """ Constructor of the POP3 authenticator.
        
        @param remotehost: The hostname or ip of the POP3 server
        @type remotehost: ip or hostname string
        @param port:  Port number of the remove service
        @type port: int
        @param usingssl: Use True to indicate a remote secured crypted service
        @type usingssl: boolean
        
        """
        self.usingssl  = usingssl
        self.remotehost = remotehost
        self.port = port
        if usingssl:
            if port is None: self.port = poplib.POP3_SSL_PORT
        else:
            if port is None: self.port = poplib.POP3_PORT

    def auth(self, user, password):
        """ Alias to validate """

        return self.validate(user, password)

    def validate(self, user, password):
        """
            Perform the connection with the POP3 server an makes the authentication.
            The POP3 validation don't requires any special permisions of the process
            (just access to the network)
            
            @param user: User name to authenticate
            @type user: string
            
            @param password: The password to use in the authentication
            @type password: string
            
            @return: Just the validation result (is validated = True or not validated = False)
            @rtype: boolean
            
        """
        if self.usingssl:
            pop = poplib.POP3_SSL(self.remotehost, self.port)
        else:
            pop = poplib.POP3(self.remotehost, self.port)
        try:
            pop.user(user)
            pop.pass_(password)
            pop.stat()
        except:
            pop.quit()
            return False
        pop.quit()
        return True

if __name__ == '__main__':
    auth = POP3Auth("localhost")
    import getpass
    print auth.auth(getpass.getuser(), getpass.getpass())
