#coding: utf8;
############################################################################
#                                                                          #
#    This file 'cache3k/__init__.py'                                       #
#    is part of 'SPyRO: Simple Python Distributed Indexing'                #
#    Copyright (C) 2004-2005 by Eric Sadit Téllez Avila                    #
#    Copyright (C) 2006      by Eric Sadit Téllez Avila                    #
#    Copyright (C) 2007      by Eric Sadit Téllez Avila                    #
#    Copyright (C) 2008      by Eric Sadit Téllez Avila                    #
#    sadit@lsc.fie.umich.mx or donsadit@gmail.com                          #
#                                                                          #
#    This program is free software; you can redistribute it and#or modify  #
#    it under the terms of the GNU General Public License as published by  #
#    the Free Software Foundation; either version 2 of the License, or     #
#    (at your option) any later version.                                   #
#                                                                          #
#    This program is distributed in the hope that it will be useful,       #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of        #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
#    GNU General Public License for more details.                          #
#                                                                          #
#    You should have received a copy of the GNU General Public License     #
#    along with this program; if not, write to the                         #
#    Free Software Foundation, Inc.,                                       #
#    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             #
############################################################################
"""
Cache3k simplifies the creation of general purpose caches of data, driven for keys.

"""
import time
import threading

class cache:
    """ The cache3k base class """
    def __init__(self, maxseconds, maxcalls, f_request):
        """ Constructor

        @param maxseconds: Maximum number of seconds to reload the cached data
        @type maxseconds: float
        @param maxcalls: Maximum number of calls to reload the cached data
        @type maxcalls: int
        @param f_request: The data generator
        @type f_request: callable

        """
        self.maxseconds = maxseconds
        self.maxcalls = maxcalls
        self.lastcall = time.time()
        self.ncalls = 0
        self.f_request = f_request
        self.xcache = f_request()
        self.lock = threading.Lock()
    
    def getvalue(self, id):
        """
        @param id: Identifier to fetch a data
        @type id: object
        @return: Data fetched from the cache
        @rtype: object
        """
        ftime = time.time()
        if ((ftime - self.lastcall) > self.maxseconds) or (self.ncalls >= self.maxcalls):
            self.calling = False
            try:
                self.lock.acquire()
                if not self.calling:
                    self.calling = True
                    self.xcache = self.f_request()
                    self.ncalls = 0
                    self.lastcall = ftime
            finally:
                self.lock.release()
        else:
            self.ncalls += 1
        return self.xcache[id]

if __name__ == '__main__':
    num = 0
    def doreq():
        print "CALLING doreq"
        global num
        if num == 6:
            raise Exception("call limit")
        num = num + 1
        return {'A': 'AA', 'B': 'BB'}
    c = cache(0.5, 100000000, doreq)
    for x in  xrange(1000000):
        print x, c.getvalue('A')
        
