############################################################################
#                                                                          #
#    This file 'spyropack.py'                                          #
#    is part of 'SPyRO: Simple Python Distributed Indexing'                #
#    Copyright (C) 2004-2005 by Eric Sadit Tellez Avila                    #
#    Copyright (C) 2006      by Eric Sadit Tellez Avila                    #
#    Copyright (C) 2007      by Eric Sadit Tellez Avila                    #
#    Copyright (C) 2008      by Eric Sadit Tellez Avila                    #
#    sadit@lsc.fie.umich.mx or donsadit@gmail.com                          #
#                                                                          #
#    This program is free software; you can redistribute it and#or modify  #
#    it under the terms of the GNU General Public License as published by  #
#    the Free Software Foundation; either version 2 of the License, or     #
#    (at your option) any later version.                                   #
#                                                                          #
#    This program is distributed in the hope that it will be useful,       #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of        #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
#    GNU General Public License for more details.                          #
#                                                                          #
#    You should have received a copy of the GNU General Public License     #
#    along with this program; if not, write to the                         #
#    Free Software Foundation, Inc.,                                       #
#    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             #
############################################################################
"""
Some important functions to handle scripts scripts

@todo: Change the content of this module to db3k
"""
import re
import warn

def sql_script_data(dbconn, data, dodebug=False):
    """
    Execute a string of sql instructions. Useful to execute a complete list of sql
    sentences like creation of tables and indices. Everything is executed in one
    transaction if dbconn's backend support for it.
    
    @param dbconn: The connection to be used to execute sentences
    @type dbconn: db3k.Connection
    @param data: The sql string sentences
    @type data: str
    @param dodebug: Show some debug output
    @type dodebug: boolean
    
    @return: Nothing
    @rtype: None
    
    """
    conn = dbconn.getconn()
    data = re.sub(r"--.*?[\n]","",data)
    dodebug=True
    try:
        for l in data.split(";"):
            l = l.strip()
            if l == '':
                continue
            if dodebug:
                warn.debug("Executing: %s"%l)
            cursor = conn.cursor()
            cursor.execute(l)
            cursor.close()
    except Exception,e:
        warn.warn("ERROR Executing sql script :", repr(e), repr(l))
        cursor.close()
        conn.rollback()
        dbconn.putconn(conn)
    else:
        conn.commit()
        dbconn.putconn(conn)

def sql_script(dbconn=None, scriptpath=None):
    """
    Executes an sql instruction file.
    
    @param dbconn: The connection to database
    @type dbconn: db3k.Connection
    
    @param scriptpath: Path to the sql file
    @type scriptpath: string
    """
    f = open(scriptpath)
    data = f.read()
    f.close()
    sql_script_data(dbconn, data)
