#!/usr/bin/python
#coding: utf8;
############################################################################
#                                                                          #
#    This file 'db3k/__init__.py'                                          #
#    is part of 'SPyRO: Simple Python Distributed Indexing'                #
#    Copyright (C) 2004-2005 by Eric Sadit Téllez Avila                    #
#    Copyright (C) 2006      by Eric Sadit Téllez Avila                    #
#    Copyright (C) 2007      by Eric Sadit Téllez Avila                    #
#    Copyright (C) 2008      by Eric Sadit Téllez Avila                    #
#    sadit@lsc.fie.umich.mx or donsadit@gmail.com                          #
#                                                                          #
#    This program is free software; you can redistribute it and#or modify  #
#    it under the terms of the GNU General Public License as published by  #
#    the Free Software Foundation; either version 2 of the License, or     #
#    (at your option) any later version.                                   #
#                                                                          #
#    This program is distributed in the hope that it will be useful,       #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of        #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
#    GNU General Public License for more details.                          #
#                                                                          #
#    You should have received a copy of the GNU General Public License     #
#    along with this program; if not, write to the                         #
#    Free Software Foundation, Inc.,                                       #
#    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             #
############################################################################

__doc__ = (""" This is the main script for the dweba project. It runs and configures """
           """ a server. It must be edited if you need a customized environments. But """
           """ it must be configured using the appconfig.py module """)
import os
import sys

try:
    import appconfig
except ImportError, e:
    print "=================> ", e
    print ("In order to run your system "
           "you must setup your system editing the file 'appconfigtemplate.py' and then rename or copy to "
           "'appconfig.py'. The default configuration will work for many apps, feel free to use it. Just "
           " copy the appconfigtemplate.py to appconfig.py and customize if you need it")
    raw_input("Press Enter>")
    sys.exit(1)
import warn
#### Checking template and appconfig
def checkappconfig():
    print "loading appconfigtemplate.py"
    import appconfigtemplate
    import types
    print "reloading appconfig.py"
    reload(appconfig)
    applist = dir(appconfig)
    templist = dir(appconfigtemplate)
    print "starting the check"
    print "<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<"
    for x in templist:
        tempattr = type(getattr(appconfigtemplate,x))
        if isinstance(tempattr, (types.ModuleType, Exception)):
            continue
        try:
            appattr = type(getattr(appconfig,x))
        except AttributeError,e:
            print "============  failure ============="
            print("There is not %s in the application appconfig.py module"%repr(x))
            print("please see its definition in appconfigtemplate.py and append it. ")
            print("If you don't need this variable please set to None")
            print(e)
    print ">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>"
    print "done"

####
import SPyRO
import SPyRO.policies, SPyRO.args
import threadutils
from auth3k import authweb
    
import db3k
import getopt


import shttp
from shttp import http
    
def getdb(spyroserver,authobj):
    import getpass
    manager = 'nodb'
    if manager.startswith("sqlite"):
        def xrewritesql(s):
            return s.replace("%s","?")
        rewritesql = xrewritesql
    else:
        rewritesql = None
    return None

def getauth(server):
    # Default authentication policies:
    auth = SPyRO.policies.PublicAllowCall()
    environ = appconfig.Environ()
    authweb.set_default_environ(environ)
    if server:
        server.setobj("auth",authweb.default_public_authenticator(environ),auth)
    return auth


from appconfig import USER_ADMIN_SPYRO, OBJECT_SPYRO, USESSL
def stopserver(addr):
    getauth(None)     # loading custom AuthEviron
    authtuple=(USER_ADMIN_SPYRO,authweb.DefaultAuthEnviron._create_authentication(USER_ADMIN_SPYRO))
    comm = SPyRO.Client(addr,
                        appconfig.spyro_url_close,
                        usingssl=USESSL)
    import socket
    try:
        comm.callMethod(OBJECT_SPYRO,'close',auth=authtuple)
        try:
            comm.close()
        except socket.error, e:
            warn.debug("Stopped")
    except socket.error, e:
        warn.debug(e)

def startserver(addr,dofork=False):
    # Append ssl_privatekey_filename="/private/key/path",ssl_certificate_filename="/certificate/path" to
    # SPyRO.Server constructor to get an HTTPS server
    #warn.debug("**********")
    if dofork:
        fout = file(appconfig.log_out,"ab",0)
        ferr = file(appconfig.log_err,"ab",0)
        threadutils.forkdaemon(new_stdout=fout, new_stderr=ferr)
    # Direct access to SPyRO's objects using XMLRPC as the default
    paths = appconfig.spyro_path
    try:
        objectpool = appconfig.spyro_object_pool
    except AttributeError:
        objectpool = SPyRO.ModulePool(paths)
    spyroserver = SPyRO.Server(appconfig.spyro_url_full, objectpool = objectpool)
    httpserver = http.HTTPServer(addr,  document_root=appconfig.document_root)

    httpserver.register_urihandler('/spyro/', http.handlespyro(spyroserver) )
    httpserver.register_urihandler('/call/', http.httpcallspyro(spyroserver,'text/html'), virtual_host='', rewrite=appconfig.main_rewrite)
    httpserver.register_urihandler('/callarg/', http.httpcallspyro(spyroserver,'text/html',mode='arg'), virtual_host='', rewrite=appconfig.main_rewrite)
    auth = getauth(spyroserver)
    if appconfig.run_proxy:
        import proxykong
        proxyhosts = [''] # All non managed
        # Those servers will be used specially to configure
        # and the first one will be used as a login server
        # The virtual hosts must not exists before or they will be overriden
        # loginhosts = ['192.168.1.38:8000'] 
        proxykong._init(spyroserver, httpserver, appconfig.proxy_setup_host, proxyhosts)
    authserver = authweb.Auth()
    authserver.permsobject[OBJECT_SPYRO] = [[], [], []]
    class StopServer:
        def __init__(self):
            pass
        def close(self):
            import asyncore
            # asyncore.socket_map.clear()
            asyncore.close_all()
    authserver.perms['close'] = [[USER_ADMIN_SPYRO],[],[]]
    spyroserver.setobj(OBJECT_SPYRO,StopServer(),authserver)
    appconfig.server_run(spyroserver, httpserver)

if __name__ == '__main__':
    dofork=False
    try:
        kwargs, args = getopt.getopt(sys.argv[1:],'mshkfc')
    except getopt.GetoptError, err:
        warn.debug('Try -h for more information')
        sys.exit(0)
    import socket
    address=appconfig.local_server_address
    for name, value in kwargs:
        if name == '-h':
            print ('usage: %s [options]\n'
                   ' -h   Shows this help\n'
                   ' -c   Runs the appconfig.py checks agains appconfigtemplate.py. This must be done after any dweba update\n'
                   ' -m   Runs as master webserver, by default in the port %s\n'
                   ' -s   Runs an SPyRO local server, useful for apache and modrewrite\n'
                   ' -f   Set as daemon (fork)\n'
                   ' -k   Stops a running server\n'
                   ''%(sys.argv[0],appconfig.port))
            sys.exit(0)
        if name == '-c':
            checkappconfig()
            sys.exit(0)
        if name == '-m':
            address=appconfig.server_address
        elif name == '-s':
            pass
        elif name == '-k':
            address = None
            stopserver(appconfig.local_server_address)
        elif name == '-f':
            dofork=True
    if address:
        print address, dofork
        startserver(address,dofork)
