#coding: utf8;
__doc__ = """ The easyform creator """

import auth3k
import auth3k.authweb as authweb
_spyro_authentication = authweb.Auth()
_spyro_authentication.permsobject['Guigo.easyform'] = ((),(),())

import re
import os, sys
import warn
import md5
import random
import template3k

def _attributes(attrs):
    return " ".join([ '%s="%s"'%(name,value) for name,value in attrs.items()])

srenoalpha = re.compile(r"[\W_]+")

def title(name):
    return srenoalpha.sub(" ",name.title())

class Entity:
    EDITSUBMIT = """<tr><td>%(markup)s</td></tr>"""
    # EDIT = """<tr><td>%(display)s: </td><td>%(markup)s</td></tr>"""
    EDIT = ("""\n<tr><td>\n"""
            """<python:locale id="%(objname)s.%(name)s">\n"""
            """[ {'encoding': 'utf8', 'locale': '%(language)s', 'translation': '%(display)s'} ]\n"""
            """</python:locale>:</td>"""
            """<td>%(markup)s</td></tr>\n""")
    def __init__(self, name, display=None, type='string', value='', constraints=(), hidden=False,
                 attributes=None, template=None, markuptemplate=None, getoptions=None, group=None,
                 language='en'):
        self.name = name
        if display is None:
            display = title(name)
        self.display = display
        self.type = type
        self.value = value
        self.hidden = hidden
        if attributes is None: attributes = {}
        self.attributes = attributes
        self.getoptions = getoptions
        self.constraints = constraints
        if template is None:
            if type == 'submit':
                template = Entity.EDITSUBMIT
            else:
                template = Entity.EDIT
        self.template = template
        self.markuptemplate = markuptemplate
        self.group = group
        self.language = language
        
    def _createoptions(self, options):
        ops = []
        default = str(self.value)
        for value,display in options:
            value = str(value)
            if default == value:
                selected = 'selected="selected"'
            else:
                selected = ''
            ops.append('<option value="%s" %s>%s</option>'%(value, selected, display))
        return "\n".join(ops)
    
    def _markup(self):
        type = t = self.type
        if t in ('string','int','float'):
            try:
                self.attributes['size']
            except KeyError:
                self.attributes['size'] = '40'
        
        attrs = _attributes(self.attributes)
        self.attrs = attrs
        name = self.name
        value = '<python:keyword id="%s" />'%self.name
        group = self.group
        constraints = ":".join(self.constraints)
        
        if self.markuptemplate:
            self.markup = self.markuptemplate%locals()
        elif t == 'hidden':
            self.markup = '<input name="%s:hidden:%s" type="hidden" value="%s" />'%(name, constraints, value)
        elif t == 'text':
            self.markup = '<textarea cols="50" rows="12" name="%s:text:%s" %s>%s</textarea>'%(name, constraints, attrs, value)
        elif t in ('string','int','float'):
            self.markup = '<input name="%s:%s:%s" type="text" value="%s" %s />'%(name, t, constraints, value, attrs)
        elif t == 'date':
            self.markup = """
            <python:locale id="date.format_date" />
            <input type="text" name="%(name)s:date:%(constraints)s" value="%(value)s" onclick="Easyform_frameReadDate(DOM3K_readInput(this), DOM3K_updateInput, this, event.clientY, event.clientX)" %(attrs)s />
            """%locals()
        elif t == 'time':
            self.markup = """
            <python:locale id="date.format_time" />
            <input type="text" name="%(name)s:time:%(constraints)s" value="%(value)s" onclick="Easyform_frameReadTime(DOM3K_readInput(this), DOM3K_updateInput, this, event.clientY, event.clientX)" %(attrs)s />
            """%locals()
        elif t == 'datetime':
            self.markup = """
            <python:locale id="date.format_date" /> 
            <python:locale id="date.format_time" /> 
            <input type="text" name="%(name)s:datetime:%(constraints)s" value="%(value)s" onclick="Easyform_frameReadDatetime(DOM3K_readInput(this), DOM3K_updateInput, this, event.clientY, event.clientX)" %(attrs)s />
            """%locals()
        elif t == 'boolean':
            if value == 't':
                tval = 'selected="true"'
                fval = ''
            else:
                tval = ''
                fval = 'selected="true"'
            self.markup = """
            <select name="%s:boolean:%s">
                <option value="t" %s><python:locale id="types.true" /></option>
                <option value="f" %s><python:locale id="types.false" /></option>
            </select>
            """%(name, constraints, tval, fval)
        elif t == 'list':
            if self.getoptions:
                options = self._createoptions(self.getoptions(name))
            else:
                options = ""
            self.markup = '<select name="%s:list:%s">\n%s\n</select>'%(name,constraints,options)
        elif t == 'submit':
            self.markup = '<input type="submit" name="%s:submit" value="<python:locale id="general.submit" />" />'%(name)
        else:
            raise "Unsupported Entity type"

    def __str__(self):
        self._markup()
        self.objname = self.group
        return self.template%self.__dict__

    def getdictentry(self):
        return """'%s' : %s"""%(self.name, repr(self.value))
    
class EntityString:
    def __init__(self, string, name):
        self.string = string
        self.name = name
        self.group = None
        self.language = None
        
    def __str__(self):
        return string

    def getdictentry(self):
        return None
    
class Group:
    def __init__(self, name, action = None, display = None, attributes = None, language = 'en', onsubmit=None, internal_fields = None):
        self.name = name
        if action is None:
            action = 'javascript: void(0);'
        self.action = action
        if display is None: display = "<strong>" + title(name) + "</strong>"
        self.display = display
        if attributes is None: attributes = {}
        self.attributes = attributes
        self.entities = []
        self.entitiesdict = {}
        self._template = None
        self.language = language
        if onsubmit is None: onsubmit = ''
        self.onsubmit = onsubmit
        if internal_fields is None: internal_fields = []
        self.internal_fields = internal_fields
    
    def __delitem__(self, name):
        del self.entitiesdict[name]
        self.entities.remove(name) # It is efficient when group has few entities
    
    def __getitem__(self, name):
        return self.entitiesdict[name]
    
    def __iter__(self):
        return iter([ self.entitiesdict[name] for name in self.entities ])

    def keys(self):
        return self.entities

    def values(self):
        return self.entitiesdict.values()

    def items(self):
        return self.entitiesdict.items()
    
    def append(self, entity):
        self.entitiesdict[entity.name] = entity
        self.entities.append(entity.name)
        entity.group = self.name
        entity.language = self.language
        
    def settemplate(self, template):
        self._template = template
    
    def gettemplate(self):
        if self._template is None:
            ents = []
            for name in self.entities:
                if name in self.internal_fields: continue
                ents.append( "%%(%s)s"%name )
            ents = "\n".join(ents)
            attrs = _attributes(self.attributes)
            self._template = """
            %s
            <form method="POST" enctype="multipart/form-data" id="form-%s<python:keyword id='__id__' />" action="%s" onsubmit="%s" name="%s" %s>
                <table><tbody>
            """%(self.display, self.name, self.action, self.onsubmit, self.name, attrs) + ents + """
                </tbody></table>
            </form>
            <python:keyword id="__bottom__"></python:keyword>
            """
        return self._template
    
    def __str__(self):
        # print self.gettemplate()
        return self.gettemplate()%self.entitiesdict

class PythonContainer:
    def __init__(self):
        self.forms = []

    def begin(self):
        pass

    def append(self,name,group):
        self.forms.append((name,group))

    def end(self,config,connui):
        D = {}
        schema = config['dbschema']
        D['QuotedDbname'] = repr(config['dbname'])
        D['Dbname'] = config['dbname']
        D['QuotedDbuser'] = repr(config['dbuser'])
        D['QuotedDbpass'] = repr(config['dbpass'])
        D['QuotedDbhost'] = repr(config['dbhost'])
        D['QuotedDbmanager'] = repr(config['dbmanager'])
        D['Objname'] = config['fullobjname']
        D['QuotedObjname'] = repr(config['fullobjname'])
        D['Title'] = repr(config['fullobjname'].title())
        res = []
        res.append("\n")
        res.append("    res = []\n")
        r = []
        for x,group in self.forms:
            r.append(x)
        D['ListForms'] = repr(r)
        output = file(config['fullobjpath'] + ".py","w")
        output.write(template3k.get_template(config['tempobject']).merge(D))
        for x,group in self.forms:
            try:
                D['Display'] = repr(group.entities[1])
                if D['Display'] == '': raise IndexError,"Fake"
            except IndexError:
                D['Display'] = repr(group.entities[0])
            if schema is None or schema == 'public':
                tabname = x
            else:
                tabname = "%s.%s"%(schema,x)
            D['Tabname'] = tabname
            D['QuotedTabname'] = repr(tabname)
            D['InternalFields'] = repr(group.internal_fields)
            D['DictDefault'] = ",".join(filter(None,map(Entity.getdictentry, group.values())))
            D['FormElement'] = str(group).replace('"""',"'''")
            D['FormElementNoMandatory'] = str(group).replace('"""',"'''").replace(":mandatory","")
            D['ListFields'] = repr(filter(lambda x:'' != x,group.keys()))
            D['ForeignDictionary'] = repr(connui.foreigns_fields_dictionary(tabname))
            output.write(template3k.get_template(config['tempform']).merge(D))
        output.close()
