__doc__ = """

Module for managing users. This object provides modules and functions to manage users in many
different ways: User handled by functions, simple file, database, or anything that you want to customize.

It uses and extends the aut3k.authweb package

TODO: Object-Package manager: online editor, push zip, tars, etc.
TODO: Start / Stop
"""

import os
import re
import shutil
import db3k
import time
import template3k
import warn
import zipfile
try:
    from cStringIO import StringIO
except ImportError:
    from StringIO import StringIO
warn = warn.Warn("packadmin")
import auth3k
import auth3k.authweb as authweb
from shttp import http
import SPyRO  as spyro
_spyro_authentication = authweb.Auth()
# To set permisions to the global module or object, uncomment the following
# [ [allowed users list], [ allowed groups list], [ denied user list] ]
_spyro_authentication.permsobject['packadmin'] = [[],['admin'],[]]
# Note: In module objects the object name must be the name of the object. If an authobject is
# manages many objects, there may exists many object names

# For method level management
_spyro_authentication.perms['only_admin'] = [[],['admin'],[]]
_spyro_authentication.perms['only_valid'] = None

import appconfig
ObjectPath=appconfig.spyro_path
ValidPath=['db', 'html','sql','templates']
CompletePath = ObjectPath + ValidPath
AbsolutePath=[]
for op in CompletePath:
    AbsolutePath.append( os.path.abspath(op) )

class PathNotAllowed(Exception): pass
class BackupError(Exception): pass

def getabsname(filename):
    if re.search('(pyc|~)$',filename,re.I):
        raise PathNotAllowed("You can't modify this file path %s"%filename)
    absname = os.path.abspath(filename)
    for base in AbsolutePath:
        if absname.startswith(base):
            return absname
    raise PathNotAllowed("getabsname: %s is not in the spyro's object path"%filename)

def in_object_path(path):
    for base in ObjectPath:
        if path.startswith(base):
            return True
    return False


import posixpath
def get_objname_from_path(path):
    path = posixpath.normpath(path)
    for base in ObjectPath:
        if path.startswith(base):
            return path[len(base)+1:].replace("/",".")
    raise PathNotAllowed("get_objname_from_path: %s is not in the object path"%repr(path))
    
def see_file(fname):
    absname = getabsname(fname)
    f = file(absname)
    r = f.read()
    f.close()
    return r

def upload_file(fname=None, data=None, **kwargs):
    if fname[-1] == '/':
        _fname = fname + os.path.basename(data.filename)
    else:
        _fname = fname
    save_file(_fname, data.value)
    return template3k.get_template("templates/upload_response.html").merge(IFrameName=fname,
                                                                           InnerContent=list_packages())

def save_file(fname, data):
    absname = getabsname(fname)
    f = file(absname,"w")
    r = f.write(data)
    f.close()
    return ("Saved",list_packages())

def list_packages():
    xres = {}
    for _path in CompletePath:
        xres[_path] = _list_packages(_path, "/")
    xkeys = xres.keys()
    xkeys.sort()
    res = [ """<a name="/"><strong>Directory Index:</strong></a>"""," &nbsp;|&nbsp; ".join([ """<a href="#%s">%s</a>"""%(x,x) for x in xkeys ]) ]
    res.append("""<div><strong>Fast Action Index: </strong>"""
               """<a href="javascript: viewweb('../packadmin/admin_objects/index'); void(0);">Admin Objects</a> | """
               """<a href="javascript: sendfile('/','upload_package'); void(0);">Upload a global update</a> """
               """(a dweba's package from threekong.com)"""
               """</div>""")
    res.extend([ xres[x] for x in xkeys ])
    return "\n".join(res)

def upload_file_form(namename, namevalue, dataname, action):
    return template3k.get_template("templates/upload_request.html").merge(
        Namename=namename,
        Namevalue=namevalue,
        Dataname=dataname,
        Action=action)

def upload_package(fname=None, data=None, **kwargs):
    if fname == '/':
        assert appconfig.support_global_updates, "This instance can't receive global updates because appconfig.support_global_updates is False"
        abspath = os.path.abspath(".")
    else:
        abspath = getabsname(fname)
    zf = zipfile.ZipFile(StringIO(data.value),"r")
    warn.debug("====== test zip ", abspath,":", zf.testzip())
    for name in zf.namelist():
        transname = name.replace("\\","/")
        if fname == '/':
            transname = "/".join(transname.split("/")[1:])
        fullname = os.path.join(abspath, transname) # This works in unix and windows
        # print fullname
        f = file(fullname,"w")
        f.write(zf.read(name))
        f.close()
    return template3k.get_template("templates/upload_response.html").merge( IFrameName=fname,
                                                                            InnerContent=list_packages())

def create_package(path):
    abspath = getabsname(path)
    zfile = StringIO()
    z = zipfile.ZipFile(zfile,"w")
    for xpath,xdirs,xfiles in os.walk(path):
        for xf in xfiles:
            a = os.path.join(xpath, xf)
            b = a #os.path.join(path, xf)
            try:
                z.write(a, b)
            except OSError,e:
                warn.debug("Error in path: %s. "%a, e)
                continue
    z.close()
    raise http.CustomResponse(zfile.getvalue(),content_type="application/zip")

def rename_file(oldpath, newpath):
    oldabsname = getabsname(oldpath)
    newabsname = getabsname(newpath)
    shutil.move(oldabsname, newabsname)
    return _list_parent_packages(oldpath) + _list_parent_packages(newpath)

def remove_backup(path):
    absname = getabsname(path)
    if not re.search("bak-\d+$",path):
        raise BackupError("""Only backups can be removed. For security reasons """
                          """If you need to delete an ordinary file, you must """
                          """rename and set and extension .bak-0 and then delete it.""")
    os.unlink(absname)
    return _list_parent_packages(path)

def create_backup(path):
    absname = getabsname(path)
    for x in xrange(200):
        dest = absname + (".bak-%d"%(x))
        if os.path.exists(dest):
            continue
        shutil.copy2(absname, dest)
        return _list_parent_packages(path)
    raise BackupError("Too many backups")

def create_directory(path):
    absname = getabsname(path)
    os.mkdir(path)
    return _list_parent_packages(path) + _list_parent_packages(path, 0)

def download(path=None):
    absname = getabsname(path)
    f = file(absname)
    d = f.read()
    f.close()
    raise http.CustomResponse(d, content_type_from_path=path, status=200)

_spyro_authentication.perms["_list_parent_packages"] = False
_spyro_authentication.perms["_list_packages"] = False

def list_parent_packages(_path,parent=1):
    return _list_parent_packages(_path,parent)

def _list_parent_packages(_path, parent=1):
    if parent:
        _path = os.path.dirname(_path)
    print "_path: ", _path
    if _path == '':
        return [['',list_packages(),'Content']]
    getabsname(_path)
    _parpath = os.path.dirname(_path)
    _parpath = os.path.dirname(_parpath)
    if _parpath == '':
        _parpath = '/'
    return [ ['',_list_packages(_path, _parpath),"path-%s"%_path, 'replace'] ]

def _list_packages(_path, parentpath):
    r = [ """<div class="packpath" id="path-%(_path)s">"""%locals() ]
    if os.path.islink(_path):
        r.append("%(_path)s is a symbolic link, stopping recursion</div>"%locals())
        return "".join(r)
    imgnewdir="""<img style="border:0pt;" src="../../icons/actions/folder_new.png" />"""
    imgnewfile="""<img style="border:0pt;" src="../../icons/actions/filenew.png" />"""
    imgpackup="""<img style="border:0pt;" src="../../icons/actions/2uparrow.png" />"""
    imgpackdown="""<img style="border:0pt;" src="../../icons/actions/2downarrow.png" />"""
    imgreload="""<img style="border:0pt;" src="../../icons/actions/reload.png" />"""
    imgup="""<img style="border:0pt;" src="../../icons/actions/up.png" />"""
    r.append(
        """<h1 class="firsttitle" style="font-size: small">"""
        """<a name="%(_path)s"></a>"""
        """<a href="javascript: callto({'handler': Show}, 'packadmin', 'list_parent_packages', ['%(_path)s',1]); void(0);">"""
        """-&nbsp;Package Path: %(_path)s.</h1>"""
        """</a>"""
        """<div style="padding: 2pt;">"""
        """<table class="packlist packpath"><tbody><tr>"""
        """<th>In this path:</th>"""
        """<td><a href="javascript: callto({'handler': Show}, 'packadmin', 'list_parent_packages', ['%(_path)s',0]); void(0);">"""
        """%(imgreload)s<br/>Reload"""
        """</a></td>"""
        """<td><a href="javascript: sendfile('%(_path)s','upload_package'); void(0);">%(imgpackup)s<br/>Upload package</a></td>"""
        """<td><a href="create_package?path=%(_path)s">%(imgpackdown)s<br/>Pack as package</a></td>"""
        """<td><a href="javascript: showprompt('Directory name: ', function(d){createelement2('%(_path)s',d,'create_directory');}, 'Select a name for the new directory'); void(0);">%(imgnewdir)s"""
        """<br/>Create Directory</a></td>"""
        """<td><a href="javascript: showprompt('File name: ', function(d){createfile('%(_path)s',d)}, 'Select a name for the new file'); void(0);">"""
        """%(imgnewfile)s<br/>Create File</a></td> """
        """<td><a href="javascript: sendfile('%(_path)s/','upload_file'); void(0);">%(imgup)s"""
        """<br/>Upload File</a></td>"""
        """</tr></tbody></table>"""
        """</div>"""
        ""%locals())
    r.append("""<div class="thirdtitle">""")
    if in_object_path(_path):
        r.append("""<a href="javascript: viewweb('../Guigo/list_projects?objpath=%(_path)s'); void(0);">Launch Guigo!</a>. """
                 ""%locals())
    dirs = []
    files = []
    for path in os.listdir(_path):
        xfile = os.path.join(_path, path)
        if os.path.isdir(xfile):
            dirs.append(xfile)
        else:
            files.append(xfile)
    dirs.sort()
    files.sort()
    r.append("""<strong>Go to:</strong> <a href="#%s">Parent</a>. """%parentpath)
    if len(dirs) > 0:
        r.append("<strong>Local Index:</strong>")
        dd = [ x.replace(os.path.sep,"/") for x in dirs ]
        r.append(" &nbsp;|&nbsp; ".join([ """<a href="#%s">%s</a>"""%(x,x.replace(_path+"/",'')) for x in dd ]))
                                          #])) for emacs :(
    r.append("</div>")                                      
    r.append("""<table class="packpath" width="98%"><tbody>"""
             """<tr><th>Filename</th><th>Time</th><th>Size</th></tr>""")
    for f in files:
        try:
            abspath=getabsname(f)
        except PathNotAllowed:
            continue
        base = os.path.basename(f)
        f = f.replace(os.path.sep,"/")
        base = base.replace(os.path.sep,"/")
        if base.endswith(".py") or base.endswith(".pyc"):
            execname = f.split('/')[1:]
            if execname[-1] == '__init__.py':
                execname = execname[:-1]
            else:
                execname[-1] = execname[-1].split('.')[0]
            execname = "../%s/index"%("/".join(execname))
        else:
            execname = "download?path=%s"%f
        try:
            st = os.stat(abspath)
        except:
            warn.warn("Error in ", abspath)
            continue
        st_mtime = time.strftime("%Y-%m-%d<br/>%H:%M:%S",time.localtime(st.st_mtime)) # time.ctime(st.st_mtime)
        st_size = "%0.1f"%(st.st_size / 1024.0)
        imgview="""<img style="border:0pt;" src="../../icons/actions/webpres.png" /><br/>View"""
        imgedit="""<img style="border:0pt;" src="../../icons/actions/edit.png" /><br/>Edit"""
        imgdel="""<img style="border:0pt;" src="../../icons/actions/mail_delete.png" /><br/>Del"""
        imgback="""<img style="border:0pt;" src="../../icons/actions/db_update.png" /><br/>Backup"""
        imgrename="""<img style="border:0pt;" src="../../icons/actions/quick_restart.png" /><br/>Move"""
        imgdown="""<img style="border:0pt;" src="../../icons/actions/download.png" /><br/>Download"""

        if f.split(".")[-1].lower() in ("py","java","html","htm","txt","c","h","sql","xml","js","css","php","text","tex","bib","sh"):
            editlink="""<a href="javascript: callto({'handler': function(d) {editfile('%(f)s',d)}},'packadmin','see_file',['%(f)s']);" title="Edit">%(imgedit)s</a>"""%locals()
        else:
            editlink="""-"""
        r.append("""<tr>"""
                 """<td>%(base)s</td>"""
                 """<td>%(st_mtime)s</td>"""
                 """<td>%(st_size)s KiB</td>"""
                 """<td><a href="javascript: viewweb('%(execname)s'); void(0);" title="Web View">%(imgview)s</td>"""
                 """<td>%(editlink)s</td>"""
                 """<td><a href="javascript: createelement1('%(f)s', 'remove_backup');void(0);" title="Remove">%(imgdel)s</a></td>"""
                 """<td><a href="javascript: createelement1('%(f)s', 'create_backup');void(0);" title="Backup">%(imgback)s</a></td>"""
                 """<td><a href="javascript: createrename('%(f)s'); void(0);" title="Rename">%(imgrename)s</a></td>"""
                 """<td><a href="download?path=%(f)s" title="Download">%(imgdown)s</a></td>"""
                 """<td><a href="javascript: sendfile('%(f)s'); void(0);" title="Upload">%(imgup)s<br/>Upload</a></td>"""
                 """</tr>"""%locals())
    r.append("</tbody></table>")
    for d in dirs:
        d = d.replace(os.path.sep,"/")
        # r.append(_list_packages(d,_path))
        r.append("""<div class="packpath" id="path-%s">"""
                 """<h1 class="firsttitle" style="font-size: 8pt;">"""
                 """<a name="%s"></a>"""
                 """<a href="javascript: callto({'handler': Show}, 'packadmin', 'list_parent_packages', ['%s',0]); void(0);">"""
                 """+&nbsp;Package Path: %s."""
                 """</a></h1></div>"""%(d,d,d,d))
    r.append("</div>")
    return "\n".join(r)

def index():
    d = {
        'Title': """Dweba's Package Manager""",
        'Content': list_packages(),
        'JavascriptInit': """DOM3K_Call(function(d){});""",
        'JavascriptGlobal': template3k.get_template("templates/manage_files.js").merge({}),
        'HeadTag': """<script type="text/javascript" src="/edit_area/edit_area_full.js"></script>"""
        }
    return template3k.get_template(appconfig.template_admin).merge(d)
