#coding: utf8;

__doc__ = ''' Easyform server to packadmin.admin_objects '''

import auth3k
import auth3k.authweb as authweb
_spyro_authentication = authweb.Auth()
# To set permisions to the global module or object, uncomment the following
# [ [allowed users list], [ allowed groups list], [ denied user list] ]
_spyro_authentication.permsobject['packadmin.admin_objects'] = [[],['admin'],[]]
# Note: In module objects the object name must be the name of the object. If an authobject is
# manages many objects, there may exists many object names

# For method level management
# _spyro_authentication.perms['methodname'] = [[],['admin'],[]]

# Gets automatically the connection because is already charged
import re
import warn
import db3k
import template3k
import SPyRO as spyro
import appconfig

db = db3k.Connection(name='admin',
                     database='admin',
                     user=None,
                     password=None,
                     host=None,
                     manager='sqlite3')

#######################################
#######################################
#############
############  INDEX
#############
#######################################
#######################################
def index(**kwargs):
    objname = 'packadmin.admin_objects'
    title = 'Packadmin.Admin_Objects'
    temp = template3k.get_template(appconfig.template_admin)
    return temp.merge(PageTitle=objname.title(),
                      Menu='''<td><a href="index">%s</a></td>'''%objname.title(),
                      JavascriptInit='''callto({'handler':Show},'packadmin.admin_objects','showpage')''',
                      DevelQuote='',
                      Title='Module ' + title,
                      Content=''
                      )

def showpage(**kwargs):
    objname = 'packadmin.admin_objects'
    
    res = []
    res.append(('Admin Objgroups',list_admin_objgroups(),'admin_objgroups'))
    res.append(('Admin Objects',list_admin_objects(),'admin_objects'))
    return res



#######################################
#####
####   COMMON CUSTOMIZABLE FUNCTIONS
#####
#######################################
def _sql_quote(f):
    return str(f).replace("'","''")

def _join_cond(d,joinmethod=None):
    try:
        items = d.items()
    except AttributeError:
        return ' 1=1 '
    if len(items) == 0:
        return ' 1=1 '
    else:
        if joinmethod == 'LIKE':
            return ' AND '.join([ "%s LIKE '%s%%'"%(_sql_quote(x[0]),_sql_quote(x[1])) for x in d.items()])
        else:
            return ' AND '.join([ "%s = '%s'"%(_sql_quote(x[0]),_sql_quote(x[1])) for x in d.items()])

# def _field_title(f):
#     return f.replace("_",' ').title()

# Field names transformations
_dictionary = {
    '_newitem' : 'New Item',
    '_delitem' : 'Delete',
    '_moditem' : 'Modify',
    }


def _title(g):
    """  To support accuted languages"""
    w = g.group(2)
    return " %s%s"%( w[0].title(),w[1:])
    
def _field_title(f):
    try:
        f = _dictionary[f]
    except KeyError:
        pass
    return re.sub(r"([\s_])([^_\s]+)",_title," " + f)
    # simpler, but doesn't support accuted characters
    # return f.replace("_",' ').title()


def _quote_html(f):
    return f.replace("<","&lt;").replace(">","&gt;") #.replace("&","&amp;")

import types
def _strip_field(value):
    # warn.debug("VVVVV   ",repr(value))
    if value is None:
        return ''
    elif isinstance(value, (types.FloatType, types.IntType, types.LongType)):
        return value
    else:
        for enc in ("utf8","latin1","latin2","latin3","utf16"):
            try:
                return value.encode(enc).decode(enc)
            except UnicodeEncodeError, e:
                warn.debug("Ignoring and Retrying on UnicodeError (enc, value, error): ", repr((enc, value, e)))
        enc = "utf8"
        return value.encode(enc).decode(enc,"replace")

def _strip_dict(d):
    for k,v in d.items():
        try:
            d[k] = _strip_field(d[k])
        except AttributeError:
            pass
    return d

def _pager(begin, count, total, template, num=15):
    xres = [ "Viewing %s elements (starting at %sth) from %s total entries "%(min(count,total-begin), begin, total) ]
    start = max(begin - num * count, 0)
    for x in range(num*2):
        page_number = start / count + 1
        if start == begin:
            xres.append("%s&nbsp;"%page_number)
        else:
            xres.append( template % locals() )
        start += count
        if start > total:
            if page_number == 1: return []
            break
    return xres


#######################################################################################
#######################################################################################
######
######   FORMS
######
#######################################################################################
#######################################################################################

_InternalFields_admin_objgroups = []
#_spyro_authentication.perms['form_admin_objgroups'] = False
#_spyro_authentication.perms['list_admin_objgroups'] = False
#_spyro_authentication.perms['modify_admin_objgroups'] = False
#_spyro_authentication.perms['delete_admin_objgroups'] = False

def form_admin_objgroups(mode = None,condition_dict = None, **kwargs):
    tabname = 'admin_objgroups'
    objname = 'packadmin.admin_objects'
    if mode is None:
        mode = 'insert'
    dictdefault = {'' : 'Send','id' : '','description' : ''} # dictionary
    dictdefault['__objname__'] = objname
    if condition_dict is None:
        cond = '{}'
    else:
        cond = condition_dict
    dictdefault['__onsubmit__'] = r"commitform(this, '%s','%s',['%s',Easyform_formToDict(this),%s]);"%(objname,tabname,mode,cond)
    if mode == 'update':
        qupdate = 'SELECT * FROM %s WHERE %s'%(tabname,_join_cond(condition_dict))
        dictdefault.update(db.hquery(qupdate)[0])
        _strip_dict(dictdefault)
    elif mode != 'insert':
        raise AttributeError, "Attribute mode is set to an unknown value (insert|update) '%s'"%mode 
    return """
            <strong>Admin Objgroups</strong>
            <form method="POST" enctype="multipart/form-data" id="form-admin_objgroups" action="javascript: void(0);" onsubmit="%(__onsubmit__)s" name="admin_objgroups" >
                <table><tbody>
            <tr><td>Id: </td><td><input name="id:string:" type="text" value="%(id)s" size="40" /></td></tr>
<tr><td>Description: </td><td><input name="description:string:" type="text" value="%(description)s" size="40" /></td></tr>
<tr><td><input type="submit" name=":submit" value="%()s" /></td></tr>
                </tbody></table>
            </form>
            """%dictdefault

def modify_admin_objgroups(mode, data, cond, **kwargs):
    tabname = 'admin_objgroups'
    objname = 'packadmin.admin_objects'
    # auth = spyro.getrequest()['auth']
    # assert _spyro_authentication._rawauth(auth,  objname, modify_admin_objgroups)
    if mode == 'update':
        if None in (data,cond): raise Exception("Unable to update table %s with empty data"%tabname)
        if len(cond) == 0: raise Exception("Unable to update %s without condition"%tabname)
        # It's a good idea overwrite user id fields with information of the authentications
        # For example:
        #cond['userid'] = spyro.getrequest()['auth'][0]
        x = db.update(tabname, _strip_dict(data), cond)
    elif mode == 'insert':
        if data is None: raise Exception("Unable to insert into table %s with empty data"%tabname)
        #cond['userid'] = spyro.getrequest()['auth'][0]
        x = db.insert(tabname, _strip_dict(data))
    else:
        raise Exception("Unknown action to modify %s"%tabname)
    if kwargs.has_key('__dont_return_list'):
        return x
    else:
        try:
            x = kwargs['__list_kwargs']
        except:
            x = {}
        return ((tabname,(list_admin_objgroups(**x)),tabname),)

def delete_admin_objgroups(cond, **kwargs):
    tabname = 'admin_objgroups'
    if None is cond: raise Exception("Unable to delete table with empty condition")
    if len(cond) == 0: raise Exception("Unable to delete without condition")
    # auth = spyro.getrequest()['auth']
    # assert _spyro_authentication._rawauth(auth,  objname, delete_admin_objgroups)
    x = db.execute('DELETE FROM %s WHERE %s'%(tabname,_join_cond(cond)))
    if kwargs.has_key('__dont_return_list'):
        return x
    else:
        try:
            x = kwargs['__list_kwargs']
        except:
            x = {}
        return ((tabname,(list_admin_objgroups(**x)),tabname),)

def list_admin_objgroups(condition_dict = None, orderby = None, usecondmethod = None, **kwargs):
    tabname = 'admin_objgroups'
    objname = 'packadmin.admin_objects'
    suffix = 'admin_objgroups'
    # auth = spyro.getrequest()['auth']
    # assert _spyro_authentication._rawauth(auth,  objname, list_admin_objgroups)
    _newitem = _dictionary['_newitem']
    _delitem = _dictionary['_delitem']
    _moditem = _dictionary['_moditem']
    if condition_dict is None: condition_dict = {}
    condition = _join_cond(condition_dict, usecondmethod)
    #condition = ' 1 == 1'
    _orderby = orderby
    if orderby is None:
        orderby = ''
    else:
        orderby = 'ORDER BY ' + (','.join(map(_sql_quote,orderby)))
    global db
    fields = ['id', 'description']
    count = kwargs.get('count',40)
    begin = kwargs.get('begin',0)
    # Using cursors yields to a very poor performance (maybe we use it in a wrong way).
    # Using count and limit to enhance performance (2 queries :( )
    limits = 'LIMIT %d OFFSET %d'%(count,begin)
    # cursor = db.conn.cursor()
    # cursor.execute('SELECT %s FROM %s WHERE %s %s'%(",".join(fields),tabname,condition,orderby))
    # This was tested in postgresql
    rowcount = db.query('SELECT COUNT(*) FROM %s WHERE %s %s'%(tabname,condition,orderby))[0][0]
    query = db.query('SELECT %s FROM %s WHERE %s %s %s'%(",".join(fields),tabname,condition,orderby,limits))
    l = [ '<tr>' ]
    for f in fields:
        if f in _InternalFields_admin_objgroups: continue
        l.append("<th>%s</th>"%_field_title(f))
    l.append('</tr>')
    # if begin > 0: cursor.scroll(begin)
    # for q in cursor.fetchmany(count):
    for q in query:
        name_id = fields[0]
        data_id = q[0]
        if len(q) > 1:
            index = 1
        else:
            index = 0
        name_name = fields[index]
        data_name = q[index]
        r = []
        for i,f in enumerate(fields):
            if f in _InternalFields_admin_objgroups: continue
            r.append(_map_result_admin_objgroups(f, q[i]))# _strip_field(q[i]))
        controls = [
            (r"""<a href="javascript: showformmodify('%(objname)s', '%(suffix)s', """
             r"""['update', {'%(name_id)s':'%(data_id)s'}]);void(0);">%(_moditem)s</a>"""%locals()),
            (r"""<a href="javascript: calldelete('%(objname)s', '%(suffix)s',  """
             r"""[{'%(name_id)s':'%(data_id)s'}]);void(0);">%(_delitem)s</a>"""%locals())
        ]
        r.append(" &nbsp; ".join(controls))
        l.append('<tr>' + " ".join(map(lambda s: "<td>%s</td>"%s,r)) + '</tr>')
    # cursor.close()
    newitemlink = r"""<a href="javascript: showformmodify('%(objname)s', '%(suffix)s'); ">%(_newitem)s</a>"""%locals()
    if usecondmethod is None:
        strcondmethod = 'null'
    else:
        strcondmethod = repr(usecondmethod)
    if _orderby is None:
        strorderby = 'null'
    else:
        strorderby = repr(_orderby)

    tem = r"""<a href="javascript: calllist('%(objname)s', '%(suffix)s', [%(condition_dict)s,%(strorderby)s,%(strcondmethod)s], { 'begin' : %%(start)s, 'count' : %%(count)s } ) ; void(0);" >%%(page_number)s</a> """%locals()
    res = [ ]
    if len(condition_dict) > 0:
        res.append("Filter: %s"%",".join([ "%s: %s"%x for x in condition_dict.items()]))
    res.append( "".join( _pager(begin, count, rowcount, tem) ) )
    res.append("".join(['<table><tbody>', '\n'.join(l),' </tbody></table>',newitemlink," &nbsp; "]) )
    res.append(r"""<a href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\'Search and Filter on %(tabname)s\'')}}, '%(objname)s', 'form_search_%(tabname)s'); void(0);">Search</a>"""%locals())
    res.append("<br />")
    res.append(res[0])
    return "".join(res)

def form_search_admin_objgroups(**kwargs):
    tabname = 'admin_objgroups'
    objname = 'packadmin.admin_objects'
    dictdefault = {'' : 'Send','id' : '','description' : ''} # dictionary
    dictdefault['__objname__'] = objname
    dictdefault['__onsubmit__'] = r"callto({'handler': Show}, '%s','search_%s',[Easyform_formToDict(this), null, 'LIKE']);"%(objname,tabname)
    return """
            <strong>Admin Objgroups</strong>
            <form method="POST" enctype="multipart/form-data" id="form-admin_objgroups" action="javascript: void(0);" onsubmit="%(__onsubmit__)s" name="admin_objgroups" >
                <table><tbody>
            <tr><td>Id: </td><td><input name="id:string:" type="text" value="%(id)s" size="40" /></td></tr>
<tr><td>Description: </td><td><input name="description:string:" type="text" value="%(description)s" size="40" /></td></tr>
<tr><td><input type="submit" name=":submit" value="%()s" /></td></tr>
                </tbody></table>
            </form>
            """%dictdefault

def search_admin_objgroups(cond,orderby=None,usecondmethod=None,**kwargs):
    tabname = 'admin_objgroups'
    objname = 'packadmin.admin_objects'
    return (('', list_admin_objgroups(cond, orderby, usecondmethod=usecondmethod), tabname),)

def preview_admin_objgroups(id, condition_dict=None, display = None, **kwargs):
    if condition_dict is None: condition_dict = {}
    tabname = 'admin_objgroups'
    # auth = spyro.getrequest()['auth']
    # assert _spyro_authentication._rawauth(auth,  objname, preview_admin_objgroups)
    # cond = ' 1=1 '
    condition = _join_cond(condition_dict)
    if display is None:
        display = 'description'
    return db.query('SELECT %s,%s FROM %s WHERE %s'%(_sql_quote(id),_sql_quote(display),tabname,condition))

def _map_result_admin_objgroups(fieldname, value):
    foreigns = {}
    try:
        # This is the default behaviour, you must create caches of this values in order to speed up
        # this requests
        foreigntable, foreignfield = foreigns[fieldname]
        res = eval("preview_%s"%foreigntable)(foreignfield,{foreignfield:value})
        value = res[0][1]
    except (KeyError, IndexError):
        pass
    return _strip_field(value)

_InternalFields_admin_objects = []
#_spyro_authentication.perms['form_admin_objects'] = False
#_spyro_authentication.perms['list_admin_objects'] = False
#_spyro_authentication.perms['modify_admin_objects'] = False
#_spyro_authentication.perms['delete_admin_objects'] = False

def form_admin_objects(mode = None,condition_dict = None, **kwargs):
    tabname = 'admin_objects'
    objname = 'packadmin.admin_objects'
    if mode is None:
        mode = 'insert'
    dictdefault = {'' : 'Send','name' : '','license' : '','ownermail' : '','imageurl' : '','enabled' : '','indexurl' : '','precedence' : '','groupname' : '','version' : '','provider' : '','homepage' : '','description' : ''} # dictionary
    dictdefault['__objname__'] = objname
    if condition_dict is None:
        cond = '{}'
    else:
        cond = condition_dict
    dictdefault['__onsubmit__'] = r"commitform(this, '%s','%s',['%s',Easyform_formToDict(this),%s]);"%(objname,tabname,mode,cond)
    if mode == 'update':
        qupdate = 'SELECT * FROM %s WHERE %s'%(tabname,_join_cond(condition_dict))
        dictdefault.update(db.hquery(qupdate)[0])
        _strip_dict(dictdefault)
    elif mode != 'insert':
        raise AttributeError, "Attribute mode is set to an unknown value (insert|update) '%s'"%mode 
    return """
            <strong>Admin Objects</strong>
            <form method="POST" enctype="multipart/form-data" id="form-admin_objects" action="javascript: void(0);" onsubmit="%(__onsubmit__)s" name="admin_objects" >
                <table><tbody>
            <tr><td>Name: </td><td><input name="name:string:" type="text" value="%(name)s" size="40" /></td></tr>
<tr><td>Description: </td><td><input name="description:string:" type="text" value="%(description)s" size="40" /></td></tr>
<tr><td>Imageurl: </td><td><input name="imageurl:string:" type="text" value="%(imageurl)s" size="40" /></td></tr>
<tr><td>Version: </td><td><input name="version:string:" type="text" value="%(version)s" size="40" /></td></tr>
<tr><td>Indexurl: </td><td><input name="indexurl:string:" type="text" value="%(indexurl)s" size="40" /></td></tr>
<tr><td>License: </td><td><input name="license:string:" type="text" value="%(license)s" size="40" /></td></tr>
<tr><td>Ownermail: </td><td><input name="ownermail:string:" type="text" value="%(ownermail)s" size="40" /></td></tr>
<tr><td>Homepage: </td><td><input name="homepage:string:" type="text" value="%(homepage)s" size="40" /></td></tr>
<tr><td>Provider: </td><td><input name="provider:string:" type="text" value="%(provider)s" size="40" /></td></tr>
<tr><td>Groupname: </td><td><select onfocus="callto({'rawhandler': createhandlerselect('%(groupname)s'), 'handlerargs': this},'%(__objname__)s','preview_admin_objgroups',['id'])" name="groupname:string"></select><a style="text-decoration: none; font-family: sans-serif;" href="javascript: showformmodify('%(__objname__)s', 'admin_objgroups'); void(0);">+</a>\n</td></tr>
<tr><td>Precedence: </td><td><input name="precedence:int:" type="text" value="%(precedence)s" size="40" /></td></tr>
<tr><td>Enabled: </td><td><input name="enabled:int:" type="text" value="%(enabled)s" size="40" /></td></tr>
<tr><td><input type="submit" name=":submit" value="%()s" /></td></tr>
                </tbody></table>
            </form>
            """%dictdefault

def modify_admin_objects(mode, data, cond, **kwargs):
    tabname = 'admin_objects'
    objname = 'packadmin.admin_objects'
    # auth = spyro.getrequest()['auth']
    # assert _spyro_authentication._rawauth(auth,  objname, modify_admin_objects)
    if mode == 'update':
        if None in (data,cond): raise Exception("Unable to update table %s with empty data"%tabname)
        if len(cond) == 0: raise Exception("Unable to update %s without condition"%tabname)
        # It's a good idea overwrite user id fields with information of the authentications
        # For example:
        #cond['userid'] = spyro.getrequest()['auth'][0]
        x = db.update(tabname, _strip_dict(data), cond)
    elif mode == 'insert':
        if data is None: raise Exception("Unable to insert into table %s with empty data"%tabname)
        #cond['userid'] = spyro.getrequest()['auth'][0]
        x = db.insert(tabname, _strip_dict(data))
    else:
        raise Exception("Unknown action to modify %s"%tabname)
    if kwargs.has_key('__dont_return_list'):
        return x
    else:
        try:
            x = kwargs['__list_kwargs']
        except:
            x = {}
        return ((tabname,(list_admin_objects(**x)),tabname),)

def delete_admin_objects(cond, **kwargs):
    tabname = 'admin_objects'
    if None is cond: raise Exception("Unable to delete table with empty condition")
    if len(cond) == 0: raise Exception("Unable to delete without condition")
    # auth = spyro.getrequest()['auth']
    # assert _spyro_authentication._rawauth(auth,  objname, delete_admin_objects)
    x = db.execute('DELETE FROM %s WHERE %s'%(tabname,_join_cond(cond)))
    if kwargs.has_key('__dont_return_list'):
        return x
    else:
        try:
            x = kwargs['__list_kwargs']
        except:
            x = {}
        return ((tabname,(list_admin_objects(**x)),tabname),)

def list_admin_objects(condition_dict = None, orderby = None, usecondmethod = None, **kwargs):
    tabname = 'admin_objects'
    objname = 'packadmin.admin_objects'
    suffix = 'admin_objects'
    # auth = spyro.getrequest()['auth']
    # assert _spyro_authentication._rawauth(auth,  objname, list_admin_objects)
    _newitem = _dictionary['_newitem']
    _delitem = _dictionary['_delitem']
    _moditem = _dictionary['_moditem']
    if condition_dict is None: condition_dict = {}
    condition = _join_cond(condition_dict, usecondmethod)
    #condition = ' 1 == 1'
    _orderby = orderby
    if orderby is None:
        orderby = ''
    else:
        orderby = 'ORDER BY ' + (','.join(map(_sql_quote,orderby)))
    global db
    # fields = ['name', 'description', 'imageurl', 'version', 'indexurl', 'license', 'ownermail', 'homepage', 'provider', 'groupname', 'precedence', 'enabled']
    fields = ['name', 'description', 'version', 'ownermail', 'homepage', 'provider', 'precedence', 'enabled']
    count = kwargs.get('count',40)
    begin = kwargs.get('begin',0)
    # Using cursors yields to a very poor performance (maybe we use it in a wrong way).
    # Using count and limit to enhance performance (2 queries :( )
    limits = 'LIMIT %d OFFSET %d'%(count,begin)
    # cursor = db.conn.cursor()
    # cursor.execute('SELECT %s FROM %s WHERE %s %s'%(",".join(fields),tabname,condition,orderby))
    # This was tested in postgresql
    rowcount = db.query('SELECT COUNT(*) FROM %s WHERE %s %s'%(tabname,condition,orderby))[0][0]
    query = db.query('SELECT %s FROM %s WHERE %s %s %s'%(",".join(fields),tabname,condition,orderby,limits))
    l = [ '<tr>' ]
    for f in fields:
        if f in _InternalFields_admin_objects: continue
        l.append("<th>%s</th>"%_field_title(f))
    l.append('</tr>')
    # if begin > 0: cursor.scroll(begin)
    # for q in cursor.fetchmany(count):
    for q in query:
        name_id = fields[0]
        data_id = q[0]
        if len(q) > 1:
            index = 1
        else:
            index = 0
        name_name = fields[index]
        data_name = q[index]
        r = []
        for i,f in enumerate(fields):
            if f in _InternalFields_admin_objects: continue
            r.append(_map_result_admin_objects(f, q[i]))# _strip_field(q[i]))
        controls = [
            (r"""<a href="javascript: showformmodify('%(objname)s', '%(suffix)s', """
             r"""['update', {'%(name_id)s':'%(data_id)s'}]);void(0);">%(_moditem)s</a>"""%locals()),
            (r"""<a href="javascript: calldelete('%(objname)s', '%(suffix)s',  """
             r"""[{'%(name_id)s':'%(data_id)s'}]);void(0);">%(_delitem)s</a>"""%locals())
        ]
        r.append(" &nbsp; ".join(controls))
        l.append('<tr>' + " ".join(map(lambda s: "<td>%s</td>"%s,r)) + '</tr>')
    # cursor.close()
    newitemlink = r"""<a href="javascript: showformmodify('%(objname)s', '%(suffix)s'); ">%(_newitem)s</a>"""%locals()
    if usecondmethod is None:
        strcondmethod = 'null'
    else:
        strcondmethod = repr(usecondmethod)
    if _orderby is None:
        strorderby = 'null'
    else:
        strorderby = repr(_orderby)

    tem = r"""<a href="javascript: calllist('%(objname)s', '%(suffix)s', [%(condition_dict)s,%(strorderby)s,%(strcondmethod)s], { 'begin' : %%(start)s, 'count' : %%(count)s } ) ; void(0);" >%%(page_number)s</a> """%locals()
    res = [ ]
    if len(condition_dict) > 0:
        res.append("Filter: %s"%",".join([ "%s: %s"%x for x in condition_dict.items()]))
    res.append( "".join( _pager(begin, count, rowcount, tem) ) )
    res.append("".join(['<table><tbody>', '\n'.join(l),' </tbody></table>',newitemlink," &nbsp; "]) )
    res.append(r"""<a href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\'Search and Filter on %(tabname)s\'')}}, '%(objname)s', 'form_search_%(tabname)s'); void(0);">Search</a>"""%locals())
    res.append("<br />")
    res.append(res[0])
    return "".join(res)

def form_search_admin_objects(**kwargs):
    tabname = 'admin_objects'
    objname = 'packadmin.admin_objects'
    dictdefault = {'' : 'Send','name' : '','license' : '','ownermail' : '','imageurl' : '','enabled' : '','indexurl' : '','precedence' : '','groupname' : '','version' : '','provider' : '','homepage' : '','description' : ''} # dictionary
    dictdefault['__objname__'] = objname
    dictdefault['__onsubmit__'] = r"callto({'handler': Show}, '%s','search_%s',[Easyform_formToDict(this), null, 'LIKE']);"%(objname,tabname)
    return """
            <strong>Admin Objects</strong>
            <form method="POST" enctype="multipart/form-data" id="form-admin_objects" action="javascript: void(0);" onsubmit="%(__onsubmit__)s" name="admin_objects" >
                <table><tbody>
            <tr><td>Name: </td><td><input name="name:string:" type="text" value="%(name)s" size="40" /></td></tr>
<tr><td>Description: </td><td><input name="description:string:" type="text" value="%(description)s" size="40" /></td></tr>
<tr><td>Imageurl: </td><td><input name="imageurl:string:" type="text" value="%(imageurl)s" size="40" /></td></tr>
<tr><td>Version: </td><td><input name="version:string:" type="text" value="%(version)s" size="40" /></td></tr>
<tr><td>Indexurl: </td><td><input name="indexurl:string:" type="text" value="%(indexurl)s" size="40" /></td></tr>
<tr><td>License: </td><td><input name="license:string:" type="text" value="%(license)s" size="40" /></td></tr>
<tr><td>Ownermail: </td><td><input name="ownermail:string:" type="text" value="%(ownermail)s" size="40" /></td></tr>
<tr><td>Homepage: </td><td><input name="homepage:string:" type="text" value="%(homepage)s" size="40" /></td></tr>
<tr><td>Provider: </td><td><input name="provider:string:" type="text" value="%(provider)s" size="40" /></td></tr>
<tr><td>Groupname: </td><td><select onfocus="callto({'rawhandler': createhandlerselect('%(groupname)s'), 'handlerargs': this},'%(__objname__)s','preview_admin_objgroups',['id'])" name="groupname:string"></select><a style="text-decoration: none; font-family: sans-serif;" href="javascript: showformmodify('%(__objname__)s', 'admin_objgroups'); void(0);">+</a>\n</td></tr>
<tr><td>Precedence: </td><td><input name="precedence:int:" type="text" value="%(precedence)s" size="40" /></td></tr>
<tr><td>Enabled: </td><td><input name="enabled:int:" type="text" value="%(enabled)s" size="40" /></td></tr>
<tr><td><input type="submit" name=":submit" value="%()s" /></td></tr>
                </tbody></table>
            </form>
            """%dictdefault

def search_admin_objects(cond,orderby=None,usecondmethod=None,**kwargs):
    tabname = 'admin_objects'
    objname = 'packadmin.admin_objects'
    return (('', list_admin_objects(cond, orderby, usecondmethod=usecondmethod), tabname),)

def preview_admin_objects(id, condition_dict=None, display = None, **kwargs):
    if condition_dict is None: condition_dict = {}
    tabname = 'admin_objects'
    # auth = spyro.getrequest()['auth']
    # assert _spyro_authentication._rawauth(auth,  objname, preview_admin_objects)
    # cond = ' 1=1 '
    condition = _join_cond(condition_dict)
    if display is None:
        display = 'description'
    return db.query('SELECT %s,%s FROM %s WHERE %s'%(_sql_quote(id),_sql_quote(display),tabname,condition))

def _map_result_admin_objects(fieldname, value):
    foreigns = {u'groupname': (u'admin_objgroups', u'id')}
    try:
        # This is the default behaviour, you must create caches of this values in order to speed up
        # this requests
        foreigntable, foreignfield = foreigns[fieldname]
        res = eval("preview_%s"%foreigntable)(foreignfield,{foreignfield:value})
        value = res[0][1]
    except (KeyError, IndexError):
        pass
    return _strip_field(value)
