__doc__ = """

Module for managing users. This object provides modules and functions to manage users in many
different ways: User handled by functions, simple file, database, or anything that you want to customize.

It uses and extends the aut3k.authweb package

TODO: Object-Package manager: online editor, push zip, tars, etc.
TODO: Start / Stop
"""

import os
import db3k
import warn
import template3k

# warn.debug(db3k.get_database_data(name="useradmin"))
dbconn = db3k.Connection(name="useradmin",database="useradmin",manager="sqlite3")

########################################
## testing the availability of the tables


import appconfig
try:
    _cryptpassword = appconfig.cryptpassword
except AttributeError:
    import chococrypt
    _cryptpassword = chococrypt._cryptpassword_md5
    
try:
    dbconn.query("""SELECT * FROM auth_users, auth_usergroups """
                 """WHERE auth_users.username = 'admin' """
                 """  AND auth_usergroups.username = 'admin' """)
except Exception:
    import spyropack
    spyropack.sql_script(dbconn, "sql/useradmin.sql")
    dbconn.insert('auth_users',{'username': 'admin',
                                'password': _cryptpassword('hello','admin')})
    dbconn.insert('auth_usergroups', {'username': 'admin',
                                      'groupname': 'admin'})

class LookInside:
    def __init__(self, groupname):
        self.groupname = groupname
        
    def __contains__(self, username):
        res = dbconn.query("SELECT COUNT(*) "
                           "FROM auth_usergroups "
                           "WHERE groupname = ? AND username = ?",(self.groupname,username))[0][0]
        return res > 0

try:
    appconfig.getgroup
    appconfig.validate_user
    def getgroup(groupname):
        return appconfig.getgroup(groupname)
    
    def validate_user(user, password):
        return appconfig.validate_user(user, password)

except AttributeError:
    def getgroup(groupname):
        return LookInside(groupname)
    
    def validate_user(user, password):
        try:
            storedpass = dbconn.query("SELECT password FROM auth_users WHERE username = ?", (user,))[0][0]
        except IndexError:
            return False
        return storedpass == _cryptpassword(password,user,storedpass)

def index():
    return template3k.get_template(appconfig.template_admin).merge({
        'Title': """User Manager""",
        'JavascriptInit': """callto({'handler' : Show}, 'useradmin.useradmin','showpage');""",
        'Content': """""",
        })
