#coding: utf8;

__doc__ = """ Easyform server to useradmin.useradmin """

import warn
warn = warn.Warn("useradmin.useradmin")
import auth3k
import auth3k.authweb as authweb
import appconfig
try:
    _cryptpassword = appconfig.cryptpassword
except AttributeError:
    import chococrypt
    _cryptpassword = chococrypt._cryptpassword_md5

import SPyRO  as spyro
_spyro_authentication = authweb.Auth()
# To set permisions to the global module or object, uncomment the following
# [ [allowed users list], [ allowed groups list], [ denied user list] ]
_spyro_authentication.permsobject['useradmin.useradmin'] = [[],['admin'],[]]
# Note: In module objects the object name must be the name of the object. If an authobject is
# manages many objects, there may exists many object names

# For method level management
_spyro_authentication.perms['only_admin'] = [[],['admin'],[]]
_spyro_authentication.perms['only_valid'] = None

import db3k
db = db3k.Connection(name="useradmin")

def _sql_quote(f):
    return str(f).replace("'","''")

def _join_cond(d):
    try:
        items = d.items()
    except AttributeError:
        return ' 1==1 '
    if len(items) == 0:
        return ' 1==1 '
    else:
        return ' AND '.join([ "%s = '%s'"%(_sql_quote(x[0]),_sql_quote(x[1])) for x in d.items()])

# def _field_title(f):
#     return f.replace("_",' ').title()

# Field names transformations
_dictionary = {
    '_newitem' : 'New Item',
    '_delitem' : 'Delete',
    '_moditem' : 'Modify',
    }

import re


def _title(g):
    """  To support accuted languages"""
    w = g.group(2)
    return " %s%s"%( w[0].title(),w[1:])
    
def _field_title(f):
    try:
        f = _dictionary[f]
    except KeyError:
        pass
    return re.sub(r"([\s_])([^_\s]+)",_title," " + f)
    # simpler, but doesn't support accuted characters
    # return f.replace("_",' ').title()


def _quote_html(f):
    return f.replace("<","&lt;").replace(">","&gt;") #.replace("&","&amp;")

def _strip_field(f):
    if f is None: return ''
    try:
        # return _quote_html(f.encode("utf8").rstrip())
        return _quote_html(f.encode("utf8").rstrip())
    except:
        return f

def _strip_dict(d):
    for k,v in d.items():
        try:
            d[k] = _strip_field(d[k])
        except AttributeError:
            pass
    return d

def _pager(begin, count, total, template, num=15):
    xres = [ "Viewing %s elements (starting at %sth) from %s total entries "%(min(count,total-begin), begin, total) ]
    start = max(begin - num * count, 0)
    for x in range(num*2):
        page_number = start / count + 1
        if start == begin:
            xres.append("%s&nbsp;"%page_number)
        else:
            xres.append( template % locals() )
        start += count
        if start > total:
            if page_number == 1: return []
            break
    return xres

    

_InternalFields_auth_users = []
#_spyro_authentication.perms['form_auth_users'] = False
#_spyro_authentication.perms['list_auth_users'] = False
#_spyro_authentication.perms['modify_auth_users'] = False
#_spyro_authentication.perms['delete_auth_users'] = False

def form_auth_users(mode = None,condition_dict = None, **kwargs):
    if mode is None: mode = 'insert'
    dictdefault = {'username' : '','' : 'Send'}
    dictdefault['__mode__'] = mode
    if condition_dict is None:
        dictdefault['__cond__'] = '{}'
    else:
        dictdefault['__cond__'] = condition_dict
    if mode == 'update':
        dictdefault.update( db.hquery('SELECT * FROM auth_users WHERE %s'%_join_cond(condition_dict))[0] )
        _strip_dict(dictdefault)
    elif mode != 'insert':
        raise AttributeError, "Attribute mode is set to an unknown value (insert|update) '%s'"%mode 
    return r"""
            <strong>Auth Users</strong>
            <form method="POST" enctype="multipart/form-data" id="auth_users" action="javascript: void(0);" onsubmit="commitform(this, 'useradmin.useradmin','auth_users',['%(__mode__)s',Easyform_formToDict(this),%(__cond__)s]);" name="auth_users" >
                <table><tbody>
<tr><td>Username:        </td><td><input name="username:string:mandatory" type="text" value="%(username)s" size="40" /></td></tr>
<tr><td>Password:        </td><td><input name="password:string:mandatory" type="password" value="" size="40" /></td></tr>
<tr><td>Password(Again): </td><td><input name="password2:string:mandatory" type="password" value="" size="40" /></td></tr>
<tr><td><input type="submit" name=":submit" value="%()s" /></td></tr>
                </tbody></table>
            </form>
            """%dictdefault


def modify_auth_users(mode, data, cond, **kwargs):
    auth = spyro.getrequest()['auth']
    if not _spyro_authentication._rawauth(auth, 'useradmin.useradmin', 'only_admin'):
        mode = 'update'
        cond["username"] = auth[0]
    warn.debug(mode, data, cond)
    if len(data["password"]) < 5:
        raise Exception("ERROR: The password must have 5 characters at least")
    if data["password2"] != data["password"]:
        raise Exception("ERROR: The passwords doesn't match")
    del data["password2"]
    data["password"] = _cryptpassword(data["password"], data["username"])
    if mode == 'update':
        if None in (data,cond): raise "Unable to update table with empty data"
        if len(cond) == 0: raise "Unable to update without condition"
        #cond['idusuario'] = kwargs['auth'][0]
        x = db.update('auth_users', _strip_dict(data), cond)
    elif mode == 'insert':
        if data is None: raise "Unable to insert into table with empty data"
        #data['idusuario'] = kwargs['auth'][0]
        x = db.insert('auth_users', _strip_dict(data))
    else:
        raise "Unknown action to modify auth_users "
    if kwargs.has_key('__dont_return_list'):
        return x
    else:
        try:
            x = kwargs['__list_kwargs']
        except:
            x = {}
        return (('auth_users',(list_auth_users(**x)),'auth_users'),)

def delete_auth_users(cond, **kwargs):
    if None is cond: raise "Unable to delete table with empty condition"
    if len(cond) == 0: raise "Unable to delete without condition"
    auth = spyro.getrequest()['auth']
    assert _spyro_authentication._rawauth(auth, 'useradmin.useradmin', 'only_admin')
    #cond['idusuario'] = kwargs['auth'][0]
    x = db.execute('DELETE FROM auth_users WHERE %s'%_join_cond(cond))
    x = db.execute('DELETE FROM auth_usergroups WHERE %s'%_join_cond(cond))
    if kwargs.has_key('__dont_return_list'):
        return x
    else:
        try:
            x = kwargs['__list_kwargs']
        except:
            x = {}
        return (('auth_users',(list_auth_users(**x)),'auth_users'),)


def list_auth_users(condition_dict = None, orderby = None, **kwargs):
    tabname = 'auth_users'
    objname = 'useradmin.useradmin'
    suffix = 'auth_users'
    _newitem = _dictionary['_newitem']
    _delitem = _dictionary['_delitem']
    _moditem = _dictionary['_moditem']    
    if condition_dict is None: condition_dict = {}
    #assert _spyro_authentication._rawauth(kwargs['auth'], 'useradmin.useradmin', 'list_auth_users')
    #condition_dict['idusuario'] = kwargs['auth'][0]
    condition = _join_cond(condition_dict)
    if orderby is None:
        orderby = ''
    else:
        orderby = 'ORDER BY ' + (','.join(map(_sql_quote,orderby)))
    global db
    fields = ['username']
    count = kwargs.get('count',30)
    begin = kwargs.get('begin',0)
    # Using cursors yields to a very poor performance (maybe we use in a wrong way). Using count and limit to enhance performance (2 queries :( )
    limits = 'LIMIT %d OFFSET %d'%(count,begin)
    # cursor = db.conn.cursor()
    # cursor.execute('SELECT %s FROM %s WHERE %s %s'%(",".join(fields),tabname,condition,orderby))
    # This was tested in postgresql
    rowcount = db.query('SELECT COUNT(*) FROM %s WHERE %s %s'%(tabname,condition,orderby))[0][0]
    query = db.query('SELECT %s FROM %s WHERE %s %s %s'%(",".join(fields),tabname,condition,orderby,limits))
    l = [ '<tr><th>Username</th><th>Commands</th><th>Groups</th><th>Group Commands</th></tr>' ]
    # if begin > 0: cursor.scroll(begin)
    # for q in cursor.fetchmany(count):
    for q in query:
        name_id = fields[0]
        data_id = q[0]
        if len(q) > 1:
            index = 1
        else:
            index = 0
        name_name = fields[index]
        data_name = q[index]
        r = []
        for i,f in enumerate(fields):
            if f in _InternalFields_auth_users: continue
            r.append(_strip_field(q[i]))
        controls = [
            r"""<a href="javascript: showformmodify('%(objname)s', '%(suffix)s', ['update', {'%(name_id)s':'%(data_id)s'}]);void(0);">%(_moditem)s</a>"""%locals(),
            r"""<a href="javascript: calldelete('%(objname)s', '%(suffix)s',  [{'%(name_id)s':'%(data_id)s'}]);void(0);">%(_delitem)s</a>"""%locals()
        ]
        r.append("&nbsp;".join(controls))
        _r = []
        for groupname in db.query("SELECT * FROM auth_usergroups WHERE username = '%s'"%(data_id)):
            groupname = groupname[1]
            _r.append("""<a href="javascript: calldelete('%(objname)s', 'auth_usergroups', """
                     """[{'username':'%(data_id)s', 'groupname':'%(groupname)s'}]); void(0);">"""
                     """%(groupname)s</a>"""%locals())
        r.append(", ".join(_r))
        r.append("""<a href="javascript: showformmodify('%(objname)s', 'auth_usergroups', ['insert', {'username':'%(data_id)s'}]); void(0);">Attach to group</a> &nbsp; """%locals())

        l.append('<tr>' + " ".join(map(lambda s: "<td>%s</td>"%s,r)) + '</tr>')
    # cursor.close()
    newitemlink = r"""<a href="javascript: showformmodify('%(objname)s', '%(suffix)s'); ">%(_newitem)s</a>"""%locals()
    tem = r"""<a href="javascript: calllist('%(objname)s', '%(suffix)s', [%(condition_dict)s], { 'begin' : %%(start)s, 'count' : %%(count)s } ) ; void(0);" >%%(page_number)s</a> """%locals()

    res = [ "".join( _pager(begin, count, rowcount, tem) ) ]
    res.append( "".join(['<table><tbody>', '\n'.join(l),' </tbody></table>',newitemlink]) )
    res.append("<br />")
    res.append(res[0])
    return "".join(res)

_InternalFields_auth_usergroups = []
#_spyro_authentication.perms['form_auth_usergroups'] = False
#_spyro_authentication.perms['list_auth_usergroups'] = False
#_spyro_authentication.perms['modify_auth_usergroups'] = False
#_spyro_authentication.perms['delete_auth_usergroups'] = False

def form_auth_usergroups(mode = None,condition_dict = None, **kwargs):
    if mode is None: mode = 'insert'
    dictdefault = {'username' : '','groupname' : '','' : 'Send'}
    dictdefault.update(condition_dict)
    if mode != 'insert':
        raise AttributeError, "Attribute mode is set to an unknown value (insert|update) '%s'"%mode 
    return r"""
            <strong>Auth Usergroups</strong>
            <form method="POST" enctype="multipart/form-data" id="auth_usergroups" action="javascript: void(0);" onsubmit="commitform(this, 'useradmin.useradmin','auth_usergroups',['insert',Easyform_formToDict(this)]);" name="auth_usergroups">
<table><tbody>
<tr><td>Username: </td><td>%(username)s<input name="username:string:mandatory" type="hidden" value="%(username)s" /></td></tr>
<tr><td>Groupname: </td><td><input name="groupname:string:mandatory" type="text" value="%(groupname)s" size="40" /></td></tr>
<tr><td><input type="submit" name=":submit" value="%()s" /></td></tr>
</tbody></table>
            </form>
            """%dictdefault

def modify_auth_usergroups(mode, data, cond='', **kwargs):
    # assert _spyro_authentication._rawauth(kwargs['auth'], 'useradmin.useradmin', 'modify_auth_usergroups')
    if mode == 'insert':
        if data is None:
            raise "Unable to insert into table with empty data"
        # warn.debug("XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX::: ",data)
        x = db.insert('auth_usergroups', _strip_dict(data))
    else:
        raise "Unknown action to modify auth_usergroups "
    if kwargs.has_key('__dont_return_list'):
        return x
    else:
        try:
            x = kwargs['__list_kwargs']
        except:
            x = {}
        return (('auth_users',(list_auth_users(**x)),'auth_users'),)

def delete_auth_usergroups(cond, **kwargs):
    if None is cond: raise "Unable to delete table with empty condition"
    if len(cond) == 0: raise "Unable to delete without condition"
    auth = spyro.getrequest()['auth']
    authuser = auth[0]
    if not _spyro_authentication._rawauth(auth, 'useradmin.useradmin', 'only_admin'):
        cond['username'] = authuser
    x = db.execute('DELETE FROM auth_usergroups WHERE %s'%_join_cond(cond))
    if kwargs.has_key('__dont_return_list'):
        return x
    else:
        try:
            x = kwargs['__list_kwargs']
        except:
            x = {}
        return (('auth_users',(list_auth_users(**x)),'auth_users'),)


def preview_auth_usergroups(id, condition_dict=None, display = None, **kwargs):
    if condition_dict is None: condition_dict = {}
    #assert _spyro_authentication._rawauth(kwargs['auth'], 'useradmin.useradmin', 'preview_auth_usergroups')
    #condition_dict['idusuario'] = kwargs['auth'][0]
    #cond = _join_cond(condition_dict)
    cond = 'true'
    if display is None: display = 'groupname'
    return db.query('SELECT %s,%s FROM auth_usergroups WHERE %s'%(_sql_quote(id),_sql_quote(display),cond))

def showpage(**kwargs):
   res = []
   res.append(('Auth Users',list_auth_users(),'auth_users'))
   return res

