
CREATE TABLE persona (
	id integer PRIMARY KEY,
	nombre VARCHAR(64) NOT NULL,
	apellidos VARCHAR(64) NOT NULL,
	fecha_nacimiento DATE,
	sexo VARCHAR(1),
	telefono1 VARCHAR(12),
	telefono2 VARCHAR(12),
	calle VARCHAR(32),
	numero VARCHAR(12),
	colonia VARCHAR(32),
	municipio VARCHAR(32),	
	estado VARCHAR(32),
	nacionalidad VARCHAR(32) NOT NULL,
	curp VARCHAR(64) UNIQUE NOT NULL,
	-- otros
	login VARCHAR(23),
	matricula VARCHAR(13) UNIQUE,
	-- o numero de empleado o login de tutor
	tipo VARCHAR(8),
	-- PROFESOR,ALUMNO,TUTOR (Sin catalogo, para simplicidad)
	observaciones TEXT
	);

CREATE INDEX idx_persona_nacimiento ON persona(fecha_nacimiento);
CREATE INDEX idx_persona_matricula ON persona(matricula);
CREATE INDEX idx_persona_tipo ON persona(tipo);
CREATE INDEX idx_persona_curp ON persona(curp);

CREATE TABLE alumno_tutor (
	alumno VARCHAR(64) REFERENCES persona(id) ON UPDATE CASCADE ON DELETE CASCADE,
	tutor VARCHAR(64) REFERENCES persona(id) ON UPDATE CASCADE ON DELETE CASCADE
);

CREATE INDEX idx_alumno_tutor ON alumno_tutor(alumno, tutor);

CREATE TABLE materia (
	id integer PRIMARY KEY,
	nombre VARCHAR(16) NOT NULL,
	creditos integer,
	nivel integer
);


CREATE TABLE persona_materia (
        id integer primary key,
	idpersona integer REFERENCES persona(id) ON UPDATE CASCADE,
	idmateria integer REFERENCES materia(id) ON UPDATE CASCADE,
	ciclo INTERVAL NOT NULL,
	calificacion_final FLOAT DEFAULT 0, -- SI ES ALUMNO
	observaciones TEXT
);

CREATE TABLE calificacion_parcial (
	nivel integer,
	calificacion FLOAT,
	idpersona VARCHAR(64) REFERENCES persona_materia(id) ON UPDATE CASCADE
);

CREATE INDEX _index_calificacion_parcial_idpersona ON calificacion_parcial(idpersona);

CREATE TABLE mensaje (
	fecha_emision TIMESTAMP NOT NULL,
	remitente VARCHAR(64) REFERENCES persona(id) ON UPDATE CASCADE ON DELETE CASCADE,
	destinatario VARCHAR(64) REFERENCES persona(id) ON UPDATE CASCADE ON DELETE CASCADE,
	contenido TEXT NOT NULL
);

