#coding: utf8;
"""
Easyform server to catalogos.catalogos
"""

import auth3k
import auth3k.authweb as authweb
_spyro_authentication = authweb.Auth()
# To set permisions to the global module or object, uncomment the following
# [ [allowed users list], [ allowed groups list], [ denied user list] ]
_spyro_authentication.permsobject['catalogos.catalogos'] = [[],['admin'],[]]
# Note: In module objects the object name must be the name of the object. If an authobject is
# manages many objects, there may exists many object names

# For method level management
# _spyro_authentication.perms['methodname'] = [[], ['admin'], []]

# Gets automatically the connection because is already charged
import re
import warn
import db3k
import time
import locale3k
import template3k
import SPyRO as spyro
import appconfig
import ft3k
import types
# import marshal

dbmanager='sqlite3'
if dbmanager.startswith('sqlite'): # param style qmark
    # sqlite can't use indexes with like operator,  we use a work around (see _like_cond function for details)
    likechar = None 
    _kwargs = {'timeout': 60 * 60}
else:
    likechar = '%%'
    _kwargs = {}
# elif param style = qmark: likechar = '%s'

# db = db3k.Connection(name='db/catalogos',
#                      database='db/catalogos',
#                      user='',
#                      password='',
#                      host=None,
#                      manager=dbmanager,
#                      kwargs=_kwargs)

ftname="ft%s"%('db/catalogos')
def ftpool_create(name):
    conn = db3k.create2(name=name)
    spyro.register_exit_handler(ftpool_exit_handler)
    return conn

ftpool=db3k.ThreadPool(ftname, dbmanager, ftpool_create)
dbftindex = db3k.Connection(
                     name=ftname,
                     database='db/catalogos',
                     user='',
                     password='',
                     host=None,
                     manager=dbmanager,
                     pool=ftpool)
db = dbftindex # transactional database connection

def ftpool_exit_handler():
    try:
        ftpool.delconn(1)
    except KeyError, e:
        warn.debug("ftpool_exit_handler ", 'catalogos.catalogos', ": ", e)

#######################################
#######################################
#############
############  INDEX
#############
#######################################
#######################################
def index(**kwargs):
    objname = 'catalogos.catalogos'
    title = 'Catalogos.Catalogos'
    temp = template3k.get_template(appconfig.template_admin)
    return temp.merge(PageTitle=objname.title(),
                      Menu='''<td><a href="index">%s</a></td>'''%objname.title(),
                      JavascriptInit='''callto({'handler': handle_commands},'catalogos.catalogos','showpage')''',
                      DevelQuote='',
                      Title='Module ' + title,
                      Content=''
                      )

TABLES=['pais', 'estado', 'ciudad', 'persona', 'categorias', 'producto', 'proveedor', 'catalogo', 'catalogo_producto', 'estados_venta', 'estados_fiscales', 'datos_facturacion', 'venta', 'venta_productos', 'comentario', 'producto_comentarios', 'tipo_pago', 'pagos']
def showpage(tables=TABLES):
    objname = 'catalogos.catalogos'
    itime = time.time()
    warn.debug("""If you want to index for a first time an existing database, """
               """you must enable the following piece of code (%s.showpage)"""%objname)
    if False:
        for x in TABLES:
            # print "DDDDD manage ft: ", x
            eval("manage_fulltext_%s"%x)(0, 1)
        print "***********************************> index time: ", time.time() - itime
    res = []
    for tabname in tables:
        res.extend(_list_method(tabname, None, None))
    return res

#######################################
#####
####   COMMON CUSTOMIZABLE FUNCTIONS
#####
#######################################
def _sql_quote(f):
    return _strip_field(f).replace("'","''")

def _like_cond(f, a):
    if likechar is None:
        b = a[:-1] + chr(ord(a[-1]) + 1)
        return "(%s >= '%s' AND %s < '%s')"%(f, a, f, b)
    else:
        return "%s LIKE '%s%s'"%(f, a, likechar)

def _join_cond(d,joinmethod=None):
    try:
        items = d.items()
    except AttributeError:
        return ' 1=1 '
    if len(items) == 0:
        return ' 1=1 '
    else:
        if joinmethod == 'LIKE':
            return ' AND '.join([ _like_cond(_sql_quote(k), _sql_quote(v)) for k, v in d.items() ])
        else:
            return ' AND '.join([ "%s = '%s'"%(_sql_quote(k), _sql_quote(v)) for k, v in d.items() ])

def _quote_html(f):
    return f.replace("<","&lt;").replace(">","&gt;") #.replace("&","&amp;")

def _strip_field(value):
    if value is None:
        return ''
    elif type(value) in types.StringTypes:
        for enc in ("utf8","latin1","latin2","latin3","utf16"):
            try:
                return value.encode(enc).decode(enc)
                # return value.encode(enc)
            except (UnicodeDecodeError,UnicodeEncodeError), e:
                warn.debug("Ignoring and Retrying on UnicodeError (enc, value, error): ", repr((enc, value, e)))
        enc = "utf8"
        return value.decode(enc,"replace")
        # return value.encode(enc,"replace")
    else:
        return str(value)

def _strip_dict(d):
    for k,v in d.items():
        try:
            d[k] = _strip_field(d[k])
        except AttributeError:
            pass
    return d

def _pager(begin, count, total, template, num=15):
    xres = [ "Viewing %s elements (starting at %sth) from %s total entries "%(min(count,total-begin), begin, total) ]
    start = max(begin - num * count, 0)
    for x in range(num*2):
        page_number = start / count + 1
        if start == begin:
            xres.append("%s&nbsp;"%page_number)
        else:
            xres.append( template % locals() )
        start += count
        if start > total:
            if page_number == 1: return []
            break
    return xres

def _get_config(config, tabname):
    if config is None:
        return modconfig[tabname]
    try:
        config['__sentinel__']
    except KeyError:
        _config = dict(modconfig[tabname])
        _config.update(config)
    else:
        _config = config
    return _config
        
def _control_generator(xlocals):
    res = [
        (r"""<a href="javascript: showformmodify('%(objname)s', '%(suffix)s', """
        r"""['update', %(rowid)s]);void(0);">%(_moditem)s</a>"""%xlocals),
        (r"""<a href="javascript: calldelete('%(objname)s', '%(suffix)s',  """
        r"""[%(rowid)s]);void(0);">%(_delitem)s</a>"""%xlocals)
        ]
    if xlocals['config']['indexable']():
        res.append("""<a href="javascript: void(0)" """
                   """onclick="guigo_related(this, '%(objname)s', '%(tabname)s');">%(_related)s</a>"""%xlocals)
    return res

######
##### Global data
######

objname = 'catalogos.catalogos'

#######################################################################################
#######################################################################################
######
######   FORMS
######
#######################################################################################
#######################################################################################
modconfig = {}

def _map_result(tabname, fieldname, value, outmap=None, config=None):
    objname = 'catalogos.catalogos'
    config = _get_config(config, tabname)
    foreigns = config['foreign_mapper']
    try:
        # This is the default behaviour, you must create caches of this values in order to speed up
        # this requests
        foreigntable, foreignfield = foreigns[fieldname]
        res = _preview(foreigntable, foreignfield, {foreignfield:value})
        display = _strip_field(res[0][1])
        if outmap is None:
            return display
        else:
            return outmap(locals())            
    except (KeyError, IndexError), e:
        return _strip_field(value)

def _preview(tabname, id, condition_dict=None, display=None, config=None, preview_maxsize=None, **kwargs):
    # tabname = <-python:keyword id="QuotedTabname" />
    config = _get_config(config, tabname)
    if condition_dict is None: condition_dict = {}
    # auth = spyro.getrequest()['auth']
    # assert _spyro_authentication._rawauth(auth,  objname, preview_<-python:keyword id="Tabname" />)
    # cond = ' 1=1 '
    condition = _join_cond(condition_dict)
    if display is None:
        fields = config['preview_fields']
        display = ",".join(fields)
    else:
        fields = None
    res = []
    internal = config['internal_fields']
    if not preview_maxsize:
        preview_maxsize = config['preview_maxsize']
    for qres in db.hquery('SELECT %s,%s FROM %s WHERE %s'%(_sql_quote(id), _sql_quote(display), tabname, condition)):
        row = []
        if fields is None:
            fields = qres.keys()
            fields.sort()
        for qk in fields:
            qv = qres[qk]
            if qk in internal:
                continue
            if qv:
                row.append(_map_result(tabname, qk, qv)) # posible infinite loops due to foreign resolution
        res.append([qres[id], ", ".join(row)[:preview_maxsize]])
    return res

def _get_rowid(tabname, cond, config=None):
    """ Returns the rowid identifing any row in the table, it's calculated
    using the condition or the insertion data. This method must be replaced
    using knowledge about the table. You must override this function with
    the correct behaviour if the default is not what you need.
    """
    config = _get_config(config, tabname)
    idname = config['list_fields'][0]
    if isinstance(cond, types.DictType):
        try:
            data = cond[idname]
        except KeyError:
            return repr(_strip_dict(cond)).replace("u'","'")
    else:
        data = cond[0]
    x = repr({str(idname): _strip_field(data)})
    x = x.replace("u'","'")
    return x

def _out_list(tabname, fields, query, dopager=None, config=None):
    objname = 'catalogos.catalogos'
    suffix = tabname
    config = _get_config(config, tabname)
    locale = locale3k.getweblocale()
    _newitem = locale3k.getname("actions", "new", locale)
    _delitem = locale3k.getname("actions", "delete", locale)
    _moditem = locale3k.getname("actions", "modify", locale)
    _related = locale3k.getname("actions","related",locale)
    editable = config['editable']()
    internal = config['internal_fields']
    try:
        hgen = config['header_generator']
        # it must has the same prototype than locale3k.getname and can be None, when it's None
        # the generation of headers will be skipped
    except KeyError:
        hgen = locale3k.getname
    out = []
    l = []
    viewfields = []
    for i, f in enumerate(fields):
        if f in internal:
            continue
        viewfields.append((i, f))
        if hgen:
            out.append(hgen(tabname, f, locale))
    l.append( config['output_table_header'](out) )
    try:
        outmap = config['outmap']
    except KeyError:
        outmap = lambda x: """<a href="javascript: callto({'handler': handle_commands_frames()}, '%(objname)s', 'list_view_%(foreigntable)s', [{'%(foreignfield)s':  '%(value)s'}]); void(0);">%(display)s</a>"""%x
    for q in query:
        if len(q) > 1:
            index = 1
        else:
            index = 0
        name_name = fields[index]
        data_name = q[index]
        r = []
        try:
            dgen = config['map_result']
        except KeyError:
            dgen = _map_result
        for i, f in viewfields:
            r.append(dgen(tabname, f, q[i], outmap, config))
        l.append(config['output_table_row'](r, q, fields))
        if editable:
            rowid = _get_rowid(tabname, q, config)
            controls = config['control_generator'](locals())
            l.append(config['output_table_control_row'](r, controls, q, fields))
            # l.append('<tr><td style="text-align: right;" colspan="%s">%s</td></tr>'%(len(r), " &nbsp; ".join(controls)))
    # cursor.close()
    if editable:
        newitemlink = r"""<a href="javascript: showformmodify('%(objname)s', '%(suffix)s'); ">%(_newitem)s</a>"""%locals()
    else:
        newitemlink = ""
    res = []
    searchname = locale3k.getname("actions","search",locale)
    insertable=config['insertable']()
    if insertable:
        res.append(newitemlink)
    if config['structured_search']():
        searchlink = r""" <a href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\'Search %(tabname)s\'')}}, '%(objname)s', 'form_search_%(tabname)s'); void(0);">%(searchname)s</a> &nbsp; """%locals()
    else:
        searchlink = ''
    if config['indexable']():
        searchname_fulltext = locale3k.getname("actions","search_fulltext",locale)
        searchlink += r"""<span id="%(tabname)s-ftsearch"><a href="javascript: callto({'showresults': function(d) {DOM3K_updateElement(d,'%(tabname)s-ftsearch')}}, '%(objname)s', 'form_search_fulltext_%(tabname)s'); void(0);">%(searchname_fulltext)s</a></span> &nbsp;"""%locals()
    if dopager:
        searchlink += dopager() 
        res.append(searchlink)
    res.append(config['output_table'](l, query, fields))
    if insertable:
        res.append(newitemlink)
    res.append(" &nbsp; ")
    res.append(searchlink)
    return "".join(res)

def _list_select(db, tabname, config, condition, orderby, begin, count):
    limits = 'LIMIT %d OFFSET %d'%(count,begin)
    if orderby is None:
        orderby = ''
    else:
        orderby = 'ORDER BY ' + (','.join(map(_sql_quote, orderby)))
    rowcount = db.query('SELECT COUNT(*) FROM %s WHERE %s %s'%(tabname, condition, orderby))[0][0]
    query = db.query('SELECT %s FROM %s WHERE %s %s %s'%(",".join(config['list_fields']), tabname, condition, orderby, limits))
    return query, rowcount

def _list(tabname, condition_dict=None, orderby=None, usecondmethod=None, list_condition_dict=None, begin=None, count=None, config=None, **kwargs):
    objname = 'catalogos.catalogos'
    suffix = tabname
    config = _get_config(config, tabname)
    # auth = spyro.getrequest()['auth']
    emptycond = False
    if condition_dict is None:
        condition_dict = {}
        emptycond = True
    if (list_condition_dict is None) or (len(list_condition_dict) == 0):
        list_condition_dict = [condition_dict]
    else:
        emptycond = False
    _orderby = orderby
    fields = config['list_fields']
    if count is None:
        count = config['count']
    if begin is None:
        begin = 0
    condition = " OR ".join( [ "(%s)"%_join_cond(cd, usecondmethod) for cd in list_condition_dict ] )
    # Using cursors yields to a very poor performance (maybe we use it in a wrong way).
    # Using count and limit to enhance performance (2 queries :( )
    # cursor = db.conn.cursor()
    # cursor.execute('SELECT %s FROM %s WHERE %s %s'%(",".join(fields),tabname,condition,orderby))
    # This was tested in postgresql
    if config['list_with_empty_condition']() or not emptycond:
        query, rowcount = config['select'](db, tabname, config, condition, orderby, begin, count)
    else:
        rowcount = 0
        query = []
    if usecondmethod is None:
        strcondmethod = 'null'
    else:
        strcondmethod = repr(usecondmethod)
    if len(list_condition_dict) == 0 or len(list_condition_dict[0]) == 0:
        str_list_condition_dict = 'null'
    else:
        str_list_condition_dict = repr(list_condition_dict)
    if _orderby is None:
        strorderby = 'null'
    else:
        strorderby = repr(_orderby)
    locs = locals()
    def dopager():
        res = []
        if len(list_condition_dict) > 0:
            qx = []
            for qc in list_condition_dict:
                try:
                    items = qc.items()
                except AttributeError:
                    items = ()
                qx.append(", ".join([ "%s: %s"%(k,v) for k,v in items ]))
            if len(qx) > 0:
                res.append("<div>%s</div>"%". ".join(qx))
        tem = r"""<a href="javascript: DOM3K_calllist('%(objname)s', '%(suffix)s', [null,%(strorderby)s,%(strcondmethod)s,%(str_list_condition_dict)s], { 'begin' : %%(start)s, 'count' : %%(count)s } ) ; void(0);" >%%(page_number)s</a> """%locs
        res.extend(_pager(begin, count, rowcount, tem))
        return "".join(res)
    return _out_list(tabname, fields, query, dopager, config)

def _out_choose_search(tabname, fields, query, idtag, previewsize, config=None):
    config = _get_config(config, tabname)
    # auth = spyro.getrequest()['auth']
    # assert _spyro_authentication._rawauth(auth,  objname, list_<-python:keyword id="Tabname" />)
    locale = locale3k.getweblocale()
    l = [ '<table><tbody><tr>' ]
    internal = config['internal_fields']    
    for f in fields:
        if f in internal:
            continue
        l.append("<th>%s</th>"%locale3k.getname(tabname, f, locale))
    l.append('</tr>')
    # if begin > 0: cursor.scroll(begin)
    # for q in cursor.fetchmany(count):
    outmap = lambda x: """<a href="javascript: callto({'handler': handle_commands_frames()}, '%(objname)s', 'list_view_%(foreigntable)s', [{'%(foreignfield)s':  '%(value)s'}]); void(0);">%(display)s</a>"""%x
    for q in query:
        name_id = fields[0]
        data_id = str(q[0])
        r = []
        for i,f in enumerate(fields):
            if f in internal:
                continue
            r.append(_map_result(tabname, f, q[i])) # outmap
        name = ". ".join(r)
        r.append("""<a href="javascript: void(0);" onclick="DOM3K_replaceListOptions([[%s,'%s']], %s); DOM3K_deleteFrame(this); void(0);">%s</a>"""%(repr(data_id),_strip_field(name),repr(idtag),locale3k.getname("general","choose_this",locale)))
        l.append('<tr>' + " ".join(map(lambda s: "<td>%s</td>"%s, r)) + '</tr>')
    if len(query) >= previewsize:
        l.append("""<tr><td>...</td><td class="thirdtitle" colspan="%s">%s</td></tr>"""%(len(fields),locale3k.getname("general","too_many_results", locale)))
    l.append("</tbody></table>")
    return "".join(l)

def _form(tabname, mode, condition_dict, config=None, **kwargs):
    config = _get_config(config, tabname)
    if mode is None:
        mode = 'insert'
    dict_default = dict(config['dict_default'])
    if condition_dict is None:
        cond = '{}'
    else:
        cond = condition_dict
    if config['editable']():
        dict_default['__onsubmit__'] = r"DOM3K_commitform(this, '%s','%s',['%s',Easyform_formToDict(this),%s]);"%(objname,tabname,mode,cond)
    else:
        dict_default['__onsubmit__'] = r"DOM3K_deleteFrame(this);"
        dict_default['__editmode__'] = ""
    if mode == 'update':
        qupdate = 'SELECT * FROM %s WHERE %s'%(tabname,_join_cond(condition_dict))
        dict_default.update(db.hquery(qupdate)[0])
        _strip_dict(dict_default)
    elif mode != 'insert':
        raise AttributeError, "Attribute mode is set to an unknown value (insert|update) '%s'"%mode
    dict_default['__id__'] = str(time.time())
    return config['form_element'].merge(dict_default)

def _list_method(tabname, mode, condition_dict, config=None, **kwargs):
    config = _get_config(config, tabname)
    return (config['output_showpage'](tabname, _list(tabname, condition_dict, config=config, **kwargs), tabname), )

def _modify(tabname, mode, data, cond, dbconn=None, doreturn=1, config=None, **kwargs):
    config = _get_config(config, tabname)
    if dbconn is None:
        dbconn = db
    # auth = spyro.getrequest()['auth']
    # assert _spyro_authentication._rawauth(auth,  objname, modify_<-python:keyword id="Tabname" />)
    if mode == 'update':
        if None in (data,cond): raise Exception("Unable to update table %s with empty data"%tabname)
        if len(cond) == 0: raise Exception("Unable to update %s without condition"%tabname)
        # It's a good idea overwrite user id fields with information of the authentications
        # For example:
        #cond['userid'] = spyro.getrequest()['auth'][0]
        x = dbconn.update(tabname, _strip_dict(data), cond)
        if config['indexable']():
            rowid = _get_rowid(tabname, cond, config)
            ftindex = config['ftindex']
            ftindex.handle_update(_preview(tabname, '1', data, config=config, preview_maxsize=512)[0][1], rowid)
    elif mode == 'insert':
        if data is None: raise Exception("Unable to insert into table %s with empty data"%tabname)
        #cond['userid'] = spyro.getrequest()['auth'][0]
        x = dbconn.insert(tabname, _strip_dict(data))
        if config['indexable']():
            xdata = _preview(tabname, config['list_fields'][0], data, config=config, preview_maxsize=512)[0]
            rowid = _get_rowid(tabname, xdata, config)
            ftindex = config['ftindex']
            ftindex.handle_insert(xdata[1], rowid)
    else:
        raise Exception("Unknown action to modify %s"%tabname)
    if doreturn:
        return _list_method(tabname, mode, cond)
    else:
        return None

def _delete(tabname, cond, dbconn = None, doreturn = 1, config = None, **kwargs):
    config = _get_config(config, tabname)
    if None is cond: raise Exception("Unable to delete table with empty condition")
    if len(cond) == 0: raise Exception("Unable to delete without condition")
    # auth = spyro.getrequest()['auth']
    # assert _spyro_authentication._rawauth(auth,  objname, delete_<-python:keyword id="Tabname" />)
    if dbconn is None:
        dbconn = db
    if config['indexable']():
        rowid = _get_rowid(tabname, cond, config)
        try:
            xdata = _preview(tabname, '1', cond, config=config, preview_maxsize=512)[0][1]
        except IndexError:
            pass
        else:
            ftindex = config['ftindex']
            ftindex.handle_delete(xdata, rowid)
    x = dbconn.execute('DELETE FROM %s WHERE %s'%(tabname, _join_cond(cond)))
    if doreturn:
        return _list_method(tabname, "delete", cond)
    else:
        return None

def _search_fulltext(tabname, query, output="", idtag="", residtag="", config=None, **kwargs):
    config = _get_config(config, tabname)
    ftindex = config['ftindex']
    size = config['ftsearch_size']
    searchres = ftindex.search(query, size)
    res = []
    fields = config['list_fields']
    for rank, cond in searchres:
        cond = eval(cond)
        res.extend( db.query("""SELECT %s FROM %s WHERE %s LIMIT 1"""%(", ".join(fields),tabname,_join_cond(cond,"="))) )
    # we loose the ranking, but we standardize the output
    if output == 'raw':
        return res
    if output == 'choose_search':
        return _out_choose_search(tabname, fields, res, idtag, size)
    else:
        x = _out_list(tabname, fields, res, idtag)
        if len(res) >= size:
            x+="<br/>" + locale3k.getname("general","too_many_results")
        return (config['output_showpage'](tabname, x, tabname), )

def _choose_search(tabname, idtag, condition_dict = None, method="LIKE", outmethod="simple",
                    previewsize=10, list_condition_dict = None, config=None, **kwargs):
    config = _get_config(config, tabname)
    # auth = spyro.getrequest()['auth']
    # assert _spyro_authentication._rawauth(auth,  objname, list_<-python:keyword id="Tabname" />)
    emptycond = False
    if condition_dict is None:
        condition_dict = {}
        emptycond = True
    if (list_condition_dict is None) or (len(list_condition_dict) == 0):
        list_condition_dict = [condition_dict]
    else:
        emptycond = False
    fields = config['list_fields']
    count = previewsize
    begin = 0
    condition = " OR ".join( [ "(%s)"%_join_cond(cd, method) for cd in list_condition_dict ] )
    limits = 'LIMIT %d OFFSET %d'%(count,begin)
    if config['list_with_empty_condition']() or not emptycond:
        query = db.query('SELECT %s FROM %s WHERE %s %s'%(",".join(fields),tabname,condition,limits))
    else:
        query = []
    return _out_choose_search(tabname, fields, query, idtag, previewsize)

####
##
## Particular methods
##
###################################################################################################
###############################################################################################
###
### Table pais UI
###

### Configuring pais

# TODO: Change all values of editable, insertable, indexable, etc to predicates
# TODO: in order to be able to give a behaviour dependable of the user, or auth

modconfig['pais'] = {
    'internal_fields' : [u'xid'],
    'dict_default': { '' : '<python:locale id="general.submit" />','nombre' : '','xid' : '','nombre_corto' : '',
                     '__objname__': 'catalogos.catalogos',
                     '__tabname__': 'pais',
                     #'__fulltextsearch__': '', # disables the fulltext search in the modify frame
                    },
    'editable': lambda: True,
    'insertable': lambda: True,
    'indexable': lambda: True,
    'structured_search': lambda: True,
    'ftindex': ft3k.sqlindex(name='pais', conn=dbftindex),
    'ftsearch_size': 10,
    'count': 30,
    'list_fields': ['xid', 'nombre', 'nombre_corto'],
    'preview_fields': ['xid', 'nombre', 'nombre_corto'],
    'list_with_empty_condition': lambda: True,
    'control_generator': _control_generator,
    'preview_maxsize': 64,
    'output_table_header': lambda r: "<tr>%s</tr>"%("<th>%s</th>"%("</th><th>".join(r))),
    'output_table_row': lambda r, q, fields: "<tr>%s</tr>"%("<td>%s</td>"%("</td><td>".join(r))),
    'output_table_control_row': lambda r, controls, q, fields: '<tr><td style="text-align: right;" colspan="%s">%s</td></tr>'%(len(r), " &nbsp; ".join(controls)),
    'output_table': lambda r, query, fields: """<table style="width: 100%%"><tbody>%s</tbody></table>"""%("".join(r)),
    'output_showpage': lambda title, data, id: ('insert',
                                                       {'title': title,
                                                        'data': data,
                                                        'id': id,
                                                        'rowclasses': (['cell1', 'cell1', 'cell2', 'cell2'], 'header')
                                                       }),
    'select': _list_select,
    'foreign_mapper': {},
    'form_element': template3k.get_template("""pais:form_element""",
                                            """
            <strong>Pais</strong>
            <form method="POST" enctype="multipart/form-data" id="form-pais<python:keyword id='__id__' />" action="javascript: void(0);" onsubmit="<python:keyword id="__onsubmit__" />" name="pais" >
                <table><tbody>
            
<tr><td>
<python:locale id="pais.nombre">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Nombre'} ]
</python:locale>:</td><td><input name="nombre:string:mandatory" type="text" value="<python:keyword id="nombre" />" size="40" /></td></tr>


<tr><td>
<python:locale id="pais.nombre_corto">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Nombre Corto'} ]
</python:locale>:</td><td><input name="nombre_corto:string:mandatory" type="text" value="<python:keyword id="nombre_corto" />" size="40" /></td></tr>

<tr><td><input type="submit" name=":submit" value="<python:locale id="general.submit" />" /></td></tr>
                </tbody></table>
            </form>
            <python:keyword id="__bottom__"></python:keyword>
            """),
    'form_element_search': template3k.get_template("""pais:form_element_search""",
                                                   """
            <strong>Pais</strong>
            <form method="POST" enctype="multipart/form-data" id="form-pais<python:keyword id='__id__' />" action="javascript: void(0);" onsubmit="<python:keyword id="__onsubmit__" />" name="pais" >
                <table><tbody>
            
<tr><td>
<python:locale id="pais.nombre">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Nombre'} ]
</python:locale>:</td><td><input name="nombre:string" type="text" value="<python:keyword id="nombre" />" size="40" /></td></tr>


<tr><td>
<python:locale id="pais.nombre_corto">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Nombre Corto'} ]
</python:locale>:</td><td><input name="nombre_corto:string" type="text" value="<python:keyword id="nombre_corto" />" size="40" /></td></tr>

<tr><td><input type="submit" name=":submit" value="<python:locale id="general.submit" />" /></td></tr>
                </tbody></table>
            </form>
            <python:keyword id="__bottom__"></python:keyword>
            """),
    '__sentinel__': True
    }

### End configuration

###
### Beginning methods/functions

#_spyro_authentication.perms['form_pais'] = False
#_spyro_authentication.perms['list_pais'] = False
#_spyro_authentication.perms['modify_pais'] = False
#_spyro_authentication.perms['delete_pais'] = False
 
def list_view_pais(*args, **kwargs):
    tabname = 'pais'
    config = _get_config(None, tabname)
    kwargs['config'] = config
    return (config['output_showpage'](tabname, _list(tabname, *args, **kwargs), tabname), )

def form_pais(mode = None, condition_dict = None, **kwargs):
    tabname = 'pais'
    kwargs['config'] = _get_config(None, tabname)
    return _form(tabname, mode, condition_dict, **kwargs)

def modify_pais(mode, data, cond, dbconn=None, doreturn=1, **kwargs):
    tabname = 'pais'
    config = _get_config(None, tabname)
    assert config['editable'](), "View only entity, can't %s"%mode
    tabname = 'pais'
    return _modify(tabname, mode, data, cond, dbconn, doreturn, config=config, **kwargs)

def delete_pais(cond, dbconn = None, doreturn = 1, **kwargs):
    tabname = 'pais'
    config = _get_config(None, tabname)
    assert config['editable'](), "View only entity, can't delete"
    return _delete(tabname, cond, dbconn, doreturn, config=config, **kwargs)

def list_pais(*args, **kwargs):
    tabname = 'pais'
    kwargs['config'] = _get_config(None, tabname)
    # assert _spyro_authentication._rawauth(auth,  objname, list_pais)
    return _list(tabname, *args, **kwargs)

def form_search_pais(method=None, *args, **kwargs):
    tabname = 'pais'
    config = _get_config(None, tabname)
    dict_default = dict(config['dict_default']) # dictionary
    dict_default['__editmode__'] = ""
    if method == "choose_search":
        idtag = args[0]
        residtag = 'choose-search-%s'%idtag
        dict_default['__onsubmit__'] = (r"callto({'showresults': function(d){ DOM3K_updateElement(d,'%s'); """
                                        r"""DOM3K_BackgroundRowColors('%s',['rgb(200,200,230);','inherit']);}}, """
                                        r"""'%s','choose_search_%s',null,{'condition_dict': Easyform_formToDict(this), 'idtag': %s });"""
                                        r""" """%(residtag,residtag,objname,tabname,repr(idtag)))
        dict_default['__bottom__'] = """<div id="%s"></div>"""%residtag
    else:
        dict_default['__onsubmit__'] = r"callto({'handler': handle_commands}, '%s','search_%s',[Easyform_formToDict(this), null, 'LIKE']);"%(objname,tabname)
    dict_default['__id__'] = str(time.time())
    x = config['form_element_search'].merge(dict_default)
    return x

def manage_fulltext_pais(drop_index, reindex):
    tabname = 'pais'
    config = _get_config(None, tabname)
    ftindex = config['ftindex']
    if drop_index:
        ftindex.drop_index()
    if reindex:
        ftindex.create_index()
        for x in _preview(tabname, config['list_fields'][0], config=config, preview_maxsize=512):
            _rowid = _get_rowid(tabname, x)
            ftindex.handle_insert(x[1], _rowid)

def form_search_fulltext_pais(output='list', idtag='', query='',**kwargs):
    tabname = 'pais'
    dict_default = {'query': query, 'output': output, 'idtag': idtag}
    if output == "choose_search":
        residtag = 'choose-search-%s'%idtag
        dict_default['__onsubmit__'] = (r"""callto({'handler': handle_commands_frames()}, '%s', 'search_fulltext_%s', """
                                        r"""null, Easyform_formToDict(this));"""%(objname, tabname))
        dict_default['__bottom__'] = """<div id="%s"></div>"""%residtag
        dict_default['__idtag__'] = """<input name="idtag:string:mandatory" value="%s" type="hidden" />"""%idtag
        dict_default['residtag'] = residtag
    else:
        dict_default['residtag'] = tabname
        dict_default['__onsubmit__'] = r"callto({'handler': handle_commands}, '%s', 'search_fulltext_%s', null, Easyform_formToDict(this));"%(objname, tabname)
        dict_default['__idtag__'] = ""
        dict_default['__bottom__'] = ""
    dict_default['__id__'] = str(time.time())
    return """<form action="javascript: void(0);" onsubmit="%(__onsubmit__)s">
    <input name="output:string:mandatory" value="%(output)s" type="hidden" />
    <input name="query:string:mandatory" value="%(query)s" size="40" type="text" />
    <input name="residtag:string" value="residtag" type="hidden" />
    %(__idtag__)s
    <input name=":submit" value="Search" type="submit" />
    </form>
    %(__bottom__)s
    """%dict_default

def search_fulltext_pais(query, output="", idtag="", residtag="", **kwargs):
    tabname = 'pais'
    kwargs['config'] = None
    return _search_fulltext(tabname, query, output, idtag, residtag, **kwargs)


def search_pais(cond,orderby=None,usecondmethod=None,**kwargs):
    tabname = 'pais'
    config = _get_config(None, tabname)
    x = list_pais(cond, orderby, usecondmethod=usecondmethod)
    return (config['output_showpage']('Search results', x, tabname), )

def choose_search_pais(idtag, condition_dict = None, method="LIKE", outmethod="simple",
                                                  previewsize=10, list_condition_dict = None, **kwargs):
    tabname = 'pais'
    config = _get_config(None, tabname)
    kwargs['config'] = config
    return _choose_search(tabname, idtag, condition_dict, method, outmethod, previewsize, list_condition_dict, **kwargs)

def preview_pais(*args, **kwargs):
    tabname = 'pais'
    kwargs['config'] = _get_config(None, tabname)
    # kwargs['display'] = 'nombre'
    return _preview(tabname, *args, **kwargs)

###
### end of pais UI
###
###############################################################################################
###############################################################################################
###############################################################################################
###############################################################################################
###
### Table estado UI
###

### Configuring estado

# TODO: Change all values of editable, insertable, indexable, etc to predicates
# TODO: in order to be able to give a behaviour dependable of the user, or auth

modconfig['estado'] = {
    'internal_fields' : [u'xid'],
    'dict_default': { '' : '<python:locale id="general.submit" />','nombre' : '','xid' : '','nombre_corto' : '','idpais' : '',
                     '__objname__': 'catalogos.catalogos',
                     '__tabname__': 'estado',
                     #'__fulltextsearch__': '', # disables the fulltext search in the modify frame
                    },
    'editable': lambda: True,
    'insertable': lambda: True,
    'indexable': lambda: True,
    'structured_search': lambda: True,
    'ftindex': ft3k.sqlindex(name='estado', conn=dbftindex),
    'ftsearch_size': 10,
    'count': 30,
    'list_fields': ['xid', 'nombre', 'nombre_corto', 'idpais'],
    'preview_fields': ['xid', 'nombre', 'nombre_corto', 'idpais'],
    'list_with_empty_condition': lambda: True,
    'control_generator': _control_generator,
    'preview_maxsize': 64,
    'output_table_header': lambda r: "<tr>%s</tr>"%("<th>%s</th>"%("</th><th>".join(r))),
    'output_table_row': lambda r, q, fields: "<tr>%s</tr>"%("<td>%s</td>"%("</td><td>".join(r))),
    'output_table_control_row': lambda r, controls, q, fields: '<tr><td style="text-align: right;" colspan="%s">%s</td></tr>'%(len(r), " &nbsp; ".join(controls)),
    'output_table': lambda r, query, fields: """<table style="width: 100%%"><tbody>%s</tbody></table>"""%("".join(r)),
    'output_showpage': lambda title, data, id: ('insert',
                                                       {'title': title,
                                                        'data': data,
                                                        'id': id,
                                                        'rowclasses': (['cell1', 'cell1', 'cell2', 'cell2'], 'header')
                                                       }),
    'select': _list_select,
    'foreign_mapper': {u'idpais': (u'pais', u'xid')},
    'form_element': template3k.get_template("""estado:form_element""",
                                            """
            <strong>Estado</strong>
            <form method="POST" enctype="multipart/form-data" id="form-estado<python:keyword id='__id__' />" action="javascript: void(0);" onsubmit="<python:keyword id="__onsubmit__" />" name="estado" >
                <table><tbody>
            
<tr><td>
<python:locale id="estado.nombre">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Nombre'} ]
</python:locale>:</td><td><input name="nombre:string:mandatory" type="text" value="<python:keyword id="nombre" />" size="40" /></td></tr>


<tr><td>
<python:locale id="estado.nombre_corto">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Nombre Corto'} ]
</python:locale>:</td><td><input name="nombre_corto:string:mandatory" type="text" value="<python:keyword id="nombre_corto" />" size="40" /></td></tr>


<tr><td>
<python:locale id="estado.idpais">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Idpais'} ]
</python:locale>:</td><td><select onfocus="callto({'rawhandler': createhandlerselect('<python:keyword id="idpais" />'), 'handlerargs': this},'<python:keyword id="__objname__" />','preview_pais',['xid'])" name="idpais:int"  id="select-pais_xid_<python:keyword id='__id__' />" ></select>
<a style="text-decoration: none;" href="javascript: DOM3K_replaceListOptions([['','']],'select-pais_xid_<python:keyword id="__id__" />'); void(0);"><python:locale id="actions.clear" /></a><python:keyword id="__editmode__">

/
<a style="text-decoration: none;" href="javascript: showformmodify('<python:keyword id="__objname__" />', 'pais'); void(0);"><python:locale id="actions.new" /></a></python:keyword>

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_pais', ['choose_search','select-pais_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search" /></a><python:keyword id="__fulltextsearch__">

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_fulltext_pais', ['choose_search','select-pais_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search_fulltext" /></a></python:keyword>

</td></tr>

<tr><td><input type="submit" name=":submit" value="<python:locale id="general.submit" />" /></td></tr>
                </tbody></table>
            </form>
            <python:keyword id="__bottom__"></python:keyword>
            """),
    'form_element_search': template3k.get_template("""estado:form_element_search""",
                                                   """
            <strong>Estado</strong>
            <form method="POST" enctype="multipart/form-data" id="form-estado<python:keyword id='__id__' />" action="javascript: void(0);" onsubmit="<python:keyword id="__onsubmit__" />" name="estado" >
                <table><tbody>
            
<tr><td>
<python:locale id="estado.nombre">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Nombre'} ]
</python:locale>:</td><td><input name="nombre:string" type="text" value="<python:keyword id="nombre" />" size="40" /></td></tr>


<tr><td>
<python:locale id="estado.nombre_corto">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Nombre Corto'} ]
</python:locale>:</td><td><input name="nombre_corto:string" type="text" value="<python:keyword id="nombre_corto" />" size="40" /></td></tr>


<tr><td>
<python:locale id="estado.idpais">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Idpais'} ]
</python:locale>:</td><td><select onfocus="callto({'rawhandler': createhandlerselect('<python:keyword id="idpais" />'), 'handlerargs': this},'<python:keyword id="__objname__" />','preview_pais',['xid'])" name="idpais:int"  id="select-pais_xid_<python:keyword id='__id__' />" ></select>
<a style="text-decoration: none;" href="javascript: DOM3K_replaceListOptions([['','']],'select-pais_xid_<python:keyword id="__id__" />'); void(0);"><python:locale id="actions.clear" /></a><python:keyword id="__editmode__">

/
<a style="text-decoration: none;" href="javascript: showformmodify('<python:keyword id="__objname__" />', 'pais'); void(0);"><python:locale id="actions.new" /></a></python:keyword>

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_pais', ['choose_search','select-pais_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search" /></a><python:keyword id="__fulltextsearch__">

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_fulltext_pais', ['choose_search','select-pais_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search_fulltext" /></a></python:keyword>

</td></tr>

<tr><td><input type="submit" name=":submit" value="<python:locale id="general.submit" />" /></td></tr>
                </tbody></table>
            </form>
            <python:keyword id="__bottom__"></python:keyword>
            """),
    '__sentinel__': True
    }

### End configuration

###
### Beginning methods/functions

#_spyro_authentication.perms['form_estado'] = False
#_spyro_authentication.perms['list_estado'] = False
#_spyro_authentication.perms['modify_estado'] = False
#_spyro_authentication.perms['delete_estado'] = False
 
def list_view_estado(*args, **kwargs):
    tabname = 'estado'
    config = _get_config(None, tabname)
    kwargs['config'] = config
    return (config['output_showpage'](tabname, _list(tabname, *args, **kwargs), tabname), )

def form_estado(mode = None, condition_dict = None, **kwargs):
    tabname = 'estado'
    kwargs['config'] = _get_config(None, tabname)
    return _form(tabname, mode, condition_dict, **kwargs)

def modify_estado(mode, data, cond, dbconn=None, doreturn=1, **kwargs):
    tabname = 'estado'
    config = _get_config(None, tabname)
    assert config['editable'](), "View only entity, can't %s"%mode
    tabname = 'estado'
    return _modify(tabname, mode, data, cond, dbconn, doreturn, config=config, **kwargs)

def delete_estado(cond, dbconn = None, doreturn = 1, **kwargs):
    tabname = 'estado'
    config = _get_config(None, tabname)
    assert config['editable'](), "View only entity, can't delete"
    return _delete(tabname, cond, dbconn, doreturn, config=config, **kwargs)

def list_estado(*args, **kwargs):
    tabname = 'estado'
    kwargs['config'] = _get_config(None, tabname)
    # assert _spyro_authentication._rawauth(auth,  objname, list_estado)
    return _list(tabname, *args, **kwargs)

def form_search_estado(method=None, *args, **kwargs):
    tabname = 'estado'
    config = _get_config(None, tabname)
    dict_default = dict(config['dict_default']) # dictionary
    dict_default['__editmode__'] = ""
    if method == "choose_search":
        idtag = args[0]
        residtag = 'choose-search-%s'%idtag
        dict_default['__onsubmit__'] = (r"callto({'showresults': function(d){ DOM3K_updateElement(d,'%s'); """
                                        r"""DOM3K_BackgroundRowColors('%s',['rgb(200,200,230);','inherit']);}}, """
                                        r"""'%s','choose_search_%s',null,{'condition_dict': Easyform_formToDict(this), 'idtag': %s });"""
                                        r""" """%(residtag,residtag,objname,tabname,repr(idtag)))
        dict_default['__bottom__'] = """<div id="%s"></div>"""%residtag
    else:
        dict_default['__onsubmit__'] = r"callto({'handler': handle_commands}, '%s','search_%s',[Easyform_formToDict(this), null, 'LIKE']);"%(objname,tabname)
    dict_default['__id__'] = str(time.time())
    x = config['form_element_search'].merge(dict_default)
    return x

def manage_fulltext_estado(drop_index, reindex):
    tabname = 'estado'
    config = _get_config(None, tabname)
    ftindex = config['ftindex']
    if drop_index:
        ftindex.drop_index()
    if reindex:
        ftindex.create_index()
        for x in _preview(tabname, config['list_fields'][0], config=config, preview_maxsize=512):
            _rowid = _get_rowid(tabname, x)
            ftindex.handle_insert(x[1], _rowid)

def form_search_fulltext_estado(output='list', idtag='', query='',**kwargs):
    tabname = 'estado'
    dict_default = {'query': query, 'output': output, 'idtag': idtag}
    if output == "choose_search":
        residtag = 'choose-search-%s'%idtag
        dict_default['__onsubmit__'] = (r"""callto({'handler': handle_commands_frames()}, '%s', 'search_fulltext_%s', """
                                        r"""null, Easyform_formToDict(this));"""%(objname, tabname))
        dict_default['__bottom__'] = """<div id="%s"></div>"""%residtag
        dict_default['__idtag__'] = """<input name="idtag:string:mandatory" value="%s" type="hidden" />"""%idtag
        dict_default['residtag'] = residtag
    else:
        dict_default['residtag'] = tabname
        dict_default['__onsubmit__'] = r"callto({'handler': handle_commands}, '%s', 'search_fulltext_%s', null, Easyform_formToDict(this));"%(objname, tabname)
        dict_default['__idtag__'] = ""
        dict_default['__bottom__'] = ""
    dict_default['__id__'] = str(time.time())
    return """<form action="javascript: void(0);" onsubmit="%(__onsubmit__)s">
    <input name="output:string:mandatory" value="%(output)s" type="hidden" />
    <input name="query:string:mandatory" value="%(query)s" size="40" type="text" />
    <input name="residtag:string" value="residtag" type="hidden" />
    %(__idtag__)s
    <input name=":submit" value="Search" type="submit" />
    </form>
    %(__bottom__)s
    """%dict_default

def search_fulltext_estado(query, output="", idtag="", residtag="", **kwargs):
    tabname = 'estado'
    kwargs['config'] = None
    return _search_fulltext(tabname, query, output, idtag, residtag, **kwargs)


def search_estado(cond,orderby=None,usecondmethod=None,**kwargs):
    tabname = 'estado'
    config = _get_config(None, tabname)
    x = list_estado(cond, orderby, usecondmethod=usecondmethod)
    return (config['output_showpage']('Search results', x, tabname), )

def choose_search_estado(idtag, condition_dict = None, method="LIKE", outmethod="simple",
                                                  previewsize=10, list_condition_dict = None, **kwargs):
    tabname = 'estado'
    config = _get_config(None, tabname)
    kwargs['config'] = config
    return _choose_search(tabname, idtag, condition_dict, method, outmethod, previewsize, list_condition_dict, **kwargs)

def preview_estado(*args, **kwargs):
    tabname = 'estado'
    kwargs['config'] = _get_config(None, tabname)
    # kwargs['display'] = 'nombre'
    return _preview(tabname, *args, **kwargs)

###
### end of estado UI
###
###############################################################################################
###############################################################################################
###############################################################################################
###############################################################################################
###
### Table ciudad UI
###

### Configuring ciudad

# TODO: Change all values of editable, insertable, indexable, etc to predicates
# TODO: in order to be able to give a behaviour dependable of the user, or auth

modconfig['ciudad'] = {
    'internal_fields' : [u'xid'],
    'dict_default': { '' : '<python:locale id="general.submit" />','nombre' : '','xid' : '','nombre_corto' : '','idestado' : '',
                     '__objname__': 'catalogos.catalogos',
                     '__tabname__': 'ciudad',
                     #'__fulltextsearch__': '', # disables the fulltext search in the modify frame
                    },
    'editable': lambda: True,
    'insertable': lambda: True,
    'indexable': lambda: True,
    'structured_search': lambda: True,
    'ftindex': ft3k.sqlindex(name='ciudad', conn=dbftindex),
    'ftsearch_size': 10,
    'count': 30,
    'list_fields': ['xid', 'nombre', 'nombre_corto', 'idestado'],
    'preview_fields': ['xid', 'nombre', 'nombre_corto', 'idestado'],
    'list_with_empty_condition': lambda: True,
    'control_generator': _control_generator,
    'preview_maxsize': 64,
    'output_table_header': lambda r: "<tr>%s</tr>"%("<th>%s</th>"%("</th><th>".join(r))),
    'output_table_row': lambda r, q, fields: "<tr>%s</tr>"%("<td>%s</td>"%("</td><td>".join(r))),
    'output_table_control_row': lambda r, controls, q, fields: '<tr><td style="text-align: right;" colspan="%s">%s</td></tr>'%(len(r), " &nbsp; ".join(controls)),
    'output_table': lambda r, query, fields: """<table style="width: 100%%"><tbody>%s</tbody></table>"""%("".join(r)),
    'output_showpage': lambda title, data, id: ('insert',
                                                       {'title': title,
                                                        'data': data,
                                                        'id': id,
                                                        'rowclasses': (['cell1', 'cell1', 'cell2', 'cell2'], 'header')
                                                       }),
    'select': _list_select,
    'foreign_mapper': {u'idestado': (u'estado', u'xid')},
    'form_element': template3k.get_template("""ciudad:form_element""",
                                            """
            <strong>Ciudad</strong>
            <form method="POST" enctype="multipart/form-data" id="form-ciudad<python:keyword id='__id__' />" action="javascript: void(0);" onsubmit="<python:keyword id="__onsubmit__" />" name="ciudad" >
                <table><tbody>
            
<tr><td>
<python:locale id="ciudad.nombre">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Nombre'} ]
</python:locale>:</td><td><input name="nombre:string:mandatory" type="text" value="<python:keyword id="nombre" />" size="40" /></td></tr>


<tr><td>
<python:locale id="ciudad.nombre_corto">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Nombre Corto'} ]
</python:locale>:</td><td><input name="nombre_corto:string:mandatory" type="text" value="<python:keyword id="nombre_corto" />" size="40" /></td></tr>


<tr><td>
<python:locale id="ciudad.idestado">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Idestado'} ]
</python:locale>:</td><td><select onfocus="callto({'rawhandler': createhandlerselect('<python:keyword id="idestado" />'), 'handlerargs': this},'<python:keyword id="__objname__" />','preview_estado',['xid'])" name="idestado:int"  id="select-estado_xid_<python:keyword id='__id__' />" ></select>
<a style="text-decoration: none;" href="javascript: DOM3K_replaceListOptions([['','']],'select-estado_xid_<python:keyword id="__id__" />'); void(0);"><python:locale id="actions.clear" /></a><python:keyword id="__editmode__">

/
<a style="text-decoration: none;" href="javascript: showformmodify('<python:keyword id="__objname__" />', 'estado'); void(0);"><python:locale id="actions.new" /></a></python:keyword>

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_estado', ['choose_search','select-estado_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search" /></a><python:keyword id="__fulltextsearch__">

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_fulltext_estado', ['choose_search','select-estado_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search_fulltext" /></a></python:keyword>

</td></tr>

<tr><td><input type="submit" name=":submit" value="<python:locale id="general.submit" />" /></td></tr>
                </tbody></table>
            </form>
            <python:keyword id="__bottom__"></python:keyword>
            """),
    'form_element_search': template3k.get_template("""ciudad:form_element_search""",
                                                   """
            <strong>Ciudad</strong>
            <form method="POST" enctype="multipart/form-data" id="form-ciudad<python:keyword id='__id__' />" action="javascript: void(0);" onsubmit="<python:keyword id="__onsubmit__" />" name="ciudad" >
                <table><tbody>
            
<tr><td>
<python:locale id="ciudad.nombre">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Nombre'} ]
</python:locale>:</td><td><input name="nombre:string" type="text" value="<python:keyword id="nombre" />" size="40" /></td></tr>


<tr><td>
<python:locale id="ciudad.nombre_corto">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Nombre Corto'} ]
</python:locale>:</td><td><input name="nombre_corto:string" type="text" value="<python:keyword id="nombre_corto" />" size="40" /></td></tr>


<tr><td>
<python:locale id="ciudad.idestado">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Idestado'} ]
</python:locale>:</td><td><select onfocus="callto({'rawhandler': createhandlerselect('<python:keyword id="idestado" />'), 'handlerargs': this},'<python:keyword id="__objname__" />','preview_estado',['xid'])" name="idestado:int"  id="select-estado_xid_<python:keyword id='__id__' />" ></select>
<a style="text-decoration: none;" href="javascript: DOM3K_replaceListOptions([['','']],'select-estado_xid_<python:keyword id="__id__" />'); void(0);"><python:locale id="actions.clear" /></a><python:keyword id="__editmode__">

/
<a style="text-decoration: none;" href="javascript: showformmodify('<python:keyword id="__objname__" />', 'estado'); void(0);"><python:locale id="actions.new" /></a></python:keyword>

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_estado', ['choose_search','select-estado_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search" /></a><python:keyword id="__fulltextsearch__">

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_fulltext_estado', ['choose_search','select-estado_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search_fulltext" /></a></python:keyword>

</td></tr>

<tr><td><input type="submit" name=":submit" value="<python:locale id="general.submit" />" /></td></tr>
                </tbody></table>
            </form>
            <python:keyword id="__bottom__"></python:keyword>
            """),
    '__sentinel__': True
    }

### End configuration

###
### Beginning methods/functions

#_spyro_authentication.perms['form_ciudad'] = False
#_spyro_authentication.perms['list_ciudad'] = False
#_spyro_authentication.perms['modify_ciudad'] = False
#_spyro_authentication.perms['delete_ciudad'] = False
 
def list_view_ciudad(*args, **kwargs):
    tabname = 'ciudad'
    config = _get_config(None, tabname)
    kwargs['config'] = config
    return (config['output_showpage'](tabname, _list(tabname, *args, **kwargs), tabname), )

def form_ciudad(mode = None, condition_dict = None, **kwargs):
    tabname = 'ciudad'
    kwargs['config'] = _get_config(None, tabname)
    return _form(tabname, mode, condition_dict, **kwargs)

def modify_ciudad(mode, data, cond, dbconn=None, doreturn=1, **kwargs):
    tabname = 'ciudad'
    config = _get_config(None, tabname)
    assert config['editable'](), "View only entity, can't %s"%mode
    tabname = 'ciudad'
    return _modify(tabname, mode, data, cond, dbconn, doreturn, config=config, **kwargs)

def delete_ciudad(cond, dbconn = None, doreturn = 1, **kwargs):
    tabname = 'ciudad'
    config = _get_config(None, tabname)
    assert config['editable'](), "View only entity, can't delete"
    return _delete(tabname, cond, dbconn, doreturn, config=config, **kwargs)

def list_ciudad(*args, **kwargs):
    tabname = 'ciudad'
    kwargs['config'] = _get_config(None, tabname)
    # assert _spyro_authentication._rawauth(auth,  objname, list_ciudad)
    return _list(tabname, *args, **kwargs)

def form_search_ciudad(method=None, *args, **kwargs):
    tabname = 'ciudad'
    config = _get_config(None, tabname)
    dict_default = dict(config['dict_default']) # dictionary
    dict_default['__editmode__'] = ""
    if method == "choose_search":
        idtag = args[0]
        residtag = 'choose-search-%s'%idtag
        dict_default['__onsubmit__'] = (r"callto({'showresults': function(d){ DOM3K_updateElement(d,'%s'); """
                                        r"""DOM3K_BackgroundRowColors('%s',['rgb(200,200,230);','inherit']);}}, """
                                        r"""'%s','choose_search_%s',null,{'condition_dict': Easyform_formToDict(this), 'idtag': %s });"""
                                        r""" """%(residtag,residtag,objname,tabname,repr(idtag)))
        dict_default['__bottom__'] = """<div id="%s"></div>"""%residtag
    else:
        dict_default['__onsubmit__'] = r"callto({'handler': handle_commands}, '%s','search_%s',[Easyform_formToDict(this), null, 'LIKE']);"%(objname,tabname)
    dict_default['__id__'] = str(time.time())
    x = config['form_element_search'].merge(dict_default)
    return x

def manage_fulltext_ciudad(drop_index, reindex):
    tabname = 'ciudad'
    config = _get_config(None, tabname)
    ftindex = config['ftindex']
    if drop_index:
        ftindex.drop_index()
    if reindex:
        ftindex.create_index()
        for x in _preview(tabname, config['list_fields'][0], config=config, preview_maxsize=512):
            _rowid = _get_rowid(tabname, x)
            ftindex.handle_insert(x[1], _rowid)

def form_search_fulltext_ciudad(output='list', idtag='', query='',**kwargs):
    tabname = 'ciudad'
    dict_default = {'query': query, 'output': output, 'idtag': idtag}
    if output == "choose_search":
        residtag = 'choose-search-%s'%idtag
        dict_default['__onsubmit__'] = (r"""callto({'handler': handle_commands_frames()}, '%s', 'search_fulltext_%s', """
                                        r"""null, Easyform_formToDict(this));"""%(objname, tabname))
        dict_default['__bottom__'] = """<div id="%s"></div>"""%residtag
        dict_default['__idtag__'] = """<input name="idtag:string:mandatory" value="%s" type="hidden" />"""%idtag
        dict_default['residtag'] = residtag
    else:
        dict_default['residtag'] = tabname
        dict_default['__onsubmit__'] = r"callto({'handler': handle_commands}, '%s', 'search_fulltext_%s', null, Easyform_formToDict(this));"%(objname, tabname)
        dict_default['__idtag__'] = ""
        dict_default['__bottom__'] = ""
    dict_default['__id__'] = str(time.time())
    return """<form action="javascript: void(0);" onsubmit="%(__onsubmit__)s">
    <input name="output:string:mandatory" value="%(output)s" type="hidden" />
    <input name="query:string:mandatory" value="%(query)s" size="40" type="text" />
    <input name="residtag:string" value="residtag" type="hidden" />
    %(__idtag__)s
    <input name=":submit" value="Search" type="submit" />
    </form>
    %(__bottom__)s
    """%dict_default

def search_fulltext_ciudad(query, output="", idtag="", residtag="", **kwargs):
    tabname = 'ciudad'
    kwargs['config'] = None
    return _search_fulltext(tabname, query, output, idtag, residtag, **kwargs)


def search_ciudad(cond,orderby=None,usecondmethod=None,**kwargs):
    tabname = 'ciudad'
    config = _get_config(None, tabname)
    x = list_ciudad(cond, orderby, usecondmethod=usecondmethod)
    return (config['output_showpage']('Search results', x, tabname), )

def choose_search_ciudad(idtag, condition_dict = None, method="LIKE", outmethod="simple",
                                                  previewsize=10, list_condition_dict = None, **kwargs):
    tabname = 'ciudad'
    config = _get_config(None, tabname)
    kwargs['config'] = config
    return _choose_search(tabname, idtag, condition_dict, method, outmethod, previewsize, list_condition_dict, **kwargs)

def preview_ciudad(*args, **kwargs):
    tabname = 'ciudad'
    kwargs['config'] = _get_config(None, tabname)
    # kwargs['display'] = 'nombre'
    return _preview(tabname, *args, **kwargs)

###
### end of ciudad UI
###
###############################################################################################
###############################################################################################
###############################################################################################
###############################################################################################
###
### Table persona UI
###

### Configuring persona

# TODO: Change all values of editable, insertable, indexable, etc to predicates
# TODO: in order to be able to give a behaviour dependable of the user, or auth

modconfig['persona'] = {
    'internal_fields' : [u'xid'],
    'dict_default': { '' : '<python:locale id="general.submit" />','calle' : '','fax' : '','xid' : '','curp' : '','adicional' : '','colonia' : '','extnumero' : '','intnumero' : '','ciudad_idciudad' : '','rfc' : '','desde' : '','celular' : '','nombre' : '','telefono' : '','email' : '','exttelefono' : '',
                     '__objname__': 'catalogos.catalogos',
                     '__tabname__': 'persona',
                     #'__fulltextsearch__': '', # disables the fulltext search in the modify frame
                    },
    'editable': lambda: True,
    'insertable': lambda: True,
    'indexable': lambda: True,
    'structured_search': lambda: True,
    'ftindex': ft3k.sqlindex(name='persona', conn=dbftindex),
    'ftsearch_size': 10,
    'count': 30,
    'list_fields': ['xid', 'email', 'nombre', 'curp', 'rfc', 'telefono', 'celular', 'fax', 'exttelefono', 'calle', 'colonia', 'extnumero', 'intnumero', 'adicional', 'ciudad_idciudad', 'desde'],
    'preview_fields': ['xid', 'email', 'nombre', 'curp', 'rfc', 'telefono', 'celular', 'fax', 'exttelefono', 'calle', 'colonia', 'extnumero', 'intnumero', 'adicional', 'ciudad_idciudad', 'desde'],
    'list_with_empty_condition': lambda: True,
    'control_generator': _control_generator,
    'preview_maxsize': 64,
    'output_table_header': lambda r: "<tr>%s</tr>"%("<th>%s</th>"%("</th><th>".join(r))),
    'output_table_row': lambda r, q, fields: "<tr>%s</tr>"%("<td>%s</td>"%("</td><td>".join(r))),
    'output_table_control_row': lambda r, controls, q, fields: '<tr><td style="text-align: right;" colspan="%s">%s</td></tr>'%(len(r), " &nbsp; ".join(controls)),
    'output_table': lambda r, query, fields: """<table style="width: 100%%"><tbody>%s</tbody></table>"""%("".join(r)),
    'output_showpage': lambda title, data, id: ('insert',
                                                       {'title': title,
                                                        'data': data,
                                                        'id': id,
                                                        'rowclasses': (['cell1', 'cell1', 'cell2', 'cell2'], 'header')
                                                       }),
    'select': _list_select,
    'foreign_mapper': {u'ciudad_idciudad': (u'ciudad', u'xid')},
    'form_element': template3k.get_template("""persona:form_element""",
                                            """
            <strong>Persona</strong>
            <form method="POST" enctype="multipart/form-data" id="form-persona<python:keyword id='__id__' />" action="javascript: void(0);" onsubmit="<python:keyword id="__onsubmit__" />" name="persona" >
                <table><tbody>
            
<tr><td>
<python:locale id="persona.email">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Email'} ]
</python:locale>:</td><td><input name="email:string:" type="text" value="<python:keyword id="email" />" size="40" /></td></tr>


<tr><td>
<python:locale id="persona.nombre">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Nombre'} ]
</python:locale>:</td><td><input name="nombre:string:mandatory" type="text" value="<python:keyword id="nombre" />" size="40" /></td></tr>


<tr><td>
<python:locale id="persona.curp">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Curp'} ]
</python:locale>:</td><td><input name="curp:string:" type="text" value="<python:keyword id="curp" />" size="40" /></td></tr>


<tr><td>
<python:locale id="persona.rfc">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Rfc'} ]
</python:locale>:</td><td><input name="rfc:string:" type="text" value="<python:keyword id="rfc" />" size="40" /></td></tr>


<tr><td>
<python:locale id="persona.telefono">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Telefono'} ]
</python:locale>:</td><td><input name="telefono:string:" type="text" value="<python:keyword id="telefono" />" size="40" /></td></tr>


<tr><td>
<python:locale id="persona.celular">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Celular'} ]
</python:locale>:</td><td><input name="celular:string:" type="text" value="<python:keyword id="celular" />" size="40" /></td></tr>


<tr><td>
<python:locale id="persona.fax">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Fax'} ]
</python:locale>:</td><td><input name="fax:string:" type="text" value="<python:keyword id="fax" />" size="40" /></td></tr>


<tr><td>
<python:locale id="persona.exttelefono">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Exttelefono'} ]
</python:locale>:</td><td><input name="exttelefono:string:" type="text" value="<python:keyword id="exttelefono" />" size="40" /></td></tr>


<tr><td>
<python:locale id="persona.calle">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Calle'} ]
</python:locale>:</td><td><input name="calle:string:" type="text" value="<python:keyword id="calle" />" size="40" /></td></tr>


<tr><td>
<python:locale id="persona.colonia">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Colonia'} ]
</python:locale>:</td><td><input name="colonia:string:" type="text" value="<python:keyword id="colonia" />" size="40" /></td></tr>


<tr><td>
<python:locale id="persona.extnumero">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Extnumero'} ]
</python:locale>:</td><td><input name="extnumero:int:" type="text" value="<python:keyword id="extnumero" />" size="40" /></td></tr>


<tr><td>
<python:locale id="persona.intnumero">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Intnumero'} ]
</python:locale>:</td><td><input name="intnumero:int:" type="text" value="<python:keyword id="intnumero" />" size="40" /></td></tr>


<tr><td>
<python:locale id="persona.adicional">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Adicional'} ]
</python:locale>:</td><td><input name="adicional:string:" type="text" value="<python:keyword id="adicional" />" size="40" /></td></tr>


<tr><td>
<python:locale id="persona.ciudad_idciudad">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Ciudad Idciudad'} ]
</python:locale>:</td><td><select onfocus="callto({'rawhandler': createhandlerselect('<python:keyword id="ciudad_idciudad" />'), 'handlerargs': this},'<python:keyword id="__objname__" />','preview_ciudad',['xid'])" name="ciudad_idciudad:int"  id="select-ciudad_xid_<python:keyword id='__id__' />" ></select>
<a style="text-decoration: none;" href="javascript: DOM3K_replaceListOptions([['','']],'select-ciudad_xid_<python:keyword id="__id__" />'); void(0);"><python:locale id="actions.clear" /></a><python:keyword id="__editmode__">

/
<a style="text-decoration: none;" href="javascript: showformmodify('<python:keyword id="__objname__" />', 'ciudad'); void(0);"><python:locale id="actions.new" /></a></python:keyword>

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_ciudad', ['choose_search','select-ciudad_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search" /></a><python:keyword id="__fulltextsearch__">

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_fulltext_ciudad', ['choose_search','select-ciudad_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search_fulltext" /></a></python:keyword>

</td></tr>


<tr><td>
<python:locale id="persona.desde">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Desde'} ]
</python:locale>:</td><td>
            <python:locale id="date.format_date" />
            <input type="text" name="desde:date:" value="<python:keyword id="desde" />" onclick="Easyform_frameReadDate(DOM3K_readInput(this), DOM3K_updateInput, this, event.clientY, event.clientX)"  />
            </td></tr>

<tr><td><input type="submit" name=":submit" value="<python:locale id="general.submit" />" /></td></tr>
                </tbody></table>
            </form>
            <python:keyword id="__bottom__"></python:keyword>
            """),
    'form_element_search': template3k.get_template("""persona:form_element_search""",
                                                   """
            <strong>Persona</strong>
            <form method="POST" enctype="multipart/form-data" id="form-persona<python:keyword id='__id__' />" action="javascript: void(0);" onsubmit="<python:keyword id="__onsubmit__" />" name="persona" >
                <table><tbody>
            
<tr><td>
<python:locale id="persona.email">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Email'} ]
</python:locale>:</td><td><input name="email:string:" type="text" value="<python:keyword id="email" />" size="40" /></td></tr>


<tr><td>
<python:locale id="persona.nombre">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Nombre'} ]
</python:locale>:</td><td><input name="nombre:string" type="text" value="<python:keyword id="nombre" />" size="40" /></td></tr>


<tr><td>
<python:locale id="persona.curp">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Curp'} ]
</python:locale>:</td><td><input name="curp:string:" type="text" value="<python:keyword id="curp" />" size="40" /></td></tr>


<tr><td>
<python:locale id="persona.rfc">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Rfc'} ]
</python:locale>:</td><td><input name="rfc:string:" type="text" value="<python:keyword id="rfc" />" size="40" /></td></tr>


<tr><td>
<python:locale id="persona.telefono">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Telefono'} ]
</python:locale>:</td><td><input name="telefono:string:" type="text" value="<python:keyword id="telefono" />" size="40" /></td></tr>


<tr><td>
<python:locale id="persona.celular">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Celular'} ]
</python:locale>:</td><td><input name="celular:string:" type="text" value="<python:keyword id="celular" />" size="40" /></td></tr>


<tr><td>
<python:locale id="persona.fax">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Fax'} ]
</python:locale>:</td><td><input name="fax:string:" type="text" value="<python:keyword id="fax" />" size="40" /></td></tr>


<tr><td>
<python:locale id="persona.exttelefono">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Exttelefono'} ]
</python:locale>:</td><td><input name="exttelefono:string:" type="text" value="<python:keyword id="exttelefono" />" size="40" /></td></tr>


<tr><td>
<python:locale id="persona.calle">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Calle'} ]
</python:locale>:</td><td><input name="calle:string:" type="text" value="<python:keyword id="calle" />" size="40" /></td></tr>


<tr><td>
<python:locale id="persona.colonia">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Colonia'} ]
</python:locale>:</td><td><input name="colonia:string:" type="text" value="<python:keyword id="colonia" />" size="40" /></td></tr>


<tr><td>
<python:locale id="persona.extnumero">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Extnumero'} ]
</python:locale>:</td><td><input name="extnumero:int:" type="text" value="<python:keyword id="extnumero" />" size="40" /></td></tr>


<tr><td>
<python:locale id="persona.intnumero">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Intnumero'} ]
</python:locale>:</td><td><input name="intnumero:int:" type="text" value="<python:keyword id="intnumero" />" size="40" /></td></tr>


<tr><td>
<python:locale id="persona.adicional">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Adicional'} ]
</python:locale>:</td><td><input name="adicional:string:" type="text" value="<python:keyword id="adicional" />" size="40" /></td></tr>


<tr><td>
<python:locale id="persona.ciudad_idciudad">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Ciudad Idciudad'} ]
</python:locale>:</td><td><select onfocus="callto({'rawhandler': createhandlerselect('<python:keyword id="ciudad_idciudad" />'), 'handlerargs': this},'<python:keyword id="__objname__" />','preview_ciudad',['xid'])" name="ciudad_idciudad:int"  id="select-ciudad_xid_<python:keyword id='__id__' />" ></select>
<a style="text-decoration: none;" href="javascript: DOM3K_replaceListOptions([['','']],'select-ciudad_xid_<python:keyword id="__id__" />'); void(0);"><python:locale id="actions.clear" /></a><python:keyword id="__editmode__">

/
<a style="text-decoration: none;" href="javascript: showformmodify('<python:keyword id="__objname__" />', 'ciudad'); void(0);"><python:locale id="actions.new" /></a></python:keyword>

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_ciudad', ['choose_search','select-ciudad_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search" /></a><python:keyword id="__fulltextsearch__">

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_fulltext_ciudad', ['choose_search','select-ciudad_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search_fulltext" /></a></python:keyword>

</td></tr>


<tr><td>
<python:locale id="persona.desde">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Desde'} ]
</python:locale>:</td><td>
            <python:locale id="date.format_date" />
            <input type="text" name="desde:date:" value="<python:keyword id="desde" />" onclick="Easyform_frameReadDate(DOM3K_readInput(this), DOM3K_updateInput, this, event.clientY, event.clientX)"  />
            </td></tr>

<tr><td><input type="submit" name=":submit" value="<python:locale id="general.submit" />" /></td></tr>
                </tbody></table>
            </form>
            <python:keyword id="__bottom__"></python:keyword>
            """),
    '__sentinel__': True
    }

### End configuration

###
### Beginning methods/functions

#_spyro_authentication.perms['form_persona'] = False
#_spyro_authentication.perms['list_persona'] = False
#_spyro_authentication.perms['modify_persona'] = False
#_spyro_authentication.perms['delete_persona'] = False
 
def list_view_persona(*args, **kwargs):
    tabname = 'persona'
    config = _get_config(None, tabname)
    kwargs['config'] = config
    return (config['output_showpage'](tabname, _list(tabname, *args, **kwargs), tabname), )

def form_persona(mode = None, condition_dict = None, **kwargs):
    tabname = 'persona'
    kwargs['config'] = _get_config(None, tabname)
    return _form(tabname, mode, condition_dict, **kwargs)

def modify_persona(mode, data, cond, dbconn=None, doreturn=1, **kwargs):
    tabname = 'persona'
    config = _get_config(None, tabname)
    assert config['editable'](), "View only entity, can't %s"%mode
    tabname = 'persona'
    return _modify(tabname, mode, data, cond, dbconn, doreturn, config=config, **kwargs)

def delete_persona(cond, dbconn = None, doreturn = 1, **kwargs):
    tabname = 'persona'
    config = _get_config(None, tabname)
    assert config['editable'](), "View only entity, can't delete"
    return _delete(tabname, cond, dbconn, doreturn, config=config, **kwargs)

def list_persona(*args, **kwargs):
    tabname = 'persona'
    kwargs['config'] = _get_config(None, tabname)
    # assert _spyro_authentication._rawauth(auth,  objname, list_persona)
    return _list(tabname, *args, **kwargs)

def form_search_persona(method=None, *args, **kwargs):
    tabname = 'persona'
    config = _get_config(None, tabname)
    dict_default = dict(config['dict_default']) # dictionary
    dict_default['__editmode__'] = ""
    if method == "choose_search":
        idtag = args[0]
        residtag = 'choose-search-%s'%idtag
        dict_default['__onsubmit__'] = (r"callto({'showresults': function(d){ DOM3K_updateElement(d,'%s'); """
                                        r"""DOM3K_BackgroundRowColors('%s',['rgb(200,200,230);','inherit']);}}, """
                                        r"""'%s','choose_search_%s',null,{'condition_dict': Easyform_formToDict(this), 'idtag': %s });"""
                                        r""" """%(residtag,residtag,objname,tabname,repr(idtag)))
        dict_default['__bottom__'] = """<div id="%s"></div>"""%residtag
    else:
        dict_default['__onsubmit__'] = r"callto({'handler': handle_commands}, '%s','search_%s',[Easyform_formToDict(this), null, 'LIKE']);"%(objname,tabname)
    dict_default['__id__'] = str(time.time())
    x = config['form_element_search'].merge(dict_default)
    return x

def manage_fulltext_persona(drop_index, reindex):
    tabname = 'persona'
    config = _get_config(None, tabname)
    ftindex = config['ftindex']
    if drop_index:
        ftindex.drop_index()
    if reindex:
        ftindex.create_index()
        for x in _preview(tabname, config['list_fields'][0], config=config, preview_maxsize=512):
            _rowid = _get_rowid(tabname, x)
            ftindex.handle_insert(x[1], _rowid)

def form_search_fulltext_persona(output='list', idtag='', query='',**kwargs):
    tabname = 'persona'
    dict_default = {'query': query, 'output': output, 'idtag': idtag}
    if output == "choose_search":
        residtag = 'choose-search-%s'%idtag
        dict_default['__onsubmit__'] = (r"""callto({'handler': handle_commands_frames()}, '%s', 'search_fulltext_%s', """
                                        r"""null, Easyform_formToDict(this));"""%(objname, tabname))
        dict_default['__bottom__'] = """<div id="%s"></div>"""%residtag
        dict_default['__idtag__'] = """<input name="idtag:string:mandatory" value="%s" type="hidden" />"""%idtag
        dict_default['residtag'] = residtag
    else:
        dict_default['residtag'] = tabname
        dict_default['__onsubmit__'] = r"callto({'handler': handle_commands}, '%s', 'search_fulltext_%s', null, Easyform_formToDict(this));"%(objname, tabname)
        dict_default['__idtag__'] = ""
        dict_default['__bottom__'] = ""
    dict_default['__id__'] = str(time.time())
    return """<form action="javascript: void(0);" onsubmit="%(__onsubmit__)s">
    <input name="output:string:mandatory" value="%(output)s" type="hidden" />
    <input name="query:string:mandatory" value="%(query)s" size="40" type="text" />
    <input name="residtag:string" value="residtag" type="hidden" />
    %(__idtag__)s
    <input name=":submit" value="Search" type="submit" />
    </form>
    %(__bottom__)s
    """%dict_default

def search_fulltext_persona(query, output="", idtag="", residtag="", **kwargs):
    tabname = 'persona'
    kwargs['config'] = None
    return _search_fulltext(tabname, query, output, idtag, residtag, **kwargs)


def search_persona(cond,orderby=None,usecondmethod=None,**kwargs):
    tabname = 'persona'
    config = _get_config(None, tabname)
    x = list_persona(cond, orderby, usecondmethod=usecondmethod)
    return (config['output_showpage']('Search results', x, tabname), )

def choose_search_persona(idtag, condition_dict = None, method="LIKE", outmethod="simple",
                                                  previewsize=10, list_condition_dict = None, **kwargs):
    tabname = 'persona'
    config = _get_config(None, tabname)
    kwargs['config'] = config
    return _choose_search(tabname, idtag, condition_dict, method, outmethod, previewsize, list_condition_dict, **kwargs)

def preview_persona(*args, **kwargs):
    tabname = 'persona'
    kwargs['config'] = _get_config(None, tabname)
    # kwargs['display'] = 'email'
    return _preview(tabname, *args, **kwargs)

###
### end of persona UI
###
###############################################################################################
###############################################################################################
###############################################################################################
###############################################################################################
###
### Table categorias UI
###

### Configuring categorias

# TODO: Change all values of editable, insertable, indexable, etc to predicates
# TODO: in order to be able to give a behaviour dependable of the user, or auth

modconfig['categorias'] = {
    'internal_fields' : [u'xid'],
    'dict_default': { 'descripcion' : '','nombre' : '','xid' : '','' : '<python:locale id="general.submit" />',
                     '__objname__': 'catalogos.catalogos',
                     '__tabname__': 'categorias',
                     #'__fulltextsearch__': '', # disables the fulltext search in the modify frame
                    },
    'editable': lambda: True,
    'insertable': lambda: True,
    'indexable': lambda: True,
    'structured_search': lambda: True,
    'ftindex': ft3k.sqlindex(name='categorias', conn=dbftindex),
    'ftsearch_size': 10,
    'count': 30,
    'list_fields': ['xid', 'nombre', 'descripcion'],
    'preview_fields': ['xid', 'nombre', 'descripcion'],
    'list_with_empty_condition': lambda: True,
    'control_generator': _control_generator,
    'preview_maxsize': 64,
    'output_table_header': lambda r: "<tr>%s</tr>"%("<th>%s</th>"%("</th><th>".join(r))),
    'output_table_row': lambda r, q, fields: "<tr>%s</tr>"%("<td>%s</td>"%("</td><td>".join(r))),
    'output_table_control_row': lambda r, controls, q, fields: '<tr><td style="text-align: right;" colspan="%s">%s</td></tr>'%(len(r), " &nbsp; ".join(controls)),
    'output_table': lambda r, query, fields: """<table style="width: 100%%"><tbody>%s</tbody></table>"""%("".join(r)),
    'output_showpage': lambda title, data, id: ('insert',
                                                       {'title': title,
                                                        'data': data,
                                                        'id': id,
                                                        'rowclasses': (['cell1', 'cell1', 'cell2', 'cell2'], 'header')
                                                       }),
    'select': _list_select,
    'foreign_mapper': {},
    'form_element': template3k.get_template("""categorias:form_element""",
                                            """
            <strong>Categorias</strong>
            <form method="POST" enctype="multipart/form-data" id="form-categorias<python:keyword id='__id__' />" action="javascript: void(0);" onsubmit="<python:keyword id="__onsubmit__" />" name="categorias" >
                <table><tbody>
            
<tr><td>
<python:locale id="categorias.nombre">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Nombre'} ]
</python:locale>:</td><td><input name="nombre:string:mandatory" type="text" value="<python:keyword id="nombre" />" size="40" /></td></tr>


<tr><td>
<python:locale id="categorias.descripcion">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Descripcion'} ]
</python:locale>:</td><td><input name="descripcion:string:mandatory" type="text" value="<python:keyword id="descripcion" />" size="40" /></td></tr>

<tr><td><input type="submit" name=":submit" value="<python:locale id="general.submit" />" /></td></tr>
                </tbody></table>
            </form>
            <python:keyword id="__bottom__"></python:keyword>
            """),
    'form_element_search': template3k.get_template("""categorias:form_element_search""",
                                                   """
            <strong>Categorias</strong>
            <form method="POST" enctype="multipart/form-data" id="form-categorias<python:keyword id='__id__' />" action="javascript: void(0);" onsubmit="<python:keyword id="__onsubmit__" />" name="categorias" >
                <table><tbody>
            
<tr><td>
<python:locale id="categorias.nombre">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Nombre'} ]
</python:locale>:</td><td><input name="nombre:string" type="text" value="<python:keyword id="nombre" />" size="40" /></td></tr>


<tr><td>
<python:locale id="categorias.descripcion">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Descripcion'} ]
</python:locale>:</td><td><input name="descripcion:string" type="text" value="<python:keyword id="descripcion" />" size="40" /></td></tr>

<tr><td><input type="submit" name=":submit" value="<python:locale id="general.submit" />" /></td></tr>
                </tbody></table>
            </form>
            <python:keyword id="__bottom__"></python:keyword>
            """),
    '__sentinel__': True
    }

### End configuration

###
### Beginning methods/functions

#_spyro_authentication.perms['form_categorias'] = False
#_spyro_authentication.perms['list_categorias'] = False
#_spyro_authentication.perms['modify_categorias'] = False
#_spyro_authentication.perms['delete_categorias'] = False
 
def list_view_categorias(*args, **kwargs):
    tabname = 'categorias'
    config = _get_config(None, tabname)
    kwargs['config'] = config
    return (config['output_showpage'](tabname, _list(tabname, *args, **kwargs), tabname), )

def form_categorias(mode = None, condition_dict = None, **kwargs):
    tabname = 'categorias'
    kwargs['config'] = _get_config(None, tabname)
    return _form(tabname, mode, condition_dict, **kwargs)

def modify_categorias(mode, data, cond, dbconn=None, doreturn=1, **kwargs):
    tabname = 'categorias'
    config = _get_config(None, tabname)
    assert config['editable'](), "View only entity, can't %s"%mode
    tabname = 'categorias'
    return _modify(tabname, mode, data, cond, dbconn, doreturn, config=config, **kwargs)

def delete_categorias(cond, dbconn = None, doreturn = 1, **kwargs):
    tabname = 'categorias'
    config = _get_config(None, tabname)
    assert config['editable'](), "View only entity, can't delete"
    return _delete(tabname, cond, dbconn, doreturn, config=config, **kwargs)

def list_categorias(*args, **kwargs):
    tabname = 'categorias'
    kwargs['config'] = _get_config(None, tabname)
    # assert _spyro_authentication._rawauth(auth,  objname, list_categorias)
    return _list(tabname, *args, **kwargs)

def form_search_categorias(method=None, *args, **kwargs):
    tabname = 'categorias'
    config = _get_config(None, tabname)
    dict_default = dict(config['dict_default']) # dictionary
    dict_default['__editmode__'] = ""
    if method == "choose_search":
        idtag = args[0]
        residtag = 'choose-search-%s'%idtag
        dict_default['__onsubmit__'] = (r"callto({'showresults': function(d){ DOM3K_updateElement(d,'%s'); """
                                        r"""DOM3K_BackgroundRowColors('%s',['rgb(200,200,230);','inherit']);}}, """
                                        r"""'%s','choose_search_%s',null,{'condition_dict': Easyform_formToDict(this), 'idtag': %s });"""
                                        r""" """%(residtag,residtag,objname,tabname,repr(idtag)))
        dict_default['__bottom__'] = """<div id="%s"></div>"""%residtag
    else:
        dict_default['__onsubmit__'] = r"callto({'handler': handle_commands}, '%s','search_%s',[Easyform_formToDict(this), null, 'LIKE']);"%(objname,tabname)
    dict_default['__id__'] = str(time.time())
    x = config['form_element_search'].merge(dict_default)
    return x

def manage_fulltext_categorias(drop_index, reindex):
    tabname = 'categorias'
    config = _get_config(None, tabname)
    ftindex = config['ftindex']
    if drop_index:
        ftindex.drop_index()
    if reindex:
        ftindex.create_index()
        for x in _preview(tabname, config['list_fields'][0], config=config, preview_maxsize=512):
            _rowid = _get_rowid(tabname, x)
            ftindex.handle_insert(x[1], _rowid)

def form_search_fulltext_categorias(output='list', idtag='', query='',**kwargs):
    tabname = 'categorias'
    dict_default = {'query': query, 'output': output, 'idtag': idtag}
    if output == "choose_search":
        residtag = 'choose-search-%s'%idtag
        dict_default['__onsubmit__'] = (r"""callto({'handler': handle_commands_frames()}, '%s', 'search_fulltext_%s', """
                                        r"""null, Easyform_formToDict(this));"""%(objname, tabname))
        dict_default['__bottom__'] = """<div id="%s"></div>"""%residtag
        dict_default['__idtag__'] = """<input name="idtag:string:mandatory" value="%s" type="hidden" />"""%idtag
        dict_default['residtag'] = residtag
    else:
        dict_default['residtag'] = tabname
        dict_default['__onsubmit__'] = r"callto({'handler': handle_commands}, '%s', 'search_fulltext_%s', null, Easyform_formToDict(this));"%(objname, tabname)
        dict_default['__idtag__'] = ""
        dict_default['__bottom__'] = ""
    dict_default['__id__'] = str(time.time())
    return """<form action="javascript: void(0);" onsubmit="%(__onsubmit__)s">
    <input name="output:string:mandatory" value="%(output)s" type="hidden" />
    <input name="query:string:mandatory" value="%(query)s" size="40" type="text" />
    <input name="residtag:string" value="residtag" type="hidden" />
    %(__idtag__)s
    <input name=":submit" value="Search" type="submit" />
    </form>
    %(__bottom__)s
    """%dict_default

def search_fulltext_categorias(query, output="", idtag="", residtag="", **kwargs):
    tabname = 'categorias'
    kwargs['config'] = None
    return _search_fulltext(tabname, query, output, idtag, residtag, **kwargs)


def search_categorias(cond,orderby=None,usecondmethod=None,**kwargs):
    tabname = 'categorias'
    config = _get_config(None, tabname)
    x = list_categorias(cond, orderby, usecondmethod=usecondmethod)
    return (config['output_showpage']('Search results', x, tabname), )

def choose_search_categorias(idtag, condition_dict = None, method="LIKE", outmethod="simple",
                                                  previewsize=10, list_condition_dict = None, **kwargs):
    tabname = 'categorias'
    config = _get_config(None, tabname)
    kwargs['config'] = config
    return _choose_search(tabname, idtag, condition_dict, method, outmethod, previewsize, list_condition_dict, **kwargs)

def preview_categorias(*args, **kwargs):
    tabname = 'categorias'
    kwargs['config'] = _get_config(None, tabname)
    # kwargs['display'] = 'nombre'
    return _preview(tabname, *args, **kwargs)

###
### end of categorias UI
###
###############################################################################################
###############################################################################################
###############################################################################################
###############################################################################################
###
### Table producto UI
###

### Configuring producto

# TODO: Change all values of editable, insertable, indexable, etc to predicates
# TODO: in order to be able to give a behaviour dependable of the user, or auth

modconfig['producto'] = {
    'internal_fields' : [u'xid'],
    'dict_default': { '' : '<python:locale id="general.submit" />','xid' : '','clase' : '','costo_base' : '','descripcion' : '','nombre' : '','codigo_barras' : '',
                     '__objname__': 'catalogos.catalogos',
                     '__tabname__': 'producto',
                     #'__fulltextsearch__': '', # disables the fulltext search in the modify frame
                    },
    'editable': lambda: True,
    'insertable': lambda: True,
    'indexable': lambda: True,
    'structured_search': lambda: True,
    'ftindex': ft3k.sqlindex(name='producto', conn=dbftindex),
    'ftsearch_size': 10,
    'count': 30,
    'list_fields': ['xid', 'nombre', 'costo_base', 'codigo_barras', 'descripcion', 'clase'],
    'preview_fields': ['xid', 'nombre', 'costo_base', 'codigo_barras', 'descripcion', 'clase'],
    'list_with_empty_condition': lambda: True,
    'control_generator': _control_generator,
    'preview_maxsize': 64,
    'output_table_header': lambda r: "<tr>%s</tr>"%("<th>%s</th>"%("</th><th>".join(r))),
    'output_table_row': lambda r, q, fields: "<tr>%s</tr>"%("<td>%s</td>"%("</td><td>".join(r))),
    'output_table_control_row': lambda r, controls, q, fields: '<tr><td style="text-align: right;" colspan="%s">%s</td></tr>'%(len(r), " &nbsp; ".join(controls)),
    'output_table': lambda r, query, fields: """<table style="width: 100%%"><tbody>%s</tbody></table>"""%("".join(r)),
    'output_showpage': lambda title, data, id: ('insert',
                                                       {'title': title,
                                                        'data': data,
                                                        'id': id,
                                                        'rowclasses': (['cell1', 'cell1', 'cell2', 'cell2'], 'header')
                                                       }),
    'select': _list_select,
    'foreign_mapper': {u'clase': (u'categorias', u'xid')},
    'form_element': template3k.get_template("""producto:form_element""",
                                            """
            <strong>Producto</strong>
            <form method="POST" enctype="multipart/form-data" id="form-producto<python:keyword id='__id__' />" action="javascript: void(0);" onsubmit="<python:keyword id="__onsubmit__" />" name="producto" >
                <table><tbody>
            
<tr><td>
<python:locale id="producto.nombre">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Nombre'} ]
</python:locale>:</td><td><input name="nombre:string:mandatory" type="text" value="<python:keyword id="nombre" />" size="40" /></td></tr>


<tr><td>
<python:locale id="producto.costo_base">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Costo Base'} ]
</python:locale>:</td><td><input name="costo_base:float:" type="text" value="<python:keyword id="costo_base" />" size="40" /></td></tr>


<tr><td>
<python:locale id="producto.codigo_barras">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Codigo Barras'} ]
</python:locale>:</td><td><input name="codigo_barras:string:" type="text" value="<python:keyword id="codigo_barras" />" size="40" /></td></tr>


<tr><td>
<python:locale id="producto.descripcion">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Descripcion'} ]
</python:locale>:</td><td><input name="descripcion:string:" type="text" value="<python:keyword id="descripcion" />" size="40" /></td></tr>


<tr><td>
<python:locale id="producto.clase">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Clase'} ]
</python:locale>:</td><td><select onfocus="callto({'rawhandler': createhandlerselect('<python:keyword id="clase" />'), 'handlerargs': this},'<python:keyword id="__objname__" />','preview_categorias',['xid'])" name="clase:int"  id="select-categorias_xid_<python:keyword id='__id__' />" ></select>
<a style="text-decoration: none;" href="javascript: DOM3K_replaceListOptions([['','']],'select-categorias_xid_<python:keyword id="__id__" />'); void(0);"><python:locale id="actions.clear" /></a><python:keyword id="__editmode__">

/
<a style="text-decoration: none;" href="javascript: showformmodify('<python:keyword id="__objname__" />', 'categorias'); void(0);"><python:locale id="actions.new" /></a></python:keyword>

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_categorias', ['choose_search','select-categorias_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search" /></a><python:keyword id="__fulltextsearch__">

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_fulltext_categorias', ['choose_search','select-categorias_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search_fulltext" /></a></python:keyword>

</td></tr>

<tr><td><input type="submit" name=":submit" value="<python:locale id="general.submit" />" /></td></tr>
                </tbody></table>
            </form>
            <python:keyword id="__bottom__"></python:keyword>
            """),
    'form_element_search': template3k.get_template("""producto:form_element_search""",
                                                   """
            <strong>Producto</strong>
            <form method="POST" enctype="multipart/form-data" id="form-producto<python:keyword id='__id__' />" action="javascript: void(0);" onsubmit="<python:keyword id="__onsubmit__" />" name="producto" >
                <table><tbody>
            
<tr><td>
<python:locale id="producto.nombre">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Nombre'} ]
</python:locale>:</td><td><input name="nombre:string" type="text" value="<python:keyword id="nombre" />" size="40" /></td></tr>


<tr><td>
<python:locale id="producto.costo_base">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Costo Base'} ]
</python:locale>:</td><td><input name="costo_base:float:" type="text" value="<python:keyword id="costo_base" />" size="40" /></td></tr>


<tr><td>
<python:locale id="producto.codigo_barras">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Codigo Barras'} ]
</python:locale>:</td><td><input name="codigo_barras:string:" type="text" value="<python:keyword id="codigo_barras" />" size="40" /></td></tr>


<tr><td>
<python:locale id="producto.descripcion">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Descripcion'} ]
</python:locale>:</td><td><input name="descripcion:string:" type="text" value="<python:keyword id="descripcion" />" size="40" /></td></tr>


<tr><td>
<python:locale id="producto.clase">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Clase'} ]
</python:locale>:</td><td><select onfocus="callto({'rawhandler': createhandlerselect('<python:keyword id="clase" />'), 'handlerargs': this},'<python:keyword id="__objname__" />','preview_categorias',['xid'])" name="clase:int"  id="select-categorias_xid_<python:keyword id='__id__' />" ></select>
<a style="text-decoration: none;" href="javascript: DOM3K_replaceListOptions([['','']],'select-categorias_xid_<python:keyword id="__id__" />'); void(0);"><python:locale id="actions.clear" /></a><python:keyword id="__editmode__">

/
<a style="text-decoration: none;" href="javascript: showformmodify('<python:keyword id="__objname__" />', 'categorias'); void(0);"><python:locale id="actions.new" /></a></python:keyword>

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_categorias', ['choose_search','select-categorias_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search" /></a><python:keyword id="__fulltextsearch__">

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_fulltext_categorias', ['choose_search','select-categorias_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search_fulltext" /></a></python:keyword>

</td></tr>

<tr><td><input type="submit" name=":submit" value="<python:locale id="general.submit" />" /></td></tr>
                </tbody></table>
            </form>
            <python:keyword id="__bottom__"></python:keyword>
            """),
    '__sentinel__': True
    }

### End configuration

###
### Beginning methods/functions

#_spyro_authentication.perms['form_producto'] = False
#_spyro_authentication.perms['list_producto'] = False
#_spyro_authentication.perms['modify_producto'] = False
#_spyro_authentication.perms['delete_producto'] = False
 
def list_view_producto(*args, **kwargs):
    tabname = 'producto'
    config = _get_config(None, tabname)
    kwargs['config'] = config
    return (config['output_showpage'](tabname, _list(tabname, *args, **kwargs), tabname), )

def form_producto(mode = None, condition_dict = None, **kwargs):
    tabname = 'producto'
    kwargs['config'] = _get_config(None, tabname)
    return _form(tabname, mode, condition_dict, **kwargs)

def modify_producto(mode, data, cond, dbconn=None, doreturn=1, **kwargs):
    tabname = 'producto'
    config = _get_config(None, tabname)
    assert config['editable'](), "View only entity, can't %s"%mode
    tabname = 'producto'
    return _modify(tabname, mode, data, cond, dbconn, doreturn, config=config, **kwargs)

def delete_producto(cond, dbconn = None, doreturn = 1, **kwargs):
    tabname = 'producto'
    config = _get_config(None, tabname)
    assert config['editable'](), "View only entity, can't delete"
    return _delete(tabname, cond, dbconn, doreturn, config=config, **kwargs)

def list_producto(*args, **kwargs):
    tabname = 'producto'
    kwargs['config'] = _get_config(None, tabname)
    # assert _spyro_authentication._rawauth(auth,  objname, list_producto)
    return _list(tabname, *args, **kwargs)

def form_search_producto(method=None, *args, **kwargs):
    tabname = 'producto'
    config = _get_config(None, tabname)
    dict_default = dict(config['dict_default']) # dictionary
    dict_default['__editmode__'] = ""
    if method == "choose_search":
        idtag = args[0]
        residtag = 'choose-search-%s'%idtag
        dict_default['__onsubmit__'] = (r"callto({'showresults': function(d){ DOM3K_updateElement(d,'%s'); """
                                        r"""DOM3K_BackgroundRowColors('%s',['rgb(200,200,230);','inherit']);}}, """
                                        r"""'%s','choose_search_%s',null,{'condition_dict': Easyform_formToDict(this), 'idtag': %s });"""
                                        r""" """%(residtag,residtag,objname,tabname,repr(idtag)))
        dict_default['__bottom__'] = """<div id="%s"></div>"""%residtag
    else:
        dict_default['__onsubmit__'] = r"callto({'handler': handle_commands}, '%s','search_%s',[Easyform_formToDict(this), null, 'LIKE']);"%(objname,tabname)
    dict_default['__id__'] = str(time.time())
    x = config['form_element_search'].merge(dict_default)
    return x

def manage_fulltext_producto(drop_index, reindex):
    tabname = 'producto'
    config = _get_config(None, tabname)
    ftindex = config['ftindex']
    if drop_index:
        ftindex.drop_index()
    if reindex:
        ftindex.create_index()
        for x in _preview(tabname, config['list_fields'][0], config=config, preview_maxsize=512):
            _rowid = _get_rowid(tabname, x)
            ftindex.handle_insert(x[1], _rowid)

def form_search_fulltext_producto(output='list', idtag='', query='',**kwargs):
    tabname = 'producto'
    dict_default = {'query': query, 'output': output, 'idtag': idtag}
    if output == "choose_search":
        residtag = 'choose-search-%s'%idtag
        dict_default['__onsubmit__'] = (r"""callto({'handler': handle_commands_frames()}, '%s', 'search_fulltext_%s', """
                                        r"""null, Easyform_formToDict(this));"""%(objname, tabname))
        dict_default['__bottom__'] = """<div id="%s"></div>"""%residtag
        dict_default['__idtag__'] = """<input name="idtag:string:mandatory" value="%s" type="hidden" />"""%idtag
        dict_default['residtag'] = residtag
    else:
        dict_default['residtag'] = tabname
        dict_default['__onsubmit__'] = r"callto({'handler': handle_commands}, '%s', 'search_fulltext_%s', null, Easyform_formToDict(this));"%(objname, tabname)
        dict_default['__idtag__'] = ""
        dict_default['__bottom__'] = ""
    dict_default['__id__'] = str(time.time())
    return """<form action="javascript: void(0);" onsubmit="%(__onsubmit__)s">
    <input name="output:string:mandatory" value="%(output)s" type="hidden" />
    <input name="query:string:mandatory" value="%(query)s" size="40" type="text" />
    <input name="residtag:string" value="residtag" type="hidden" />
    %(__idtag__)s
    <input name=":submit" value="Search" type="submit" />
    </form>
    %(__bottom__)s
    """%dict_default

def search_fulltext_producto(query, output="", idtag="", residtag="", **kwargs):
    tabname = 'producto'
    kwargs['config'] = None
    return _search_fulltext(tabname, query, output, idtag, residtag, **kwargs)


def search_producto(cond,orderby=None,usecondmethod=None,**kwargs):
    tabname = 'producto'
    config = _get_config(None, tabname)
    x = list_producto(cond, orderby, usecondmethod=usecondmethod)
    return (config['output_showpage']('Search results', x, tabname), )

def choose_search_producto(idtag, condition_dict = None, method="LIKE", outmethod="simple",
                                                  previewsize=10, list_condition_dict = None, **kwargs):
    tabname = 'producto'
    config = _get_config(None, tabname)
    kwargs['config'] = config
    return _choose_search(tabname, idtag, condition_dict, method, outmethod, previewsize, list_condition_dict, **kwargs)

def preview_producto(*args, **kwargs):
    tabname = 'producto'
    kwargs['config'] = _get_config(None, tabname)
    # kwargs['display'] = 'nombre'
    return _preview(tabname, *args, **kwargs)

###
### end of producto UI
###
###############################################################################################
###############################################################################################
###############################################################################################
###############################################################################################
###
### Table proveedor UI
###

### Configuring proveedor

# TODO: Change all values of editable, insertable, indexable, etc to predicates
# TODO: in order to be able to give a behaviour dependable of the user, or auth

modconfig['proveedor'] = {
    'internal_fields' : [u'xid'],
    'dict_default': { '' : '<python:locale id="general.submit" />','nombre' : '','contactid' : '','xid' : '',
                     '__objname__': 'catalogos.catalogos',
                     '__tabname__': 'proveedor',
                     #'__fulltextsearch__': '', # disables the fulltext search in the modify frame
                    },
    'editable': lambda: True,
    'insertable': lambda: True,
    'indexable': lambda: True,
    'structured_search': lambda: True,
    'ftindex': ft3k.sqlindex(name='proveedor', conn=dbftindex),
    'ftsearch_size': 10,
    'count': 30,
    'list_fields': ['xid', 'contactid', 'nombre'],
    'preview_fields': ['xid', 'contactid', 'nombre'],
    'list_with_empty_condition': lambda: True,
    'control_generator': _control_generator,
    'preview_maxsize': 64,
    'output_table_header': lambda r: "<tr>%s</tr>"%("<th>%s</th>"%("</th><th>".join(r))),
    'output_table_row': lambda r, q, fields: "<tr>%s</tr>"%("<td>%s</td>"%("</td><td>".join(r))),
    'output_table_control_row': lambda r, controls, q, fields: '<tr><td style="text-align: right;" colspan="%s">%s</td></tr>'%(len(r), " &nbsp; ".join(controls)),
    'output_table': lambda r, query, fields: """<table style="width: 100%%"><tbody>%s</tbody></table>"""%("".join(r)),
    'output_showpage': lambda title, data, id: ('insert',
                                                       {'title': title,
                                                        'data': data,
                                                        'id': id,
                                                        'rowclasses': (['cell1', 'cell1', 'cell2', 'cell2'], 'header')
                                                       }),
    'select': _list_select,
    'foreign_mapper': {u'contactid': (u'persona', u'xid')},
    'form_element': template3k.get_template("""proveedor:form_element""",
                                            """
            <strong>Proveedor</strong>
            <form method="POST" enctype="multipart/form-data" id="form-proveedor<python:keyword id='__id__' />" action="javascript: void(0);" onsubmit="<python:keyword id="__onsubmit__" />" name="proveedor" >
                <table><tbody>
            
<tr><td>
<python:locale id="proveedor.contactid">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Contactid'} ]
</python:locale>:</td><td><select onfocus="callto({'rawhandler': createhandlerselect('<python:keyword id="contactid" />'), 'handlerargs': this},'<python:keyword id="__objname__" />','preview_persona',['xid'])" name="contactid:int"  id="select-persona_xid_<python:keyword id='__id__' />" ></select>
<a style="text-decoration: none;" href="javascript: DOM3K_replaceListOptions([['','']],'select-persona_xid_<python:keyword id="__id__" />'); void(0);"><python:locale id="actions.clear" /></a><python:keyword id="__editmode__">

/
<a style="text-decoration: none;" href="javascript: showformmodify('<python:keyword id="__objname__" />', 'persona'); void(0);"><python:locale id="actions.new" /></a></python:keyword>

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_persona', ['choose_search','select-persona_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search" /></a><python:keyword id="__fulltextsearch__">

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_fulltext_persona', ['choose_search','select-persona_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search_fulltext" /></a></python:keyword>

</td></tr>


<tr><td>
<python:locale id="proveedor.nombre">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Nombre'} ]
</python:locale>:</td><td><input name="nombre:string:mandatory" type="text" value="<python:keyword id="nombre" />" size="40" /></td></tr>

<tr><td><input type="submit" name=":submit" value="<python:locale id="general.submit" />" /></td></tr>
                </tbody></table>
            </form>
            <python:keyword id="__bottom__"></python:keyword>
            """),
    'form_element_search': template3k.get_template("""proveedor:form_element_search""",
                                                   """
            <strong>Proveedor</strong>
            <form method="POST" enctype="multipart/form-data" id="form-proveedor<python:keyword id='__id__' />" action="javascript: void(0);" onsubmit="<python:keyword id="__onsubmit__" />" name="proveedor" >
                <table><tbody>
            
<tr><td>
<python:locale id="proveedor.contactid">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Contactid'} ]
</python:locale>:</td><td><select onfocus="callto({'rawhandler': createhandlerselect('<python:keyword id="contactid" />'), 'handlerargs': this},'<python:keyword id="__objname__" />','preview_persona',['xid'])" name="contactid:int"  id="select-persona_xid_<python:keyword id='__id__' />" ></select>
<a style="text-decoration: none;" href="javascript: DOM3K_replaceListOptions([['','']],'select-persona_xid_<python:keyword id="__id__" />'); void(0);"><python:locale id="actions.clear" /></a><python:keyword id="__editmode__">

/
<a style="text-decoration: none;" href="javascript: showformmodify('<python:keyword id="__objname__" />', 'persona'); void(0);"><python:locale id="actions.new" /></a></python:keyword>

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_persona', ['choose_search','select-persona_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search" /></a><python:keyword id="__fulltextsearch__">

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_fulltext_persona', ['choose_search','select-persona_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search_fulltext" /></a></python:keyword>

</td></tr>


<tr><td>
<python:locale id="proveedor.nombre">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Nombre'} ]
</python:locale>:</td><td><input name="nombre:string" type="text" value="<python:keyword id="nombre" />" size="40" /></td></tr>

<tr><td><input type="submit" name=":submit" value="<python:locale id="general.submit" />" /></td></tr>
                </tbody></table>
            </form>
            <python:keyword id="__bottom__"></python:keyword>
            """),
    '__sentinel__': True
    }

### End configuration

###
### Beginning methods/functions

#_spyro_authentication.perms['form_proveedor'] = False
#_spyro_authentication.perms['list_proveedor'] = False
#_spyro_authentication.perms['modify_proveedor'] = False
#_spyro_authentication.perms['delete_proveedor'] = False
 
def list_view_proveedor(*args, **kwargs):
    tabname = 'proveedor'
    config = _get_config(None, tabname)
    kwargs['config'] = config
    return (config['output_showpage'](tabname, _list(tabname, *args, **kwargs), tabname), )

def form_proveedor(mode = None, condition_dict = None, **kwargs):
    tabname = 'proveedor'
    kwargs['config'] = _get_config(None, tabname)
    return _form(tabname, mode, condition_dict, **kwargs)

def modify_proveedor(mode, data, cond, dbconn=None, doreturn=1, **kwargs):
    tabname = 'proveedor'
    config = _get_config(None, tabname)
    assert config['editable'](), "View only entity, can't %s"%mode
    tabname = 'proveedor'
    return _modify(tabname, mode, data, cond, dbconn, doreturn, config=config, **kwargs)

def delete_proveedor(cond, dbconn = None, doreturn = 1, **kwargs):
    tabname = 'proveedor'
    config = _get_config(None, tabname)
    assert config['editable'](), "View only entity, can't delete"
    return _delete(tabname, cond, dbconn, doreturn, config=config, **kwargs)

def list_proveedor(*args, **kwargs):
    tabname = 'proveedor'
    kwargs['config'] = _get_config(None, tabname)
    # assert _spyro_authentication._rawauth(auth,  objname, list_proveedor)
    return _list(tabname, *args, **kwargs)

def form_search_proveedor(method=None, *args, **kwargs):
    tabname = 'proveedor'
    config = _get_config(None, tabname)
    dict_default = dict(config['dict_default']) # dictionary
    dict_default['__editmode__'] = ""
    if method == "choose_search":
        idtag = args[0]
        residtag = 'choose-search-%s'%idtag
        dict_default['__onsubmit__'] = (r"callto({'showresults': function(d){ DOM3K_updateElement(d,'%s'); """
                                        r"""DOM3K_BackgroundRowColors('%s',['rgb(200,200,230);','inherit']);}}, """
                                        r"""'%s','choose_search_%s',null,{'condition_dict': Easyform_formToDict(this), 'idtag': %s });"""
                                        r""" """%(residtag,residtag,objname,tabname,repr(idtag)))
        dict_default['__bottom__'] = """<div id="%s"></div>"""%residtag
    else:
        dict_default['__onsubmit__'] = r"callto({'handler': handle_commands}, '%s','search_%s',[Easyform_formToDict(this), null, 'LIKE']);"%(objname,tabname)
    dict_default['__id__'] = str(time.time())
    x = config['form_element_search'].merge(dict_default)
    return x

def manage_fulltext_proveedor(drop_index, reindex):
    tabname = 'proveedor'
    config = _get_config(None, tabname)
    ftindex = config['ftindex']
    if drop_index:
        ftindex.drop_index()
    if reindex:
        ftindex.create_index()
        for x in _preview(tabname, config['list_fields'][0], config=config, preview_maxsize=512):
            _rowid = _get_rowid(tabname, x)
            ftindex.handle_insert(x[1], _rowid)

def form_search_fulltext_proveedor(output='list', idtag='', query='',**kwargs):
    tabname = 'proveedor'
    dict_default = {'query': query, 'output': output, 'idtag': idtag}
    if output == "choose_search":
        residtag = 'choose-search-%s'%idtag
        dict_default['__onsubmit__'] = (r"""callto({'handler': handle_commands_frames()}, '%s', 'search_fulltext_%s', """
                                        r"""null, Easyform_formToDict(this));"""%(objname, tabname))
        dict_default['__bottom__'] = """<div id="%s"></div>"""%residtag
        dict_default['__idtag__'] = """<input name="idtag:string:mandatory" value="%s" type="hidden" />"""%idtag
        dict_default['residtag'] = residtag
    else:
        dict_default['residtag'] = tabname
        dict_default['__onsubmit__'] = r"callto({'handler': handle_commands}, '%s', 'search_fulltext_%s', null, Easyform_formToDict(this));"%(objname, tabname)
        dict_default['__idtag__'] = ""
        dict_default['__bottom__'] = ""
    dict_default['__id__'] = str(time.time())
    return """<form action="javascript: void(0);" onsubmit="%(__onsubmit__)s">
    <input name="output:string:mandatory" value="%(output)s" type="hidden" />
    <input name="query:string:mandatory" value="%(query)s" size="40" type="text" />
    <input name="residtag:string" value="residtag" type="hidden" />
    %(__idtag__)s
    <input name=":submit" value="Search" type="submit" />
    </form>
    %(__bottom__)s
    """%dict_default

def search_fulltext_proveedor(query, output="", idtag="", residtag="", **kwargs):
    tabname = 'proveedor'
    kwargs['config'] = None
    return _search_fulltext(tabname, query, output, idtag, residtag, **kwargs)


def search_proveedor(cond,orderby=None,usecondmethod=None,**kwargs):
    tabname = 'proveedor'
    config = _get_config(None, tabname)
    x = list_proveedor(cond, orderby, usecondmethod=usecondmethod)
    return (config['output_showpage']('Search results', x, tabname), )

def choose_search_proveedor(idtag, condition_dict = None, method="LIKE", outmethod="simple",
                                                  previewsize=10, list_condition_dict = None, **kwargs):
    tabname = 'proveedor'
    config = _get_config(None, tabname)
    kwargs['config'] = config
    return _choose_search(tabname, idtag, condition_dict, method, outmethod, previewsize, list_condition_dict, **kwargs)

def preview_proveedor(*args, **kwargs):
    tabname = 'proveedor'
    kwargs['config'] = _get_config(None, tabname)
    # kwargs['display'] = 'contactid'
    return _preview(tabname, *args, **kwargs)

###
### end of proveedor UI
###
###############################################################################################
###############################################################################################
###############################################################################################
###############################################################################################
###
### Table catalogo UI
###

### Configuring catalogo

# TODO: Change all values of editable, insertable, indexable, etc to predicates
# TODO: in order to be able to give a behaviour dependable of the user, or auth

modconfig['catalogo'] = {
    'internal_fields' : [u'xid'],
    'dict_default': { 'descripcion' : '','nombre' : '','xid' : '','' : '<python:locale id="general.submit" />','propietario' : '',
                     '__objname__': 'catalogos.catalogos',
                     '__tabname__': 'catalogo',
                     #'__fulltextsearch__': '', # disables the fulltext search in the modify frame
                    },
    'editable': lambda: True,
    'insertable': lambda: True,
    'indexable': lambda: True,
    'structured_search': lambda: True,
    'ftindex': ft3k.sqlindex(name='catalogo', conn=dbftindex),
    'ftsearch_size': 10,
    'count': 30,
    'list_fields': ['xid', 'nombre', 'propietario', 'descripcion'],
    'preview_fields': ['xid', 'nombre', 'propietario', 'descripcion'],
    'list_with_empty_condition': lambda: True,
    'control_generator': _control_generator,
    'preview_maxsize': 64,
    'output_table_header': lambda r: "<tr>%s</tr>"%("<th>%s</th>"%("</th><th>".join(r))),
    'output_table_row': lambda r, q, fields: "<tr>%s</tr>"%("<td>%s</td>"%("</td><td>".join(r))),
    'output_table_control_row': lambda r, controls, q, fields: '<tr><td style="text-align: right;" colspan="%s">%s</td></tr>'%(len(r), " &nbsp; ".join(controls)),
    'output_table': lambda r, query, fields: """<table style="width: 100%%"><tbody>%s</tbody></table>"""%("".join(r)),
    'output_showpage': lambda title, data, id: ('insert',
                                                       {'title': title,
                                                        'data': data,
                                                        'id': id,
                                                        'rowclasses': (['cell1', 'cell1', 'cell2', 'cell2'], 'header')
                                                       }),
    'select': _list_select,
    'foreign_mapper': {u'propietario': (u'persona', u'xid')},
    'form_element': template3k.get_template("""catalogo:form_element""",
                                            """
            <strong>Catalogo</strong>
            <form method="POST" enctype="multipart/form-data" id="form-catalogo<python:keyword id='__id__' />" action="javascript: void(0);" onsubmit="<python:keyword id="__onsubmit__" />" name="catalogo" >
                <table><tbody>
            
<tr><td>
<python:locale id="catalogo.nombre">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Nombre'} ]
</python:locale>:</td><td><input name="nombre:string:mandatory" type="text" value="<python:keyword id="nombre" />" size="40" /></td></tr>


<tr><td>
<python:locale id="catalogo.propietario">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Propietario'} ]
</python:locale>:</td><td><select onfocus="callto({'rawhandler': createhandlerselect('<python:keyword id="propietario" />'), 'handlerargs': this},'<python:keyword id="__objname__" />','preview_persona',['xid'])" name="propietario:int"  id="select-persona_xid_<python:keyword id='__id__' />" ></select>
<a style="text-decoration: none;" href="javascript: DOM3K_replaceListOptions([['','']],'select-persona_xid_<python:keyword id="__id__" />'); void(0);"><python:locale id="actions.clear" /></a><python:keyword id="__editmode__">

/
<a style="text-decoration: none;" href="javascript: showformmodify('<python:keyword id="__objname__" />', 'persona'); void(0);"><python:locale id="actions.new" /></a></python:keyword>

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_persona', ['choose_search','select-persona_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search" /></a><python:keyword id="__fulltextsearch__">

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_fulltext_persona', ['choose_search','select-persona_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search_fulltext" /></a></python:keyword>

</td></tr>


<tr><td>
<python:locale id="catalogo.descripcion">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Descripcion'} ]
</python:locale>:</td><td><textarea cols="50" rows="12" name="descripcion:text:" ><python:keyword id="descripcion" /></textarea></td></tr>

<tr><td><input type="submit" name=":submit" value="<python:locale id="general.submit" />" /></td></tr>
                </tbody></table>
            </form>
            <python:keyword id="__bottom__"></python:keyword>
            """),
    'form_element_search': template3k.get_template("""catalogo:form_element_search""",
                                                   """
            <strong>Catalogo</strong>
            <form method="POST" enctype="multipart/form-data" id="form-catalogo<python:keyword id='__id__' />" action="javascript: void(0);" onsubmit="<python:keyword id="__onsubmit__" />" name="catalogo" >
                <table><tbody>
            
<tr><td>
<python:locale id="catalogo.nombre">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Nombre'} ]
</python:locale>:</td><td><input name="nombre:string" type="text" value="<python:keyword id="nombre" />" size="40" /></td></tr>


<tr><td>
<python:locale id="catalogo.propietario">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Propietario'} ]
</python:locale>:</td><td><select onfocus="callto({'rawhandler': createhandlerselect('<python:keyword id="propietario" />'), 'handlerargs': this},'<python:keyword id="__objname__" />','preview_persona',['xid'])" name="propietario:int"  id="select-persona_xid_<python:keyword id='__id__' />" ></select>
<a style="text-decoration: none;" href="javascript: DOM3K_replaceListOptions([['','']],'select-persona_xid_<python:keyword id="__id__" />'); void(0);"><python:locale id="actions.clear" /></a><python:keyword id="__editmode__">

/
<a style="text-decoration: none;" href="javascript: showformmodify('<python:keyword id="__objname__" />', 'persona'); void(0);"><python:locale id="actions.new" /></a></python:keyword>

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_persona', ['choose_search','select-persona_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search" /></a><python:keyword id="__fulltextsearch__">

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_fulltext_persona', ['choose_search','select-persona_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search_fulltext" /></a></python:keyword>

</td></tr>


<tr><td>
<python:locale id="catalogo.descripcion">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Descripcion'} ]
</python:locale>:</td><td><textarea cols="50" rows="12" name="descripcion:text:" ><python:keyword id="descripcion" /></textarea></td></tr>

<tr><td><input type="submit" name=":submit" value="<python:locale id="general.submit" />" /></td></tr>
                </tbody></table>
            </form>
            <python:keyword id="__bottom__"></python:keyword>
            """),
    '__sentinel__': True
    }

### End configuration

###
### Beginning methods/functions

#_spyro_authentication.perms['form_catalogo'] = False
#_spyro_authentication.perms['list_catalogo'] = False
#_spyro_authentication.perms['modify_catalogo'] = False
#_spyro_authentication.perms['delete_catalogo'] = False
 
def list_view_catalogo(*args, **kwargs):
    tabname = 'catalogo'
    config = _get_config(None, tabname)
    kwargs['config'] = config
    return (config['output_showpage'](tabname, _list(tabname, *args, **kwargs), tabname), )

def form_catalogo(mode = None, condition_dict = None, **kwargs):
    tabname = 'catalogo'
    kwargs['config'] = _get_config(None, tabname)
    return _form(tabname, mode, condition_dict, **kwargs)

def modify_catalogo(mode, data, cond, dbconn=None, doreturn=1, **kwargs):
    tabname = 'catalogo'
    config = _get_config(None, tabname)
    assert config['editable'](), "View only entity, can't %s"%mode
    tabname = 'catalogo'
    return _modify(tabname, mode, data, cond, dbconn, doreturn, config=config, **kwargs)

def delete_catalogo(cond, dbconn = None, doreturn = 1, **kwargs):
    tabname = 'catalogo'
    config = _get_config(None, tabname)
    assert config['editable'](), "View only entity, can't delete"
    return _delete(tabname, cond, dbconn, doreturn, config=config, **kwargs)

def list_catalogo(*args, **kwargs):
    tabname = 'catalogo'
    kwargs['config'] = _get_config(None, tabname)
    # assert _spyro_authentication._rawauth(auth,  objname, list_catalogo)
    return _list(tabname, *args, **kwargs)

def form_search_catalogo(method=None, *args, **kwargs):
    tabname = 'catalogo'
    config = _get_config(None, tabname)
    dict_default = dict(config['dict_default']) # dictionary
    dict_default['__editmode__'] = ""
    if method == "choose_search":
        idtag = args[0]
        residtag = 'choose-search-%s'%idtag
        dict_default['__onsubmit__'] = (r"callto({'showresults': function(d){ DOM3K_updateElement(d,'%s'); """
                                        r"""DOM3K_BackgroundRowColors('%s',['rgb(200,200,230);','inherit']);}}, """
                                        r"""'%s','choose_search_%s',null,{'condition_dict': Easyform_formToDict(this), 'idtag': %s });"""
                                        r""" """%(residtag,residtag,objname,tabname,repr(idtag)))
        dict_default['__bottom__'] = """<div id="%s"></div>"""%residtag
    else:
        dict_default['__onsubmit__'] = r"callto({'handler': handle_commands}, '%s','search_%s',[Easyform_formToDict(this), null, 'LIKE']);"%(objname,tabname)
    dict_default['__id__'] = str(time.time())
    x = config['form_element_search'].merge(dict_default)
    return x

def manage_fulltext_catalogo(drop_index, reindex):
    tabname = 'catalogo'
    config = _get_config(None, tabname)
    ftindex = config['ftindex']
    if drop_index:
        ftindex.drop_index()
    if reindex:
        ftindex.create_index()
        for x in _preview(tabname, config['list_fields'][0], config=config, preview_maxsize=512):
            _rowid = _get_rowid(tabname, x)
            ftindex.handle_insert(x[1], _rowid)

def form_search_fulltext_catalogo(output='list', idtag='', query='',**kwargs):
    tabname = 'catalogo'
    dict_default = {'query': query, 'output': output, 'idtag': idtag}
    if output == "choose_search":
        residtag = 'choose-search-%s'%idtag
        dict_default['__onsubmit__'] = (r"""callto({'handler': handle_commands_frames()}, '%s', 'search_fulltext_%s', """
                                        r"""null, Easyform_formToDict(this));"""%(objname, tabname))
        dict_default['__bottom__'] = """<div id="%s"></div>"""%residtag
        dict_default['__idtag__'] = """<input name="idtag:string:mandatory" value="%s" type="hidden" />"""%idtag
        dict_default['residtag'] = residtag
    else:
        dict_default['residtag'] = tabname
        dict_default['__onsubmit__'] = r"callto({'handler': handle_commands}, '%s', 'search_fulltext_%s', null, Easyform_formToDict(this));"%(objname, tabname)
        dict_default['__idtag__'] = ""
        dict_default['__bottom__'] = ""
    dict_default['__id__'] = str(time.time())
    return """<form action="javascript: void(0);" onsubmit="%(__onsubmit__)s">
    <input name="output:string:mandatory" value="%(output)s" type="hidden" />
    <input name="query:string:mandatory" value="%(query)s" size="40" type="text" />
    <input name="residtag:string" value="residtag" type="hidden" />
    %(__idtag__)s
    <input name=":submit" value="Search" type="submit" />
    </form>
    %(__bottom__)s
    """%dict_default

def search_fulltext_catalogo(query, output="", idtag="", residtag="", **kwargs):
    tabname = 'catalogo'
    kwargs['config'] = None
    return _search_fulltext(tabname, query, output, idtag, residtag, **kwargs)


def search_catalogo(cond,orderby=None,usecondmethod=None,**kwargs):
    tabname = 'catalogo'
    config = _get_config(None, tabname)
    x = list_catalogo(cond, orderby, usecondmethod=usecondmethod)
    return (config['output_showpage']('Search results', x, tabname), )

def choose_search_catalogo(idtag, condition_dict = None, method="LIKE", outmethod="simple",
                                                  previewsize=10, list_condition_dict = None, **kwargs):
    tabname = 'catalogo'
    config = _get_config(None, tabname)
    kwargs['config'] = config
    return _choose_search(tabname, idtag, condition_dict, method, outmethod, previewsize, list_condition_dict, **kwargs)

def preview_catalogo(*args, **kwargs):
    tabname = 'catalogo'
    kwargs['config'] = _get_config(None, tabname)
    # kwargs['display'] = 'nombre'
    return _preview(tabname, *args, **kwargs)

###
### end of catalogo UI
###
###############################################################################################
###############################################################################################
###############################################################################################
###############################################################################################
###
### Table catalogo_producto UI
###

### Configuring catalogo_producto

# TODO: Change all values of editable, insertable, indexable, etc to predicates
# TODO: in order to be able to give a behaviour dependable of the user, or auth

modconfig['catalogo_producto'] = {
    'internal_fields' : [u'xid'],
    'dict_default': { '' : '<python:locale id="general.submit" />','en_bodega' : '','xid' : '','impuestos' : '','costo' : '','idproducto' : '','idcatalogo' : '',
                     '__objname__': 'catalogos.catalogos',
                     '__tabname__': 'catalogo_producto',
                     #'__fulltextsearch__': '', # disables the fulltext search in the modify frame
                    },
    'editable': lambda: True,
    'insertable': lambda: True,
    'indexable': lambda: True,
    'structured_search': lambda: True,
    'ftindex': ft3k.sqlindex(name='catalogo_producto', conn=dbftindex),
    'ftsearch_size': 10,
    'count': 30,
    'list_fields': ['xid', 'idproducto', 'idcatalogo', 'costo', 'impuestos', 'en_bodega'],
    'preview_fields': ['xid', 'idproducto', 'idcatalogo', 'costo', 'impuestos', 'en_bodega'],
    'list_with_empty_condition': lambda: True,
    'control_generator': _control_generator,
    'preview_maxsize': 64,
    'output_table_header': lambda r: "<tr>%s</tr>"%("<th>%s</th>"%("</th><th>".join(r))),
    'output_table_row': lambda r, q, fields: "<tr>%s</tr>"%("<td>%s</td>"%("</td><td>".join(r))),
    'output_table_control_row': lambda r, controls, q, fields: '<tr><td style="text-align: right;" colspan="%s">%s</td></tr>'%(len(r), " &nbsp; ".join(controls)),
    'output_table': lambda r, query, fields: """<table style="width: 100%%"><tbody>%s</tbody></table>"""%("".join(r)),
    'output_showpage': lambda title, data, id: ('insert',
                                                       {'title': title,
                                                        'data': data,
                                                        'id': id,
                                                        'rowclasses': (['cell1', 'cell1', 'cell2', 'cell2'], 'header')
                                                       }),
    'select': _list_select,
    'foreign_mapper': {u'idproducto': (u'producto', u'xid'), u'idcatalogo': (u'catalogo', u'xid')},
    'form_element': template3k.get_template("""catalogo_producto:form_element""",
                                            """
            <strong>Catalogo Producto</strong>
            <form method="POST" enctype="multipart/form-data" id="form-catalogo_producto<python:keyword id='__id__' />" action="javascript: void(0);" onsubmit="<python:keyword id="__onsubmit__" />" name="catalogo_producto" >
                <table><tbody>
            
<tr><td>
<python:locale id="catalogo_producto.idproducto">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Idproducto'} ]
</python:locale>:</td><td><select onfocus="callto({'rawhandler': createhandlerselect('<python:keyword id="idproducto" />'), 'handlerargs': this},'<python:keyword id="__objname__" />','preview_producto',['xid'])" name="idproducto:int"  id="select-producto_xid_<python:keyword id='__id__' />" ></select>
<a style="text-decoration: none;" href="javascript: DOM3K_replaceListOptions([['','']],'select-producto_xid_<python:keyword id="__id__" />'); void(0);"><python:locale id="actions.clear" /></a><python:keyword id="__editmode__">

/
<a style="text-decoration: none;" href="javascript: showformmodify('<python:keyword id="__objname__" />', 'producto'); void(0);"><python:locale id="actions.new" /></a></python:keyword>

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_producto', ['choose_search','select-producto_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search" /></a><python:keyword id="__fulltextsearch__">

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_fulltext_producto', ['choose_search','select-producto_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search_fulltext" /></a></python:keyword>

</td></tr>


<tr><td>
<python:locale id="catalogo_producto.idcatalogo">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Idcatalogo'} ]
</python:locale>:</td><td><select onfocus="callto({'rawhandler': createhandlerselect('<python:keyword id="idcatalogo" />'), 'handlerargs': this},'<python:keyword id="__objname__" />','preview_catalogo',['xid'])" name="idcatalogo:int"  id="select-catalogo_xid_<python:keyword id='__id__' />" ></select>
<a style="text-decoration: none;" href="javascript: DOM3K_replaceListOptions([['','']],'select-catalogo_xid_<python:keyword id="__id__" />'); void(0);"><python:locale id="actions.clear" /></a><python:keyword id="__editmode__">

/
<a style="text-decoration: none;" href="javascript: showformmodify('<python:keyword id="__objname__" />', 'catalogo'); void(0);"><python:locale id="actions.new" /></a></python:keyword>

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_catalogo', ['choose_search','select-catalogo_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search" /></a><python:keyword id="__fulltextsearch__">

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_fulltext_catalogo', ['choose_search','select-catalogo_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search_fulltext" /></a></python:keyword>

</td></tr>


<tr><td>
<python:locale id="catalogo_producto.costo">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Costo'} ]
</python:locale>:</td><td><input name="costo:float:" type="text" value="<python:keyword id="costo" />" size="40" /></td></tr>


<tr><td>
<python:locale id="catalogo_producto.impuestos">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Impuestos'} ]
</python:locale>:</td><td><input name="impuestos:float:" type="text" value="<python:keyword id="impuestos" />" size="40" /></td></tr>


<tr><td>
<python:locale id="catalogo_producto.en_bodega">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'En Bodega'} ]
</python:locale>:</td><td><input name="en_bodega:float:" type="text" value="<python:keyword id="en_bodega" />" size="40" /></td></tr>

<tr><td><input type="submit" name=":submit" value="<python:locale id="general.submit" />" /></td></tr>
                </tbody></table>
            </form>
            <python:keyword id="__bottom__"></python:keyword>
            """),
    'form_element_search': template3k.get_template("""catalogo_producto:form_element_search""",
                                                   """
            <strong>Catalogo Producto</strong>
            <form method="POST" enctype="multipart/form-data" id="form-catalogo_producto<python:keyword id='__id__' />" action="javascript: void(0);" onsubmit="<python:keyword id="__onsubmit__" />" name="catalogo_producto" >
                <table><tbody>
            
<tr><td>
<python:locale id="catalogo_producto.idproducto">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Idproducto'} ]
</python:locale>:</td><td><select onfocus="callto({'rawhandler': createhandlerselect('<python:keyword id="idproducto" />'), 'handlerargs': this},'<python:keyword id="__objname__" />','preview_producto',['xid'])" name="idproducto:int"  id="select-producto_xid_<python:keyword id='__id__' />" ></select>
<a style="text-decoration: none;" href="javascript: DOM3K_replaceListOptions([['','']],'select-producto_xid_<python:keyword id="__id__" />'); void(0);"><python:locale id="actions.clear" /></a><python:keyword id="__editmode__">

/
<a style="text-decoration: none;" href="javascript: showformmodify('<python:keyword id="__objname__" />', 'producto'); void(0);"><python:locale id="actions.new" /></a></python:keyword>

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_producto', ['choose_search','select-producto_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search" /></a><python:keyword id="__fulltextsearch__">

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_fulltext_producto', ['choose_search','select-producto_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search_fulltext" /></a></python:keyword>

</td></tr>


<tr><td>
<python:locale id="catalogo_producto.idcatalogo">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Idcatalogo'} ]
</python:locale>:</td><td><select onfocus="callto({'rawhandler': createhandlerselect('<python:keyword id="idcatalogo" />'), 'handlerargs': this},'<python:keyword id="__objname__" />','preview_catalogo',['xid'])" name="idcatalogo:int"  id="select-catalogo_xid_<python:keyword id='__id__' />" ></select>
<a style="text-decoration: none;" href="javascript: DOM3K_replaceListOptions([['','']],'select-catalogo_xid_<python:keyword id="__id__" />'); void(0);"><python:locale id="actions.clear" /></a><python:keyword id="__editmode__">

/
<a style="text-decoration: none;" href="javascript: showformmodify('<python:keyword id="__objname__" />', 'catalogo'); void(0);"><python:locale id="actions.new" /></a></python:keyword>

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_catalogo', ['choose_search','select-catalogo_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search" /></a><python:keyword id="__fulltextsearch__">

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_fulltext_catalogo', ['choose_search','select-catalogo_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search_fulltext" /></a></python:keyword>

</td></tr>


<tr><td>
<python:locale id="catalogo_producto.costo">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Costo'} ]
</python:locale>:</td><td><input name="costo:float:" type="text" value="<python:keyword id="costo" />" size="40" /></td></tr>


<tr><td>
<python:locale id="catalogo_producto.impuestos">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Impuestos'} ]
</python:locale>:</td><td><input name="impuestos:float:" type="text" value="<python:keyword id="impuestos" />" size="40" /></td></tr>


<tr><td>
<python:locale id="catalogo_producto.en_bodega">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'En Bodega'} ]
</python:locale>:</td><td><input name="en_bodega:float:" type="text" value="<python:keyword id="en_bodega" />" size="40" /></td></tr>

<tr><td><input type="submit" name=":submit" value="<python:locale id="general.submit" />" /></td></tr>
                </tbody></table>
            </form>
            <python:keyword id="__bottom__"></python:keyword>
            """),
    '__sentinel__': True
    }

### End configuration

###
### Beginning methods/functions

#_spyro_authentication.perms['form_catalogo_producto'] = False
#_spyro_authentication.perms['list_catalogo_producto'] = False
#_spyro_authentication.perms['modify_catalogo_producto'] = False
#_spyro_authentication.perms['delete_catalogo_producto'] = False
 
def list_view_catalogo_producto(*args, **kwargs):
    tabname = 'catalogo_producto'
    config = _get_config(None, tabname)
    kwargs['config'] = config
    return (config['output_showpage'](tabname, _list(tabname, *args, **kwargs), tabname), )

def form_catalogo_producto(mode = None, condition_dict = None, **kwargs):
    tabname = 'catalogo_producto'
    kwargs['config'] = _get_config(None, tabname)
    return _form(tabname, mode, condition_dict, **kwargs)

def modify_catalogo_producto(mode, data, cond, dbconn=None, doreturn=1, **kwargs):
    tabname = 'catalogo_producto'
    config = _get_config(None, tabname)
    assert config['editable'](), "View only entity, can't %s"%mode
    tabname = 'catalogo_producto'
    return _modify(tabname, mode, data, cond, dbconn, doreturn, config=config, **kwargs)

def delete_catalogo_producto(cond, dbconn = None, doreturn = 1, **kwargs):
    tabname = 'catalogo_producto'
    config = _get_config(None, tabname)
    assert config['editable'](), "View only entity, can't delete"
    return _delete(tabname, cond, dbconn, doreturn, config=config, **kwargs)

def list_catalogo_producto(*args, **kwargs):
    tabname = 'catalogo_producto'
    kwargs['config'] = _get_config(None, tabname)
    # assert _spyro_authentication._rawauth(auth,  objname, list_catalogo_producto)
    return _list(tabname, *args, **kwargs)

def form_search_catalogo_producto(method=None, *args, **kwargs):
    tabname = 'catalogo_producto'
    config = _get_config(None, tabname)
    dict_default = dict(config['dict_default']) # dictionary
    dict_default['__editmode__'] = ""
    if method == "choose_search":
        idtag = args[0]
        residtag = 'choose-search-%s'%idtag
        dict_default['__onsubmit__'] = (r"callto({'showresults': function(d){ DOM3K_updateElement(d,'%s'); """
                                        r"""DOM3K_BackgroundRowColors('%s',['rgb(200,200,230);','inherit']);}}, """
                                        r"""'%s','choose_search_%s',null,{'condition_dict': Easyform_formToDict(this), 'idtag': %s });"""
                                        r""" """%(residtag,residtag,objname,tabname,repr(idtag)))
        dict_default['__bottom__'] = """<div id="%s"></div>"""%residtag
    else:
        dict_default['__onsubmit__'] = r"callto({'handler': handle_commands}, '%s','search_%s',[Easyform_formToDict(this), null, 'LIKE']);"%(objname,tabname)
    dict_default['__id__'] = str(time.time())
    x = config['form_element_search'].merge(dict_default)
    return x

def manage_fulltext_catalogo_producto(drop_index, reindex):
    tabname = 'catalogo_producto'
    config = _get_config(None, tabname)
    ftindex = config['ftindex']
    if drop_index:
        ftindex.drop_index()
    if reindex:
        ftindex.create_index()
        for x in _preview(tabname, config['list_fields'][0], config=config, preview_maxsize=512):
            _rowid = _get_rowid(tabname, x)
            ftindex.handle_insert(x[1], _rowid)

def form_search_fulltext_catalogo_producto(output='list', idtag='', query='',**kwargs):
    tabname = 'catalogo_producto'
    dict_default = {'query': query, 'output': output, 'idtag': idtag}
    if output == "choose_search":
        residtag = 'choose-search-%s'%idtag
        dict_default['__onsubmit__'] = (r"""callto({'handler': handle_commands_frames()}, '%s', 'search_fulltext_%s', """
                                        r"""null, Easyform_formToDict(this));"""%(objname, tabname))
        dict_default['__bottom__'] = """<div id="%s"></div>"""%residtag
        dict_default['__idtag__'] = """<input name="idtag:string:mandatory" value="%s" type="hidden" />"""%idtag
        dict_default['residtag'] = residtag
    else:
        dict_default['residtag'] = tabname
        dict_default['__onsubmit__'] = r"callto({'handler': handle_commands}, '%s', 'search_fulltext_%s', null, Easyform_formToDict(this));"%(objname, tabname)
        dict_default['__idtag__'] = ""
        dict_default['__bottom__'] = ""
    dict_default['__id__'] = str(time.time())
    return """<form action="javascript: void(0);" onsubmit="%(__onsubmit__)s">
    <input name="output:string:mandatory" value="%(output)s" type="hidden" />
    <input name="query:string:mandatory" value="%(query)s" size="40" type="text" />
    <input name="residtag:string" value="residtag" type="hidden" />
    %(__idtag__)s
    <input name=":submit" value="Search" type="submit" />
    </form>
    %(__bottom__)s
    """%dict_default

def search_fulltext_catalogo_producto(query, output="", idtag="", residtag="", **kwargs):
    tabname = 'catalogo_producto'
    kwargs['config'] = None
    return _search_fulltext(tabname, query, output, idtag, residtag, **kwargs)


def search_catalogo_producto(cond,orderby=None,usecondmethod=None,**kwargs):
    tabname = 'catalogo_producto'
    config = _get_config(None, tabname)
    x = list_catalogo_producto(cond, orderby, usecondmethod=usecondmethod)
    return (config['output_showpage']('Search results', x, tabname), )

def choose_search_catalogo_producto(idtag, condition_dict = None, method="LIKE", outmethod="simple",
                                                  previewsize=10, list_condition_dict = None, **kwargs):
    tabname = 'catalogo_producto'
    config = _get_config(None, tabname)
    kwargs['config'] = config
    return _choose_search(tabname, idtag, condition_dict, method, outmethod, previewsize, list_condition_dict, **kwargs)

def preview_catalogo_producto(*args, **kwargs):
    tabname = 'catalogo_producto'
    kwargs['config'] = _get_config(None, tabname)
    # kwargs['display'] = 'idproducto'
    return _preview(tabname, *args, **kwargs)

###
### end of catalogo_producto UI
###
###############################################################################################
###############################################################################################
###############################################################################################
###############################################################################################
###
### Table estados_venta UI
###

### Configuring estados_venta

# TODO: Change all values of editable, insertable, indexable, etc to predicates
# TODO: in order to be able to give a behaviour dependable of the user, or auth

modconfig['estados_venta'] = {
    'internal_fields' : [],
    'dict_default': { 'descripcion' : '','nombre' : '','' : '<python:locale id="general.submit" />',
                     '__objname__': 'catalogos.catalogos',
                     '__tabname__': 'estados_venta',
                     #'__fulltextsearch__': '', # disables the fulltext search in the modify frame
                    },
    'editable': lambda: True,
    'insertable': lambda: True,
    'indexable': lambda: True,
    'structured_search': lambda: True,
    'ftindex': ft3k.sqlindex(name='estados_venta', conn=dbftindex),
    'ftsearch_size': 10,
    'count': 30,
    'list_fields': ['nombre', 'descripcion'],
    'preview_fields': ['nombre', 'descripcion'],
    'list_with_empty_condition': lambda: True,
    'control_generator': _control_generator,
    'preview_maxsize': 64,
    'output_table_header': lambda r: "<tr>%s</tr>"%("<th>%s</th>"%("</th><th>".join(r))),
    'output_table_row': lambda r, q, fields: "<tr>%s</tr>"%("<td>%s</td>"%("</td><td>".join(r))),
    'output_table_control_row': lambda r, controls, q, fields: '<tr><td style="text-align: right;" colspan="%s">%s</td></tr>'%(len(r), " &nbsp; ".join(controls)),
    'output_table': lambda r, query, fields: """<table style="width: 100%%"><tbody>%s</tbody></table>"""%("".join(r)),
    'output_showpage': lambda title, data, id: ('insert',
                                                       {'title': title,
                                                        'data': data,
                                                        'id': id,
                                                        'rowclasses': (['cell1', 'cell1', 'cell2', 'cell2'], 'header')
                                                       }),
    'select': _list_select,
    'foreign_mapper': {},
    'form_element': template3k.get_template("""estados_venta:form_element""",
                                            """
            <strong>Estados Venta</strong>
            <form method="POST" enctype="multipart/form-data" id="form-estados_venta<python:keyword id='__id__' />" action="javascript: void(0);" onsubmit="<python:keyword id="__onsubmit__" />" name="estados_venta" >
                <table><tbody>
            
<tr><td>
<python:locale id="estados_venta.nombre">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Nombre'} ]
</python:locale>:</td><td><input name="nombre:string:" type="text" value="<python:keyword id="nombre" />" size="40" /></td></tr>


<tr><td>
<python:locale id="estados_venta.descripcion">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Descripcion'} ]
</python:locale>:</td><td><input name="descripcion:string:" type="text" value="<python:keyword id="descripcion" />" size="40" /></td></tr>

<tr><td><input type="submit" name=":submit" value="<python:locale id="general.submit" />" /></td></tr>
                </tbody></table>
            </form>
            <python:keyword id="__bottom__"></python:keyword>
            """),
    'form_element_search': template3k.get_template("""estados_venta:form_element_search""",
                                                   """
            <strong>Estados Venta</strong>
            <form method="POST" enctype="multipart/form-data" id="form-estados_venta<python:keyword id='__id__' />" action="javascript: void(0);" onsubmit="<python:keyword id="__onsubmit__" />" name="estados_venta" >
                <table><tbody>
            
<tr><td>
<python:locale id="estados_venta.nombre">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Nombre'} ]
</python:locale>:</td><td><input name="nombre:string:" type="text" value="<python:keyword id="nombre" />" size="40" /></td></tr>


<tr><td>
<python:locale id="estados_venta.descripcion">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Descripcion'} ]
</python:locale>:</td><td><input name="descripcion:string:" type="text" value="<python:keyword id="descripcion" />" size="40" /></td></tr>

<tr><td><input type="submit" name=":submit" value="<python:locale id="general.submit" />" /></td></tr>
                </tbody></table>
            </form>
            <python:keyword id="__bottom__"></python:keyword>
            """),
    '__sentinel__': True
    }

### End configuration

###
### Beginning methods/functions

#_spyro_authentication.perms['form_estados_venta'] = False
#_spyro_authentication.perms['list_estados_venta'] = False
#_spyro_authentication.perms['modify_estados_venta'] = False
#_spyro_authentication.perms['delete_estados_venta'] = False
 
def list_view_estados_venta(*args, **kwargs):
    tabname = 'estados_venta'
    config = _get_config(None, tabname)
    kwargs['config'] = config
    return (config['output_showpage'](tabname, _list(tabname, *args, **kwargs), tabname), )

def form_estados_venta(mode = None, condition_dict = None, **kwargs):
    tabname = 'estados_venta'
    kwargs['config'] = _get_config(None, tabname)
    return _form(tabname, mode, condition_dict, **kwargs)

def modify_estados_venta(mode, data, cond, dbconn=None, doreturn=1, **kwargs):
    tabname = 'estados_venta'
    config = _get_config(None, tabname)
    assert config['editable'](), "View only entity, can't %s"%mode
    tabname = 'estados_venta'
    return _modify(tabname, mode, data, cond, dbconn, doreturn, config=config, **kwargs)

def delete_estados_venta(cond, dbconn = None, doreturn = 1, **kwargs):
    tabname = 'estados_venta'
    config = _get_config(None, tabname)
    assert config['editable'](), "View only entity, can't delete"
    return _delete(tabname, cond, dbconn, doreturn, config=config, **kwargs)

def list_estados_venta(*args, **kwargs):
    tabname = 'estados_venta'
    kwargs['config'] = _get_config(None, tabname)
    # assert _spyro_authentication._rawauth(auth,  objname, list_estados_venta)
    return _list(tabname, *args, **kwargs)

def form_search_estados_venta(method=None, *args, **kwargs):
    tabname = 'estados_venta'
    config = _get_config(None, tabname)
    dict_default = dict(config['dict_default']) # dictionary
    dict_default['__editmode__'] = ""
    if method == "choose_search":
        idtag = args[0]
        residtag = 'choose-search-%s'%idtag
        dict_default['__onsubmit__'] = (r"callto({'showresults': function(d){ DOM3K_updateElement(d,'%s'); """
                                        r"""DOM3K_BackgroundRowColors('%s',['rgb(200,200,230);','inherit']);}}, """
                                        r"""'%s','choose_search_%s',null,{'condition_dict': Easyform_formToDict(this), 'idtag': %s });"""
                                        r""" """%(residtag,residtag,objname,tabname,repr(idtag)))
        dict_default['__bottom__'] = """<div id="%s"></div>"""%residtag
    else:
        dict_default['__onsubmit__'] = r"callto({'handler': handle_commands}, '%s','search_%s',[Easyform_formToDict(this), null, 'LIKE']);"%(objname,tabname)
    dict_default['__id__'] = str(time.time())
    x = config['form_element_search'].merge(dict_default)
    return x

def manage_fulltext_estados_venta(drop_index, reindex):
    tabname = 'estados_venta'
    config = _get_config(None, tabname)
    ftindex = config['ftindex']
    if drop_index:
        ftindex.drop_index()
    if reindex:
        ftindex.create_index()
        for x in _preview(tabname, config['list_fields'][0], config=config, preview_maxsize=512):
            _rowid = _get_rowid(tabname, x)
            ftindex.handle_insert(x[1], _rowid)

def form_search_fulltext_estados_venta(output='list', idtag='', query='',**kwargs):
    tabname = 'estados_venta'
    dict_default = {'query': query, 'output': output, 'idtag': idtag}
    if output == "choose_search":
        residtag = 'choose-search-%s'%idtag
        dict_default['__onsubmit__'] = (r"""callto({'handler': handle_commands_frames()}, '%s', 'search_fulltext_%s', """
                                        r"""null, Easyform_formToDict(this));"""%(objname, tabname))
        dict_default['__bottom__'] = """<div id="%s"></div>"""%residtag
        dict_default['__idtag__'] = """<input name="idtag:string:mandatory" value="%s" type="hidden" />"""%idtag
        dict_default['residtag'] = residtag
    else:
        dict_default['residtag'] = tabname
        dict_default['__onsubmit__'] = r"callto({'handler': handle_commands}, '%s', 'search_fulltext_%s', null, Easyform_formToDict(this));"%(objname, tabname)
        dict_default['__idtag__'] = ""
        dict_default['__bottom__'] = ""
    dict_default['__id__'] = str(time.time())
    return """<form action="javascript: void(0);" onsubmit="%(__onsubmit__)s">
    <input name="output:string:mandatory" value="%(output)s" type="hidden" />
    <input name="query:string:mandatory" value="%(query)s" size="40" type="text" />
    <input name="residtag:string" value="residtag" type="hidden" />
    %(__idtag__)s
    <input name=":submit" value="Search" type="submit" />
    </form>
    %(__bottom__)s
    """%dict_default

def search_fulltext_estados_venta(query, output="", idtag="", residtag="", **kwargs):
    tabname = 'estados_venta'
    kwargs['config'] = None
    return _search_fulltext(tabname, query, output, idtag, residtag, **kwargs)


def search_estados_venta(cond,orderby=None,usecondmethod=None,**kwargs):
    tabname = 'estados_venta'
    config = _get_config(None, tabname)
    x = list_estados_venta(cond, orderby, usecondmethod=usecondmethod)
    return (config['output_showpage']('Search results', x, tabname), )

def choose_search_estados_venta(idtag, condition_dict = None, method="LIKE", outmethod="simple",
                                                  previewsize=10, list_condition_dict = None, **kwargs):
    tabname = 'estados_venta'
    config = _get_config(None, tabname)
    kwargs['config'] = config
    return _choose_search(tabname, idtag, condition_dict, method, outmethod, previewsize, list_condition_dict, **kwargs)

def preview_estados_venta(*args, **kwargs):
    tabname = 'estados_venta'
    kwargs['config'] = _get_config(None, tabname)
    # kwargs['display'] = 'descripcion'
    return _preview(tabname, *args, **kwargs)

###
### end of estados_venta UI
###
###############################################################################################
###############################################################################################
###############################################################################################
###############################################################################################
###
### Table estados_fiscales UI
###

### Configuring estados_fiscales

# TODO: Change all values of editable, insertable, indexable, etc to predicates
# TODO: in order to be able to give a behaviour dependable of the user, or auth

modconfig['estados_fiscales'] = {
    'internal_fields' : [],
    'dict_default': { 'descripcion' : '','nombre' : '','' : '<python:locale id="general.submit" />',
                     '__objname__': 'catalogos.catalogos',
                     '__tabname__': 'estados_fiscales',
                     #'__fulltextsearch__': '', # disables the fulltext search in the modify frame
                    },
    'editable': lambda: True,
    'insertable': lambda: True,
    'indexable': lambda: True,
    'structured_search': lambda: True,
    'ftindex': ft3k.sqlindex(name='estados_fiscales', conn=dbftindex),
    'ftsearch_size': 10,
    'count': 30,
    'list_fields': ['nombre', 'descripcion'],
    'preview_fields': ['nombre', 'descripcion'],
    'list_with_empty_condition': lambda: True,
    'control_generator': _control_generator,
    'preview_maxsize': 64,
    'output_table_header': lambda r: "<tr>%s</tr>"%("<th>%s</th>"%("</th><th>".join(r))),
    'output_table_row': lambda r, q, fields: "<tr>%s</tr>"%("<td>%s</td>"%("</td><td>".join(r))),
    'output_table_control_row': lambda r, controls, q, fields: '<tr><td style="text-align: right;" colspan="%s">%s</td></tr>'%(len(r), " &nbsp; ".join(controls)),
    'output_table': lambda r, query, fields: """<table style="width: 100%%"><tbody>%s</tbody></table>"""%("".join(r)),
    'output_showpage': lambda title, data, id: ('insert',
                                                       {'title': title,
                                                        'data': data,
                                                        'id': id,
                                                        'rowclasses': (['cell1', 'cell1', 'cell2', 'cell2'], 'header')
                                                       }),
    'select': _list_select,
    'foreign_mapper': {},
    'form_element': template3k.get_template("""estados_fiscales:form_element""",
                                            """
            <strong>Estados Fiscales</strong>
            <form method="POST" enctype="multipart/form-data" id="form-estados_fiscales<python:keyword id='__id__' />" action="javascript: void(0);" onsubmit="<python:keyword id="__onsubmit__" />" name="estados_fiscales" >
                <table><tbody>
            
<tr><td>
<python:locale id="estados_fiscales.nombre">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Nombre'} ]
</python:locale>:</td><td><input name="nombre:string:" type="text" value="<python:keyword id="nombre" />" size="40" /></td></tr>


<tr><td>
<python:locale id="estados_fiscales.descripcion">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Descripcion'} ]
</python:locale>:</td><td><input name="descripcion:string:" type="text" value="<python:keyword id="descripcion" />" size="40" /></td></tr>

<tr><td><input type="submit" name=":submit" value="<python:locale id="general.submit" />" /></td></tr>
                </tbody></table>
            </form>
            <python:keyword id="__bottom__"></python:keyword>
            """),
    'form_element_search': template3k.get_template("""estados_fiscales:form_element_search""",
                                                   """
            <strong>Estados Fiscales</strong>
            <form method="POST" enctype="multipart/form-data" id="form-estados_fiscales<python:keyword id='__id__' />" action="javascript: void(0);" onsubmit="<python:keyword id="__onsubmit__" />" name="estados_fiscales" >
                <table><tbody>
            
<tr><td>
<python:locale id="estados_fiscales.nombre">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Nombre'} ]
</python:locale>:</td><td><input name="nombre:string:" type="text" value="<python:keyword id="nombre" />" size="40" /></td></tr>


<tr><td>
<python:locale id="estados_fiscales.descripcion">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Descripcion'} ]
</python:locale>:</td><td><input name="descripcion:string:" type="text" value="<python:keyword id="descripcion" />" size="40" /></td></tr>

<tr><td><input type="submit" name=":submit" value="<python:locale id="general.submit" />" /></td></tr>
                </tbody></table>
            </form>
            <python:keyword id="__bottom__"></python:keyword>
            """),
    '__sentinel__': True
    }

### End configuration

###
### Beginning methods/functions

#_spyro_authentication.perms['form_estados_fiscales'] = False
#_spyro_authentication.perms['list_estados_fiscales'] = False
#_spyro_authentication.perms['modify_estados_fiscales'] = False
#_spyro_authentication.perms['delete_estados_fiscales'] = False
 
def list_view_estados_fiscales(*args, **kwargs):
    tabname = 'estados_fiscales'
    config = _get_config(None, tabname)
    kwargs['config'] = config
    return (config['output_showpage'](tabname, _list(tabname, *args, **kwargs), tabname), )

def form_estados_fiscales(mode = None, condition_dict = None, **kwargs):
    tabname = 'estados_fiscales'
    kwargs['config'] = _get_config(None, tabname)
    return _form(tabname, mode, condition_dict, **kwargs)

def modify_estados_fiscales(mode, data, cond, dbconn=None, doreturn=1, **kwargs):
    tabname = 'estados_fiscales'
    config = _get_config(None, tabname)
    assert config['editable'](), "View only entity, can't %s"%mode
    tabname = 'estados_fiscales'
    return _modify(tabname, mode, data, cond, dbconn, doreturn, config=config, **kwargs)

def delete_estados_fiscales(cond, dbconn = None, doreturn = 1, **kwargs):
    tabname = 'estados_fiscales'
    config = _get_config(None, tabname)
    assert config['editable'](), "View only entity, can't delete"
    return _delete(tabname, cond, dbconn, doreturn, config=config, **kwargs)

def list_estados_fiscales(*args, **kwargs):
    tabname = 'estados_fiscales'
    kwargs['config'] = _get_config(None, tabname)
    # assert _spyro_authentication._rawauth(auth,  objname, list_estados_fiscales)
    return _list(tabname, *args, **kwargs)

def form_search_estados_fiscales(method=None, *args, **kwargs):
    tabname = 'estados_fiscales'
    config = _get_config(None, tabname)
    dict_default = dict(config['dict_default']) # dictionary
    dict_default['__editmode__'] = ""
    if method == "choose_search":
        idtag = args[0]
        residtag = 'choose-search-%s'%idtag
        dict_default['__onsubmit__'] = (r"callto({'showresults': function(d){ DOM3K_updateElement(d,'%s'); """
                                        r"""DOM3K_BackgroundRowColors('%s',['rgb(200,200,230);','inherit']);}}, """
                                        r"""'%s','choose_search_%s',null,{'condition_dict': Easyform_formToDict(this), 'idtag': %s });"""
                                        r""" """%(residtag,residtag,objname,tabname,repr(idtag)))
        dict_default['__bottom__'] = """<div id="%s"></div>"""%residtag
    else:
        dict_default['__onsubmit__'] = r"callto({'handler': handle_commands}, '%s','search_%s',[Easyform_formToDict(this), null, 'LIKE']);"%(objname,tabname)
    dict_default['__id__'] = str(time.time())
    x = config['form_element_search'].merge(dict_default)
    return x

def manage_fulltext_estados_fiscales(drop_index, reindex):
    tabname = 'estados_fiscales'
    config = _get_config(None, tabname)
    ftindex = config['ftindex']
    if drop_index:
        ftindex.drop_index()
    if reindex:
        ftindex.create_index()
        for x in _preview(tabname, config['list_fields'][0], config=config, preview_maxsize=512):
            _rowid = _get_rowid(tabname, x)
            ftindex.handle_insert(x[1], _rowid)

def form_search_fulltext_estados_fiscales(output='list', idtag='', query='',**kwargs):
    tabname = 'estados_fiscales'
    dict_default = {'query': query, 'output': output, 'idtag': idtag}
    if output == "choose_search":
        residtag = 'choose-search-%s'%idtag
        dict_default['__onsubmit__'] = (r"""callto({'handler': handle_commands_frames()}, '%s', 'search_fulltext_%s', """
                                        r"""null, Easyform_formToDict(this));"""%(objname, tabname))
        dict_default['__bottom__'] = """<div id="%s"></div>"""%residtag
        dict_default['__idtag__'] = """<input name="idtag:string:mandatory" value="%s" type="hidden" />"""%idtag
        dict_default['residtag'] = residtag
    else:
        dict_default['residtag'] = tabname
        dict_default['__onsubmit__'] = r"callto({'handler': handle_commands}, '%s', 'search_fulltext_%s', null, Easyform_formToDict(this));"%(objname, tabname)
        dict_default['__idtag__'] = ""
        dict_default['__bottom__'] = ""
    dict_default['__id__'] = str(time.time())
    return """<form action="javascript: void(0);" onsubmit="%(__onsubmit__)s">
    <input name="output:string:mandatory" value="%(output)s" type="hidden" />
    <input name="query:string:mandatory" value="%(query)s" size="40" type="text" />
    <input name="residtag:string" value="residtag" type="hidden" />
    %(__idtag__)s
    <input name=":submit" value="Search" type="submit" />
    </form>
    %(__bottom__)s
    """%dict_default

def search_fulltext_estados_fiscales(query, output="", idtag="", residtag="", **kwargs):
    tabname = 'estados_fiscales'
    kwargs['config'] = None
    return _search_fulltext(tabname, query, output, idtag, residtag, **kwargs)


def search_estados_fiscales(cond,orderby=None,usecondmethod=None,**kwargs):
    tabname = 'estados_fiscales'
    config = _get_config(None, tabname)
    x = list_estados_fiscales(cond, orderby, usecondmethod=usecondmethod)
    return (config['output_showpage']('Search results', x, tabname), )

def choose_search_estados_fiscales(idtag, condition_dict = None, method="LIKE", outmethod="simple",
                                                  previewsize=10, list_condition_dict = None, **kwargs):
    tabname = 'estados_fiscales'
    config = _get_config(None, tabname)
    kwargs['config'] = config
    return _choose_search(tabname, idtag, condition_dict, method, outmethod, previewsize, list_condition_dict, **kwargs)

def preview_estados_fiscales(*args, **kwargs):
    tabname = 'estados_fiscales'
    kwargs['config'] = _get_config(None, tabname)
    # kwargs['display'] = 'descripcion'
    return _preview(tabname, *args, **kwargs)

###
### end of estados_fiscales UI
###
###############################################################################################
###############################################################################################
###############################################################################################
###############################################################################################
###
### Table datos_facturacion UI
###

### Configuring datos_facturacion

# TODO: Change all values of editable, insertable, indexable, etc to predicates
# TODO: in order to be able to give a behaviour dependable of the user, or auth

modconfig['datos_facturacion'] = {
    'internal_fields' : [u'xid'],
    'dict_default': { '' : '<python:locale id="general.submit" />','folio' : '','xid' : '','razon_social' : '','direccion' : '','rfc' : '',
                     '__objname__': 'catalogos.catalogos',
                     '__tabname__': 'datos_facturacion',
                     #'__fulltextsearch__': '', # disables the fulltext search in the modify frame
                    },
    'editable': lambda: True,
    'insertable': lambda: True,
    'indexable': lambda: True,
    'structured_search': lambda: True,
    'ftindex': ft3k.sqlindex(name='datos_facturacion', conn=dbftindex),
    'ftsearch_size': 10,
    'count': 30,
    'list_fields': ['xid', 'razon_social', 'rfc', 'folio', 'direccion'],
    'preview_fields': ['xid', 'razon_social', 'rfc', 'folio', 'direccion'],
    'list_with_empty_condition': lambda: True,
    'control_generator': _control_generator,
    'preview_maxsize': 64,
    'output_table_header': lambda r: "<tr>%s</tr>"%("<th>%s</th>"%("</th><th>".join(r))),
    'output_table_row': lambda r, q, fields: "<tr>%s</tr>"%("<td>%s</td>"%("</td><td>".join(r))),
    'output_table_control_row': lambda r, controls, q, fields: '<tr><td style="text-align: right;" colspan="%s">%s</td></tr>'%(len(r), " &nbsp; ".join(controls)),
    'output_table': lambda r, query, fields: """<table style="width: 100%%"><tbody>%s</tbody></table>"""%("".join(r)),
    'output_showpage': lambda title, data, id: ('insert',
                                                       {'title': title,
                                                        'data': data,
                                                        'id': id,
                                                        'rowclasses': (['cell1', 'cell1', 'cell2', 'cell2'], 'header')
                                                       }),
    'select': _list_select,
    'foreign_mapper': {},
    'form_element': template3k.get_template("""datos_facturacion:form_element""",
                                            """
            <strong>Datos Facturacion</strong>
            <form method="POST" enctype="multipart/form-data" id="form-datos_facturacion<python:keyword id='__id__' />" action="javascript: void(0);" onsubmit="<python:keyword id="__onsubmit__" />" name="datos_facturacion" >
                <table><tbody>
            
<tr><td>
<python:locale id="datos_facturacion.razon_social">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Razon Social'} ]
</python:locale>:</td><td><input name="razon_social:string:" type="text" value="<python:keyword id="razon_social" />" size="40" /></td></tr>


<tr><td>
<python:locale id="datos_facturacion.rfc">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Rfc'} ]
</python:locale>:</td><td><input name="rfc:string:" type="text" value="<python:keyword id="rfc" />" size="40" /></td></tr>


<tr><td>
<python:locale id="datos_facturacion.folio">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Folio'} ]
</python:locale>:</td><td><input name="folio:string:" type="text" value="<python:keyword id="folio" />" size="40" /></td></tr>


<tr><td>
<python:locale id="datos_facturacion.direccion">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Direccion'} ]
</python:locale>:</td><td><input name="direccion:string:" type="text" value="<python:keyword id="direccion" />" size="40" /></td></tr>

<tr><td><input type="submit" name=":submit" value="<python:locale id="general.submit" />" /></td></tr>
                </tbody></table>
            </form>
            <python:keyword id="__bottom__"></python:keyword>
            """),
    'form_element_search': template3k.get_template("""datos_facturacion:form_element_search""",
                                                   """
            <strong>Datos Facturacion</strong>
            <form method="POST" enctype="multipart/form-data" id="form-datos_facturacion<python:keyword id='__id__' />" action="javascript: void(0);" onsubmit="<python:keyword id="__onsubmit__" />" name="datos_facturacion" >
                <table><tbody>
            
<tr><td>
<python:locale id="datos_facturacion.razon_social">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Razon Social'} ]
</python:locale>:</td><td><input name="razon_social:string:" type="text" value="<python:keyword id="razon_social" />" size="40" /></td></tr>


<tr><td>
<python:locale id="datos_facturacion.rfc">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Rfc'} ]
</python:locale>:</td><td><input name="rfc:string:" type="text" value="<python:keyword id="rfc" />" size="40" /></td></tr>


<tr><td>
<python:locale id="datos_facturacion.folio">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Folio'} ]
</python:locale>:</td><td><input name="folio:string:" type="text" value="<python:keyword id="folio" />" size="40" /></td></tr>


<tr><td>
<python:locale id="datos_facturacion.direccion">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Direccion'} ]
</python:locale>:</td><td><input name="direccion:string:" type="text" value="<python:keyword id="direccion" />" size="40" /></td></tr>

<tr><td><input type="submit" name=":submit" value="<python:locale id="general.submit" />" /></td></tr>
                </tbody></table>
            </form>
            <python:keyword id="__bottom__"></python:keyword>
            """),
    '__sentinel__': True
    }

### End configuration

###
### Beginning methods/functions

#_spyro_authentication.perms['form_datos_facturacion'] = False
#_spyro_authentication.perms['list_datos_facturacion'] = False
#_spyro_authentication.perms['modify_datos_facturacion'] = False
#_spyro_authentication.perms['delete_datos_facturacion'] = False
 
def list_view_datos_facturacion(*args, **kwargs):
    tabname = 'datos_facturacion'
    config = _get_config(None, tabname)
    kwargs['config'] = config
    return (config['output_showpage'](tabname, _list(tabname, *args, **kwargs), tabname), )

def form_datos_facturacion(mode = None, condition_dict = None, **kwargs):
    tabname = 'datos_facturacion'
    kwargs['config'] = _get_config(None, tabname)
    return _form(tabname, mode, condition_dict, **kwargs)

def modify_datos_facturacion(mode, data, cond, dbconn=None, doreturn=1, **kwargs):
    tabname = 'datos_facturacion'
    config = _get_config(None, tabname)
    assert config['editable'](), "View only entity, can't %s"%mode
    tabname = 'datos_facturacion'
    return _modify(tabname, mode, data, cond, dbconn, doreturn, config=config, **kwargs)

def delete_datos_facturacion(cond, dbconn = None, doreturn = 1, **kwargs):
    tabname = 'datos_facturacion'
    config = _get_config(None, tabname)
    assert config['editable'](), "View only entity, can't delete"
    return _delete(tabname, cond, dbconn, doreturn, config=config, **kwargs)

def list_datos_facturacion(*args, **kwargs):
    tabname = 'datos_facturacion'
    kwargs['config'] = _get_config(None, tabname)
    # assert _spyro_authentication._rawauth(auth,  objname, list_datos_facturacion)
    return _list(tabname, *args, **kwargs)

def form_search_datos_facturacion(method=None, *args, **kwargs):
    tabname = 'datos_facturacion'
    config = _get_config(None, tabname)
    dict_default = dict(config['dict_default']) # dictionary
    dict_default['__editmode__'] = ""
    if method == "choose_search":
        idtag = args[0]
        residtag = 'choose-search-%s'%idtag
        dict_default['__onsubmit__'] = (r"callto({'showresults': function(d){ DOM3K_updateElement(d,'%s'); """
                                        r"""DOM3K_BackgroundRowColors('%s',['rgb(200,200,230);','inherit']);}}, """
                                        r"""'%s','choose_search_%s',null,{'condition_dict': Easyform_formToDict(this), 'idtag': %s });"""
                                        r""" """%(residtag,residtag,objname,tabname,repr(idtag)))
        dict_default['__bottom__'] = """<div id="%s"></div>"""%residtag
    else:
        dict_default['__onsubmit__'] = r"callto({'handler': handle_commands}, '%s','search_%s',[Easyform_formToDict(this), null, 'LIKE']);"%(objname,tabname)
    dict_default['__id__'] = str(time.time())
    x = config['form_element_search'].merge(dict_default)
    return x

def manage_fulltext_datos_facturacion(drop_index, reindex):
    tabname = 'datos_facturacion'
    config = _get_config(None, tabname)
    ftindex = config['ftindex']
    if drop_index:
        ftindex.drop_index()
    if reindex:
        ftindex.create_index()
        for x in _preview(tabname, config['list_fields'][0], config=config, preview_maxsize=512):
            _rowid = _get_rowid(tabname, x)
            ftindex.handle_insert(x[1], _rowid)

def form_search_fulltext_datos_facturacion(output='list', idtag='', query='',**kwargs):
    tabname = 'datos_facturacion'
    dict_default = {'query': query, 'output': output, 'idtag': idtag}
    if output == "choose_search":
        residtag = 'choose-search-%s'%idtag
        dict_default['__onsubmit__'] = (r"""callto({'handler': handle_commands_frames()}, '%s', 'search_fulltext_%s', """
                                        r"""null, Easyform_formToDict(this));"""%(objname, tabname))
        dict_default['__bottom__'] = """<div id="%s"></div>"""%residtag
        dict_default['__idtag__'] = """<input name="idtag:string:mandatory" value="%s" type="hidden" />"""%idtag
        dict_default['residtag'] = residtag
    else:
        dict_default['residtag'] = tabname
        dict_default['__onsubmit__'] = r"callto({'handler': handle_commands}, '%s', 'search_fulltext_%s', null, Easyform_formToDict(this));"%(objname, tabname)
        dict_default['__idtag__'] = ""
        dict_default['__bottom__'] = ""
    dict_default['__id__'] = str(time.time())
    return """<form action="javascript: void(0);" onsubmit="%(__onsubmit__)s">
    <input name="output:string:mandatory" value="%(output)s" type="hidden" />
    <input name="query:string:mandatory" value="%(query)s" size="40" type="text" />
    <input name="residtag:string" value="residtag" type="hidden" />
    %(__idtag__)s
    <input name=":submit" value="Search" type="submit" />
    </form>
    %(__bottom__)s
    """%dict_default

def search_fulltext_datos_facturacion(query, output="", idtag="", residtag="", **kwargs):
    tabname = 'datos_facturacion'
    kwargs['config'] = None
    return _search_fulltext(tabname, query, output, idtag, residtag, **kwargs)


def search_datos_facturacion(cond,orderby=None,usecondmethod=None,**kwargs):
    tabname = 'datos_facturacion'
    config = _get_config(None, tabname)
    x = list_datos_facturacion(cond, orderby, usecondmethod=usecondmethod)
    return (config['output_showpage']('Search results', x, tabname), )

def choose_search_datos_facturacion(idtag, condition_dict = None, method="LIKE", outmethod="simple",
                                                  previewsize=10, list_condition_dict = None, **kwargs):
    tabname = 'datos_facturacion'
    config = _get_config(None, tabname)
    kwargs['config'] = config
    return _choose_search(tabname, idtag, condition_dict, method, outmethod, previewsize, list_condition_dict, **kwargs)

def preview_datos_facturacion(*args, **kwargs):
    tabname = 'datos_facturacion'
    kwargs['config'] = _get_config(None, tabname)
    # kwargs['display'] = 'razon_social'
    return _preview(tabname, *args, **kwargs)

###
### end of datos_facturacion UI
###
###############################################################################################
###############################################################################################
###############################################################################################
###############################################################################################
###
### Table venta UI
###

### Configuring venta

# TODO: Change all values of editable, insertable, indexable, etc to predicates
# TODO: in order to be able to give a behaviour dependable of the user, or auth

modconfig['venta'] = {
    'internal_fields' : [u'xid'],
    'dict_default': { '' : '<python:locale id="general.submit" />','startdate' : '','personaid' : '','xid' : '','estado_venta' : '','datos_factura' : '','vendedorid' : '','estado_fiscal' : '',
                     '__objname__': 'catalogos.catalogos',
                     '__tabname__': 'venta',
                     #'__fulltextsearch__': '', # disables the fulltext search in the modify frame
                    },
    'editable': lambda: True,
    'insertable': lambda: True,
    'indexable': lambda: True,
    'structured_search': lambda: True,
    'ftindex': ft3k.sqlindex(name='venta', conn=dbftindex),
    'ftsearch_size': 10,
    'count': 30,
    'list_fields': ['xid', 'personaid', 'vendedorid', 'startdate', 'estado_venta', 'estado_fiscal', 'datos_factura'],
    'preview_fields': ['xid', 'personaid', 'vendedorid', 'startdate', 'estado_venta', 'estado_fiscal', 'datos_factura'],
    'list_with_empty_condition': lambda: True,
    'control_generator': _control_generator,
    'preview_maxsize': 64,
    'output_table_header': lambda r: "<tr>%s</tr>"%("<th>%s</th>"%("</th><th>".join(r))),
    'output_table_row': lambda r, q, fields: "<tr>%s</tr>"%("<td>%s</td>"%("</td><td>".join(r))),
    'output_table_control_row': lambda r, controls, q, fields: '<tr><td style="text-align: right;" colspan="%s">%s</td></tr>'%(len(r), " &nbsp; ".join(controls)),
    'output_table': lambda r, query, fields: """<table style="width: 100%%"><tbody>%s</tbody></table>"""%("".join(r)),
    'output_showpage': lambda title, data, id: ('insert',
                                                       {'title': title,
                                                        'data': data,
                                                        'id': id,
                                                        'rowclasses': (['cell1', 'cell1', 'cell2', 'cell2'], 'header')
                                                       }),
    'select': _list_select,
    'foreign_mapper': {u'vendedorid': (u'persona', u'xid'), u'datos_factura': (u'datos_facturacion', u'xid'), u'estado_fiscal': (u'estados_fiscales', u'nombre'), u'personaid': (u'persona', u'xid'), u'estado_venta': (u'estados_venta', u'nombre')},
    'form_element': template3k.get_template("""venta:form_element""",
                                            """
            <strong>Venta</strong>
            <form method="POST" enctype="multipart/form-data" id="form-venta<python:keyword id='__id__' />" action="javascript: void(0);" onsubmit="<python:keyword id="__onsubmit__" />" name="venta" >
                <table><tbody>
            
<tr><td>
<python:locale id="venta.personaid">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Personaid'} ]
</python:locale>:</td><td><select onfocus="callto({'rawhandler': createhandlerselect('<python:keyword id="personaid" />'), 'handlerargs': this},'<python:keyword id="__objname__" />','preview_persona',['xid'])" name="personaid:int"  id="select-persona_xid_<python:keyword id='__id__' />" ></select>
<a style="text-decoration: none;" href="javascript: DOM3K_replaceListOptions([['','']],'select-persona_xid_<python:keyword id="__id__" />'); void(0);"><python:locale id="actions.clear" /></a><python:keyword id="__editmode__">

/
<a style="text-decoration: none;" href="javascript: showformmodify('<python:keyword id="__objname__" />', 'persona'); void(0);"><python:locale id="actions.new" /></a></python:keyword>

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_persona', ['choose_search','select-persona_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search" /></a><python:keyword id="__fulltextsearch__">

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_fulltext_persona', ['choose_search','select-persona_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search_fulltext" /></a></python:keyword>

</td></tr>


<tr><td>
<python:locale id="venta.vendedorid">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Vendedorid'} ]
</python:locale>:</td><td><select onfocus="callto({'rawhandler': createhandlerselect('<python:keyword id="vendedorid" />'), 'handlerargs': this},'<python:keyword id="__objname__" />','preview_persona',['xid'])" name="vendedorid:int"  id="select-persona_xid_<python:keyword id='__id__' />" ></select>
<a style="text-decoration: none;" href="javascript: DOM3K_replaceListOptions([['','']],'select-persona_xid_<python:keyword id="__id__" />'); void(0);"><python:locale id="actions.clear" /></a><python:keyword id="__editmode__">

/
<a style="text-decoration: none;" href="javascript: showformmodify('<python:keyword id="__objname__" />', 'persona'); void(0);"><python:locale id="actions.new" /></a></python:keyword>

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_persona', ['choose_search','select-persona_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search" /></a><python:keyword id="__fulltextsearch__">

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_fulltext_persona', ['choose_search','select-persona_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search_fulltext" /></a></python:keyword>

</td></tr>


<tr><td>
<python:locale id="venta.startdate">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Startdate'} ]
</python:locale>:</td><td>
            <python:locale id="date.format_date" /> 
            <python:locale id="date.format_time" /> 
            <input type="text" name="startdate:datetime:" value="<python:keyword id="startdate" />" onclick="Easyform_frameReadDatetime(DOM3K_readInput(this), DOM3K_updateInput, this, event.clientY, event.clientX)"  />
            </td></tr>


<tr><td>
<python:locale id="venta.estado_venta">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Estado Venta'} ]
</python:locale>:</td><td><select onfocus="callto({'rawhandler': createhandlerselect('<python:keyword id="estado_venta" />'), 'handlerargs': this},'<python:keyword id="__objname__" />','preview_estados_venta',['nombre'])" name="estado_venta:string"  id="select-estados_venta_nombre_<python:keyword id='__id__' />" ></select>
<a style="text-decoration: none;" href="javascript: DOM3K_replaceListOptions([['','']],'select-estados_venta_nombre_<python:keyword id="__id__" />'); void(0);"><python:locale id="actions.clear" /></a><python:keyword id="__editmode__">

/
<a style="text-decoration: none;" href="javascript: showformmodify('<python:keyword id="__objname__" />', 'estados_venta'); void(0);"><python:locale id="actions.new" /></a></python:keyword>

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_estados_venta', ['choose_search','select-estados_venta_nombre_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search" /></a><python:keyword id="__fulltextsearch__">

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_fulltext_estados_venta', ['choose_search','select-estados_venta_nombre_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search_fulltext" /></a></python:keyword>

</td></tr>


<tr><td>
<python:locale id="venta.estado_fiscal">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Estado Fiscal'} ]
</python:locale>:</td><td><select onfocus="callto({'rawhandler': createhandlerselect('<python:keyword id="estado_fiscal" />'), 'handlerargs': this},'<python:keyword id="__objname__" />','preview_estados_fiscales',['nombre'])" name="estado_fiscal:string"  id="select-estados_fiscales_nombre_<python:keyword id='__id__' />" ></select>
<a style="text-decoration: none;" href="javascript: DOM3K_replaceListOptions([['','']],'select-estados_fiscales_nombre_<python:keyword id="__id__" />'); void(0);"><python:locale id="actions.clear" /></a><python:keyword id="__editmode__">

/
<a style="text-decoration: none;" href="javascript: showformmodify('<python:keyword id="__objname__" />', 'estados_fiscales'); void(0);"><python:locale id="actions.new" /></a></python:keyword>

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_estados_fiscales', ['choose_search','select-estados_fiscales_nombre_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search" /></a><python:keyword id="__fulltextsearch__">

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_fulltext_estados_fiscales', ['choose_search','select-estados_fiscales_nombre_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search_fulltext" /></a></python:keyword>

</td></tr>


<tr><td>
<python:locale id="venta.datos_factura">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Datos Factura'} ]
</python:locale>:</td><td><select onfocus="callto({'rawhandler': createhandlerselect('<python:keyword id="datos_factura" />'), 'handlerargs': this},'<python:keyword id="__objname__" />','preview_datos_facturacion',['xid'])" name="datos_factura:int"  id="select-datos_facturacion_xid_<python:keyword id='__id__' />" ></select>
<a style="text-decoration: none;" href="javascript: DOM3K_replaceListOptions([['','']],'select-datos_facturacion_xid_<python:keyword id="__id__" />'); void(0);"><python:locale id="actions.clear" /></a><python:keyword id="__editmode__">

/
<a style="text-decoration: none;" href="javascript: showformmodify('<python:keyword id="__objname__" />', 'datos_facturacion'); void(0);"><python:locale id="actions.new" /></a></python:keyword>

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_datos_facturacion', ['choose_search','select-datos_facturacion_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search" /></a><python:keyword id="__fulltextsearch__">

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_fulltext_datos_facturacion', ['choose_search','select-datos_facturacion_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search_fulltext" /></a></python:keyword>

</td></tr>

<tr><td><input type="submit" name=":submit" value="<python:locale id="general.submit" />" /></td></tr>
                </tbody></table>
            </form>
            <python:keyword id="__bottom__"></python:keyword>
            """),
    'form_element_search': template3k.get_template("""venta:form_element_search""",
                                                   """
            <strong>Venta</strong>
            <form method="POST" enctype="multipart/form-data" id="form-venta<python:keyword id='__id__' />" action="javascript: void(0);" onsubmit="<python:keyword id="__onsubmit__" />" name="venta" >
                <table><tbody>
            
<tr><td>
<python:locale id="venta.personaid">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Personaid'} ]
</python:locale>:</td><td><select onfocus="callto({'rawhandler': createhandlerselect('<python:keyword id="personaid" />'), 'handlerargs': this},'<python:keyword id="__objname__" />','preview_persona',['xid'])" name="personaid:int"  id="select-persona_xid_<python:keyword id='__id__' />" ></select>
<a style="text-decoration: none;" href="javascript: DOM3K_replaceListOptions([['','']],'select-persona_xid_<python:keyword id="__id__" />'); void(0);"><python:locale id="actions.clear" /></a><python:keyword id="__editmode__">

/
<a style="text-decoration: none;" href="javascript: showformmodify('<python:keyword id="__objname__" />', 'persona'); void(0);"><python:locale id="actions.new" /></a></python:keyword>

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_persona', ['choose_search','select-persona_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search" /></a><python:keyword id="__fulltextsearch__">

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_fulltext_persona', ['choose_search','select-persona_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search_fulltext" /></a></python:keyword>

</td></tr>


<tr><td>
<python:locale id="venta.vendedorid">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Vendedorid'} ]
</python:locale>:</td><td><select onfocus="callto({'rawhandler': createhandlerselect('<python:keyword id="vendedorid" />'), 'handlerargs': this},'<python:keyword id="__objname__" />','preview_persona',['xid'])" name="vendedorid:int"  id="select-persona_xid_<python:keyword id='__id__' />" ></select>
<a style="text-decoration: none;" href="javascript: DOM3K_replaceListOptions([['','']],'select-persona_xid_<python:keyword id="__id__" />'); void(0);"><python:locale id="actions.clear" /></a><python:keyword id="__editmode__">

/
<a style="text-decoration: none;" href="javascript: showformmodify('<python:keyword id="__objname__" />', 'persona'); void(0);"><python:locale id="actions.new" /></a></python:keyword>

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_persona', ['choose_search','select-persona_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search" /></a><python:keyword id="__fulltextsearch__">

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_fulltext_persona', ['choose_search','select-persona_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search_fulltext" /></a></python:keyword>

</td></tr>


<tr><td>
<python:locale id="venta.startdate">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Startdate'} ]
</python:locale>:</td><td>
            <python:locale id="date.format_date" /> 
            <python:locale id="date.format_time" /> 
            <input type="text" name="startdate:datetime:" value="<python:keyword id="startdate" />" onclick="Easyform_frameReadDatetime(DOM3K_readInput(this), DOM3K_updateInput, this, event.clientY, event.clientX)"  />
            </td></tr>


<tr><td>
<python:locale id="venta.estado_venta">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Estado Venta'} ]
</python:locale>:</td><td><select onfocus="callto({'rawhandler': createhandlerselect('<python:keyword id="estado_venta" />'), 'handlerargs': this},'<python:keyword id="__objname__" />','preview_estados_venta',['nombre'])" name="estado_venta:string"  id="select-estados_venta_nombre_<python:keyword id='__id__' />" ></select>
<a style="text-decoration: none;" href="javascript: DOM3K_replaceListOptions([['','']],'select-estados_venta_nombre_<python:keyword id="__id__" />'); void(0);"><python:locale id="actions.clear" /></a><python:keyword id="__editmode__">

/
<a style="text-decoration: none;" href="javascript: showformmodify('<python:keyword id="__objname__" />', 'estados_venta'); void(0);"><python:locale id="actions.new" /></a></python:keyword>

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_estados_venta', ['choose_search','select-estados_venta_nombre_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search" /></a><python:keyword id="__fulltextsearch__">

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_fulltext_estados_venta', ['choose_search','select-estados_venta_nombre_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search_fulltext" /></a></python:keyword>

</td></tr>


<tr><td>
<python:locale id="venta.estado_fiscal">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Estado Fiscal'} ]
</python:locale>:</td><td><select onfocus="callto({'rawhandler': createhandlerselect('<python:keyword id="estado_fiscal" />'), 'handlerargs': this},'<python:keyword id="__objname__" />','preview_estados_fiscales',['nombre'])" name="estado_fiscal:string"  id="select-estados_fiscales_nombre_<python:keyword id='__id__' />" ></select>
<a style="text-decoration: none;" href="javascript: DOM3K_replaceListOptions([['','']],'select-estados_fiscales_nombre_<python:keyword id="__id__" />'); void(0);"><python:locale id="actions.clear" /></a><python:keyword id="__editmode__">

/
<a style="text-decoration: none;" href="javascript: showformmodify('<python:keyword id="__objname__" />', 'estados_fiscales'); void(0);"><python:locale id="actions.new" /></a></python:keyword>

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_estados_fiscales', ['choose_search','select-estados_fiscales_nombre_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search" /></a><python:keyword id="__fulltextsearch__">

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_fulltext_estados_fiscales', ['choose_search','select-estados_fiscales_nombre_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search_fulltext" /></a></python:keyword>

</td></tr>


<tr><td>
<python:locale id="venta.datos_factura">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Datos Factura'} ]
</python:locale>:</td><td><select onfocus="callto({'rawhandler': createhandlerselect('<python:keyword id="datos_factura" />'), 'handlerargs': this},'<python:keyword id="__objname__" />','preview_datos_facturacion',['xid'])" name="datos_factura:int"  id="select-datos_facturacion_xid_<python:keyword id='__id__' />" ></select>
<a style="text-decoration: none;" href="javascript: DOM3K_replaceListOptions([['','']],'select-datos_facturacion_xid_<python:keyword id="__id__" />'); void(0);"><python:locale id="actions.clear" /></a><python:keyword id="__editmode__">

/
<a style="text-decoration: none;" href="javascript: showformmodify('<python:keyword id="__objname__" />', 'datos_facturacion'); void(0);"><python:locale id="actions.new" /></a></python:keyword>

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_datos_facturacion', ['choose_search','select-datos_facturacion_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search" /></a><python:keyword id="__fulltextsearch__">

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_fulltext_datos_facturacion', ['choose_search','select-datos_facturacion_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search_fulltext" /></a></python:keyword>

</td></tr>

<tr><td><input type="submit" name=":submit" value="<python:locale id="general.submit" />" /></td></tr>
                </tbody></table>
            </form>
            <python:keyword id="__bottom__"></python:keyword>
            """),
    '__sentinel__': True
    }

### End configuration

###
### Beginning methods/functions

#_spyro_authentication.perms['form_venta'] = False
#_spyro_authentication.perms['list_venta'] = False
#_spyro_authentication.perms['modify_venta'] = False
#_spyro_authentication.perms['delete_venta'] = False
 
def list_view_venta(*args, **kwargs):
    tabname = 'venta'
    config = _get_config(None, tabname)
    kwargs['config'] = config
    return (config['output_showpage'](tabname, _list(tabname, *args, **kwargs), tabname), )

def form_venta(mode = None, condition_dict = None, **kwargs):
    tabname = 'venta'
    kwargs['config'] = _get_config(None, tabname)
    return _form(tabname, mode, condition_dict, **kwargs)

def modify_venta(mode, data, cond, dbconn=None, doreturn=1, **kwargs):
    tabname = 'venta'
    config = _get_config(None, tabname)
    assert config['editable'](), "View only entity, can't %s"%mode
    tabname = 'venta'
    return _modify(tabname, mode, data, cond, dbconn, doreturn, config=config, **kwargs)

def delete_venta(cond, dbconn = None, doreturn = 1, **kwargs):
    tabname = 'venta'
    config = _get_config(None, tabname)
    assert config['editable'](), "View only entity, can't delete"
    return _delete(tabname, cond, dbconn, doreturn, config=config, **kwargs)

def list_venta(*args, **kwargs):
    tabname = 'venta'
    kwargs['config'] = _get_config(None, tabname)
    # assert _spyro_authentication._rawauth(auth,  objname, list_venta)
    return _list(tabname, *args, **kwargs)

def form_search_venta(method=None, *args, **kwargs):
    tabname = 'venta'
    config = _get_config(None, tabname)
    dict_default = dict(config['dict_default']) # dictionary
    dict_default['__editmode__'] = ""
    if method == "choose_search":
        idtag = args[0]
        residtag = 'choose-search-%s'%idtag
        dict_default['__onsubmit__'] = (r"callto({'showresults': function(d){ DOM3K_updateElement(d,'%s'); """
                                        r"""DOM3K_BackgroundRowColors('%s',['rgb(200,200,230);','inherit']);}}, """
                                        r"""'%s','choose_search_%s',null,{'condition_dict': Easyform_formToDict(this), 'idtag': %s });"""
                                        r""" """%(residtag,residtag,objname,tabname,repr(idtag)))
        dict_default['__bottom__'] = """<div id="%s"></div>"""%residtag
    else:
        dict_default['__onsubmit__'] = r"callto({'handler': handle_commands}, '%s','search_%s',[Easyform_formToDict(this), null, 'LIKE']);"%(objname,tabname)
    dict_default['__id__'] = str(time.time())
    x = config['form_element_search'].merge(dict_default)
    return x

def manage_fulltext_venta(drop_index, reindex):
    tabname = 'venta'
    config = _get_config(None, tabname)
    ftindex = config['ftindex']
    if drop_index:
        ftindex.drop_index()
    if reindex:
        ftindex.create_index()
        for x in _preview(tabname, config['list_fields'][0], config=config, preview_maxsize=512):
            _rowid = _get_rowid(tabname, x)
            ftindex.handle_insert(x[1], _rowid)

def form_search_fulltext_venta(output='list', idtag='', query='',**kwargs):
    tabname = 'venta'
    dict_default = {'query': query, 'output': output, 'idtag': idtag}
    if output == "choose_search":
        residtag = 'choose-search-%s'%idtag
        dict_default['__onsubmit__'] = (r"""callto({'handler': handle_commands_frames()}, '%s', 'search_fulltext_%s', """
                                        r"""null, Easyform_formToDict(this));"""%(objname, tabname))
        dict_default['__bottom__'] = """<div id="%s"></div>"""%residtag
        dict_default['__idtag__'] = """<input name="idtag:string:mandatory" value="%s" type="hidden" />"""%idtag
        dict_default['residtag'] = residtag
    else:
        dict_default['residtag'] = tabname
        dict_default['__onsubmit__'] = r"callto({'handler': handle_commands}, '%s', 'search_fulltext_%s', null, Easyform_formToDict(this));"%(objname, tabname)
        dict_default['__idtag__'] = ""
        dict_default['__bottom__'] = ""
    dict_default['__id__'] = str(time.time())
    return """<form action="javascript: void(0);" onsubmit="%(__onsubmit__)s">
    <input name="output:string:mandatory" value="%(output)s" type="hidden" />
    <input name="query:string:mandatory" value="%(query)s" size="40" type="text" />
    <input name="residtag:string" value="residtag" type="hidden" />
    %(__idtag__)s
    <input name=":submit" value="Search" type="submit" />
    </form>
    %(__bottom__)s
    """%dict_default

def search_fulltext_venta(query, output="", idtag="", residtag="", **kwargs):
    tabname = 'venta'
    kwargs['config'] = None
    return _search_fulltext(tabname, query, output, idtag, residtag, **kwargs)


def search_venta(cond,orderby=None,usecondmethod=None,**kwargs):
    tabname = 'venta'
    config = _get_config(None, tabname)
    x = list_venta(cond, orderby, usecondmethod=usecondmethod)
    return (config['output_showpage']('Search results', x, tabname), )

def choose_search_venta(idtag, condition_dict = None, method="LIKE", outmethod="simple",
                                                  previewsize=10, list_condition_dict = None, **kwargs):
    tabname = 'venta'
    config = _get_config(None, tabname)
    kwargs['config'] = config
    return _choose_search(tabname, idtag, condition_dict, method, outmethod, previewsize, list_condition_dict, **kwargs)

def preview_venta(*args, **kwargs):
    tabname = 'venta'
    kwargs['config'] = _get_config(None, tabname)
    # kwargs['display'] = 'personaid'
    return _preview(tabname, *args, **kwargs)

###
### end of venta UI
###
###############################################################################################
###############################################################################################
###############################################################################################
###############################################################################################
###
### Table venta_productos UI
###

### Configuring venta_productos

# TODO: Change all values of editable, insertable, indexable, etc to predicates
# TODO: in order to be able to give a behaviour dependable of the user, or auth

modconfig['venta_productos'] = {
    'internal_fields' : [],
    'dict_default': { 'idventa' : '','idproducto' : '','' : '<python:locale id="general.submit" />','cantidad' : '','costo_final' : '',
                     '__objname__': 'catalogos.catalogos',
                     '__tabname__': 'venta_productos',
                     #'__fulltextsearch__': '', # disables the fulltext search in the modify frame
                    },
    'editable': lambda: True,
    'insertable': lambda: True,
    'indexable': lambda: True,
    'structured_search': lambda: True,
    'ftindex': ft3k.sqlindex(name='venta_productos', conn=dbftindex),
    'ftsearch_size': 10,
    'count': 30,
    'list_fields': ['idventa', 'idproducto', 'cantidad', 'costo_final'],
    'preview_fields': ['idventa', 'idproducto', 'cantidad', 'costo_final'],
    'list_with_empty_condition': lambda: True,
    'control_generator': _control_generator,
    'preview_maxsize': 64,
    'output_table_header': lambda r: "<tr>%s</tr>"%("<th>%s</th>"%("</th><th>".join(r))),
    'output_table_row': lambda r, q, fields: "<tr>%s</tr>"%("<td>%s</td>"%("</td><td>".join(r))),
    'output_table_control_row': lambda r, controls, q, fields: '<tr><td style="text-align: right;" colspan="%s">%s</td></tr>'%(len(r), " &nbsp; ".join(controls)),
    'output_table': lambda r, query, fields: """<table style="width: 100%%"><tbody>%s</tbody></table>"""%("".join(r)),
    'output_showpage': lambda title, data, id: ('insert',
                                                       {'title': title,
                                                        'data': data,
                                                        'id': id,
                                                        'rowclasses': (['cell1', 'cell1', 'cell2', 'cell2'], 'header')
                                                       }),
    'select': _list_select,
    'foreign_mapper': {u'idventa': (u'venta', u'xid'), u'idproducto': (u'catalogo_producto', u'xid')},
    'form_element': template3k.get_template("""venta_productos:form_element""",
                                            """
            <strong>Venta Productos</strong>
            <form method="POST" enctype="multipart/form-data" id="form-venta_productos<python:keyword id='__id__' />" action="javascript: void(0);" onsubmit="<python:keyword id="__onsubmit__" />" name="venta_productos" >
                <table><tbody>
            
<tr><td>
<python:locale id="venta_productos.idventa">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Idventa'} ]
</python:locale>:</td><td><select onfocus="callto({'rawhandler': createhandlerselect('<python:keyword id="idventa" />'), 'handlerargs': this},'<python:keyword id="__objname__" />','preview_venta',['xid'])" name="idventa:int"  id="select-venta_xid_<python:keyword id='__id__' />" ></select>
<a style="text-decoration: none;" href="javascript: DOM3K_replaceListOptions([['','']],'select-venta_xid_<python:keyword id="__id__" />'); void(0);"><python:locale id="actions.clear" /></a><python:keyword id="__editmode__">

/
<a style="text-decoration: none;" href="javascript: showformmodify('<python:keyword id="__objname__" />', 'venta'); void(0);"><python:locale id="actions.new" /></a></python:keyword>

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_venta', ['choose_search','select-venta_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search" /></a><python:keyword id="__fulltextsearch__">

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_fulltext_venta', ['choose_search','select-venta_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search_fulltext" /></a></python:keyword>

</td></tr>


<tr><td>
<python:locale id="venta_productos.idproducto">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Idproducto'} ]
</python:locale>:</td><td><select onfocus="callto({'rawhandler': createhandlerselect('<python:keyword id="idproducto" />'), 'handlerargs': this},'<python:keyword id="__objname__" />','preview_catalogo_producto',['xid'])" name="idproducto:int"  id="select-catalogo_producto_xid_<python:keyword id='__id__' />" ></select>
<a style="text-decoration: none;" href="javascript: DOM3K_replaceListOptions([['','']],'select-catalogo_producto_xid_<python:keyword id="__id__" />'); void(0);"><python:locale id="actions.clear" /></a><python:keyword id="__editmode__">

/
<a style="text-decoration: none;" href="javascript: showformmodify('<python:keyword id="__objname__" />', 'catalogo_producto'); void(0);"><python:locale id="actions.new" /></a></python:keyword>

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_catalogo_producto', ['choose_search','select-catalogo_producto_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search" /></a><python:keyword id="__fulltextsearch__">

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_fulltext_catalogo_producto', ['choose_search','select-catalogo_producto_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search_fulltext" /></a></python:keyword>

</td></tr>


<tr><td>
<python:locale id="venta_productos.cantidad">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Cantidad'} ]
</python:locale>:</td><td><input name="cantidad:float:" type="text" value="<python:keyword id="cantidad" />" size="40" /></td></tr>


<tr><td>
<python:locale id="venta_productos.costo_final">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Costo Final'} ]
</python:locale>:</td><td><input name="costo_final:float:" type="text" value="<python:keyword id="costo_final" />" size="40" /></td></tr>

<tr><td><input type="submit" name=":submit" value="<python:locale id="general.submit" />" /></td></tr>
                </tbody></table>
            </form>
            <python:keyword id="__bottom__"></python:keyword>
            """),
    'form_element_search': template3k.get_template("""venta_productos:form_element_search""",
                                                   """
            <strong>Venta Productos</strong>
            <form method="POST" enctype="multipart/form-data" id="form-venta_productos<python:keyword id='__id__' />" action="javascript: void(0);" onsubmit="<python:keyword id="__onsubmit__" />" name="venta_productos" >
                <table><tbody>
            
<tr><td>
<python:locale id="venta_productos.idventa">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Idventa'} ]
</python:locale>:</td><td><select onfocus="callto({'rawhandler': createhandlerselect('<python:keyword id="idventa" />'), 'handlerargs': this},'<python:keyword id="__objname__" />','preview_venta',['xid'])" name="idventa:int"  id="select-venta_xid_<python:keyword id='__id__' />" ></select>
<a style="text-decoration: none;" href="javascript: DOM3K_replaceListOptions([['','']],'select-venta_xid_<python:keyword id="__id__" />'); void(0);"><python:locale id="actions.clear" /></a><python:keyword id="__editmode__">

/
<a style="text-decoration: none;" href="javascript: showformmodify('<python:keyword id="__objname__" />', 'venta'); void(0);"><python:locale id="actions.new" /></a></python:keyword>

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_venta', ['choose_search','select-venta_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search" /></a><python:keyword id="__fulltextsearch__">

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_fulltext_venta', ['choose_search','select-venta_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search_fulltext" /></a></python:keyword>

</td></tr>


<tr><td>
<python:locale id="venta_productos.idproducto">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Idproducto'} ]
</python:locale>:</td><td><select onfocus="callto({'rawhandler': createhandlerselect('<python:keyword id="idproducto" />'), 'handlerargs': this},'<python:keyword id="__objname__" />','preview_catalogo_producto',['xid'])" name="idproducto:int"  id="select-catalogo_producto_xid_<python:keyword id='__id__' />" ></select>
<a style="text-decoration: none;" href="javascript: DOM3K_replaceListOptions([['','']],'select-catalogo_producto_xid_<python:keyword id="__id__" />'); void(0);"><python:locale id="actions.clear" /></a><python:keyword id="__editmode__">

/
<a style="text-decoration: none;" href="javascript: showformmodify('<python:keyword id="__objname__" />', 'catalogo_producto'); void(0);"><python:locale id="actions.new" /></a></python:keyword>

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_catalogo_producto', ['choose_search','select-catalogo_producto_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search" /></a><python:keyword id="__fulltextsearch__">

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_fulltext_catalogo_producto', ['choose_search','select-catalogo_producto_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search_fulltext" /></a></python:keyword>

</td></tr>


<tr><td>
<python:locale id="venta_productos.cantidad">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Cantidad'} ]
</python:locale>:</td><td><input name="cantidad:float:" type="text" value="<python:keyword id="cantidad" />" size="40" /></td></tr>


<tr><td>
<python:locale id="venta_productos.costo_final">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Costo Final'} ]
</python:locale>:</td><td><input name="costo_final:float:" type="text" value="<python:keyword id="costo_final" />" size="40" /></td></tr>

<tr><td><input type="submit" name=":submit" value="<python:locale id="general.submit" />" /></td></tr>
                </tbody></table>
            </form>
            <python:keyword id="__bottom__"></python:keyword>
            """),
    '__sentinel__': True
    }

### End configuration

###
### Beginning methods/functions

#_spyro_authentication.perms['form_venta_productos'] = False
#_spyro_authentication.perms['list_venta_productos'] = False
#_spyro_authentication.perms['modify_venta_productos'] = False
#_spyro_authentication.perms['delete_venta_productos'] = False
 
def list_view_venta_productos(*args, **kwargs):
    tabname = 'venta_productos'
    config = _get_config(None, tabname)
    kwargs['config'] = config
    return (config['output_showpage'](tabname, _list(tabname, *args, **kwargs), tabname), )

def form_venta_productos(mode = None, condition_dict = None, **kwargs):
    tabname = 'venta_productos'
    kwargs['config'] = _get_config(None, tabname)
    return _form(tabname, mode, condition_dict, **kwargs)

def modify_venta_productos(mode, data, cond, dbconn=None, doreturn=1, **kwargs):
    tabname = 'venta_productos'
    config = _get_config(None, tabname)
    assert config['editable'](), "View only entity, can't %s"%mode
    tabname = 'venta_productos'
    return _modify(tabname, mode, data, cond, dbconn, doreturn, config=config, **kwargs)

def delete_venta_productos(cond, dbconn = None, doreturn = 1, **kwargs):
    tabname = 'venta_productos'
    config = _get_config(None, tabname)
    assert config['editable'](), "View only entity, can't delete"
    return _delete(tabname, cond, dbconn, doreturn, config=config, **kwargs)

def list_venta_productos(*args, **kwargs):
    tabname = 'venta_productos'
    kwargs['config'] = _get_config(None, tabname)
    # assert _spyro_authentication._rawauth(auth,  objname, list_venta_productos)
    return _list(tabname, *args, **kwargs)

def form_search_venta_productos(method=None, *args, **kwargs):
    tabname = 'venta_productos'
    config = _get_config(None, tabname)
    dict_default = dict(config['dict_default']) # dictionary
    dict_default['__editmode__'] = ""
    if method == "choose_search":
        idtag = args[0]
        residtag = 'choose-search-%s'%idtag
        dict_default['__onsubmit__'] = (r"callto({'showresults': function(d){ DOM3K_updateElement(d,'%s'); """
                                        r"""DOM3K_BackgroundRowColors('%s',['rgb(200,200,230);','inherit']);}}, """
                                        r"""'%s','choose_search_%s',null,{'condition_dict': Easyform_formToDict(this), 'idtag': %s });"""
                                        r""" """%(residtag,residtag,objname,tabname,repr(idtag)))
        dict_default['__bottom__'] = """<div id="%s"></div>"""%residtag
    else:
        dict_default['__onsubmit__'] = r"callto({'handler': handle_commands}, '%s','search_%s',[Easyform_formToDict(this), null, 'LIKE']);"%(objname,tabname)
    dict_default['__id__'] = str(time.time())
    x = config['form_element_search'].merge(dict_default)
    return x

def manage_fulltext_venta_productos(drop_index, reindex):
    tabname = 'venta_productos'
    config = _get_config(None, tabname)
    ftindex = config['ftindex']
    if drop_index:
        ftindex.drop_index()
    if reindex:
        ftindex.create_index()
        for x in _preview(tabname, config['list_fields'][0], config=config, preview_maxsize=512):
            _rowid = _get_rowid(tabname, x)
            ftindex.handle_insert(x[1], _rowid)

def form_search_fulltext_venta_productos(output='list', idtag='', query='',**kwargs):
    tabname = 'venta_productos'
    dict_default = {'query': query, 'output': output, 'idtag': idtag}
    if output == "choose_search":
        residtag = 'choose-search-%s'%idtag
        dict_default['__onsubmit__'] = (r"""callto({'handler': handle_commands_frames()}, '%s', 'search_fulltext_%s', """
                                        r"""null, Easyform_formToDict(this));"""%(objname, tabname))
        dict_default['__bottom__'] = """<div id="%s"></div>"""%residtag
        dict_default['__idtag__'] = """<input name="idtag:string:mandatory" value="%s" type="hidden" />"""%idtag
        dict_default['residtag'] = residtag
    else:
        dict_default['residtag'] = tabname
        dict_default['__onsubmit__'] = r"callto({'handler': handle_commands}, '%s', 'search_fulltext_%s', null, Easyform_formToDict(this));"%(objname, tabname)
        dict_default['__idtag__'] = ""
        dict_default['__bottom__'] = ""
    dict_default['__id__'] = str(time.time())
    return """<form action="javascript: void(0);" onsubmit="%(__onsubmit__)s">
    <input name="output:string:mandatory" value="%(output)s" type="hidden" />
    <input name="query:string:mandatory" value="%(query)s" size="40" type="text" />
    <input name="residtag:string" value="residtag" type="hidden" />
    %(__idtag__)s
    <input name=":submit" value="Search" type="submit" />
    </form>
    %(__bottom__)s
    """%dict_default

def search_fulltext_venta_productos(query, output="", idtag="", residtag="", **kwargs):
    tabname = 'venta_productos'
    kwargs['config'] = None
    return _search_fulltext(tabname, query, output, idtag, residtag, **kwargs)


def search_venta_productos(cond,orderby=None,usecondmethod=None,**kwargs):
    tabname = 'venta_productos'
    config = _get_config(None, tabname)
    x = list_venta_productos(cond, orderby, usecondmethod=usecondmethod)
    return (config['output_showpage']('Search results', x, tabname), )

def choose_search_venta_productos(idtag, condition_dict = None, method="LIKE", outmethod="simple",
                                                  previewsize=10, list_condition_dict = None, **kwargs):
    tabname = 'venta_productos'
    config = _get_config(None, tabname)
    kwargs['config'] = config
    return _choose_search(tabname, idtag, condition_dict, method, outmethod, previewsize, list_condition_dict, **kwargs)

def preview_venta_productos(*args, **kwargs):
    tabname = 'venta_productos'
    kwargs['config'] = _get_config(None, tabname)
    # kwargs['display'] = 'idproducto'
    return _preview(tabname, *args, **kwargs)

###
### end of venta_productos UI
###
###############################################################################################
###############################################################################################
###############################################################################################
###############################################################################################
###
### Table comentario UI
###

### Configuring comentario

# TODO: Change all values of editable, insertable, indexable, etc to predicates
# TODO: in order to be able to give a behaviour dependable of the user, or auth

modconfig['comentario'] = {
    'internal_fields' : [u'xid'],
    'dict_default': { '' : '<python:locale id="general.submit" />','xid' : '','fecha_publicacion' : '','receptor' : '','nombre' : '','remitente' : '','comentario' : '',
                     '__objname__': 'catalogos.catalogos',
                     '__tabname__': 'comentario',
                     #'__fulltextsearch__': '', # disables the fulltext search in the modify frame
                    },
    'editable': lambda: True,
    'insertable': lambda: True,
    'indexable': lambda: True,
    'structured_search': lambda: True,
    'ftindex': ft3k.sqlindex(name='comentario', conn=dbftindex),
    'ftsearch_size': 10,
    'count': 30,
    'list_fields': ['xid', 'nombre', 'comentario', 'fecha_publicacion', 'remitente', 'receptor'],
    'preview_fields': ['xid', 'nombre', 'comentario', 'fecha_publicacion', 'remitente', 'receptor'],
    'list_with_empty_condition': lambda: True,
    'control_generator': _control_generator,
    'preview_maxsize': 64,
    'output_table_header': lambda r: "<tr>%s</tr>"%("<th>%s</th>"%("</th><th>".join(r))),
    'output_table_row': lambda r, q, fields: "<tr>%s</tr>"%("<td>%s</td>"%("</td><td>".join(r))),
    'output_table_control_row': lambda r, controls, q, fields: '<tr><td style="text-align: right;" colspan="%s">%s</td></tr>'%(len(r), " &nbsp; ".join(controls)),
    'output_table': lambda r, query, fields: """<table style="width: 100%%"><tbody>%s</tbody></table>"""%("".join(r)),
    'output_showpage': lambda title, data, id: ('insert',
                                                       {'title': title,
                                                        'data': data,
                                                        'id': id,
                                                        'rowclasses': (['cell1', 'cell1', 'cell2', 'cell2'], 'header')
                                                       }),
    'select': _list_select,
    'foreign_mapper': {u'remitente': (u'persona', u'xid'), u'receptor': (u'persona', u'xid')},
    'form_element': template3k.get_template("""comentario:form_element""",
                                            """
            <strong>Comentario</strong>
            <form method="POST" enctype="multipart/form-data" id="form-comentario<python:keyword id='__id__' />" action="javascript: void(0);" onsubmit="<python:keyword id="__onsubmit__" />" name="comentario" >
                <table><tbody>
            
<tr><td>
<python:locale id="comentario.nombre">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Nombre'} ]
</python:locale>:</td><td><input name="nombre:string:mandatory" type="text" value="<python:keyword id="nombre" />" size="40" /></td></tr>


<tr><td>
<python:locale id="comentario.comentario">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Comentario'} ]
</python:locale>:</td><td><textarea cols="50" rows="12" name="comentario:text:mandatory" ><python:keyword id="comentario" /></textarea></td></tr>


<tr><td>
<python:locale id="comentario.fecha_publicacion">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Fecha Publicacion'} ]
</python:locale>:</td><td>
            <python:locale id="date.format_date" /> 
            <python:locale id="date.format_time" /> 
            <input type="text" name="fecha_publicacion:datetime:mandatory" value="<python:keyword id="fecha_publicacion" />" onclick="Easyform_frameReadDatetime(DOM3K_readInput(this), DOM3K_updateInput, this, event.clientY, event.clientX)"  />
            </td></tr>


<tr><td>
<python:locale id="comentario.remitente">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Remitente'} ]
</python:locale>:</td><td><select onfocus="callto({'rawhandler': createhandlerselect('<python:keyword id="remitente" />'), 'handlerargs': this},'<python:keyword id="__objname__" />','preview_persona',['xid'])" name="remitente:int"  id="select-persona_xid_<python:keyword id='__id__' />" ></select>
<a style="text-decoration: none;" href="javascript: DOM3K_replaceListOptions([['','']],'select-persona_xid_<python:keyword id="__id__" />'); void(0);"><python:locale id="actions.clear" /></a><python:keyword id="__editmode__">

/
<a style="text-decoration: none;" href="javascript: showformmodify('<python:keyword id="__objname__" />', 'persona'); void(0);"><python:locale id="actions.new" /></a></python:keyword>

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_persona', ['choose_search','select-persona_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search" /></a><python:keyword id="__fulltextsearch__">

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_fulltext_persona', ['choose_search','select-persona_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search_fulltext" /></a></python:keyword>

</td></tr>


<tr><td>
<python:locale id="comentario.receptor">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Receptor'} ]
</python:locale>:</td><td><select onfocus="callto({'rawhandler': createhandlerselect('<python:keyword id="receptor" />'), 'handlerargs': this},'<python:keyword id="__objname__" />','preview_persona',['xid'])" name="receptor:int"  id="select-persona_xid_<python:keyword id='__id__' />" ></select>
<a style="text-decoration: none;" href="javascript: DOM3K_replaceListOptions([['','']],'select-persona_xid_<python:keyword id="__id__" />'); void(0);"><python:locale id="actions.clear" /></a><python:keyword id="__editmode__">

/
<a style="text-decoration: none;" href="javascript: showformmodify('<python:keyword id="__objname__" />', 'persona'); void(0);"><python:locale id="actions.new" /></a></python:keyword>

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_persona', ['choose_search','select-persona_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search" /></a><python:keyword id="__fulltextsearch__">

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_fulltext_persona', ['choose_search','select-persona_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search_fulltext" /></a></python:keyword>

</td></tr>

<tr><td><input type="submit" name=":submit" value="<python:locale id="general.submit" />" /></td></tr>
                </tbody></table>
            </form>
            <python:keyword id="__bottom__"></python:keyword>
            """),
    'form_element_search': template3k.get_template("""comentario:form_element_search""",
                                                   """
            <strong>Comentario</strong>
            <form method="POST" enctype="multipart/form-data" id="form-comentario<python:keyword id='__id__' />" action="javascript: void(0);" onsubmit="<python:keyword id="__onsubmit__" />" name="comentario" >
                <table><tbody>
            
<tr><td>
<python:locale id="comentario.nombre">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Nombre'} ]
</python:locale>:</td><td><input name="nombre:string" type="text" value="<python:keyword id="nombre" />" size="40" /></td></tr>


<tr><td>
<python:locale id="comentario.comentario">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Comentario'} ]
</python:locale>:</td><td><textarea cols="50" rows="12" name="comentario:text" ><python:keyword id="comentario" /></textarea></td></tr>


<tr><td>
<python:locale id="comentario.fecha_publicacion">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Fecha Publicacion'} ]
</python:locale>:</td><td>
            <python:locale id="date.format_date" /> 
            <python:locale id="date.format_time" /> 
            <input type="text" name="fecha_publicacion:datetime" value="<python:keyword id="fecha_publicacion" />" onclick="Easyform_frameReadDatetime(DOM3K_readInput(this), DOM3K_updateInput, this, event.clientY, event.clientX)"  />
            </td></tr>


<tr><td>
<python:locale id="comentario.remitente">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Remitente'} ]
</python:locale>:</td><td><select onfocus="callto({'rawhandler': createhandlerselect('<python:keyword id="remitente" />'), 'handlerargs': this},'<python:keyword id="__objname__" />','preview_persona',['xid'])" name="remitente:int"  id="select-persona_xid_<python:keyword id='__id__' />" ></select>
<a style="text-decoration: none;" href="javascript: DOM3K_replaceListOptions([['','']],'select-persona_xid_<python:keyword id="__id__" />'); void(0);"><python:locale id="actions.clear" /></a><python:keyword id="__editmode__">

/
<a style="text-decoration: none;" href="javascript: showformmodify('<python:keyword id="__objname__" />', 'persona'); void(0);"><python:locale id="actions.new" /></a></python:keyword>

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_persona', ['choose_search','select-persona_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search" /></a><python:keyword id="__fulltextsearch__">

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_fulltext_persona', ['choose_search','select-persona_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search_fulltext" /></a></python:keyword>

</td></tr>


<tr><td>
<python:locale id="comentario.receptor">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Receptor'} ]
</python:locale>:</td><td><select onfocus="callto({'rawhandler': createhandlerselect('<python:keyword id="receptor" />'), 'handlerargs': this},'<python:keyword id="__objname__" />','preview_persona',['xid'])" name="receptor:int"  id="select-persona_xid_<python:keyword id='__id__' />" ></select>
<a style="text-decoration: none;" href="javascript: DOM3K_replaceListOptions([['','']],'select-persona_xid_<python:keyword id="__id__" />'); void(0);"><python:locale id="actions.clear" /></a><python:keyword id="__editmode__">

/
<a style="text-decoration: none;" href="javascript: showformmodify('<python:keyword id="__objname__" />', 'persona'); void(0);"><python:locale id="actions.new" /></a></python:keyword>

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_persona', ['choose_search','select-persona_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search" /></a><python:keyword id="__fulltextsearch__">

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_fulltext_persona', ['choose_search','select-persona_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search_fulltext" /></a></python:keyword>

</td></tr>

<tr><td><input type="submit" name=":submit" value="<python:locale id="general.submit" />" /></td></tr>
                </tbody></table>
            </form>
            <python:keyword id="__bottom__"></python:keyword>
            """),
    '__sentinel__': True
    }

### End configuration

###
### Beginning methods/functions

#_spyro_authentication.perms['form_comentario'] = False
#_spyro_authentication.perms['list_comentario'] = False
#_spyro_authentication.perms['modify_comentario'] = False
#_spyro_authentication.perms['delete_comentario'] = False
 
def list_view_comentario(*args, **kwargs):
    tabname = 'comentario'
    config = _get_config(None, tabname)
    kwargs['config'] = config
    return (config['output_showpage'](tabname, _list(tabname, *args, **kwargs), tabname), )

def form_comentario(mode = None, condition_dict = None, **kwargs):
    tabname = 'comentario'
    kwargs['config'] = _get_config(None, tabname)
    return _form(tabname, mode, condition_dict, **kwargs)

def modify_comentario(mode, data, cond, dbconn=None, doreturn=1, **kwargs):
    tabname = 'comentario'
    config = _get_config(None, tabname)
    assert config['editable'](), "View only entity, can't %s"%mode
    tabname = 'comentario'
    return _modify(tabname, mode, data, cond, dbconn, doreturn, config=config, **kwargs)

def delete_comentario(cond, dbconn = None, doreturn = 1, **kwargs):
    tabname = 'comentario'
    config = _get_config(None, tabname)
    assert config['editable'](), "View only entity, can't delete"
    return _delete(tabname, cond, dbconn, doreturn, config=config, **kwargs)

def list_comentario(*args, **kwargs):
    tabname = 'comentario'
    kwargs['config'] = _get_config(None, tabname)
    # assert _spyro_authentication._rawauth(auth,  objname, list_comentario)
    return _list(tabname, *args, **kwargs)

def form_search_comentario(method=None, *args, **kwargs):
    tabname = 'comentario'
    config = _get_config(None, tabname)
    dict_default = dict(config['dict_default']) # dictionary
    dict_default['__editmode__'] = ""
    if method == "choose_search":
        idtag = args[0]
        residtag = 'choose-search-%s'%idtag
        dict_default['__onsubmit__'] = (r"callto({'showresults': function(d){ DOM3K_updateElement(d,'%s'); """
                                        r"""DOM3K_BackgroundRowColors('%s',['rgb(200,200,230);','inherit']);}}, """
                                        r"""'%s','choose_search_%s',null,{'condition_dict': Easyform_formToDict(this), 'idtag': %s });"""
                                        r""" """%(residtag,residtag,objname,tabname,repr(idtag)))
        dict_default['__bottom__'] = """<div id="%s"></div>"""%residtag
    else:
        dict_default['__onsubmit__'] = r"callto({'handler': handle_commands}, '%s','search_%s',[Easyform_formToDict(this), null, 'LIKE']);"%(objname,tabname)
    dict_default['__id__'] = str(time.time())
    x = config['form_element_search'].merge(dict_default)
    return x

def manage_fulltext_comentario(drop_index, reindex):
    tabname = 'comentario'
    config = _get_config(None, tabname)
    ftindex = config['ftindex']
    if drop_index:
        ftindex.drop_index()
    if reindex:
        ftindex.create_index()
        for x in _preview(tabname, config['list_fields'][0], config=config, preview_maxsize=512):
            _rowid = _get_rowid(tabname, x)
            ftindex.handle_insert(x[1], _rowid)

def form_search_fulltext_comentario(output='list', idtag='', query='',**kwargs):
    tabname = 'comentario'
    dict_default = {'query': query, 'output': output, 'idtag': idtag}
    if output == "choose_search":
        residtag = 'choose-search-%s'%idtag
        dict_default['__onsubmit__'] = (r"""callto({'handler': handle_commands_frames()}, '%s', 'search_fulltext_%s', """
                                        r"""null, Easyform_formToDict(this));"""%(objname, tabname))
        dict_default['__bottom__'] = """<div id="%s"></div>"""%residtag
        dict_default['__idtag__'] = """<input name="idtag:string:mandatory" value="%s" type="hidden" />"""%idtag
        dict_default['residtag'] = residtag
    else:
        dict_default['residtag'] = tabname
        dict_default['__onsubmit__'] = r"callto({'handler': handle_commands}, '%s', 'search_fulltext_%s', null, Easyform_formToDict(this));"%(objname, tabname)
        dict_default['__idtag__'] = ""
        dict_default['__bottom__'] = ""
    dict_default['__id__'] = str(time.time())
    return """<form action="javascript: void(0);" onsubmit="%(__onsubmit__)s">
    <input name="output:string:mandatory" value="%(output)s" type="hidden" />
    <input name="query:string:mandatory" value="%(query)s" size="40" type="text" />
    <input name="residtag:string" value="residtag" type="hidden" />
    %(__idtag__)s
    <input name=":submit" value="Search" type="submit" />
    </form>
    %(__bottom__)s
    """%dict_default

def search_fulltext_comentario(query, output="", idtag="", residtag="", **kwargs):
    tabname = 'comentario'
    kwargs['config'] = None
    return _search_fulltext(tabname, query, output, idtag, residtag, **kwargs)


def search_comentario(cond,orderby=None,usecondmethod=None,**kwargs):
    tabname = 'comentario'
    config = _get_config(None, tabname)
    x = list_comentario(cond, orderby, usecondmethod=usecondmethod)
    return (config['output_showpage']('Search results', x, tabname), )

def choose_search_comentario(idtag, condition_dict = None, method="LIKE", outmethod="simple",
                                                  previewsize=10, list_condition_dict = None, **kwargs):
    tabname = 'comentario'
    config = _get_config(None, tabname)
    kwargs['config'] = config
    return _choose_search(tabname, idtag, condition_dict, method, outmethod, previewsize, list_condition_dict, **kwargs)

def preview_comentario(*args, **kwargs):
    tabname = 'comentario'
    kwargs['config'] = _get_config(None, tabname)
    # kwargs['display'] = 'nombre'
    return _preview(tabname, *args, **kwargs)

###
### end of comentario UI
###
###############################################################################################
###############################################################################################
###############################################################################################
###############################################################################################
###
### Table producto_comentarios UI
###

### Configuring producto_comentarios

# TODO: Change all values of editable, insertable, indexable, etc to predicates
# TODO: in order to be able to give a behaviour dependable of the user, or auth

modconfig['producto_comentarios'] = {
    'internal_fields' : [],
    'dict_default': { '' : '<python:locale id="general.submit" />','comentario_idcomentario' : '','producto_idproducto' : '',
                     '__objname__': 'catalogos.catalogos',
                     '__tabname__': 'producto_comentarios',
                     #'__fulltextsearch__': '', # disables the fulltext search in the modify frame
                    },
    'editable': lambda: True,
    'insertable': lambda: True,
    'indexable': lambda: True,
    'structured_search': lambda: True,
    'ftindex': ft3k.sqlindex(name='producto_comentarios', conn=dbftindex),
    'ftsearch_size': 10,
    'count': 30,
    'list_fields': ['producto_idproducto', 'comentario_idcomentario'],
    'preview_fields': ['producto_idproducto', 'comentario_idcomentario'],
    'list_with_empty_condition': lambda: True,
    'control_generator': _control_generator,
    'preview_maxsize': 64,
    'output_table_header': lambda r: "<tr>%s</tr>"%("<th>%s</th>"%("</th><th>".join(r))),
    'output_table_row': lambda r, q, fields: "<tr>%s</tr>"%("<td>%s</td>"%("</td><td>".join(r))),
    'output_table_control_row': lambda r, controls, q, fields: '<tr><td style="text-align: right;" colspan="%s">%s</td></tr>'%(len(r), " &nbsp; ".join(controls)),
    'output_table': lambda r, query, fields: """<table style="width: 100%%"><tbody>%s</tbody></table>"""%("".join(r)),
    'output_showpage': lambda title, data, id: ('insert',
                                                       {'title': title,
                                                        'data': data,
                                                        'id': id,
                                                        'rowclasses': (['cell1', 'cell1', 'cell2', 'cell2'], 'header')
                                                       }),
    'select': _list_select,
    'foreign_mapper': {u'comentario_idcomentario': (u'comentario', u'id'), u'producto_idproducto': (u'producto', u'xid')},
    'form_element': template3k.get_template("""producto_comentarios:form_element""",
                                            """
            <strong>Producto Comentarios</strong>
            <form method="POST" enctype="multipart/form-data" id="form-producto_comentarios<python:keyword id='__id__' />" action="javascript: void(0);" onsubmit="<python:keyword id="__onsubmit__" />" name="producto_comentarios" >
                <table><tbody>
            
<tr><td>
<python:locale id="producto_comentarios.producto_idproducto">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Producto Idproducto'} ]
</python:locale>:</td><td><select onfocus="callto({'rawhandler': createhandlerselect('<python:keyword id="producto_idproducto" />'), 'handlerargs': this},'<python:keyword id="__objname__" />','preview_producto',['xid'])" name="producto_idproducto:int"  id="select-producto_xid_<python:keyword id='__id__' />" ></select>
<a style="text-decoration: none;" href="javascript: DOM3K_replaceListOptions([['','']],'select-producto_xid_<python:keyword id="__id__" />'); void(0);"><python:locale id="actions.clear" /></a><python:keyword id="__editmode__">

/
<a style="text-decoration: none;" href="javascript: showformmodify('<python:keyword id="__objname__" />', 'producto'); void(0);"><python:locale id="actions.new" /></a></python:keyword>

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_producto', ['choose_search','select-producto_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search" /></a><python:keyword id="__fulltextsearch__">

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_fulltext_producto', ['choose_search','select-producto_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search_fulltext" /></a></python:keyword>

</td></tr>


<tr><td>
<python:locale id="producto_comentarios.comentario_idcomentario">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Comentario Idcomentario'} ]
</python:locale>:</td><td><select onfocus="callto({'rawhandler': createhandlerselect('<python:keyword id="comentario_idcomentario" />'), 'handlerargs': this},'<python:keyword id="__objname__" />','preview_comentario',['id'])" name="comentario_idcomentario:int"  id="select-comentario_id_<python:keyword id='__id__' />" ></select>
<a style="text-decoration: none;" href="javascript: DOM3K_replaceListOptions([['','']],'select-comentario_id_<python:keyword id="__id__" />'); void(0);"><python:locale id="actions.clear" /></a><python:keyword id="__editmode__">

/
<a style="text-decoration: none;" href="javascript: showformmodify('<python:keyword id="__objname__" />', 'comentario'); void(0);"><python:locale id="actions.new" /></a></python:keyword>

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_comentario', ['choose_search','select-comentario_id_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search" /></a><python:keyword id="__fulltextsearch__">

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_fulltext_comentario', ['choose_search','select-comentario_id_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search_fulltext" /></a></python:keyword>

</td></tr>

<tr><td><input type="submit" name=":submit" value="<python:locale id="general.submit" />" /></td></tr>
                </tbody></table>
            </form>
            <python:keyword id="__bottom__"></python:keyword>
            """),
    'form_element_search': template3k.get_template("""producto_comentarios:form_element_search""",
                                                   """
            <strong>Producto Comentarios</strong>
            <form method="POST" enctype="multipart/form-data" id="form-producto_comentarios<python:keyword id='__id__' />" action="javascript: void(0);" onsubmit="<python:keyword id="__onsubmit__" />" name="producto_comentarios" >
                <table><tbody>
            
<tr><td>
<python:locale id="producto_comentarios.producto_idproducto">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Producto Idproducto'} ]
</python:locale>:</td><td><select onfocus="callto({'rawhandler': createhandlerselect('<python:keyword id="producto_idproducto" />'), 'handlerargs': this},'<python:keyword id="__objname__" />','preview_producto',['xid'])" name="producto_idproducto:int"  id="select-producto_xid_<python:keyword id='__id__' />" ></select>
<a style="text-decoration: none;" href="javascript: DOM3K_replaceListOptions([['','']],'select-producto_xid_<python:keyword id="__id__" />'); void(0);"><python:locale id="actions.clear" /></a><python:keyword id="__editmode__">

/
<a style="text-decoration: none;" href="javascript: showformmodify('<python:keyword id="__objname__" />', 'producto'); void(0);"><python:locale id="actions.new" /></a></python:keyword>

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_producto', ['choose_search','select-producto_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search" /></a><python:keyword id="__fulltextsearch__">

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_fulltext_producto', ['choose_search','select-producto_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search_fulltext" /></a></python:keyword>

</td></tr>


<tr><td>
<python:locale id="producto_comentarios.comentario_idcomentario">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Comentario Idcomentario'} ]
</python:locale>:</td><td><select onfocus="callto({'rawhandler': createhandlerselect('<python:keyword id="comentario_idcomentario" />'), 'handlerargs': this},'<python:keyword id="__objname__" />','preview_comentario',['id'])" name="comentario_idcomentario:int"  id="select-comentario_id_<python:keyword id='__id__' />" ></select>
<a style="text-decoration: none;" href="javascript: DOM3K_replaceListOptions([['','']],'select-comentario_id_<python:keyword id="__id__" />'); void(0);"><python:locale id="actions.clear" /></a><python:keyword id="__editmode__">

/
<a style="text-decoration: none;" href="javascript: showformmodify('<python:keyword id="__objname__" />', 'comentario'); void(0);"><python:locale id="actions.new" /></a></python:keyword>

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_comentario', ['choose_search','select-comentario_id_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search" /></a><python:keyword id="__fulltextsearch__">

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_fulltext_comentario', ['choose_search','select-comentario_id_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search_fulltext" /></a></python:keyword>

</td></tr>

<tr><td><input type="submit" name=":submit" value="<python:locale id="general.submit" />" /></td></tr>
                </tbody></table>
            </form>
            <python:keyword id="__bottom__"></python:keyword>
            """),
    '__sentinel__': True
    }

### End configuration

###
### Beginning methods/functions

#_spyro_authentication.perms['form_producto_comentarios'] = False
#_spyro_authentication.perms['list_producto_comentarios'] = False
#_spyro_authentication.perms['modify_producto_comentarios'] = False
#_spyro_authentication.perms['delete_producto_comentarios'] = False
 
def list_view_producto_comentarios(*args, **kwargs):
    tabname = 'producto_comentarios'
    config = _get_config(None, tabname)
    kwargs['config'] = config
    return (config['output_showpage'](tabname, _list(tabname, *args, **kwargs), tabname), )

def form_producto_comentarios(mode = None, condition_dict = None, **kwargs):
    tabname = 'producto_comentarios'
    kwargs['config'] = _get_config(None, tabname)
    return _form(tabname, mode, condition_dict, **kwargs)

def modify_producto_comentarios(mode, data, cond, dbconn=None, doreturn=1, **kwargs):
    tabname = 'producto_comentarios'
    config = _get_config(None, tabname)
    assert config['editable'](), "View only entity, can't %s"%mode
    tabname = 'producto_comentarios'
    return _modify(tabname, mode, data, cond, dbconn, doreturn, config=config, **kwargs)

def delete_producto_comentarios(cond, dbconn = None, doreturn = 1, **kwargs):
    tabname = 'producto_comentarios'
    config = _get_config(None, tabname)
    assert config['editable'](), "View only entity, can't delete"
    return _delete(tabname, cond, dbconn, doreturn, config=config, **kwargs)

def list_producto_comentarios(*args, **kwargs):
    tabname = 'producto_comentarios'
    kwargs['config'] = _get_config(None, tabname)
    # assert _spyro_authentication._rawauth(auth,  objname, list_producto_comentarios)
    return _list(tabname, *args, **kwargs)

def form_search_producto_comentarios(method=None, *args, **kwargs):
    tabname = 'producto_comentarios'
    config = _get_config(None, tabname)
    dict_default = dict(config['dict_default']) # dictionary
    dict_default['__editmode__'] = ""
    if method == "choose_search":
        idtag = args[0]
        residtag = 'choose-search-%s'%idtag
        dict_default['__onsubmit__'] = (r"callto({'showresults': function(d){ DOM3K_updateElement(d,'%s'); """
                                        r"""DOM3K_BackgroundRowColors('%s',['rgb(200,200,230);','inherit']);}}, """
                                        r"""'%s','choose_search_%s',null,{'condition_dict': Easyform_formToDict(this), 'idtag': %s });"""
                                        r""" """%(residtag,residtag,objname,tabname,repr(idtag)))
        dict_default['__bottom__'] = """<div id="%s"></div>"""%residtag
    else:
        dict_default['__onsubmit__'] = r"callto({'handler': handle_commands}, '%s','search_%s',[Easyform_formToDict(this), null, 'LIKE']);"%(objname,tabname)
    dict_default['__id__'] = str(time.time())
    x = config['form_element_search'].merge(dict_default)
    return x

def manage_fulltext_producto_comentarios(drop_index, reindex):
    tabname = 'producto_comentarios'
    config = _get_config(None, tabname)
    ftindex = config['ftindex']
    if drop_index:
        ftindex.drop_index()
    if reindex:
        ftindex.create_index()
        for x in _preview(tabname, config['list_fields'][0], config=config, preview_maxsize=512):
            _rowid = _get_rowid(tabname, x)
            ftindex.handle_insert(x[1], _rowid)

def form_search_fulltext_producto_comentarios(output='list', idtag='', query='',**kwargs):
    tabname = 'producto_comentarios'
    dict_default = {'query': query, 'output': output, 'idtag': idtag}
    if output == "choose_search":
        residtag = 'choose-search-%s'%idtag
        dict_default['__onsubmit__'] = (r"""callto({'handler': handle_commands_frames()}, '%s', 'search_fulltext_%s', """
                                        r"""null, Easyform_formToDict(this));"""%(objname, tabname))
        dict_default['__bottom__'] = """<div id="%s"></div>"""%residtag
        dict_default['__idtag__'] = """<input name="idtag:string:mandatory" value="%s" type="hidden" />"""%idtag
        dict_default['residtag'] = residtag
    else:
        dict_default['residtag'] = tabname
        dict_default['__onsubmit__'] = r"callto({'handler': handle_commands}, '%s', 'search_fulltext_%s', null, Easyform_formToDict(this));"%(objname, tabname)
        dict_default['__idtag__'] = ""
        dict_default['__bottom__'] = ""
    dict_default['__id__'] = str(time.time())
    return """<form action="javascript: void(0);" onsubmit="%(__onsubmit__)s">
    <input name="output:string:mandatory" value="%(output)s" type="hidden" />
    <input name="query:string:mandatory" value="%(query)s" size="40" type="text" />
    <input name="residtag:string" value="residtag" type="hidden" />
    %(__idtag__)s
    <input name=":submit" value="Search" type="submit" />
    </form>
    %(__bottom__)s
    """%dict_default

def search_fulltext_producto_comentarios(query, output="", idtag="", residtag="", **kwargs):
    tabname = 'producto_comentarios'
    kwargs['config'] = None
    return _search_fulltext(tabname, query, output, idtag, residtag, **kwargs)


def search_producto_comentarios(cond,orderby=None,usecondmethod=None,**kwargs):
    tabname = 'producto_comentarios'
    config = _get_config(None, tabname)
    x = list_producto_comentarios(cond, orderby, usecondmethod=usecondmethod)
    return (config['output_showpage']('Search results', x, tabname), )

def choose_search_producto_comentarios(idtag, condition_dict = None, method="LIKE", outmethod="simple",
                                                  previewsize=10, list_condition_dict = None, **kwargs):
    tabname = 'producto_comentarios'
    config = _get_config(None, tabname)
    kwargs['config'] = config
    return _choose_search(tabname, idtag, condition_dict, method, outmethod, previewsize, list_condition_dict, **kwargs)

def preview_producto_comentarios(*args, **kwargs):
    tabname = 'producto_comentarios'
    kwargs['config'] = _get_config(None, tabname)
    # kwargs['display'] = 'comentario_idcomentario'
    return _preview(tabname, *args, **kwargs)

###
### end of producto_comentarios UI
###
###############################################################################################
###############################################################################################
###############################################################################################
###############################################################################################
###
### Table tipo_pago UI
###

### Configuring tipo_pago

# TODO: Change all values of editable, insertable, indexable, etc to predicates
# TODO: in order to be able to give a behaviour dependable of the user, or auth

modconfig['tipo_pago'] = {
    'internal_fields' : [],
    'dict_default': { 'descripcion' : '','nombre' : '','' : '<python:locale id="general.submit" />',
                     '__objname__': 'catalogos.catalogos',
                     '__tabname__': 'tipo_pago',
                     #'__fulltextsearch__': '', # disables the fulltext search in the modify frame
                    },
    'editable': lambda: True,
    'insertable': lambda: True,
    'indexable': lambda: True,
    'structured_search': lambda: True,
    'ftindex': ft3k.sqlindex(name='tipo_pago', conn=dbftindex),
    'ftsearch_size': 10,
    'count': 30,
    'list_fields': ['nombre', 'descripcion'],
    'preview_fields': ['nombre', 'descripcion'],
    'list_with_empty_condition': lambda: True,
    'control_generator': _control_generator,
    'preview_maxsize': 64,
    'output_table_header': lambda r: "<tr>%s</tr>"%("<th>%s</th>"%("</th><th>".join(r))),
    'output_table_row': lambda r, q, fields: "<tr>%s</tr>"%("<td>%s</td>"%("</td><td>".join(r))),
    'output_table_control_row': lambda r, controls, q, fields: '<tr><td style="text-align: right;" colspan="%s">%s</td></tr>'%(len(r), " &nbsp; ".join(controls)),
    'output_table': lambda r, query, fields: """<table style="width: 100%%"><tbody>%s</tbody></table>"""%("".join(r)),
    'output_showpage': lambda title, data, id: ('insert',
                                                       {'title': title,
                                                        'data': data,
                                                        'id': id,
                                                        'rowclasses': (['cell1', 'cell1', 'cell2', 'cell2'], 'header')
                                                       }),
    'select': _list_select,
    'foreign_mapper': {},
    'form_element': template3k.get_template("""tipo_pago:form_element""",
                                            """
            <strong>Tipo Pago</strong>
            <form method="POST" enctype="multipart/form-data" id="form-tipo_pago<python:keyword id='__id__' />" action="javascript: void(0);" onsubmit="<python:keyword id="__onsubmit__" />" name="tipo_pago" >
                <table><tbody>
            
<tr><td>
<python:locale id="tipo_pago.nombre">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Nombre'} ]
</python:locale>:</td><td><input name="nombre:string:mandatory" type="text" value="<python:keyword id="nombre" />" size="40" /></td></tr>


<tr><td>
<python:locale id="tipo_pago.descripcion">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Descripcion'} ]
</python:locale>:</td><td><input name="descripcion:string:mandatory" type="text" value="<python:keyword id="descripcion" />" size="40" /></td></tr>

<tr><td><input type="submit" name=":submit" value="<python:locale id="general.submit" />" /></td></tr>
                </tbody></table>
            </form>
            <python:keyword id="__bottom__"></python:keyword>
            """),
    'form_element_search': template3k.get_template("""tipo_pago:form_element_search""",
                                                   """
            <strong>Tipo Pago</strong>
            <form method="POST" enctype="multipart/form-data" id="form-tipo_pago<python:keyword id='__id__' />" action="javascript: void(0);" onsubmit="<python:keyword id="__onsubmit__" />" name="tipo_pago" >
                <table><tbody>
            
<tr><td>
<python:locale id="tipo_pago.nombre">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Nombre'} ]
</python:locale>:</td><td><input name="nombre:string" type="text" value="<python:keyword id="nombre" />" size="40" /></td></tr>


<tr><td>
<python:locale id="tipo_pago.descripcion">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Descripcion'} ]
</python:locale>:</td><td><input name="descripcion:string" type="text" value="<python:keyword id="descripcion" />" size="40" /></td></tr>

<tr><td><input type="submit" name=":submit" value="<python:locale id="general.submit" />" /></td></tr>
                </tbody></table>
            </form>
            <python:keyword id="__bottom__"></python:keyword>
            """),
    '__sentinel__': True
    }

### End configuration

###
### Beginning methods/functions

#_spyro_authentication.perms['form_tipo_pago'] = False
#_spyro_authentication.perms['list_tipo_pago'] = False
#_spyro_authentication.perms['modify_tipo_pago'] = False
#_spyro_authentication.perms['delete_tipo_pago'] = False
 
def list_view_tipo_pago(*args, **kwargs):
    tabname = 'tipo_pago'
    config = _get_config(None, tabname)
    kwargs['config'] = config
    return (config['output_showpage'](tabname, _list(tabname, *args, **kwargs), tabname), )

def form_tipo_pago(mode = None, condition_dict = None, **kwargs):
    tabname = 'tipo_pago'
    kwargs['config'] = _get_config(None, tabname)
    return _form(tabname, mode, condition_dict, **kwargs)

def modify_tipo_pago(mode, data, cond, dbconn=None, doreturn=1, **kwargs):
    tabname = 'tipo_pago'
    config = _get_config(None, tabname)
    assert config['editable'](), "View only entity, can't %s"%mode
    tabname = 'tipo_pago'
    return _modify(tabname, mode, data, cond, dbconn, doreturn, config=config, **kwargs)

def delete_tipo_pago(cond, dbconn = None, doreturn = 1, **kwargs):
    tabname = 'tipo_pago'
    config = _get_config(None, tabname)
    assert config['editable'](), "View only entity, can't delete"
    return _delete(tabname, cond, dbconn, doreturn, config=config, **kwargs)

def list_tipo_pago(*args, **kwargs):
    tabname = 'tipo_pago'
    kwargs['config'] = _get_config(None, tabname)
    # assert _spyro_authentication._rawauth(auth,  objname, list_tipo_pago)
    return _list(tabname, *args, **kwargs)

def form_search_tipo_pago(method=None, *args, **kwargs):
    tabname = 'tipo_pago'
    config = _get_config(None, tabname)
    dict_default = dict(config['dict_default']) # dictionary
    dict_default['__editmode__'] = ""
    if method == "choose_search":
        idtag = args[0]
        residtag = 'choose-search-%s'%idtag
        dict_default['__onsubmit__'] = (r"callto({'showresults': function(d){ DOM3K_updateElement(d,'%s'); """
                                        r"""DOM3K_BackgroundRowColors('%s',['rgb(200,200,230);','inherit']);}}, """
                                        r"""'%s','choose_search_%s',null,{'condition_dict': Easyform_formToDict(this), 'idtag': %s });"""
                                        r""" """%(residtag,residtag,objname,tabname,repr(idtag)))
        dict_default['__bottom__'] = """<div id="%s"></div>"""%residtag
    else:
        dict_default['__onsubmit__'] = r"callto({'handler': handle_commands}, '%s','search_%s',[Easyform_formToDict(this), null, 'LIKE']);"%(objname,tabname)
    dict_default['__id__'] = str(time.time())
    x = config['form_element_search'].merge(dict_default)
    return x

def manage_fulltext_tipo_pago(drop_index, reindex):
    tabname = 'tipo_pago'
    config = _get_config(None, tabname)
    ftindex = config['ftindex']
    if drop_index:
        ftindex.drop_index()
    if reindex:
        ftindex.create_index()
        for x in _preview(tabname, config['list_fields'][0], config=config, preview_maxsize=512):
            _rowid = _get_rowid(tabname, x)
            ftindex.handle_insert(x[1], _rowid)

def form_search_fulltext_tipo_pago(output='list', idtag='', query='',**kwargs):
    tabname = 'tipo_pago'
    dict_default = {'query': query, 'output': output, 'idtag': idtag}
    if output == "choose_search":
        residtag = 'choose-search-%s'%idtag
        dict_default['__onsubmit__'] = (r"""callto({'handler': handle_commands_frames()}, '%s', 'search_fulltext_%s', """
                                        r"""null, Easyform_formToDict(this));"""%(objname, tabname))
        dict_default['__bottom__'] = """<div id="%s"></div>"""%residtag
        dict_default['__idtag__'] = """<input name="idtag:string:mandatory" value="%s" type="hidden" />"""%idtag
        dict_default['residtag'] = residtag
    else:
        dict_default['residtag'] = tabname
        dict_default['__onsubmit__'] = r"callto({'handler': handle_commands}, '%s', 'search_fulltext_%s', null, Easyform_formToDict(this));"%(objname, tabname)
        dict_default['__idtag__'] = ""
        dict_default['__bottom__'] = ""
    dict_default['__id__'] = str(time.time())
    return """<form action="javascript: void(0);" onsubmit="%(__onsubmit__)s">
    <input name="output:string:mandatory" value="%(output)s" type="hidden" />
    <input name="query:string:mandatory" value="%(query)s" size="40" type="text" />
    <input name="residtag:string" value="residtag" type="hidden" />
    %(__idtag__)s
    <input name=":submit" value="Search" type="submit" />
    </form>
    %(__bottom__)s
    """%dict_default

def search_fulltext_tipo_pago(query, output="", idtag="", residtag="", **kwargs):
    tabname = 'tipo_pago'
    kwargs['config'] = None
    return _search_fulltext(tabname, query, output, idtag, residtag, **kwargs)


def search_tipo_pago(cond,orderby=None,usecondmethod=None,**kwargs):
    tabname = 'tipo_pago'
    config = _get_config(None, tabname)
    x = list_tipo_pago(cond, orderby, usecondmethod=usecondmethod)
    return (config['output_showpage']('Search results', x, tabname), )

def choose_search_tipo_pago(idtag, condition_dict = None, method="LIKE", outmethod="simple",
                                                  previewsize=10, list_condition_dict = None, **kwargs):
    tabname = 'tipo_pago'
    config = _get_config(None, tabname)
    kwargs['config'] = config
    return _choose_search(tabname, idtag, condition_dict, method, outmethod, previewsize, list_condition_dict, **kwargs)

def preview_tipo_pago(*args, **kwargs):
    tabname = 'tipo_pago'
    kwargs['config'] = _get_config(None, tabname)
    # kwargs['display'] = 'descripcion'
    return _preview(tabname, *args, **kwargs)

###
### end of tipo_pago UI
###
###############################################################################################
###############################################################################################
###############################################################################################
###############################################################################################
###
### Table pagos UI
###

### Configuring pagos

# TODO: Change all values of editable, insertable, indexable, etc to predicates
# TODO: in order to be able to give a behaviour dependable of the user, or auth

modconfig['pagos'] = {
    'internal_fields' : [],
    'dict_default': { '' : '<python:locale id="general.submit" />','txid' : '','idventa' : '','costo' : '','type' : '','fecha_pago' : '',
                     '__objname__': 'catalogos.catalogos',
                     '__tabname__': 'pagos',
                     #'__fulltextsearch__': '', # disables the fulltext search in the modify frame
                    },
    'editable': lambda: True,
    'insertable': lambda: True,
    'indexable': lambda: True,
    'structured_search': lambda: True,
    'ftindex': ft3k.sqlindex(name='pagos', conn=dbftindex),
    'ftsearch_size': 10,
    'count': 30,
    'list_fields': ['idventa', 'fecha_pago', 'costo', 'txid', 'type'],
    'preview_fields': ['idventa', 'fecha_pago', 'costo', 'txid', 'type'],
    'list_with_empty_condition': lambda: True,
    'control_generator': _control_generator,
    'preview_maxsize': 64,
    'output_table_header': lambda r: "<tr>%s</tr>"%("<th>%s</th>"%("</th><th>".join(r))),
    'output_table_row': lambda r, q, fields: "<tr>%s</tr>"%("<td>%s</td>"%("</td><td>".join(r))),
    'output_table_control_row': lambda r, controls, q, fields: '<tr><td style="text-align: right;" colspan="%s">%s</td></tr>'%(len(r), " &nbsp; ".join(controls)),
    'output_table': lambda r, query, fields: """<table style="width: 100%%"><tbody>%s</tbody></table>"""%("".join(r)),
    'output_showpage': lambda title, data, id: ('insert',
                                                       {'title': title,
                                                        'data': data,
                                                        'id': id,
                                                        'rowclasses': (['cell1', 'cell1', 'cell2', 'cell2'], 'header')
                                                       }),
    'select': _list_select,
    'foreign_mapper': {u'idventa': (u'venta', u'xid'), u'type': (u'tipo_pago', u'nombre')},
    'form_element': template3k.get_template("""pagos:form_element""",
                                            """
            <strong>Pagos</strong>
            <form method="POST" enctype="multipart/form-data" id="form-pagos<python:keyword id='__id__' />" action="javascript: void(0);" onsubmit="<python:keyword id="__onsubmit__" />" name="pagos" >
                <table><tbody>
            
<tr><td>
<python:locale id="pagos.idventa">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Idventa'} ]
</python:locale>:</td><td><select onfocus="callto({'rawhandler': createhandlerselect('<python:keyword id="idventa" />'), 'handlerargs': this},'<python:keyword id="__objname__" />','preview_venta',['xid'])" name="idventa:int"  id="select-venta_xid_<python:keyword id='__id__' />" ></select>
<a style="text-decoration: none;" href="javascript: DOM3K_replaceListOptions([['','']],'select-venta_xid_<python:keyword id="__id__" />'); void(0);"><python:locale id="actions.clear" /></a><python:keyword id="__editmode__">

/
<a style="text-decoration: none;" href="javascript: showformmodify('<python:keyword id="__objname__" />', 'venta'); void(0);"><python:locale id="actions.new" /></a></python:keyword>

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_venta', ['choose_search','select-venta_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search" /></a><python:keyword id="__fulltextsearch__">

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_fulltext_venta', ['choose_search','select-venta_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search_fulltext" /></a></python:keyword>

</td></tr>


<tr><td>
<python:locale id="pagos.fecha_pago">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Fecha Pago'} ]
</python:locale>:</td><td>
            <python:locale id="date.format_date" /> 
            <python:locale id="date.format_time" /> 
            <input type="text" name="fecha_pago:datetime:" value="<python:keyword id="fecha_pago" />" onclick="Easyform_frameReadDatetime(DOM3K_readInput(this), DOM3K_updateInput, this, event.clientY, event.clientX)"  />
            </td></tr>


<tr><td>
<python:locale id="pagos.costo">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Costo'} ]
</python:locale>:</td><td><input name="costo:float:" type="text" value="<python:keyword id="costo" />" size="40" /></td></tr>


<tr><td>
<python:locale id="pagos.txid">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Txid'} ]
</python:locale>:</td><td><input name="txid:string:" type="text" value="<python:keyword id="txid" />" size="40" /></td></tr>


<tr><td>
<python:locale id="pagos.type">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Type'} ]
</python:locale>:</td><td><select onfocus="callto({'rawhandler': createhandlerselect('<python:keyword id="type" />'), 'handlerargs': this},'<python:keyword id="__objname__" />','preview_tipo_pago',['nombre'])" name="type:string"  id="select-tipo_pago_nombre_<python:keyword id='__id__' />" ></select>
<a style="text-decoration: none;" href="javascript: DOM3K_replaceListOptions([['','']],'select-tipo_pago_nombre_<python:keyword id="__id__" />'); void(0);"><python:locale id="actions.clear" /></a><python:keyword id="__editmode__">

/
<a style="text-decoration: none;" href="javascript: showformmodify('<python:keyword id="__objname__" />', 'tipo_pago'); void(0);"><python:locale id="actions.new" /></a></python:keyword>

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_tipo_pago', ['choose_search','select-tipo_pago_nombre_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search" /></a><python:keyword id="__fulltextsearch__">

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_fulltext_tipo_pago', ['choose_search','select-tipo_pago_nombre_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search_fulltext" /></a></python:keyword>

</td></tr>

<tr><td><input type="submit" name=":submit" value="<python:locale id="general.submit" />" /></td></tr>
                </tbody></table>
            </form>
            <python:keyword id="__bottom__"></python:keyword>
            """),
    'form_element_search': template3k.get_template("""pagos:form_element_search""",
                                                   """
            <strong>Pagos</strong>
            <form method="POST" enctype="multipart/form-data" id="form-pagos<python:keyword id='__id__' />" action="javascript: void(0);" onsubmit="<python:keyword id="__onsubmit__" />" name="pagos" >
                <table><tbody>
            
<tr><td>
<python:locale id="pagos.idventa">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Idventa'} ]
</python:locale>:</td><td><select onfocus="callto({'rawhandler': createhandlerselect('<python:keyword id="idventa" />'), 'handlerargs': this},'<python:keyword id="__objname__" />','preview_venta',['xid'])" name="idventa:int"  id="select-venta_xid_<python:keyword id='__id__' />" ></select>
<a style="text-decoration: none;" href="javascript: DOM3K_replaceListOptions([['','']],'select-venta_xid_<python:keyword id="__id__" />'); void(0);"><python:locale id="actions.clear" /></a><python:keyword id="__editmode__">

/
<a style="text-decoration: none;" href="javascript: showformmodify('<python:keyword id="__objname__" />', 'venta'); void(0);"><python:locale id="actions.new" /></a></python:keyword>

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_venta', ['choose_search','select-venta_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search" /></a><python:keyword id="__fulltextsearch__">

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_fulltext_venta', ['choose_search','select-venta_xid_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search_fulltext" /></a></python:keyword>

</td></tr>


<tr><td>
<python:locale id="pagos.fecha_pago">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Fecha Pago'} ]
</python:locale>:</td><td>
            <python:locale id="date.format_date" /> 
            <python:locale id="date.format_time" /> 
            <input type="text" name="fecha_pago:datetime:" value="<python:keyword id="fecha_pago" />" onclick="Easyform_frameReadDatetime(DOM3K_readInput(this), DOM3K_updateInput, this, event.clientY, event.clientX)"  />
            </td></tr>


<tr><td>
<python:locale id="pagos.costo">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Costo'} ]
</python:locale>:</td><td><input name="costo:float:" type="text" value="<python:keyword id="costo" />" size="40" /></td></tr>


<tr><td>
<python:locale id="pagos.txid">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Txid'} ]
</python:locale>:</td><td><input name="txid:string:" type="text" value="<python:keyword id="txid" />" size="40" /></td></tr>


<tr><td>
<python:locale id="pagos.type">
[ {'encoding': 'utf8', 'locale': 'en', 'translation': 'Type'} ]
</python:locale>:</td><td><select onfocus="callto({'rawhandler': createhandlerselect('<python:keyword id="type" />'), 'handlerargs': this},'<python:keyword id="__objname__" />','preview_tipo_pago',['nombre'])" name="type:string"  id="select-tipo_pago_nombre_<python:keyword id='__id__' />" ></select>
<a style="text-decoration: none;" href="javascript: DOM3K_replaceListOptions([['','']],'select-tipo_pago_nombre_<python:keyword id="__id__" />'); void(0);"><python:locale id="actions.clear" /></a><python:keyword id="__editmode__">

/
<a style="text-decoration: none;" href="javascript: showformmodify('<python:keyword id="__objname__" />', 'tipo_pago'); void(0);"><python:locale id="actions.new" /></a></python:keyword>

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_tipo_pago', ['choose_search','select-tipo_pago_nombre_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search" /></a><python:keyword id="__fulltextsearch__">

/
<a style="text-decoration: none;" href="javascript: callto({'showresults': function(d) {DOM3K_createFrame(d,'title=\\'<python:locale id="actions.search_form" />\\' ');}},  '<python:keyword id="__objname__" />', 'form_search_fulltext_tipo_pago', ['choose_search','select-tipo_pago_nombre_<python:keyword id="__id__" />']); void(0);"><python:locale id="actions.search_fulltext" /></a></python:keyword>

</td></tr>

<tr><td><input type="submit" name=":submit" value="<python:locale id="general.submit" />" /></td></tr>
                </tbody></table>
            </form>
            <python:keyword id="__bottom__"></python:keyword>
            """),
    '__sentinel__': True
    }

### End configuration

###
### Beginning methods/functions

#_spyro_authentication.perms['form_pagos'] = False
#_spyro_authentication.perms['list_pagos'] = False
#_spyro_authentication.perms['modify_pagos'] = False
#_spyro_authentication.perms['delete_pagos'] = False
 
def list_view_pagos(*args, **kwargs):
    tabname = 'pagos'
    config = _get_config(None, tabname)
    kwargs['config'] = config
    return (config['output_showpage'](tabname, _list(tabname, *args, **kwargs), tabname), )

def form_pagos(mode = None, condition_dict = None, **kwargs):
    tabname = 'pagos'
    kwargs['config'] = _get_config(None, tabname)
    return _form(tabname, mode, condition_dict, **kwargs)

def modify_pagos(mode, data, cond, dbconn=None, doreturn=1, **kwargs):
    tabname = 'pagos'
    config = _get_config(None, tabname)
    assert config['editable'](), "View only entity, can't %s"%mode
    tabname = 'pagos'
    return _modify(tabname, mode, data, cond, dbconn, doreturn, config=config, **kwargs)

def delete_pagos(cond, dbconn = None, doreturn = 1, **kwargs):
    tabname = 'pagos'
    config = _get_config(None, tabname)
    assert config['editable'](), "View only entity, can't delete"
    return _delete(tabname, cond, dbconn, doreturn, config=config, **kwargs)

def list_pagos(*args, **kwargs):
    tabname = 'pagos'
    kwargs['config'] = _get_config(None, tabname)
    # assert _spyro_authentication._rawauth(auth,  objname, list_pagos)
    return _list(tabname, *args, **kwargs)

def form_search_pagos(method=None, *args, **kwargs):
    tabname = 'pagos'
    config = _get_config(None, tabname)
    dict_default = dict(config['dict_default']) # dictionary
    dict_default['__editmode__'] = ""
    if method == "choose_search":
        idtag = args[0]
        residtag = 'choose-search-%s'%idtag
        dict_default['__onsubmit__'] = (r"callto({'showresults': function(d){ DOM3K_updateElement(d,'%s'); """
                                        r"""DOM3K_BackgroundRowColors('%s',['rgb(200,200,230);','inherit']);}}, """
                                        r"""'%s','choose_search_%s',null,{'condition_dict': Easyform_formToDict(this), 'idtag': %s });"""
                                        r""" """%(residtag,residtag,objname,tabname,repr(idtag)))
        dict_default['__bottom__'] = """<div id="%s"></div>"""%residtag
    else:
        dict_default['__onsubmit__'] = r"callto({'handler': handle_commands}, '%s','search_%s',[Easyform_formToDict(this), null, 'LIKE']);"%(objname,tabname)
    dict_default['__id__'] = str(time.time())
    x = config['form_element_search'].merge(dict_default)
    return x

def manage_fulltext_pagos(drop_index, reindex):
    tabname = 'pagos'
    config = _get_config(None, tabname)
    ftindex = config['ftindex']
    if drop_index:
        ftindex.drop_index()
    if reindex:
        ftindex.create_index()
        for x in _preview(tabname, config['list_fields'][0], config=config, preview_maxsize=512):
            _rowid = _get_rowid(tabname, x)
            ftindex.handle_insert(x[1], _rowid)

def form_search_fulltext_pagos(output='list', idtag='', query='',**kwargs):
    tabname = 'pagos'
    dict_default = {'query': query, 'output': output, 'idtag': idtag}
    if output == "choose_search":
        residtag = 'choose-search-%s'%idtag
        dict_default['__onsubmit__'] = (r"""callto({'handler': handle_commands_frames()}, '%s', 'search_fulltext_%s', """
                                        r"""null, Easyform_formToDict(this));"""%(objname, tabname))
        dict_default['__bottom__'] = """<div id="%s"></div>"""%residtag
        dict_default['__idtag__'] = """<input name="idtag:string:mandatory" value="%s" type="hidden" />"""%idtag
        dict_default['residtag'] = residtag
    else:
        dict_default['residtag'] = tabname
        dict_default['__onsubmit__'] = r"callto({'handler': handle_commands}, '%s', 'search_fulltext_%s', null, Easyform_formToDict(this));"%(objname, tabname)
        dict_default['__idtag__'] = ""
        dict_default['__bottom__'] = ""
    dict_default['__id__'] = str(time.time())
    return """<form action="javascript: void(0);" onsubmit="%(__onsubmit__)s">
    <input name="output:string:mandatory" value="%(output)s" type="hidden" />
    <input name="query:string:mandatory" value="%(query)s" size="40" type="text" />
    <input name="residtag:string" value="residtag" type="hidden" />
    %(__idtag__)s
    <input name=":submit" value="Search" type="submit" />
    </form>
    %(__bottom__)s
    """%dict_default

def search_fulltext_pagos(query, output="", idtag="", residtag="", **kwargs):
    tabname = 'pagos'
    kwargs['config'] = None
    return _search_fulltext(tabname, query, output, idtag, residtag, **kwargs)


def search_pagos(cond,orderby=None,usecondmethod=None,**kwargs):
    tabname = 'pagos'
    config = _get_config(None, tabname)
    x = list_pagos(cond, orderby, usecondmethod=usecondmethod)
    return (config['output_showpage']('Search results', x, tabname), )

def choose_search_pagos(idtag, condition_dict = None, method="LIKE", outmethod="simple",
                                                  previewsize=10, list_condition_dict = None, **kwargs):
    tabname = 'pagos'
    config = _get_config(None, tabname)
    kwargs['config'] = config
    return _choose_search(tabname, idtag, condition_dict, method, outmethod, previewsize, list_condition_dict, **kwargs)

def preview_pagos(*args, **kwargs):
    tabname = 'pagos'
    kwargs['config'] = _get_config(None, tabname)
    # kwargs['display'] = 'fecha_pago'
    return _preview(tabname, *args, **kwargs)

###
### end of pagos UI
###
###############################################################################################
###############################################################################################
